Package: OhdsiReportGenerator
Type: Package
Title: Observational Health Data Sciences and Informatics Report
        Generator
Version: 2.2.0
Date: 2026-05-15
Authors@R: c(
  person("Jenna", "Reps", email = "jreps@its.jnj.com", role = c("aut", "cre")),
  person("Anthony", "Sena", email = "sena@ohdsi.org", role = c("aut"))
  )
Maintainer: Jenna Reps <jreps@its.jnj.com>
Description: Extract results into R from the Observational Health Data Sciences and Informatics result database (see <https://ohdsi.github.io/Strategus/results-schema/index.html>) and generate reports/presentations via 'quarto' that summarize results in HTML format. Learn more about 'OhdsiReportGenerator' at <https://ohdsi.github.io/OhdsiReportGenerator/>.
License: Apache License 2.0
URL: https://ohdsi.github.io/OhdsiReportGenerator/,
        https://github.com/OHDSI/OhdsiReportGenerator
BugReports: https://github.com/OHDSI/OhdsiReportGenerator/issues
VignetteBuilder: knitr
Depends: R (>= 4.1.0)
Imports: CirceR, DatabaseConnector, forestploter, dplyr, ggplot2,
        grDevices, grid, gt, gtExtras, kableExtra, ParallelLogger,
        quarto, rlang, rmarkdown, SqlRender, tidyr
Suggests: htmltools, knitr, markdown, ResultModelManager, RSQLite,
        testthat (>= 3.0.0)
RoxygenNote: 7.3.3
Encoding: UTF-8
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-16 02:02:23 UTC; jreps
Author: Jenna Reps [aut, cre],
  Anthony Sena [aut]
Repository: CRAN
Date/Publication: 2026-05-16 23:10:02 UTC
Built: R 4.6.0; ; 2026-05-17 01:28:48 UTC; unix

# Generated by roxygen2: do not edit by hand

export(addTarColumn)
export(createCharacterizationIndexes)
export(createCohortIndexes)
export(createIncidenceIndexes)
export(createPredictionReport)
export(createSccsIndexes)
export(formatBinaryCovariateName)
export(generateFullReport)
export(generateSummaryPredictionReport)
export(getAnalysisCohorts)
export(getBinaryCaseSeries)
export(getBinaryRiskFactors)
export(getBinaryTargetBaseline)
export(getCMEstimation)
export(getCaseCounts)
export(getCaseTargetCounts)
export(getCharacterizationCohortBinary)
export(getCharacterizationCohortContinuous)
export(getCharacterizationDemographics)
export(getCharacterizationOutcomes)
export(getCharacterizationTargets)
export(getCmDiagnosticsData)
export(getCmMetaEstimation)
export(getCmNegativeControlEstimates)
export(getCmOutcomes)
export(getCmPropensityModel)
export(getCmTable)
export(getCmTargets)
export(getCohortAttrition)
export(getCohortCounts)
export(getCohortDefinitions)
export(getCohortInclusionRules)
export(getCohortInclusionStats)
export(getCohortInclusionSummary)
export(getCohortMeta)
export(getCohortSubsetAttrition)
export(getCohortSubsetDefinitions)
export(getContinuousCaseSeries)
export(getContinuousRiskFactors)
export(getDatabaseDetails)
export(getDechallengeRechallenge)
export(getDechallengeRechallengeFails)
export(getEventDuration)
export(getExampleConnectionDetails)
export(getFullPredictionPerformances)
export(getIncidenceOutcomes)
export(getIncidenceRates)
export(getIncidenceTargets)
export(getOutcomeTable)
export(getPredictionAggregateTopPredictors)
export(getPredictionCohorts)
export(getPredictionCovariates)
export(getPredictionDiagnosticTable)
export(getPredictionDiagnostics)
export(getPredictionHyperParamSearch)
export(getPredictionIntercept)
export(getPredictionLift)
export(getPredictionModelDesigns)
export(getPredictionOutcomes)
export(getPredictionPerformanceTable)
export(getPredictionPerformances)
export(getPredictionTargets)
export(getPredictionTopPredictors)
export(getSccsDiagnosticsData)
export(getSccsEstimation)
export(getSccsMetaEstimation)
export(getSccsModel)
export(getSccsNegativeControlEstimates)
export(getSccsOutcomes)
export(getSccsTable)
export(getSccsTargets)
export(getSccsTimeToEvent)
export(getSubsetText)
export(getTargetBinaryFeatures)
export(getTargetContinuousFeatures)
export(getTargetTable)
export(getTimeToEvent)
export(getTreatmentPathways)
export(kableDark)
export(plotAgeDistributions)
export(plotCmEstimates)
export(plotSccsEstimates)
export(plotSexDistributions)
export(processCohortDefinitionsForQuarto)
export(processCohorts)
export(removeSpaces)
export(restrictCohortDefinitionsForQuarto)
export(viewIncidenceRate)
importFrom(dplyr,"%>%")
importFrom(rlang,":=")
importFrom(rlang,.data)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)
library(dplyr)

## ----echo=TRUE----------------------------------------------------------------
library(OhdsiReportGenerator)
# create a connection details object to 
connectionDetails <- getExampleConnectionDetails()

# create a connection handler to the results
library(ResultModelManager)
ConnectionHandler <- ResultModelManager::ConnectionHandler$new(connectionDetails)


## ----echo=TRUE----------------------------------------------------------------

cohorts <- getCohortDefinitions(
    connectionHandler = ConnectionHandler,
    schema = 'main',
    cgTablePrefix = 'cg_',
    targetIds = NULL
)

knitr::kable(
  x = cohorts %>% 
    dplyr::select(-"json", -"sqlCommand"), 
  caption = 'The data.frame extracted containing the cohort details minus the json and sqlCommand columns.'
  )


## ----echo=TRUE----------------------------------------------------------------
if(nrow(cohorts) > 0){
processedCohorts <- processCohorts(cohorts)

knitr::kable(
  x = data.frame(
    parentId = processedCohorts$parents,
    parentName= names(processedCohorts$parents)
    ), 
  caption = 'The parent cohorts.'
  )

knitr::kable(
  x = processedCohorts$cohortList[[1]] %>% 
    dplyr::select(-"json", -"sqlCommand"),
  caption = 'The children/subset cohorts for the first parent cohort.'
  )
}


## ----echo=TRUE----------------------------------------------------------------

subsets <- getCohortSubsetDefinitions(
    connectionHandler = ConnectionHandler,
    schema = 'main',
    cgTablePrefix = 'cg_'
)

knitr::kable(
  x = subsets, 
  caption = 'The subset cohort logic used in the analysis.'
  )


## ----echo=TRUE----------------------------------------------------------------

tte <- getTimeToEvent(
    connectionHandler = ConnectionHandler,
    schema = 'main'
)

knitr::kable(
  x = tte %>% dplyr::filter(.data$timeScale == 'per 365-day'), 
  caption = 'Example time-to-event results for the 365-day scale.'
  )


## ----echo=TRUE----------------------------------------------------------------

drc <- getDechallengeRechallenge(
    connectionHandler = ConnectionHandler,
    schema = 'main'
)

knitr::kable(
  x = drc, 
  caption = 'Example dechallenge-rechallenge results.'
  )


## ----echo=TRUE----------------------------------------------------------------

ir <- getIncidenceRates(
    connectionHandler = ConnectionHandler,
    schema = 'main', 
    targetIds = 1
)

knitr::kable(
  x = ir %>% dplyr::filter(
    subgroupName == 'All' &
    ageGroupName == 'Any' &	
    genderName == 'Any' &	
    startYear == 'Any'
  ), 
  caption = 'Example incidence rate results.'
  )


## ----echo=TRUE----------------------------------------------------------------

rf <- getBinaryRiskFactors(
    connectionHandler = ConnectionHandler,
    schema = 'main', 
    targetId = 1, 
    outcomeId = 3
)

knitr::kable(
  x = rf, 
  caption = 'Example risk factors for binary features results.'
  )


## ----echo=TRUE----------------------------------------------------------------

rf <- getContinuousRiskFactors(
    connectionHandler = ConnectionHandler,
    schema = 'main', 
    targetId = 1, 
    outcomeId = 3
)

knitr::kable(
  x = rf, 
  caption = 'Example risk factors for continuous features results.'
  )


## ----echo=TRUE----------------------------------------------------------------

modelDesigns <- getPredictionModelDesigns(
    connectionHandler = ConnectionHandler,
    schema = 'main', 
    targetIds = 1002, 
    outcomeIds = 3
)

knitr::kable(
  x = modelDesigns %>%
    dplyr::select(
      "modelDesignId",	
      "modelType",
      "developmentTargetId",
      "developmentTargetName",
      "developmentOutcomeId",	
      "developmentOutcomeName",	
      "timeAtRisk"
    ), 
  caption = 'Example model designs for the prediction results.'
  )


## ----echo=TRUE----------------------------------------------------------------

perform <- getFullPredictionPerformances(
    connectionHandler = ConnectionHandler,
    schema = 'main', 
    modelDesignId = 1
)

knitr::kable(
  x = perform, 
  caption = 'Example performance for the prediction results.'
  )


## ----echo=TRUE----------------------------------------------------------------

top5 <- getPredictionTopPredictors(
    connectionHandler = ConnectionHandler,
    schema = 'main', 
    targetIds = 1002, 
    outcomeIds = 3,
    numberPredictors = 5
)

knitr::kable(
  x = top5 , 
  caption = 'Example top five predictors.'
  )


## ----echo=TRUE----------------------------------------------------------------

attrition <- getPredictionPerformanceTable(
    connectionHandler = ConnectionHandler,
    schema = 'main', 
    table = 'attrition',
    performanceId = 1
)

knitr::kable(
  x = attrition , 
  caption = 'Example prediction attrition for performance 1.'
  )


## ----echo=TRUE----------------------------------------------------------------

cmEst <- getCMEstimation(
    connectionHandler = ConnectionHandler,
    schema = 'main', 
    targetIds = 1002, 
    comparatorIds = 2002,
    outcomeIds = 3
)

knitr::kable(
  x = cmEst , 
  caption = 'Example cohort method estimation results.'
  )


## ----echo=TRUE----------------------------------------------------------------

cmMe <- getCmMetaEstimation(
    connectionHandler = ConnectionHandler,
    schema = 'main'
)

knitr::kable(
  x = cmMe , 
  caption = 'Example cohort method meta analysis estimation results.'
  )


## ----echo=TRUE----------------------------------------------------------------

plotCmEstimates(
  cmData = cmEst, 
  cmMeta = NULL,
  cohortNames = data.frame(
    cohortName = c('Example Target', 'Example Comp'),
    cohortId= c(1002,2002)
  ),
  selectedAnalysisId = 1
)

#' Extract the database used in the analyses
#' @description
#' This function extracts the databases and their information.
#'
#' @details
#' Specify the connectionHandler, the schema and the database table name
#'
#' @template connectionHandler
#' @template schema
#' @template databaseTable
#' @family Database
#' @return
#' Returns a data.frame with the columns:
#' \itemize{
#'  \item{databaseFullName the full name of the database}
#'  \item{databaseName the friendly name of the database}
#'  \item{cdmHolder the license holder of the database}
#'  \item{sourceDescription a description of the database}
#'  \item{sourceDocumentationReference a link to the database information document}
#'  \item{cdmEtlReference a link to the ETL document}
#'  \item{sourceReleaseDatethe release date for the source database}
#'  \item(cdmReleaseDate the release date for the database mapped to the OMOP CDM)
#'  \item{cdmVersion the OMOP CDM version of the database}
#'  \item{cdmVersionConceptId the CDM version concept ID}
#'  \item{vocabularyVersion the database's vocabulary version}
#'  \item{databaseId a unique identifier for the database}
#'  \item{maxObsPeriodEndDate the last observational period end date in the database}
#'  } 
#' 
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' ir <- getIncidenceRates(
#' connectionHandler = connectionHandler, 
#' schema = 'main'
#' )
#'
getDatabaseDetails <- function(
    connectionHandler,
    schema,
    databaseTable = 'database_meta_data'
    ){
  
  sql <- "SELECT 
  cdm_source_name as database_full_name,
  cdm_source_abbreviation as database_name,
  cdm_holder,
  source_description,
  ISNULL(source_documentation_reference,'None') as source_documentation_reference,
  ISNULL(cdm_etl_reference,'None') as cdm_etl_reference,
  source_release_date,
  cdm_release_date,
  cdm_version,
  cdm_version_concept_id,
  vocabulary_version,
  database_id,
  max_obs_period_end_date
  
  from @schema.@database_table
  ;"

  result <- connectionHandler$queryDb(
      sql = sql,
      schema = schema,
      database_table = databaseTable
    )

  return(result)
}

test_that("getExampleConnectionDetails", {
  
  dbs <- getDatabaseDetails(
    connectionHandler = connectionHandler, 
    schema = schema
  )
  
  testthat::expect_true(inherits(dbs, 'data.frame'))
  testthat::expect_true(nrow(dbs) > 0)
  
})

#' A function to add indexes to the Cohort Generator results
#'
#' @details
#' Specify the connectionHandler, the schema and the prefixes
#'
#' @template connectionHandler
#' @template schema
#' @template cgTablePrefix
#' @family Indexes
#' 
#' @return
#' NULL
#'
#' @export
createCohortIndexes <- function(
    connectionHandler,
    schema,
    cgTablePrefix = 'cg_'
){
  
  cgVersion <- round(
    .getCgVersion(
      connectionHandler = connectionHandler,
      schema = schema,
      cgTablePrefix = cgTablePrefix
    )
  )
  
  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/cohort/indexesCgV", cgVersion, ".sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
  connectionHandler$executeSql(
    sql = sql,
    schema = schema,
    cg_table_prefix = cgTablePrefix
  )
  
  return(invisible(NULL))
  
}


#' A function to add indexes to the incidence results
#'
#' @details
#' Specify the connectionHandler, the schema and the prefixes
#'
#' @template connectionHandler
#' @template schema
#' @template ciTablePrefix
#' @family Indexes
#' 
#' @return
#' NULL
#'
#' @export
createIncidenceIndexes <- function(
    connectionHandler,
    schema,
    ciTablePrefix = 'ci_'
){
  
  ciVersion <- .getCIVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    ciTablePrefix = ciTablePrefix
  )
  
  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/characterization/indexesCiV", ciVersion, ".sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
  connectionHandler$executeSql(
    sql = sql,
    schema = schema,
    ci_table_prefix = ciTablePrefix
  )
    
  return(invisible(NULL))
  
}


#' A function to add indexes to the Characterization results
#'
#' @details
#' Specify the connectionHandler, the schema and the prefixes
#'
#' @template connectionHandler
#' @template schema
#' @template cTablePrefix
#' @family Indexes
#' 
#' @return
#' NULL
#'
#' @export
createCharacterizationIndexes <- function(
    connectionHandler,
    schema,
    cTablePrefix = 'c_'
){
  
  tteVersion <- 0
  
  cVersion <- .getCVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cTablePrefix = cTablePrefix
  )
  
  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/characterization/indexesCharacterizationV", cVersion, ".sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
  connectionHandler$executeSql(
    sql = sql,
    schema = schema,
    c_table_prefix = cTablePrefix
  )
  
  # now add the time to event optimization table
  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/characterization/tteOptimizationV", tteVersion, ".sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
  connectionHandler$executeSql(
    sql = sql,
    schema = schema,
    c_table_prefix = cTablePrefix
  )
  
  return(invisible(NULL))
  
}


#' A function to add indexes to the SCCS results
#'
#' @details
#' Specify the connectionHandler, the schema and the prefixes
#'
#' @template connectionHandler
#' @template schema
#' @template sccsTablePrefix
#' @family Indexes
#' 
#' @return
#' NULL
#'
#' @export
createSccsIndexes <- function(
    connectionHandler,
    schema,
    sccsTablePrefix = 'sccs_'
){
  
  sccsVersion <- 0
  
  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/estimation/indexesSccsV", sccsVersion, ".sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
  connectionHandler$executeSql(
    sql = sql,
    schema = schema,
    sccs_table_prefix = sccsTablePrefix
  )
  
  return(invisible(NULL))
  
}


# CM has no required indexes 

test_that("Characterization indexes", {
  
  # now add indexes
  nullReturn <- createCharacterizationIndexes(
    connectionHandler = connectionHandler, 
    schema = schema, 
    cTablePrefix = 'c_'
  )
  testthat::expect_true(is.null(nullReturn))

  cTargets <- getCharacterizationTargets(
    connectionHandler = connectionHandler, 
    schema = schema, 
    cTablePrefix = 'c_', 
    cgTablePrefix = 'cg_'
  )
  
  testthat::expect_true(nrow(cTargets) >0)
  
})


test_that("Cohort indexes", {
  
  # now add indexes
  nullReturn <- createCohortIndexes(
    connectionHandler = connectionHandler, 
    schema = schema, 
    cgTablePrefix = 'cg_'
  )
  testthat::expect_true(is.null(nullReturn))
  
  ccounts <- getCohortCounts(
    connectionHandler = connectionHandler, 
    schema = schema, 
    cgTablePrefix = 'cg_'
  )
  
  testthat::expect_true(nrow(ccounts) >0)
  
})

test_that("Cohort incidence indexes", {
  
  # now add indexes
  nullReturn <- createIncidenceIndexes(
    connectionHandler = connectionHandler, 
    schema = schema, 
    ciTablePrefix = 'ci_'
  )
  testthat::expect_true(is.null(nullReturn))
  
  cinc <- getIncidenceRates(
    connectionHandler = connectionHandler, 
    schema = schema, 
    ciTablePrefix = 'ci_',
    cgTablePrefix = 'cg_'
  )
  
  testthat::expect_true(nrow(cinc) >0)
  
})



test_that("createSccsIndexes indexes", {
  
  # now add indexes
  nullReturn <- createSccsIndexes(
    connectionHandler = connectionHandler, 
    schema = schema, 
    sccsTablePrefix = 'sccs_'
  )
  testthat::expect_true(is.null(nullReturn))
  
  sccsE <- getSccsEstimation(
    connectionHandler = connectionHandler, 
    schema = schema, 
    sccsTablePrefix = 'sccs_',
    cgTablePrefix = 'cg_'
  )
  
  testthat::expect_true(nrow(sccsE) >0)
  
})

# cohort method

#' An internal function to determine the version of CohortMethod used 
#' to store results
#'
#' @details
#' Specify the connectionHandler, the schema and the prefixes. This
#' query will attempt to identify if CohortMethod v6 was used by 
#' inspecing the migration table. When the migration_order is >= 3
#' then v6 of CohortMethod was used.
#'
#' @template connectionHandler
#' @template schema
#' @template cmTablePrefix
#' @family Estimation
#' 
#' @return
#' A integer with the major version number of cohort method
#'
.getCmVersion <- function(
    connectionHandler,
    schema,
    cmTablePrefix = 'cm_'
){
  version <- 5 # Default to v5
  tryCatch(
    {
      sql <- "
      SELECT MAX(migration_order) max_migration_order
      FROM @schema.@cm_table_prefixmigration
      ;"
      
      maxMigrationOrder <- connectionHandler$queryDb(
        sql = sql,
        schema = schema,
        cm_table_prefix = cmTablePrefix
      ) %>%
        dplyr::pull(maxMigrationOrder) %>%
        dplyr::first()
      if (maxMigrationOrder >= 3) {
        version <- 6
      }
    },
    error = function(e) {
      # Do nothing - most likely the migration table does not exist so assume
      # CohortMethod v5
    }
  )
  return(version)
}

#' A function to extract the targets found in cohort method
#'
#' @details
#' Specify the connectionHandler, the schema and the prefixes
#'
#' @template connectionHandler
#' @template schema
#' @template cmTablePrefix
#' @template cgTablePrefix
#' @family Estimation
#' 
#' @return
#' A data.frame with the cohort method target cohort ids and names.
#'
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' cohorts <- getCmTargets(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getCmTargets <- function(
    connectionHandler,
    schema,
    cmTablePrefix = 'cm_',
    cgTablePrefix = 'cg_'
){
  cmVersion <- .getCmVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cmTablePrefix = cmTablePrefix
  )
  
  sql <- SqlRender::readSql(
    sourceFile = system.file(
      paste0("sql/sql_server/estimation/getCmTargetsV",cmVersion,".sql"),
      package = "OhdsiReportGenerator",
      mustWork = TRUE
    )
  )

  targets <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    cm_table_prefix = cmTablePrefix,
    cg_table_prefix = cgTablePrefix
  ) %>%
    tidyr::pivot_wider(
      id_cols = c("cohortName", "cohortDefinitionId"), 
      names_from = "type", 
      values_from = c("value")
    )
  
  return(targets)
  
}


#' A function to extract the outcomes found in cohort method
#'
#' @details
#' Specify the connectionHandler, the schema and the prefixes
#'
#' @template connectionHandler
#' @template schema
#' @template cmTablePrefix
#' @template cgTablePrefix
#' @template targetId
#' @family Estimation
#' 
#' @return
#' A data.frame with the cohort method outcome ids and names.
#'
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' outcomes <- getCmOutcomes(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getCmOutcomes <- function(
    connectionHandler,
    schema,
    cmTablePrefix = 'cm_',
    cgTablePrefix = 'cg_',
    targetId = NULL
){
  cmVersion <- .getCmVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cmTablePrefix = cmTablePrefix
  )
  
  sql <- SqlRender::readSql(
    sourceFile = system.file(
      paste0("sql/sql_server/estimation/getCmOutcomesV",cmVersion,".sql"),
      package = "OhdsiReportGenerator",
      mustWork = TRUE
    )
  )
  
  outcomes <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    cm_table_prefix = cmTablePrefix,
    cg_table_prefix = cgTablePrefix,
    use_target = !is.null(targetId),
    target_id = paste0(targetId, collapse = ',')
  ) %>%
    tidyr::pivot_wider(
      id_cols = c("cohortName", "cohortDefinitionId"), 
      names_from = "type", 
      values_from = c("value")
    )
  
  return(outcomes)
  
}

#' Extract the cohort method results 
#' @description
#' This function extracts the single database cohort method estimates for results that can be unblinded and have a calibrated RR
#'
#' @details
#' Specify the connectionHandler, the schema and the target/comparator/outcome cohort IDs
#'
#' @template connectionHandler
#' @template schema
#' @template cmTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @template targetIds
#' @template outcomeIds
#' @param indicationIds A vector of cohort ids for the indication (nesting id) to restrict to
#' @template comparatorIds
#' @family Estimation
#' @return
#' Returns a data.frame with the columns:
#' \itemize{
#'  \item{databaseName the name of the database}
#'  \item{databaseId the unqiue identifier of the database}
#'  \item{analysisId the analysis design unique identifier}
#'  \item{description the analysis design description}
#'  \item{targetName the target cohort name}
#'  \item{targetId the target cohort unique identifier}
#'  \item{comparatorName the comparator cohort name}
#'  \item{comparatorId the comparator cohort unique identifier}
#'  \item{outcomeName the outcome name}
#'  \item{outcomeId the outcome unique identifier}
#'  \item{calibratedRr the calibrated relative risk}
#'  \item{calibratedRrCi95Lb the calibrated relative risk 95 percent confidence interval lower bound}
#'  \item{calibratedRrCi95Ub the calibrated relative risk 95 percent confidence interval upper bound}
#'  \item{calibratedP the two sided calibrated p value}
#'  \item{calibratedOneSidedP the one sided calibrated p value}
#'  \item{calibratedLogRr the calibrated relative risk logged}
#'  \item{calibratedSeLogRr the standard error of the calibrated relative risk logged}
#'  \item{targetSubjects the number of people in the target cohort}
#'  \item{comparatorSubjects the number of people in the comparator cohort}
#'  \item{targetDays the total number of days at risk across the target cohort people}
#'  \item{comparatorDays the total number of days at risk across the comparator cohort people}
#'  \item{targetOutcomes the total number of outcomes occuring during the time at risk for the target cohort people}
#'  \item{comparatorOutcomes the total number of outcomes occuring during the time at risk for the comparator cohort people}
#'  \item{Unblind Whether the results passed diagnostics and were unblinded}
#'  \item{unblindForEvidenceSynthesis whether the results can be unblinded for the meta analysis.}
#'  \item{targetEstimator ...}
#'  } 
#' 
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' cmEst <- getCMEstimation(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main',
#'   targetIds = 1,
#'   outcomeIds = 3
#' )
#' 
getCMEstimation <- function(
    connectionHandler,
    schema,
    cmTablePrefix = 'cm_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    targetIds = NULL,
    outcomeIds = NULL,
    indicationIds = NULL,
    comparatorIds = NULL
){
  cmVersion <- .getCmVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cmTablePrefix = cmTablePrefix
  )
  
  sql <- SqlRender::readSql(
    sourceFile = system.file(
      paste0("sql/sql_server/estimation/getCMEstimationV",cmVersion,".sql"),
      package = "OhdsiReportGenerator",
      mustWork = TRUE
    )
  )
  
  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    database_table = databaseTable,
    cm_table_prefix = cmTablePrefix,
    cg_table_prefix = cgTablePrefix,
    outcome_id = paste0(outcomeIds, collapse = ','),
    restrict_outcome = !is.null(outcomeIds),
    target_id = paste0(targetIds, collapse = ','),
    restrict_target = !is.null(targetIds),
    indication_id = paste0(indicationIds, collapse = ','),
    restrict_indication = !is.null(indicationIds),
    comparator_id = paste(comparatorIds, collapse = ','),
    restrict_comparator = !is.null(comparatorIds)
  )
  
  return(result)
}

#' Extract the cohort method diagostic results 
#' @description
#' This function extracts the cohort method diagnostics that examine whether the analyses were sufficiently powered
#' and checks for different types of bias.
#'
#' @details
#' Specify the connectionHandler, the schema and the target/comparator/outcome cohort IDs
#'
#' @template connectionHandler
#' @template schema
#' @template cmTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @template targetIds
#' @template outcomeIds
#' @param indicationIds A vector of cohort ids for the indication (nesting id) to restrict to
#' @template comparatorIds
#' @param analysisIds An optional vector of analysisIds to filter to
#' @param databaseIds An optional vector of databaseIds to filter to
#' @family Estimation
#' @return
#' Returns a data.frame with the columns:
#' \itemize{
#'  \item{databaseName the name of the database}
#'  \item{databaseId the unqiue identifier of the database}
#'  \item{analysisId the analysis unique identifier}
#'  \item{description a description of the analysis}
#'  \item{targetName the target cohort name}
#'  \item{targetId the target cohort unique identifier}
#'  \item{comparatorName the comparator cohort name}
#'  \item{comparatorId the comparator cohort unique identifier}
#'  \item{outcomeName the outcome name}
#'  \item{outcomeId the outcome cohort unique identifier}
#'  \item{maxSdm max allowed standardized difference of means when comparing the target to the comparator after PS adjustment for the ballance diagnostic diagnostic to pass.}
#'  \item{sharedMaxSdm max allowed standardized difference of means when comparing the target to the comparator after PS adjustment for the ballance diagnostic diagnostic to pass.}
#'  \item{equipoise the bounds on the preference score to determine whether a subject is in equipoise.}
#'  \item{mdrr the maximum passable minimum detectable relative risk (mdrr) value.  If the mdrr is greater than this the diagnostics will fail.}
#'  \item{attritionFraction (depreciated) the minmum attrition before the diagnostics fails.}
#'  \item{ease The expected absolute systematic error (ease) measures residual bias.}
#'  \item{balanceDiagnostic whether the balance diagnostic passed or failed.}
#'  \item{sharedBalanceDiagnostic whether the shared balance diagnostic passed or failed.}
#'  \item{equipoiseDiagnostic whether the equipose diagnostic passed or failed.}
#'  \item{mdrrDiagnostic whether the mdrr (power) diagnostic passed or failed.}
#'  \item{attritionDiagnostic (depreciated) whether the attrition diagnostic passed or failed.}
#'  \item{easeDiagnostic whether the ease diagnostic passed or failed.}
#'  \item{unblindForEvidenceSynthesis whether the results can be unblinded for the meta analysis.}
#'  \item{unblind whether the results can be unblinded.}
#'  \item{summaryValue summary of diagnostics results. FAIL, PASS or number of warnings.}
#'  } 
#' 
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' cmDiag <- getCmDiagnosticsData(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main',
#'   targetIds = 1,
#'   outcomeIds = 3
#' )
#' 
getCmDiagnosticsData <- function(
    connectionHandler,
    schema,
    cmTablePrefix = 'cm_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    targetIds = NULL,
    outcomeIds = NULL,
    comparatorIds = NULL,
    indicationIds = NULL,
    analysisIds = NULL,
    databaseIds = NULL
){
  cmVersion <- .getCmVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cmTablePrefix = cmTablePrefix
  )
  
  sql <- SqlRender::readSql(
    sourceFile = system.file(
      paste0("sql/sql_server/estimation/getCmDiagnosticsDataV",cmVersion,".sql"),
      package = "OhdsiReportGenerator",
      mustWork = TRUE
    )
  )
  
  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    cm_table_prefix = cmTablePrefix,
    cg_table_prefix = cgTablePrefix,
    database_table = databaseTable,
    targets = paste0(targetIds, collapse = ','),
    use_target = !is.null(targetIds),
    comparators = paste0(comparatorIds, collapse = ','),
    use_comparator = !is.null(comparatorIds),
    outcomes = paste0(outcomeIds, collapse = ','),
    use_outcome = !is.null(outcomeIds),
    
    indication_ids = paste0(indicationIds, collapse = ','),
    use_indication = !is.null(indicationIds),

    database_ids = paste0("'",databaseIds,"'", collapse = ','),
    use_database = !is.null(databaseIds),
    analysis_ids = paste0(analysisIds, collapse = ','),
    use_analysis = !is.null(analysisIds)
  )
  
  # adding percent fail for summary
  result$summaryValue <- apply(
    X = result[, grep('Diagnostic', colnames(result))], 
    MARGIN = 1, 
    FUN = function(x){
      
      if(sum(x %in% c('FAIL'))>0){
        return('Fail')
      } else if(sum(x %in% c('WARNING')) >0){
        return(sum(x %in% c('WARNING')))
      } else{
        return('Pass')
      }
    }
  )
  
  result <- result %>% 
    dplyr::relocate("summaryValue", .after = "outcomeName")
  
  return(result)
}

#' Extract the cohort method meta analysis results
#' @description
#' This function extracts any meta analysis estimation results for cohort method.
#'
#' @details
#' Specify the connectionHandler, the schema and the target/comparator/outcome cohort IDs
#'
#' @template connectionHandler
#' @template schema
#' @template cmTablePrefix
#' @template cgTablePrefix
#' @template esTablePrefix
#' @template targetIds
#' @template outcomeIds
#' @param indicationIds A vector of cohort ids for the indication (nesting id) to restrict to
#' @template comparatorIds
#' @param includeOneSidedP This lets you extract from older results that do not have the one sided p by setting this to FALSE
#' @family Estimation
#' @return
#' Returns a data.frame with the columns:
#' \itemize{
#'  \item{databaseName the name of the database}
#'  \item{analysisId the analysis unique identifier}
#'  \item{description a description of the analysis}
#'  \item{targetName the target cohort name}
#'  \item{targetId the target cohort unique identifier}
#'  \item{comparatorName the comparator cohort name}
#'  \item{comparatorId the comparator cohort unique identifier}
#'  \item{outcomeName the outcome name}
#'  \item{outcomeId the outcome cohort unique identifier}
#'  \item{calibratedRr the calibrated relative risk}
#'  \item{calibratedRrCi95Lb the calibrated relative risk 95 percent confidence interval lower bound}
#'  \item{calibratedRrCi95Ub the calibrated relative risk 95 percent confidence interval upper bound}
#'  \item{calibratedP the two sided calibrated p value}
#'  \item{calibratedOneSidedP the one sided calibrated p value}
#'  \item{calibratedLogRr the calibrated relative risk logged}
#'  \item{calibratedSeLogRr the standard error of the calibrated relative risk logged}
#'  
#'  \item{targetSubjects the number of people in the target cohort across included database}
#'  \item{comparatorSubjects the number of people in the comparator cohort across included database}
#'  \item{targetDays the total number of days at risk across the target cohort people across included database}
#'  \item{comparatorDays the total number of days at risk across the comparator cohort people across included database}
#'  \item{targetOutcomes the total number of outcomes occuring during the time at risk for the target cohort people across included database}
#'  \item{comparatorOutcomes the total number of outcomes occuring during the time at risk for the comparator cohort people across included database}
#'  
#'  \item{unblind whether the results can be unblinded.}
#'  \item{nDatabases the number of databases included}
#'  } 
#' 
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' cmMeta <- getCmMetaEstimation(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main',
#'   targetIds = 1,
#'   outcomeIds = 3
#' )
#' 
getCmMetaEstimation <- function(
    connectionHandler,
    schema,
    cmTablePrefix = 'cm_',
    cgTablePrefix = 'cg_',
    esTablePrefix = 'es_',
    targetIds = NULL,
    outcomeIds = NULL,
    indicationIds = NULL,
    comparatorIds = NULL,
    includeOneSidedP = TRUE
){
  cmVersion <- .getCmVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cmTablePrefix = cmTablePrefix
  )
  
  sql <- SqlRender::readSql(
    sourceFile = system.file(
      paste0("sql/sql_server/estimation/getCmMetaEstimationV",cmVersion,".sql"),
      package = "OhdsiReportGenerator",
      mustWork = TRUE
    )
  )
  
  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    cm_table_prefix = cmTablePrefix,
    cg_table_prefix = cgTablePrefix,
    es_table_prefix = esTablePrefix,
    target_id = paste0(targetIds, collapse = ','),
    include_target = !is.null(targetIds),
    outcome_id = paste0(outcomeIds, collapse = ','),
    include_outcome = !is.null(outcomeIds),
    indication_id = paste0(indicationIds, collapse = ','),
    include_indication = !is.null(indicationIds),
    comparator_id = paste0(comparatorIds, collapse = ','),
    include_comparator = !is.null(comparatorIds),
    include_one_sided_p = includeOneSidedP
  )
  
  return(unique(result))
}

#' Extract the cohort method table specified
#' @description
#' This function extracts the specific cohort method table.
#'
#' @details
#' Specify the connectionHandler, the schema and optionally the target/comparator/outcome/analysis/database IDs
#'
#' @template connectionHandler
#' @template schema
#' @param table The result table to extract
#' @template cmTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @template targetIds
#' @param indicationIds The indications that the target & comparator was nested to
#' @template outcomeIds
#' @template comparatorIds
#' @param analysisIds the analysis IDs to restrict to 
#' @param databaseIds the database IDs to restrict to 
#' @family Estimation
#' @return
#' Returns a data.frame with the cohort method requested table
#' 
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' cmTable <- getCmTable(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main',
#'   table = 'attrition'
#' )
#' 
getCmTable <- function(
    connectionHandler,
    schema,
    table = c('attrition', 'follow_up_dist', 'interaction_result',
              'covariate_balance', 'kaplan_meier_dist', 'likelihood_profile',
              'preference_score_dist', 'propensity_model',
              'shared_covariate_balance')[1],
    cmTablePrefix = 'cm_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    targetIds = NULL,
    indicationIds = NULL,
    outcomeIds = NULL,
    comparatorIds = NULL,
    analysisIds = NULL,
    databaseIds = NULL
){
  
  # check table valid:
  if(!table %in% c('attrition', 'follow_up_dist', 'interaction_result',
                  'covariate_balance', 'kaplan_meier_dist', 'likelihood_profile',
                  'preference_score_dist', 'propensity_model',
                  'shared_covariate_balance')){
    stop('Invalid table')
  }
  
  useOutcome = !is.null(outcomeIds)
  # some tables dont have outcomeIds
  if(table %in% c('preference_score_dist', 'propensity_model',
     'shared_covariate_balance')){
    useOutcome = FALSE
  }
  
  addCovariateName <- table %in% c('shared_covariate_balance','covariate_balance')
  
  cmVersion <- .getCmVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cmTablePrefix = cmTablePrefix
  )
  
  sql <- SqlRender::readSql(
    sourceFile = system.file(
      paste0("sql/sql_server/estimation/getCmTableV",cmVersion,".sql"),
      package = "OhdsiReportGenerator",
      mustWork = TRUE
    )
  )
  
  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    cm_table_prefix = cmTablePrefix,
    cg_table_prefix = cgTablePrefix,
    table = table,
    target_id = paste0(targetIds, collapse = ','),
    include_target = !is.null(targetIds),
    outcome_id = paste0(outcomeIds, collapse = ','),
    include_outcome = useOutcome,
    comparator_id = paste0(comparatorIds, collapse = ','),
    include_comparator = !is.null(comparatorIds),
    indication_id = paste0(indicationIds, collapse = ','),
    include_indication = !is.null(indicationIds),
    database_id = paste0("'",databaseIds,"'", collapse = ','),
    include_database = !is.null(databaseIds),
    analysis_id = paste0(analysisIds, collapse = ','),
    include_analyses = !is.null(analysisIds),
    database_table = databaseTable,
    
    include_covariate_name = addCovariateName
  )
  
  
  # edit covariateName 
  ##if(addCovariateName){
  ##  result$covariateName
  ##}
  
  
  return(unique(result))
}

# add cohort and database names:

#' Extract the cohort method negative controls
#' @description
#' This function extracts the cohort method negative control table.
#'
#' @details
#' Specify the connectionHandler, the schema and optionally the target/comparator/outcome/analysis/database IDs
#'
#' @template connectionHandler
#' @template schema
#' @template cmTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @template targetIds
#' @template comparatorIds
#' @param indicationIds The indications that the target & comparator was nested to
#' @param analysisIds the analysis IDs to restrict to 
#' @param databaseIds the database IDs to restrict to 
#' @param excludePositiveControls Whether to exclude the positive controls 
#' @family Estimation
#' @return
#' Returns a data.frame with the cohort method negative controls
#' 
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' cmNc <- getCmNegativeControlEstimates(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getCmNegativeControlEstimates <- function(
  connectionHandler,
  schema,
  cmTablePrefix = 'cm_',
  cgTablePrefix = 'cg_',
  databaseTable = 'database_meta_data',
  targetIds = NULL,
  comparatorIds = NULL,
  indicationIds = NULL,
  analysisIds = NULL,
  databaseIds = NULL,
  excludePositiveControls = TRUE
){
  cmVersion <- .getCmVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cmTablePrefix = cmTablePrefix
  )
  
  sql <- SqlRender::readSql(
    sourceFile = system.file(
      paste0("sql/sql_server/estimation/getCmNegativeControlEstimatesV",cmVersion,".sql"),
      package = "OhdsiReportGenerator",
      mustWork = TRUE
    )
  )
  
  results <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    cm_table_prefix = cmTablePrefix,
    
    use_target = !is.null(targetIds),
    target_ids = paste0(targetIds, collapse = ','),
    use_comparator = !is.null(comparatorIds),
    comparator_ids = paste0(comparatorIds, collapse = ','),
    use_indication = !is.null(indicationIds),
    indication_ids = paste0(indicationIds, collapse = ','),
    

    use_analysis = !is.null(analysisIds),
    analysis_ids = paste0(analysisIds, collapse = ','),
    use_database = !is.null(databaseIds),
    database_ids = paste0("'",databaseIds,"'", collapse = ','),
    exclude_positive_controls = excludePositiveControls
  )
  
  return(results)
}


#' Extract the cohort method model
#' @description
#' This function extracts the cohort method model.
#'
#' @details
#' Specify the connectionHandler, the schema and optionally the target/comparator/analysis/database IDs
#'
#' @template connectionHandler
#' @template schema
#' @template cmTablePrefix
#' @template targetId
#' @param comparatorId the comparator ID of interest
#' @param indicationId The indications that the target & comparator was nested to
#' @param analysisId the analysis ID to restrict to 
#' @param databaseId the database ID to restrict to 
#' @family Estimation
#' @return
#' Returns a data.frame with the cohort method model
#' 
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' cmModel <- getCmPropensityModel(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getCmPropensityModel <- function(
    connectionHandler, 
    schema,
    cmTablePrefix = 'cm_',
    targetId = NULL, 
    comparatorId = NULL,
    indicationId = NULL,
    analysisId = NULL, 
    databaseId = NULL
) {
  
  if(is.null(targetId)){
    message('Please specify targetId')
    return(NULL)
  }
  
  cmVersion <- .getCmVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cmTablePrefix = cmTablePrefix
  )
  
  sql <- SqlRender::readSql(
    sourceFile = system.file(
      paste0("sql/sql_server/estimation/getCmPropensityModelV",cmVersion,".sql"),
      package = "OhdsiReportGenerator",
      mustWork = TRUE
    )
  )
  
  model <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    cm_table_prefix = cmTablePrefix,
    target_id = targetId,
    comparator_id = comparatorId,
    use_indication = !is.null(indicationId),
    indication_id = indicationId,
    analysis_id = analysisId,
    database_id = databaseId
  )
  
  model <- model %>%
    dplyr::arrange(dplyr::desc(abs(.data$coefficient)))
  
  return(model)
}


#' An internal function to determine the version of SCCS used 
#' to store results
#'
#' @details
#' Specify the connectionHandler, the schema and the prefixes. This
#' query will attempt to identify if SelfControlledCaseSeries v6.1.4 was used by 
#' inspecing the migration table. When the migration_order is >= 3
#' then v6.1.4 of SelfControlledCaseSeries was used.
#'
#' @template connectionHandler
#' @template schema
#' @template sccsTablePrefix
#' @family Estimation
#' 
#' @return
#' A integer with the major version number of cohort method
#'
.getSccsVersion <- function(
    connectionHandler,
    schema,
    sccsTablePrefix = 'sccs_'
){
  version <- 0 # Default to v0
  tryCatch(
    {
      sql <- "
      SELECT MAX(migration_order) max_migration_order
      FROM @schema.@sccs_table_prefixmigration
      ;"
      
      maxMigrationOrder <- connectionHandler$queryDb(
        sql = sql,
        schema = schema,
        sccs_table_prefix = sccsTablePrefix
      ) %>%
        dplyr::pull(maxMigrationOrder) %>%
        dplyr::first()
      if (maxMigrationOrder >= 3) {
        version <- '6_1_4'
      }
    },
    error = function(e) {
      # Do nothing - most likely the migration table does not exist so assume
      # older SCCS
    }
  )
  return(version)
}

#' A function to extract the targets found in self controlled case series
#'
#' @details
#' Specify the connectionHandler, the schema and the prefixes
#'
#' @template connectionHandler
#' @template schema
#' @template sccsTablePrefix
#' @template cgTablePrefix
#' @family Estimation
#' 
#' @return
#' A data.frame with the self controlled case series target cohort ids and names.
#'
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' cohorts <- getSccsTargets(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getSccsTargets <- function(
    connectionHandler,
    schema,
    sccsTablePrefix = 'sccs_',
    cgTablePrefix = 'cg_'
){
  
  sql <- "SELECT distinct 
    cd.cohort_name,
    sc.era_id as cohort_definition_id, 
    'selfControlledCaseSeries' as type,
    1 as value

       FROM
      
      @schema.@sccs_table_prefixresult as sr
      
      INNER JOIN 
      @schema.@sccs_table_prefixcovariate sc ON (
    sc.exposures_outcome_set_id = sr.exposures_outcome_set_id AND
    sc.database_id = sr.database_id AND
    sc.analysis_id = sr.analysis_id AND
    sc.covariate_id = sr.covariate_id
  )
          
      inner join @schema.@cg_table_prefixcohort_definition cd
      
      on sc.era_id = cd.cohort_definition_id
      ;"
  
  targets <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    sccs_table_prefix = sccsTablePrefix,
    cg_table_prefix = cgTablePrefix
  ) %>%
    tidyr::pivot_wider(
      id_cols = c("cohortName", "cohortDefinitionId"), 
      names_from = "type", 
      values_from = c("value")
    )
  
  return(targets)
  
}


#' A function to extract the outcomes found in self controlled case series
#'
#' @details
#' Specify the connectionHandler, the schema and the prefixes
#'
#' @template connectionHandler
#' @template schema
#' @template sccsTablePrefix
#' @template cgTablePrefix
#' @template targetId
#' @family Estimation
#' 
#' @return
#' A data.frame with the self controlled case series outcome ids and names.
#'
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' outcomes <- getSccsOutcomes(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getSccsOutcomes <- function(
    connectionHandler,
    schema,
    sccsTablePrefix = 'sccs_',
    cgTablePrefix = 'cg_',
    targetId = NULL
){
  
  sql <- "SELECT distinct 
    cd.cohort_name,
    eos.outcome_id as cohort_definition_id, 
    'selfControlledCaseSeries' as type,
    1 as value

       FROM
      
      @schema.@sccs_table_prefixresult as sr
      
      inner join
      
      @schema.@sccs_table_prefixexposures_outcome_set as eos
      on eos.exposures_outcome_set_id = sr.exposures_outcome_set_id
      
      inner join
      @schema.@sccs_table_prefixcovariate sc ON (
    sc.exposures_outcome_set_id = sr.exposures_outcome_set_id AND
    sc.database_id = sr.database_id AND
    sc.analysis_id = sr.analysis_id AND
    sc.covariate_id = sr.covariate_id
  )
  
  inner join
  @schema.@sccs_table_prefixexposure e ON
  (
  e.exposures_outcome_set_id = sc.exposures_outcome_set_id AND
  e.era_id = sc.era_id
  )


      inner join @schema.@cg_table_prefixcohort_definition cd
      
      on eos.outcome_id = cd.cohort_definition_id
      
      where e.true_effect_size is NULL
  {@use_target}?{ AND sc.era_id in (@target_id)}
      
      ;"
  
  outcomes <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    sccs_table_prefix = sccsTablePrefix,
    cg_table_prefix = cgTablePrefix,
    use_target = !is.null(targetId),
    target_id = paste0(targetId, collapse = ',')
  ) %>%
    tidyr::pivot_wider(
      id_cols = c("cohortName", "cohortDefinitionId"), 
      names_from = "type", 
      values_from = c("value")
    )
  
  return(outcomes)
  
}

#' Extract the self controlled case series (sccs) results 
#' @description
#' This function extracts the single database sccs estimates
#'
#' @details
#' Specify the connectionHandler, the schema and the target/outcome cohort IDs
#'
#' @template connectionHandler
#' @template schema
#' @template sccsTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @template targetIds
#' @template outcomeIds
#' @family Estimation
#' @return
#' Returns a data.frame with the columns:
#' \itemize{
#'  \item{databaseName the database name}
#'  \item{databaseId the database id}
#'  \item{exposuresOutcomeSetId the exposure outcome set identifier}
#'  \item{analysisId the analysis unique identifier}
#'  \item{description an analysis description}
#'  \item{targetName the target name}
#'  \item{targetId the target cohort id}
#'  \item{outcomeName the outcome name}
#'  \item{outcomeId the outcome cohort id}
#'  \item{indicationName the indication name}
#'  \item{indicatonId the indication cohort id}
#'  \item{covariateName whether main or secondary analysis}
#'  \item{covariateId the analysis id}
#'  \item{outcomeSubjects The number of subjects with at least one outcome.}
#'  \item{outcomeEvents The number of outcome events.}
#'  \item{outcomeObservationPeriods The number of observation periods containing at least one outcome.}
#'  \item{covariateSubjects The number of subjects having the covariate.}
#'  \item{covariateDays The total covariate time in days.}
#'  \item{covariateEras The number of continuous eras of the covariate.}
#'  \item{covariateOutcomes The number of outcomes observed during the covariate time.}
#'  \item{observedDays The number of days subjects were observed.}
#'  \item{rr the relative risk}
#'  \item{ci95Lb the lower bound of the 95 percent confidence interval for the relative risk}
#'  \item{ci95Ub the upper bound of the 95 percent confidence interval for the relative risk}
#'  \item{p the p-value for the relative risk}
#'  \item{logRr the log of the relative risk}
#'  \item{seLogRr the standard error or the log of the relative risk}
#'  \item{calibratedRr the calibrated relative risk}
#'  \item{calibratedCi95Lb the lower bound of the 95 percent confidence interval for the calibrated relative risk}
#'  \item{calibratedCi95Ub the upper bound of the 95 percent confidence interval for the calibrated relative risk}
#'  \item{calibratedP the calibrated p-value}
#'  \item{calibratedOneSidedP the calibrated one sided p-value}
#'  \item{calibratedLogRr the calibrated log of the relative risk}
#'  \item{calibratedSeLogRr the calibrated log of the relative risk standard error}
#'  \item{llr The log of the likelihood ratio (of the MLE vs the null hypothesis of no effect).}
#'  \item{mdrr The minimum detectable relative risk.}
#'  \item{unblind Whether the results can be unblinded}
#'  \item{unblindForEvidenceSynthesis whether the results can be unblinded for the meta analysis.}
#'  } 
#' 
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' sccsEst <- getSccsEstimation(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main',
#'   targetIds = 1,
#'   outcomeIds = 3
#' )
#' 
getSccsEstimation <- function(
    connectionHandler,
    schema,
    sccsTablePrefix = 'sccs_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    targetIds = NULL,
    outcomeIds = NULL
) {

  sql <- "
  SELECT
  
    ds.cdm_source_abbreviation as database_name,
    ds.database_id,
    sr.exposures_outcome_set_id,
    sr.analysis_id,
    a.description,
    cg2.cohort_name as target_name,
    sc.era_id as target_id,
    cg1.cohort_name as outcome_name,
    eos.outcome_id,
    cg3.cohort_name as indication_name,
    eos.nesting_cohort_id as indication_id,
    sc.covariate_name,
    sc.covariate_id,
    
  sr.outcome_subjects,
  sr.outcome_events,
  sr.outcome_observation_periods,
  sr.covariate_subjects, 
  sr.covariate_days,
  sr.covariate_eras,
  sr.covariate_outcomes,
  sr.observed_days,

  case when COALESCE(sds.unblind, 0) = 0 then NULL else sr.rr end rr,
  case when COALESCE(sds.unblind, 0) = 0 then NULL else sr.ci_95_lb end ci_95_lb,
  case when COALESCE(sds.unblind, 0) = 0 then NULL else sr.ci_95_ub end ci_95_ub,
  case when COALESCE(sds.unblind, 0) = 0 then NULL else sr.p end p,
  case when COALESCE(sds.unblind, 0) = 0 then NULL else sr.log_rr end log_rr,
  case when COALESCE(sds.unblind, 0) = 0 then NULL else sr.se_log_rr end se_log_rr,
  case when COALESCE(sds.unblind, 0) = 0 then NULL else sr.calibrated_rr end calibrated_rr,
  case when COALESCE(sds.unblind, 0) = 0 then NULL else sr.calibrated_ci_95_lb end calibrated_ci_95_lb,
  case when COALESCE(sds.unblind, 0) = 0 then NULL else sr.calibrated_ci_95_ub end calibrated_ci_95_ub,
  case when COALESCE(sds.unblind, 0) = 0 then NULL else sr.calibrated_p end calibrated_p,
  case when COALESCE(sds.unblind, 0) = 0 then NULL else sr.calibrated_one_sided_p end calibrated_one_sided_p,
  case when COALESCE(sds.unblind, 0) = 0 then NULL else sr.calibrated_log_rr end calibrated_log_rr,
  case when COALESCE(sds.unblind, 0) = 0 then NULL else sr.calibrated_se_log_rr end calibrated_se_log_rr,
  
  case when COALESCE(sds.unblind, 0) = 0 then NULL else sr.llr end llr,


    sds.mdrr,
  --sds.ease,
  --sds.time_trend_p,
  --sds.pre_exposure_p,
  --sds.mdrr_diagnostic,
  --sds.ease_diagnostic,
  --sds.time_trend_diagnostic,
  --sds.pre_exposure_diagnostic,
  sds.unblind,
  sds.unblind_for_evidence_synthesis
  
  FROM @schema.@sccs_table_prefixresult sr
  INNER JOIN 
  @schema.@database_table ds 
  ON sr.database_id = ds.database_id
  INNER JOIN 
  @schema.@sccs_table_prefixdiagnostics_summary sds ON (
    sds.exposures_outcome_set_id = sr.exposures_outcome_set_id AND
    sds.database_id = sr.database_id AND
    sds.analysis_id = sr.analysis_id AND
    sds.covariate_id = sr.covariate_id
  )
  INNER JOIN 
  @schema.@sccs_table_prefixcovariate sc ON (
    sc.exposures_outcome_set_id = sr.exposures_outcome_set_id AND
    sc.database_id = sr.database_id AND
    sc.analysis_id = sr.analysis_id AND
    sc.covariate_id = sr.covariate_id
  )
  INNER JOIN @schema.@sccs_table_prefixexposures_outcome_set eos
  ON 
  eos.exposures_outcome_set_id = sr.exposures_outcome_set_id
  INNER JOIN
  @schema.@sccs_table_prefixanalysis a
  on a.analysis_id = sr.analysis_id
  
  inner join 
  @schema.@cg_table_prefixcohort_definition cg1
	on cg1.cohort_definition_id = eos.outcome_id
	
	inner join 
  @schema.@cg_table_prefixcohort_definition cg2
	on cg2.cohort_definition_id = sc.era_id
	
	 left join
   @schema.@cg_table_prefixcohort_definition as cg3
   on eos.nesting_cohort_id = cg3.cohort_definition_id
  
  WHERE
  1 = 1 
  {@restrict_outcome}?{ and eos.outcome_id IN (@outcome_ids)}
  {@restrict_target}?{ and sc.era_id IN (@exposure_ids)}
  ;
  "
  
  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    database_table = databaseTable,
    sccs_table_prefix = sccsTablePrefix,
    cg_table_prefix = cgTablePrefix,
    #database_ids = paste(quoteLiterals(databaseIds), collapse = ','),
    #analysis_ids = paste(analysisIds, collapse = ','),
    outcome_ids = paste(outcomeIds, collapse = ','),
    restrict_outcome = !is.null(outcomeIds),
    exposure_ids = paste(targetIds, collapse = ','),
    restrict_target = !is.null(targetIds)
  )
  
  return(result)
}

#' Extract the self controlled case series (sccs) diagostic results 
#' @description
#' This function extracts the sccs diagnostics that examine whether the analyses were sufficiently powered
#' and checks for different types of bias.
#'
#' @details
#' Specify the connectionHandler, the schema and the target/outcome cohort IDs
#'
#' @template connectionHandler
#' @template schema
#' @template sccsTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @template targetIds
#' @template outcomeIds
#' @family Estimation
#' @return
#' Returns a data.frame with the columns:
#' \itemize{
#'  \item{databaseName the database name}
#'  \item{databaseId the unique identifier for the database}
#'  \item{analysisId the analysis unique identifier}
#'  \item{description an analysis description}
#'  \item{targetName the target name}
#'  \item{targetId the target cohort id}
#'  \item{outcomeName the outcome name}
#'  \item{outcomeId the outcome cohort id}
#'  \item{indicationName the indication name}
#'  \item{indicatonId the indication cohort id}
#'  \item{covariateName whether main or secondary analysis}
#'  \item{mdrr the maximum passable minimum detectable relative risk (mdrr) value.  If the mdrr is greater than this the diagnostics will fail.}
#'  \item{ease The expected absolute systematic error (ease) measures residual bias.}
#'  \item{timeTrendP (Depreciated to timeStabilityP) The p for whether the mean monthly ratio between observed and expected is no greater than 1.25.}
#'  \item{preExposureP (Depreciated) One-sided p-value for whether the rate before expore is higher than after, against the null of no difference.}
#'  \item{mdrrDiagnostic whether the mdrr (power) diagnostic passed or failed.}
#'  \item{easeDiagnostic whether the ease diagnostic passed or failed.}
#'  \item{timeStabilityP (New) The p for whether the mean monthly ratio between observed and expected exceeds the specified threshold.}
#'  \item{eventExposureLb (New) Lower bound of the 95\% CI for the pre-expososure estimate.}
#'  \item{eventExposureUb (New) Upper bound of the 95\% CI for the pre-expososure estimate.}
#'  \item{eventObservationLb (New) Lower bound of the 95\% CI for the end of observation probe estimate.}
#'  \item{eventObservationUb (New) Upper bound of the 95\% CI for the end of observation probe estimate.}
#'  \item{rareOutcomePrevalence (New) The proportion of people in the underlying population who have the outcome at least once.}
#'  \item{timeTrendDiagnostic (Depreciated) Pass / warning / fail / not evaluated classification of the time trend (unstalbe months) diagnostic.}
#'  \item{preExposureDiagnostic (Depreciated) Pass / warning / fail / not evaluated classification of the time trend (unstalbe months) diagnostic.}
#'  \item{timeStabilityDiagnostic (New) Pass / fail / not evaluated classification of the time stability diagnostic.}
#'  \item{eventExposureDiagnostic (New) Pass / fail / not evaluated classification of the event-exposure independence diagnostic.}
#'  \item{eventObservationDiagnostic (New) Pass / fail / not evaluated classification of the event-observation period dependence diagnostic.}
#'  \item{rareOutcomeDiagnostic (New) Pass / fail / not evaluated classification of the rare outcome diagnostic.}
#'  \item{unblind whether the results can be unblinded.}
#'  \item{unblindForEvidenceSynthesis whether the results can be unblinded for the meta analysis.}

#'  \item{summaryValue summary of diagnostics results. FAIL, PASS or number of warnings.}
#'  } 
#' 
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' sccsDiag <- getSccsDiagnosticsData(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main',
#'   targetIds = 1,
#'   outcomeIds = 3
#' )
#' 
getSccsDiagnosticsData <- function(
    connectionHandler,
    schema,
    sccsTablePrefix = 'sccs_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    targetIds = NULL,
    outcomeIds = NULL
) {
  
 sccsVersion <- .getSccsVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    sccsTablePrefix = sccsTablePrefix
  )
  
  sql <- SqlRender::readSql(
    sourceFile = system.file(
      paste0("sql/sql_server/estimation/getSccsMetaEstimationV",sccsVersion,".sql"),
      package = "OhdsiReportGenerator",
      mustWork = TRUE
    )
  )


  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    cg_table_prefix = 'cg_',
    sccs_table_prefix = 'sccs_',
    database_table = 'database_meta_data',
    target_ids = paste0(targetIds, collapse = ','),
    restrict_target = !is.null(targetIds),
    outcome_ids = paste0(outcomeIds, collapse = ','),
    restrict_outcome = !is.null(outcomeIds)
  )
  
  
  # restrict to columns in the output description
  columnsToInclude <- c(
    "databaseName","databaseId","analysisId","description", 
    "targetName","targetId","outcomeName","outcomeId","indicationName", 
    "indicationId","covariateName","mdrr","ease","timeTrendP","preExposureP", 
    "timeStabilityP","eventExposureLb","eventExposureUb","eventObservationLb", 
    "eventObservationUb","rareOutcomePrevalence","mdrrDiagnostic","easeDiagnostic", 
    "timeTrendDiagnostic","preExposureDiagnostic","timeStabilityDiagnostic", 
    "eventExposureDiagnostic","eventObservationDiagnostic","rareOutcomeDiagnostic",
    "unblind","unblindForEvidenceSynthesis"
    )
  
  columnsToInclude <- columnsToInclude[columnsToInclude %in% colnames(result)]
  result <- result[, columnsToInclude]
  
  result$summaryValue <- apply(
    X = result[, grep('Diagnostic', colnames(result))], 
    MARGIN = 1, 
    FUN = function(x){
      
      if(sum(x %in% c('FAIL'))>0){
        return('Fail')
      } else if(sum(x %in% c('WARNING')) >0){
        return(sum(x %in% c('WARNING'), na.rm = TRUE))
      } else{
        return('Pass')
      }
    }
  )
  
  result <- result %>% 
    dplyr::relocate("summaryValue", .after = "covariateName")
  
  return(result)  
  
}

#' Extract the self controlled case series (sccs) meta analysis results
#' @description
#' This function extracts any meta analysis estimation results for sccs.
#'
#' @details
#' Specify the connectionHandler, the schema and the targetoutcome cohort IDs
#'
#' @template connectionHandler
#' @template schema
#' @template sccsTablePrefix
#' @template cgTablePrefix
#' @template esTablePrefix
#' @template targetIds
#' @template outcomeIds
#' @param includeOneSidedP This lets you extract from older results that do not have the one sided p by setting this to FALSE
#' @family Estimation
#' @return
#' Returns a data.frame with the columns:
#' \itemize{
#'  #'  \item{databaseName the database name}
#'  \item{analysisId the analysis unique identifier}
#'  \item{description an analysis description}
#'  \item{targetName the target name}
#'  \item{targetId the target cohort id}
#'  \item{outcomeName the outcome name}
#'  \item{outcomeId the outcome cohort id}
#'  \item{indicationName the indicationname}
#'  \item{indicationId the indication cohort id}
#'  \item{covariateName whether main or secondary analysis}
#'  \item{outcomeSubjects The number of subjects with at least one outcome.}
#'  \item{outcomeEvents The number of outcome events.}
#'  \item{outcomeObservationPeriods The number of observation periods containing at least one outcome.}
#'  \item{covariateSubjects The number of subjects having the covariate.}
#'  \item{covariateDays The total covariate time in days.}
#'  \item{covariateEras The number of continuous eras of the covariate.}
#'  \item{covariateOutcomes The number of outcomes observed during the covariate time.}
#'  \item{observedDays The number of days subjects were observed.}
#'  \item{calibratedRr the calibrated relative risk}
#'  \item{calibratedCi95Lb the lower bound of the 95 percent confidence interval for the calibrated relative risk}
#'  \item{calibratedCi95Ub the upper bound of the 95 percent confidence interval for the calibrated relative risk}
#'  \item{calibratedP the calibrated p-value}
#'  \item{calibratedOneSidedP the calibrated one sided p-value}
#'  \item{calibratedLogRr the calibrated log of the relative risk}
#'  \item{calibratedSeLogRr the calibrated log of the relative risk standard error}
#'  \item{nDatabases The number of databases included in the estimate.}
#'  } 
#' 
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' sccsMeta <- getSccsMetaEstimation(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main',
#'   targetIds = 1,
#'   outcomeIds = 3
#' )
#' 
getSccsMetaEstimation <- function(
    connectionHandler,
    schema,
    sccsTablePrefix = 'sccs_',
    cgTablePrefix = 'cg_',
    esTablePrefix = 'es_',
    targetIds = NULL,
    outcomeIds = NULL,
    includeOneSidedP = TRUE
) {

  # figure out if PI cols exist
  sql <- "SELECT * from @schema.@es_table_prefixsccs_result
          WHERE exposures_outcome_set_id = 0;"
  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    es_table_prefix = esTablePrefix
  )
  piCols <- 'pi95Lb' %in% colnames(result)
  
  sql <- "select distinct
    ev.evidence_synthesis_description as database_name, 
    r.analysis_id, 
    a.description,
  c1.cohort_name as target_name,
  cov.era_id as target_id, 
  c3.cohort_name as outcome_name,
  eos.outcome_id,
  cg3.cohort_name as indication_name,
  eos.nesting_cohort_id as indication_id,
  cov.covariate_name,

  r.outcome_subjects,
  r.outcome_events,
  r.outcome_observation_periods,
  r.covariate_subjects,
  r.covariate_days,
  r.covariate_eras,
  r.covariate_outcomes,
  r.observed_days,
  
  CASE WHEN unblind.unblind = 1 THEN r.calibrated_rr ELSE NULL END calibrated_rr,
  CASE WHEN unblind.unblind = 1 THEN r.calibrated_ci_95_lb ELSE NULL END calibrated_ci_95_lb,
  CASE WHEN unblind.unblind = 1 THEN r.calibrated_ci_95_ub ELSE NULL END calibrated_ci_95_ub,
  CASE WHEN unblind.unblind = 1 THEN r.calibrated_p ELSE NULL END calibrated_p,
  {@include_one_sided_p}?{CASE WHEN unblind.unblind = 1 THEN r.calibrated_one_sided_p ELSE NULL END calibrated_one_sided_p,}
  
  CASE WHEN unblind.unblind = 1 THEN r.calibrated_log_rr ELSE NULL END calibrated_log_rr,
  CASE WHEN unblind.unblind = 1 THEN r.calibrated_se_log_rr ELSE NULL END calibrated_se_log_rr,
  
  r.n_databases,
  
  {@pi_cols}?{CASE WHEN unblind.unblind = 1 THEN r.pi_95_lb ELSE NULL END pi_95_lb,}:{NULL as pi_95_lb,}
	{@pi_cols}?{CASE WHEN unblind.unblind = 1 THEN r.pi_95_ub ELSE NULL END pi_95_ub,}:{NULL as pi_95_ub,}
	{@pi_cols}?{CASE WHEN unblind.unblind = 1 THEN r.calibrated_pi_95_lb ELSE NULL END calibrated_pi_95_lb,}:{NULL as calibrated_pi_95_lb,}
	{@pi_cols}?{CASE WHEN unblind.unblind = 1 THEN r.calibrated_pi_95_ub ELSE NULL END calibrated_pi_95_ub,}:{NULL as calibrated_pi_95_ub,}
	unblind.tau,
	unblind.i_2
  
  from 
   @schema.@es_table_prefixsccs_result as r
   inner join 
   @schema.@sccs_table_prefixexposures_outcome_set as eos
   on 
   r.exposures_outcome_set_id = eos.exposures_outcome_set_id
   
   inner join
   @schema.@sccs_table_prefixcovariate as cov
   on 
   r.covariate_id = cov.covariate_id and
   r.analysis_id = cov.analysis_id and
   r.exposures_outcome_set_id = cov.exposures_outcome_set_id
   
   inner join
   @schema.@sccs_table_prefixexposure as ex
   on 
   ex.era_id = cov.era_id and
   ex.exposures_outcome_set_id = cov.exposures_outcome_set_id
   
   inner join
   
   @schema.@es_table_prefixsccs_diagnostics_summary as unblind
   on
   r.analysis_id = unblind.analysis_id and 
   r.exposures_outcome_set_id = unblind.exposures_outcome_set_id and 
   r.covariate_id = unblind.covariate_id and
   r.evidence_synthesis_analysis_id = unblind.evidence_synthesis_analysis_id
   
   inner join
   @schema.@cg_table_prefixcohort_definition as c1
   on c1.cohort_definition_id = cov.era_id

   inner join
   @schema.@cg_table_prefixcohort_definition as c3
   on c3.cohort_definition_id = eos.outcome_id
   
   inner join
   @schema.@sccs_table_prefixanalysis as a
   on a.analysis_id = r.analysis_id
   
   inner join
   @schema.@es_table_prefixanalysis as ev
   on ev.evidence_synthesis_analysis_id = r.evidence_synthesis_analysis_id
   
   left join
   @schema.@cg_table_prefixcohort_definition as cg3
   on eos.nesting_cohort_id = cg3.cohort_definition_id
   
   where 
   r.calibrated_rr != 0
   --AND ex.true_effect_size != 1
   AND cov.covariate_name in ('Main', 'Second dose') 
   {@include_target}?{AND cov.era_id in (@target_id)}
   {@include_outcome}?{AND eos.outcome_id in(@outcome_id)}
  ;"  
  
  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    sccs_table_prefix = sccsTablePrefix,
    cg_table_prefix = cgTablePrefix,
    es_table_prefix = esTablePrefix,
    target_id = paste0(targetIds, collapse = ','),
    include_target = !is.null(targetIds),
    outcome_id = paste0(outcomeIds, collapse = ','),
    include_outcome = !is.null(outcomeIds),
    include_one_sided_p = includeOneSidedP,
    pi_cols = piCols
  )
  
  return(result)
}


#' Extract the SCCS table specified
#' @description
#' This function extracts the specific cohort method table.
#'
#' @details
#' Specify the connectionHandler, the schema and optionally the target/outcome/analysis/database IDs
#'
#' @template connectionHandler
#' @template schema
#' @param table The result table to extract
#' @template sccsTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @param indicationIds The indications that the target was nested to
#' @template outcomeIds
#' @param analysisIds the analysis IDs to restrict to 
#' @param databaseIds the database IDs to restrict to 
#' @param exposureOutcomeIds the exposureOutcomeIds to restrict to
#' @param covariateIds the covariateIds to restrict to
#' @family Estimation
#' @return
#' Returns a data.frame with the cohort method requested table
#' 
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' sccsTable <- getSccsTable(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main',
#'   table = 'attrition'
#' )
#' 
getSccsTable <- function(
    connectionHandler,
    schema,
    table = c('attrition', 'time_trend', 'event_dep_observation',
              'age_spanning', 'calendar_time_spanning', 'spline')[1],
    sccsTablePrefix = 'sccs_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    indicationIds = NULL,
    outcomeIds = NULL,
    analysisIds = NULL,
    databaseIds = NULL,
    exposureOutcomeIds = NULL,
    covariateIds = NULL
){
  
  # check table valid:
  if(!table %in% c('attrition', 'time_trend', 'event_dep_observation',
                   'age_spanning', 'calendar_time_spanning', 'spline')){
    stop('Invalid table')
  }
  
  # only let users use covariated for valid tables
  includeCovariates <- !is.null(covariateIds)
  if(table %in% c('time_trend', 'event_dep_observation',
                  'age_spanning', 'calendar_time_spanning',
                  'spline')){
    includeCovariates <- FALSE
  }
  
  sql <- "select 
  dmd.cdm_source_abbreviation database_name,
  a.description as analysis_description,
  c2.cohort_name as outcome_name,
  eos.outcome_id,
  c3.cohort_name as indication_name,
  eos.nesting_cohort_id,
  tab.*

  from 
   @schema.@sccs_table_prefix@table as tab

  inner join
  @schema.@sccs_table_prefixexposures_outcome_set eos
  on eos.exposures_outcome_set_id = tab.exposures_outcome_set_id
  
   inner join
   @schema.@cg_table_prefixcohort_definition as c2
   on c2.cohort_definition_id = eos.outcome_id
   
   left join
   @schema.@cg_table_prefixcohort_definition as c3
   on c3.cohort_definition_id = eos.nesting_cohort_id
   
   inner join
   @schema.@sccs_table_prefixanalysis as a
   on a.analysis_id = tab.analysis_id
   
   inner join
   @schema.@database_table as dmd
   on dmd.database_id = tab.database_id
   
   where 
   1 = 1 
  {@include_outcome}?{and eos.outcome_id in (@outcome_ids)}
  {@include_indication}?{and eos.nesting_cohort_id in (@indication_ids)}
  {@include_database}?{and tab.database_id in (@database_ids)}
  {@include_analyses}?{and tab.analysis_id in (@analysis_ids)}
  {@include_eos}?{and tab.exposures_outcome_set_id in (@exposures_outcome_set_ids)}
  {@include_covariates}?{AND tab.covariate_id in (@covariate_ids)}
  ;"
  
  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    sccs_table_prefix = sccsTablePrefix,
    cg_table_prefix = cgTablePrefix,
    table = table,
    
    include_covariates = includeCovariates,
    covariate_ids = paste0(covariateIds, collapse = ','),
    
    outcome_ids = paste0(outcomeIds, collapse = ','),
    include_outcome = !is.null(outcomeIds),
    
    indication_ids = paste0(indicationIds, collapse = ','),
    include_indication = !is.null(indicationIds),
    
    database_ids = paste0("'",databaseIds,"'", collapse = ','),
    include_database = !is.null(databaseIds),
    
    analysis_ids = paste0(analysisIds, collapse = ','),
    include_analyses = !is.null(analysisIds),
    
    include_eos = !is.null(exposureOutcomeIds),
    exposures_outcome_set_ids = paste0(exposureOutcomeIds, collapse = ','),
    
    database_table = databaseTable
  )
  
  return(unique(result))
}

#' Extract the SCCS model table
#' @description
#' This function extracts the sccs model table.
#'
#' @details
#' Specify the connectionHandler, the schema and optionally the target/outcome/analysis/database IDs
#'
#' @template connectionHandler
#' @template schema
#' @template sccsTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @param exposureOutcomeSetIds the exposureOutcomeIds to restrict to
#' @param indicationIds The indications that the target was nested to
#' @template outcomeIds
#' @param databaseIds the database IDs to restrict to 
#' @param analysisIds the analysis IDs to restrict to 
#' @template targetIds
#' @family Estimation
#' @return
#' Returns a data.frame with the SCCS model table
#' 
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' sccsModels <- getSccsModel(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getSccsModel <- function(
    connectionHandler,
    schema,
    sccsTablePrefix = 'sccs_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    exposureOutcomeSetIds = NULL,
    indicationIds = NULL,
    outcomeIds = NULL,
    databaseIds = NULL,
    analysisIds = NULL,
    targetIds = NULL
) {
  
  sql <- "
  SELECT
  dmd.cdm_source_abbreviation as database_name,
  sc.database_id,
  a.description as analysis_description,
  sc.analysis_id,
  cd.cohort_name as target_name,
  sc.era_id as target_id,
  c2.cohort_name as outcome_name,
  eos.outcome_id,
  c3.cohort_name as indication_name,
  eos.nesting_cohort_id as indication_id,
  scr.exposures_outcome_set_id,
  
    CASE
       WHEN era.era_name IS NULL THEN sc.covariate_name
       ELSE CONCAT(sc.covariate_name, ' : ', era.era_name)
    END as covariate_name,
    scr.covariate_id, scr.rr, scr.ci_95_lb, scr.ci_95_ub
  FROM 
  
  @schema.@sccs_table_prefixcovariate_result scr
  INNER JOIN @schema.@sccs_table_prefixcovariate sc ON (
    sc.exposures_outcome_set_id = scr.exposures_outcome_set_id AND
    sc.database_id = scr.database_id AND
    sc.analysis_id = scr.analysis_id AND
    sc.covariate_id = scr.covariate_id
  )
  
  INNER JOIN
  @schema.@sccs_table_prefixexposures_outcome_set eos
  on eos.exposures_outcome_set_id = scr.exposures_outcome_set_id
  
  LEFT JOIN @schema.@cg_table_prefixcohort_definition cd 
  ON cd.cohort_definition_id = sc.era_id
  
  LEFT JOIN @schema.@sccs_table_prefixera era ON (
    era.exposures_outcome_set_id = scr.exposures_outcome_set_id AND
    era.database_id = scr.database_id AND
    era.analysis_id = scr.analysis_id AND
    era.era_id = sc.era_id
  )
  
   inner join
   @schema.@cg_table_prefixcohort_definition as c2
   on c2.cohort_definition_id = eos.outcome_id
   
   left join
   @schema.@cg_table_prefixcohort_definition as c3
   on c3.cohort_definition_id = eos.nesting_cohort_id
   
   inner join
   @schema.@sccs_table_prefixanalysis as a
   on a.analysis_id = scr.analysis_id
   
   inner join
   @schema.@database_table as dmd
   on dmd.database_id = scr.database_id

  WHERE 
  1 = 1
  {@use_databases}?{AND  scr.database_id in (@database_ids)}
  {@use_analyses}?{AND  scr.analysis_id in (@analysis_ids)}
  {@use_targets}?{AND  sc.era_id in (@target_ids)}
  {@use_eos}?{AND scr.exposures_outcome_set_id in (@exposures_outcome_set_ids)}
  {@use_outcomes}?{AND eos.outcome_id in (@outcome_ids)}
  {@use_indications}?{AND eos.nesting_id in (@indication_ids)}
  ;
  "
  
  connectionHandler$queryDb(
    sql,
    schema = schema,
    sccs_table_prefix = sccsTablePrefix,
    cg_table_prefix = cgTablePrefix,
    
    use_databases = !is.null(databaseIds),
    database_ids = paste0("'",databaseIds,"'", collapse = ','),
    
    use_analyses = !is.null(analysisIds),
    analysis_ids = paste0(analysisIds, collapse = ','),
    
    use_targets = !is.null(targetIds),
    target_ids = paste0(targetIds, collapse = ','),
    
    use_outcomes = !is.null(outcomeIds),
    outcome_ids = paste0(outcomeIds, collapse = ','),
    
    use_indications = !is.null(indicationIds),
    indication_ids = paste0(indicationIds, collapse = ','),
    
    use_eos = !is.null(exposureOutcomeSetIds),
    exposures_outcome_set_ids = paste0(exposureOutcomeSetIds, collapse = ','),
    
    database_table = databaseTable,
    
    snakeCaseToCamelCase = TRUE
    )
}



#' Extract the SCCS negative controls
#' @description
#' This function extracts the sccs negative controls.
#'
#' @details
#' Specify the connectionHandler, the schema and optionally the target/outcome/analysis/database IDs
#'
#' @template connectionHandler
#' @template schema
#' @template sccsTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @param databaseIds the database IDs to restrict to 
#' @param exposuresOutcomeSetIds the exposureOutcomeIds to restrict to
#' @param indicationIds The indications that the target was nested to
#' @template outcomeIds
#' @template targetIds
#' @param analysisIds the analysis IDs to restrict to 
#' @param covariateIds the covariate IDs to restrict to 
#' @param covariateAnalysisIds the covariate analysis IDs to restrict to 
#' @family Estimation
#' @return
#' Returns a data.frame with the SCCS negative controls
#' 
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' sccsNcs <- getSccsNegativeControlEstimates(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getSccsNegativeControlEstimates <- function(
    connectionHandler,
    schema,
    sccsTablePrefix = 'sccs_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    databaseIds = NULL,
    exposuresOutcomeSetIds = NULL,
    indicationIds = NULL,
    outcomeIds  = NULL,
    targetIds  = NULL, #  eraId,
    analysisIds  = NULL,
    covariateIds  = NULL, # needed?
    covariateAnalysisIds  = NULL # needed?
) {
  
  sql <- "
  SELECT 
  c.exposures_outcome_set_id,
  eos.nesting_cohort_id as indication_id,
  c1.cohort_name as indication_name,
  c.era_id as target_id,
  c2.cohort_name as target_name,
  outcome_id as outcome_id,
  c3.cohort_name as outcome_name,
  db.database_id as database_id,
  db.cdm_source_abbreviation as database_name,
  a.description as analysis_description,
  r.ci_95_lb, 
  r.ci_95_ub, 
  r.log_rr, 
  r.se_log_rr, 
  r.calibrated_ci_95_lb, 
  r.calibrated_ci_95_ub, 
  r.calibrated_log_rr,
  r.calibrated_se_log_rr, 
  e.true_effect_size,
  ds.ease
  
  FROM 
  @schema.@sccs_table_prefixresult r
  INNER JOIN
   @schema.@sccs_table_prefixexposure e
   on r.exposures_outcome_set_id = e.exposures_outcome_set_id

   INNER JOIN 
   @schema.@sccs_table_prefixcovariate c
   on e.era_id = c.era_id 
   and e.exposures_outcome_set_id = c.exposures_outcome_set_id
   and c.database_id = r.database_id
   and c.analysis_id = r.analysis_id
   and c.covariate_id = r.covariate_id
   
   INNER JOIN  
   @schema.@sccs_table_prefixexposures_outcome_set eos
   on eos.exposures_outcome_set_id = r.exposures_outcome_set_id
   
   left join
   @schema.@cg_table_prefixcohort_definition as c2
   on c2.cohort_definition_id = e.era_id
   
   left join
   @schema.@cg_table_prefixcohort_definition as c3
   on c3.cohort_definition_id = eos.outcome_id
   
   left join
   @schema.@cg_table_prefixcohort_definition as c1
   on c1.cohort_definition_id = eos.nesting_cohort_id
   
   inner join
   @schema.@sccs_table_prefixanalysis as a
   on a.analysis_id = r.analysis_id
   
   inner join
   @schema.@database_table as db
   on db.database_id = r.database_id
   
   left join
   @schema.@sccs_table_prefixdiagnostics_summary ds
   ON ds.database_id = r.database_id
   AND ds.analysis_id = r.analysis_id
   AND ds.covariate_id = r.covariate_id
   AND ds.exposures_outcome_set_id = r.exposures_outcome_set_id
  
   WHERE 
   e.true_effect_size is not NULL
  {@use_eos_id}?{AND r.exposures_outcome_set_id in (@eos_ids)}
  {@use_outcome_id}?{AND eos.outcome_id in (@outcome_ids)}
  {@use_target_id}?{AND e.era_id in (@target_ids)}
   {@use_database_id}?{AND r.database_id in (@database_ids)}
   {@use_analysis_id}?{AND r.analysis_id in (@analysis_ids)}
   {@use_covariate_id}?{AND r.covariate_id in (@covariate_ids)}
  {@use_indication_id}?{AND eos.nesting_cohort_id in (@indication_ids)}
   {@use_covariate_analysis_id}?{AND c.covariate_analysis_id in (@covariate_analysis_ids)}
  ;
  "
  
  result <- connectionHandler$queryDb(
    sql,
    schema = schema,
    sccs_table_prefix = sccsTablePrefix,
    cg_table_prefix = cgTablePrefix,
    database_table = databaseTable,
    use_eos_id = !is.null(exposuresOutcomeSetIds),
    eos_ids = paste0(exposuresOutcomeSetIds, collapse = ','),
    use_outcome_id = !is.null(outcomeIds),
    outcome_ids = paste0(outcomeIds, collapse = ','),
    use_target_id = !is.null(targetIds),
    target_ids = paste0(targetIds, collapse = ','),
    use_database_id = !is.null(databaseIds),
    database_ids = paste0("'",databaseIds,"'", collapse = ','),
    use_analysis_id = !is.null(analysisIds),
    analysis_ids = paste0(analysisIds, collapse = ','),
    use_covariate_id = !is.null(covariateIds),
    covariate_ids = paste0(covariateIds, collapse = ','),
    use_indication_id = !is.null(indicationIds),
    indication_ids = paste0(indicationIds, collapse = ','),
    use_covariate_analysis_id = !is.null(covariateAnalysisIds),
    covariate_analysis_ids = paste0(covariateAnalysisIds, collapse = ','),
    snakeCaseToCamelCase = TRUE
  )
  

  return(result)
}

#' Extract the SCCS time-to-event
#' @description
#' This function extracts the SCCS time-to-event.
#'
#' @details
#' Specify the connectionHandler, the schema and optionally the target/outcome/analysis/database IDs
#'
#' @template connectionHandler
#' @template schema
#' @template sccsTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @param databaseIds the database IDs to restrict to 
#' @param exposuresOutcomeSetIds the exposureOutcomeIds to restrict to
#' @param indicationIds The indications that the target was nested to
#' @template outcomeIds
#' @template targetIds
#' @param analysisIds the analysis IDs to restrict to 
#' @family Estimation
#' @return
#' Returns a data.frame with the SCCS time-to-event
#' 
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' getSccsTimeToEvent <- getSccsNegativeControlEstimates(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getSccsTimeToEvent <- function(
    connectionHandler,
    schema,
    sccsTablePrefix = 'sccs_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    databaseIds = NULL,
    exposuresOutcomeSetIds = NULL,
    indicationIds = NULL,
    outcomeIds = NULL,
    targetIds = NULL,
    analysisIds = NULL 
    # add covariateIds?
) {
  
  sql <- "
  SELECT 
  eos.nesting_cohort_id as indication_id,
  c1.cohort_name as indication_name,
  tte.era_id as target_id,
  c2.cohort_name as target_name,
  eos.outcome_id as outcome_id,
  c3.cohort_name as outcome_name,
  db.cdm_source_abbreviation as database_name,
  a.description as analysis_description,
  c.covariate_id,
  c.covariate_name,
  tte.* , 
  ds.pre_exposure_p as p
  
  FROM 
  @schema.@sccs_table_prefixtime_to_event tte
  
  inner join
  @schema.@sccs_table_prefixcovariate c
  ON c.analysis_id = tte.analysis_id
  AND c.exposures_outcome_set_id = tte.exposures_outcome_set_id
  AND c.era_id = tte.era_id
  AND c.database_id = tte.database_id
  
  inner join
  @schema.@sccs_table_prefixdiagnostics_summary ds
  ON ds.database_id = tte.database_id
  AND ds.covariate_id  = c.covariate_id 
  AND ds.analysis_id = tte.analysis_id
  AND ds.exposures_outcome_set_id = tte.exposures_outcome_set_id
  
  INNER JOIN  
   @schema.@sccs_table_prefixexposures_outcome_set eos
   on eos.exposures_outcome_set_id = tte.exposures_outcome_set_id
   
   left join
   @schema.@cg_table_prefixcohort_definition as c2
   on c2.cohort_definition_id = tte.era_id
   
   left join
   @schema.@cg_table_prefixcohort_definition as c3
   on c3.cohort_definition_id = eos.outcome_id
   
   left join
   @schema.@cg_table_prefixcohort_definition as c1
   on c1.cohort_definition_id = eos.nesting_cohort_id
   
   inner join
   @schema.@sccs_table_prefixanalysis as a
   on a.analysis_id = tte.analysis_id
   
   inner join
   @schema.@database_table as db
   on db.database_id = tte.database_id
  

  WHERE 
  1 = 1
  {@use_databases}?{ AND tte.database_id in (@database_ids) }
  {@use_targets}?{ AND tte.era_id  in (@target_ids)}
  {@use_analyses}?{ AND tte.analysis_id in (@analysis_ids)}
  {@use_eos}?{ AND tte.exposures_outcome_set_id in (@exposures_outcome_set_ids)}
  {@use_outcomes}?{ AND eos.outcome_id in (@outcome_ids)}
  {@use_indications}?{ AND eos.nesting_id in (@indication_ids)}
  ;
  "
  
  timeToEvent <- connectionHandler$queryDb(
    sql,
    schema = schema,
    sccs_table_prefix = sccsTablePrefix,
    cg_table_prefix = cgTablePrefix,
    database_table = databaseTable,
    use_eos = !is.null(exposuresOutcomeSetIds),
    exposures_outcome_set_ids = paste0(exposuresOutcomeSetIds, collapse = ','),
    use_outcomes = !is.null(outcomeIds),
    outcome_ids = paste0(outcomeIds, collapse = ','),
    use_targets = !is.null(targetIds),
    target_ids = paste0(targetIds, collapse = ','),
    use_databases = !is.null(databaseIds),
    database_ids = paste0("'",databaseIds,"'", collapse = ','),
    use_analyses = !is.null(analysisIds),
    analysis_ids = paste0(analysisIds, collapse = ','),
    use_indications = !is.null(indicationIds),
    indication_ids = paste0(indicationIds, collapse = ','),
    snakeCaseToCamelCase = TRUE
  )
  
  return(timeToEvent)
}

test_that("getCMEstimation", {
  
  data <- getCMEstimation(
    connectionHandler = connectionHandler,
    schema = schema
  )
  
  testthat::expect_true(nrow(data) > 0)
  
  testthat::expect_true('databaseName' %in% colnames(data))
  testthat::expect_true('analysisId' %in% colnames(data))
  testthat::expect_true('description' %in% colnames(data))
  testthat::expect_true('targetName' %in% colnames(data))
  testthat::expect_true('comparatorName' %in% colnames(data))
  testthat::expect_true('outcomeName' %in% colnames(data))
  testthat::expect_true('calibratedRr' %in% colnames(data))
  testthat::expect_true('calibratedP' %in% colnames(data))
  testthat::expect_true('calibratedOneSidedP' %in% colnames(data))
  testthat::expect_true('calibratedLogRr' %in% colnames(data))
  testthat::expect_true('targetSubjects' %in% colnames(data))
  testthat::expect_true('comparatorSubjects' %in% colnames(data))
  testthat::expect_true('targetDays' %in% colnames(data))
  testthat::expect_true('comparatorDays' %in% colnames(data))
  testthat::expect_true('targetOutcomes' %in% colnames(data))
  testthat::expect_true('comparatorOutcomes' %in% colnames(data))
})


test_that("getCmDiagnosticsData", {
  
  data <- getCmDiagnosticsData(
    connectionHandler = connectionHandler,
    schema = schema
  )
  
  testthat::expect_true(nrow(data) > 0)
  
  testthat::expect_true('databaseName' %in% colnames(data))
  testthat::expect_true('analysisId' %in% colnames(data))
  testthat::expect_true('description' %in% colnames(data))
  testthat::expect_true('targetName' %in% colnames(data))
  testthat::expect_true('comparatorName' %in% colnames(data))
  testthat::expect_true('outcomeName' %in% colnames(data))
  testthat::expect_true('maxSdm' %in% colnames(data))
  testthat::expect_true('sharedMaxSdm' %in% colnames(data))
  testthat::expect_true('equipoise' %in% colnames(data))
  testthat::expect_true('mdrr' %in% colnames(data))
  testthat::expect_true('attritionFraction' %in% colnames(data))
  testthat::expect_true('ease' %in% colnames(data))
  testthat::expect_true('balanceDiagnostic' %in% colnames(data))
  testthat::expect_true('sharedBalanceDiagnostic' %in% colnames(data))
  testthat::expect_true('equipoiseDiagnostic' %in% colnames(data))
  testthat::expect_true('mdrrDiagnostic' %in% colnames(data))
  testthat::expect_true('attritionDiagnostic' %in% colnames(data))
  testthat::expect_true('easeDiagnostic' %in% colnames(data))
  
  testthat::expect_true('unblind' %in% colnames(data))
  testthat::expect_true('summaryValue' %in% colnames(data))
  
})


test_that("getCmMetaEstimation", {
  
  # TODO get results into example?
  data <- getCmMetaEstimation(
    connectionHandler = connectionHandler,
    schema = schema
  )
  
  testthat::expect_true('databaseName' %in% colnames(data))
  testthat::expect_true('analysisId' %in% colnames(data))
  testthat::expect_true('description' %in% colnames(data))
  testthat::expect_true('targetName' %in% colnames(data))
  testthat::expect_true('comparatorName' %in% colnames(data))
  testthat::expect_true('outcomeName' %in% colnames(data))
  testthat::expect_true('calibratedRr' %in% colnames(data))
  testthat::expect_true('calibratedP' %in% colnames(data))
  testthat::expect_true('calibratedOneSidedP' %in% colnames(data))
  testthat::expect_true('calibratedLogRr' %in% colnames(data))
  testthat::expect_true('targetSubjects' %in% colnames(data))
  testthat::expect_true('comparatorSubjects' %in% colnames(data))
  testthat::expect_true('targetDays' %in% colnames(data))
  testthat::expect_true('comparatorDays' %in% colnames(data))
  testthat::expect_true('targetOutcomes' %in% colnames(data))
  testthat::expect_true('comparatorOutcomes' %in% colnames(data))
  testthat::expect_true('nDatabases' %in% colnames(data))
})


test_that("getCmPropensityModel", {
  
  data <- getCmPropensityModel(
    connectionHandler = connectionHandler,
    schema = schema,
    targetId = 1, 
    comparatorId = 2, 
    analysisId = 1, 
    databaseId = 388020256
  )
  
  testthat::expect_true('covariateName' %in% colnames(data))
  testthat::expect_true('coefficient' %in% colnames(data))
  
})

test_that("getCmNegativeControlEstimates", {
  
  data <- getCmNegativeControlEstimates(
    connectionHandler = connectionHandler,
    schema = schema
  )
  
  testthat::expect_true('effectSize' %in% colnames(data))
  
})

test_that("getCmTable", {
  
  data <- getCmTable(
    connectionHandler = connectionHandler,
    schema = schema, 
    table = 'attrition'
  )
  
  testthat::expect_true(nrow(data) > 0)
  
})


##=========== SCCS

test_that("getSccsEstimation", {
  
  data <- getSccsEstimation(
    connectionHandler = connectionHandler,
    schema = schema
  )
  
  testthat::expect_true(nrow(data) > 0)
  
  testthat::expect_true('databaseName' %in% colnames(data))
  testthat::expect_true('analysisId' %in% colnames(data))
  testthat::expect_true('description' %in% colnames(data))
  testthat::expect_true('targetName' %in% colnames(data))
  testthat::expect_true('outcomeName' %in% colnames(data))
  testthat::expect_true('calibratedRr' %in% colnames(data))
  testthat::expect_true('calibratedP' %in% colnames(data))
  testthat::expect_true('calibratedLogRr' %in% colnames(data))
  testthat::expect_true('calibratedSeLogRr' %in% colnames(data))
  testthat::expect_true('outcomeSubjects' %in% colnames(data))
  testthat::expect_true('outcomeEvents' %in% colnames(data))
  testthat::expect_true('outcomeObservationPeriods' %in% colnames(data))
  testthat::expect_true('covariateSubjects' %in% colnames(data))
  testthat::expect_true('covariateDays' %in% colnames(data))
  testthat::expect_true('covariateEras' %in% colnames(data))
  testthat::expect_true('covariateOutcomes' %in% colnames(data))
  testthat::expect_true('observedDays' %in% colnames(data))
})


test_that("getSccsDiagnosticsData", {
  
  data <- getSccsDiagnosticsData(
    connectionHandler = connectionHandler,
    schema = schema
  )
  
  testthat::expect_true(nrow(data) > 0)
  
  testthat::expect_true('databaseName' %in% colnames(data))
  testthat::expect_true('analysisId' %in% colnames(data))
  testthat::expect_true('description' %in% colnames(data))
  testthat::expect_true('targetName' %in% colnames(data))
  testthat::expect_true('outcomeName' %in% colnames(data))
  testthat::expect_true('mdrr' %in% colnames(data))
  testthat::expect_true('ease' %in% colnames(data))
  testthat::expect_true('timeTrendP' %in% colnames(data))
  testthat::expect_true('preExposureP' %in% colnames(data))
  testthat::expect_true('mdrrDiagnostic' %in% colnames(data))
  testthat::expect_true('easeDiagnostic' %in% colnames(data))
  testthat::expect_true('timeTrendDiagnostic' %in% colnames(data))
  testthat::expect_true('preExposureDiagnostic' %in% colnames(data))
  testthat::expect_true('unblind' %in% colnames(data))
  testthat::expect_true('unblindForEvidenceSynthesis' %in% colnames(data))
  testthat::expect_true('summaryValue' %in% colnames(data))
  
})


test_that("getSccsMetaEstimation", {
  
  # TODO get results into example?
  data <- getSccsMetaEstimation(
    connectionHandler = connectionHandler,
    schema = schema
  )
  
  testthat::expect_true('databaseName' %in% colnames(data))
  testthat::expect_true('analysisId' %in% colnames(data))
  testthat::expect_true('description' %in% colnames(data))
  testthat::expect_true('targetName' %in% colnames(data))
  testthat::expect_true('outcomeName' %in% colnames(data))
  
  testthat::expect_true('outcomeSubjects' %in% colnames(data))
  testthat::expect_true('outcomeEvents' %in% colnames(data))
  testthat::expect_true('outcomeObservationPeriods' %in% colnames(data))
  testthat::expect_true('covariateSubjects' %in% colnames(data))
  testthat::expect_true('observedDays' %in% colnames(data))

  testthat::expect_true('calibratedRr' %in% colnames(data))
  testthat::expect_true('calibratedP' %in% colnames(data))
  testthat::expect_true('calibratedOneSidedP' %in% colnames(data))
  testthat::expect_true('calibratedLogRr' %in% colnames(data))
  testthat::expect_true('calibratedSeLogRr' %in% colnames(data))
  testthat::expect_true('nDatabases' %in% colnames(data))
})


test_that("getCmOutcomes", {
  
  data <- getCmOutcomes(
    connectionHandler = connectionHandler, 
    schema = schema
  )
  
  testthat::expect_true(nrow(data) > 0)
  testthat::expect_true(data$cohortDefinitionId[1] == 3)
  
  data <- getCmOutcomes(
    connectionHandler = connectionHandler, 
    schema = schema, 
    targetId = c(1002,1)
  )
  
  testthat::expect_true(nrow(data) > 0)
  
  # not a target in the prediction results
  data <- getCmOutcomes(
    connectionHandler = connectionHandler, 
    schema = schema, 
    targetId = 5
  )
  
  testthat::expect_true(nrow(data) == 0)
  
  
  
})

test_that("getSccsOutcomes", {
  
  data <- getSccsOutcomes(
    connectionHandler = connectionHandler, 
    schema = schema
  )
  
  testthat::expect_true(nrow(data) > 0)
  testthat::expect_true(data$cohortDefinitionId[1] == 3)
  
  data <- getSccsOutcomes(
    connectionHandler = connectionHandler, 
    schema = schema, 
    targetId = c(1002,1, 1001)
  )
  
  testthat::expect_true(nrow(data) > 0)
  
  # not a target in the prediction results
  data <- getSccsOutcomes(
    connectionHandler = connectionHandler, 
    schema = schema, 
    targetId = 5
  )
  
  testthat::expect_true(nrow(data) == 0)
  
  
  
})

test_that("getSccsTable", {
  
dataA <- OhdsiReportGenerator:::getSccsTable(
    connectionHandler = connectionHandler,
    schema = schema,
    table = 'attrition',
    sccsTablePrefix = 'sccs_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data'
)

testthat::expect_true(nrow(dataA) > 0)
testthat::expect_true("databaseName" %in% colnames(dataA))
testthat::expect_true("outcomeName" %in% colnames(dataA))
testthat::expect_true("indicationName" %in% colnames(dataA))
testthat::expect_true("outcomeSubjects" %in% colnames(dataA))

dataAsub <- OhdsiReportGenerator:::getSccsTable(
  connectionHandler = connectionHandler,
  schema = schema,
  table = 'attrition',
  sccsTablePrefix = 'sccs_',
  cgTablePrefix = 'cg_',
  databaseTable = 'database_meta_data',
  analysisIds =  1, 
  databaseIds = 388020256, 
  covariateIds = 1001,
  exposureOutcomeIds = 2086096871
)

# check filtering works
testthat::expect_true(nrow(dataAsub) > 0)
testthat::expect_true(unique(dataAsub$exposuresOutcomeSetId) == 2086096871)
testthat::expect_true(unique(dataAsub$covariateId) == 1001)
testthat::expect_true(unique(dataAsub$databaseId) == 388020256)
testthat::expect_true(unique(dataAsub$analysisId) == 1)

dataAsub2 <- OhdsiReportGenerator:::getSccsTable(
  connectionHandler = connectionHandler,
  schema = schema,
  table = 'attrition',
  sccsTablePrefix = 'sccs_',
  cgTablePrefix = 'cg_',
  databaseTable = 'database_meta_data',
  analysisIds =  1, 
  databaseIds = 388020256, 
  covariateIds = 1001,
  outcomeIds = 3
)
# check filtering bt outcomeId works
testthat::expect_true(nrow(dataAsub2) > 0)
testthat::expect_true(unique(dataAsub2$outcomeId) == 3)
testthat::expect_true(unique(dataAsub2$covariateId) == 1001)
testthat::expect_true(unique(dataAsub2$databaseId) == 388020256)
testthat::expect_true(unique(dataAsub2$analysisId) == 1)


dataTt <- getSccsTable(
  connectionHandler = connectionHandler,
  schema = schema,
  table = 'time_trend',
  sccsTablePrefix = 'sccs_',
  cgTablePrefix = 'cg_',
  databaseTable = 'database_meta_data'
)
testthat::expect_true(nrow(dataTt) > 0)
testthat::expect_true("databaseName" %in% colnames(dataTt))
testthat::expect_true("outcomeName" %in% colnames(dataTt))
testthat::expect_true("indicationName" %in% colnames(dataTt))
testthat::expect_true("calendarYear" %in% colnames(dataTt))
testthat::expect_true("calendarMonth" %in% colnames(dataTt))
testthat::expect_true("observedSubjects" %in% colnames(dataTt))

dataTtSub <- getSccsTable(
  connectionHandler = connectionHandler,
  schema = schema,
  table = 'time_trend',
  sccsTablePrefix = 'sccs_',
  cgTablePrefix = 'cg_',
  databaseTable = 'database_meta_data',
  analysisIds =  1, 
  databaseIds = 388020256, 
  covariateIds = 1001,
  exposureOutcomeIds = 2086096871
)

# check filtering works
testthat::expect_true(nrow(dataTtSub) > 0)
testthat::expect_true(unique(dataTtSub$exposuresOutcomeSetId) == 2086096871)
testthat::expect_true(unique(dataTtSub$databaseId) == 388020256)
testthat::expect_true(unique(dataTtSub$analysisId) == 1)


dataSub <- getSccsTable(
  connectionHandler = connectionHandler,
  schema = schema,
  table = 'event_dep_observation',
  sccsTablePrefix = 'sccs_',
  cgTablePrefix = 'cg_',
  databaseTable = 'database_meta_data',
  analysisIds =  1, 
  databaseIds = 388020256, 
  covariateIds = 1001,
  exposureOutcomeIds = 2086096871
)

# check filtering works
testthat::expect_true(nrow(dataSub) > 0)
testthat::expect_true(unique(dataSub$exposuresOutcomeSetId) == 2086096871)
testthat::expect_true(unique(dataSub$databaseId) == 388020256)
testthat::expect_true(unique(dataSub$analysisId) == 1)

dataSub <- getSccsTable(
  connectionHandler = connectionHandler,
  schema = schema,
  table = 'age_spanning',
  sccsTablePrefix = 'sccs_',
  cgTablePrefix = 'cg_',
  databaseTable = 'database_meta_data',
  analysisIds =  1, 
  databaseIds = 388020256, 
  covariateIds = 1001,
  exposureOutcomeIds = 2086096871
)
# check filtering works
testthat::expect_true(nrow(dataSub) > 0)
testthat::expect_true(unique(dataSub$exposuresOutcomeSetId) == 2086096871)
testthat::expect_true(unique(dataSub$databaseId) == 388020256)
testthat::expect_true(unique(dataSub$analysisId) == 1)

dataSub <- getSccsTable(
  connectionHandler = connectionHandler,
  schema = schema,
  table = 'calendar_time_spanning',
  sccsTablePrefix = 'sccs_',
  cgTablePrefix = 'cg_',
  databaseTable = 'database_meta_data',
  analysisIds =  1, 
  databaseIds = 388020256, 
  covariateIds = 1001,
  exposureOutcomeIds = 2086096871
)
# check filtering works
testthat::expect_true(nrow(dataSub) > 0)
testthat::expect_true(unique(dataSub$exposuresOutcomeSetId) == 2086096871)
testthat::expect_true(unique(dataSub$databaseId) == 388020256)
testthat::expect_true(unique(dataSub$analysisId) == 1)


dataSub <- getSccsTable(
  connectionHandler = connectionHandler,
  schema = schema,
  table =  'spline',
  sccsTablePrefix = 'sccs_',
  cgTablePrefix = 'cg_',
  databaseTable = 'database_meta_data',
  analysisIds =  1, 
  databaseIds = 388020256, 
  covariateIds = 1001,
  exposureOutcomeIds = 2086096871
)
# check filtering works
testthat::expect_true(nrow(dataSub) > 0)
testthat::expect_true(unique(dataSub$exposuresOutcomeSetId) == 2086096871)
testthat::expect_true(unique(dataSub$databaseId) == 388020256)
testthat::expect_true(unique(dataSub$analysisId) == 1)

# invalid table
testthat::expect_error(getSccsTable(
  connectionHandler = connectionHandler,
  schema = schema,
  table =  'madeup',
  sccsTablePrefix = 'sccs_',
  cgTablePrefix = 'cg_',
  databaseTable = 'database_meta_data'
))

})


test_that("getSccsModel", {
  
allModels <- getSccsModel(
    connectionHandler = connectionHandler,
    schema = schema
    )
testthat::expect_true(nrow(allModels) > 0)
testthat::expect_true("databaseName" %in% colnames(allModels))
testthat::expect_true("analysisDescription" %in% colnames(allModels))
testthat::expect_true("targetName" %in% colnames(allModels))
testthat::expect_true("outcomeName" %in% colnames(allModels))
testthat::expect_true("indicationName" %in% colnames(allModels))
testthat::expect_true("covariateName" %in% colnames(allModels))
testthat::expect_true("rr" %in% colnames(allModels))
testthat::expect_true("exposuresOutcomeSetId" %in% colnames(allModels))

noModels <- getSccsModel(
  connectionHandler = connectionHandler,
  schema = schema, 
  databaseIds = 535454 # fake 
)
testthat::expect_true(nrow(noModels) == 0)

noModels <- getSccsModel(
  connectionHandler = connectionHandler,
  schema = schema, 
  exposureOutcomeSetIds = 4343 # fake
)
testthat::expect_true(nrow(noModels) == 0)


})


test_that("getSccsNegativeControlEstimates", {
  
  allNCs <- getSccsNegativeControlEstimates(
    connectionHandler = connectionHandler,
    schema = schema
  )
  testthat::expect_true(nrow(allNCs) > 0)
  testthat::expect_true("databaseName" %in% colnames(allNCs))
  testthat::expect_true("analysisDescription" %in% colnames(allNCs))
  testthat::expect_true("targetName" %in% colnames(allNCs))
  testthat::expect_true("outcomeName" %in% colnames(allNCs))
  testthat::expect_true("indicationName" %in% colnames(allNCs))
  testthat::expect_true("exposuresOutcomeSetId" %in% colnames(allNCs))
  
  dbNCs <- getSccsNegativeControlEstimates(
    connectionHandler = connectionHandler,
    schema = schema, 
    databaseIds = 388020256, 
    targetIds = 1
  )
  testthat::expect_true(unique(dbNCs$databaseId) == 388020256)
  testthat::expect_true(unique(dbNCs$targetId) == 1)
  
  oneNCs <- getSccsNegativeControlEstimates(
    connectionHandler = connectionHandler,
    schema = schema, 
    databaseIds = 388020256, 
    exposuresOutcomeSetIds = 1513783871
  )
  testthat::expect_true(unique(oneNCs$exposuresOutcomeSetId) == 1513783871)
  testthat::expect_true(unique(oneNCs$databaseId) == 388020256)
  
})



test_that("getSccsTimeToEvent", {
  
  allTte <- getSccsTimeToEvent(
    connectionHandler = connectionHandler,
    schema = schema
  )
  testthat::expect_true(nrow( allTte) > 0)
  testthat::expect_true("databaseName" %in% colnames( allTte))
  testthat::expect_true("analysisDescription" %in% colnames( allTte))
  testthat::expect_true("targetName" %in% colnames( allTte))
  testthat::expect_true("outcomeName" %in% colnames( allTte))
  testthat::expect_true("indicationName" %in% colnames( allTte))
  testthat::expect_true("exposuresOutcomeSetId" %in% colnames( allTte))
  
  
  # check filter
  filterTte <- getSccsTimeToEvent(
    connectionHandler = connectionHandler,
    schema = schema, 
    databaseIds = 388020256, 
    exposuresOutcomeSetIds = 2086096871, 
    analysisIds = 1
  )
  
  testthat::expect_true(unique(filterTte$databaseId) == 388020256)
  testthat::expect_true(unique(filterTte$exposuresOutcomeSetId) == 2086096871)
  testthat::expect_true(unique(filterTte$analysisId) == 1)
  
})

test_that("plotCmEstimates", {
  
  data <- getCMEstimation(
    connectionHandler = connectionHandler,
    schema = schema, 
    targetIds = 1002, 
    outcomeIds = 3, 
    comparatorIds = 2002
  )
  
  p <- plotCmEstimates(
    cmData = data,
    cmMeta = NULL, 
    cohortNames = data.frame(
      cohortName = c('target', 'comp'), 
      cohortId = c(1002,2002)
    ),
    selectedAnalysisId = 1
  )
  
  testthat::expect_type(p, "list")
  testthat::expect_s3_class(p[[1]], "forestplot")
  
  # check without counts
  p <- plotCmEstimates(
    cmData = data,
    cmMeta = NULL, 
    cohortNames = data.frame(
      cohortName = c('target', 'comp'), 
      cohortId = c(1002,2002)
    ),
    includeCounts = FALSE,
    selectedAnalysisId = 1
  )
  
  testthat::expect_type(p, "list")
  testthat::expect_s3_class(p[[1]], "forestplot")
  
  # check with meta
  cmMeta <- getCmMetaEstimation(
    connectionHandler = connectionHandler,
    schema = schema, 
    targetIds = 1002, 
    outcomeIds = 3, 
    comparatorIds = 2002
  )
  
  p <- plotCmEstimates(
    cmData = data,
    cmMeta = cmMeta, 
    cohortNames = data.frame(
      cohortName = c('target', 'comp'), 
      cohortId = c(1002,2002)
    ),
    includeCounts = FALSE,
    selectedAnalysisId = 1
  )
  
  testthat::expect_type(p, "list")
  testthat::expect_s3_class(p[[1]], "forestplot")
  
  # check with dignostics

  cmDiagnostics <- getCmDiagnosticsData(
    connectionHandler = connectionHandler,
    schema = schema, 
    targetIds = 1002, 
    outcomeIds = 3, 
    comparatorIds = 2002
  )
  
  p <- plotCmEstimates(
    cmData = data,
    cmMeta = cmMeta, 
    cmDiagnostics = cmDiagnostics,
    cohortNames = data.frame(
      cohortName = c('target', 'comp'), 
      cohortId = c(1002,2002)
    ),
    includeCounts = FALSE,
    selectedAnalysisId = 1
  )
  
  testthat::expect_type(p, "list")
  testthat::expect_s3_class(p[[1]], "forestplot")
  
})


test_that("plotSccsEstimates", {
  
  data <- getSccsEstimation(
    connectionHandler = connectionHandler,
    schema = schema, 
    targetIds = 1, 
    outcomeIds = 3 
  )
  
  meta <- getSccsMetaEstimation(
    connectionHandler = connectionHandler,
    schema = schema, 
    targetIds = 1, 
    outcomeIds = 3 
  )
  
  diag <- getSccsDiagnosticsData(
    connectionHandler = connectionHandler,
    schema = schema, 
    targetIds = 1, 
    outcomeIds = 3 
  )
  
  p <- plotSccsEstimates(
    sccsData = data,
    sccsMeta = NULL,
    includeCounts = TRUE, 
    selectedAnalysisId = 1
  )
  
  testthat::expect_type(p, "list")
  testthat::expect_s3_class(p[[1]], "forestplot")
  
  p <- plotSccsEstimates(
    sccsData = data,
    sccsMeta = NULL,
    includeCounts = FALSE, 
    selectedAnalysisId = 1
  )
  
  testthat::expect_type(p, "list")
  testthat::expect_s3_class(p[[1]], "forestplot")
  
  p <- plotSccsEstimates(
    sccsData = data,
    sccsMeta = meta,
    includeCounts = TRUE, 
    selectedAnalysisId = 1
  )
  
  testthat::expect_type(p, "list")
  testthat::expect_s3_class(p[[1]], "forestplot")
  
  p <- plotSccsEstimates(
    sccsData = data,
    sccsMeta = meta,
    sccsDiagnostics = diag,
    includeCounts = TRUE, 
    selectedAnalysisId = 1
  )
  
  testthat::expect_type(p, "list")
  testthat::expect_s3_class(p[[1]], "forestplot")
  
})

#plotSccsEstimates

.getCVersion <- function(
    connectionHandler,
    schema,
    cTablePrefix = 'c_'
){
  majorVersion <- 0 # Default to v0
  minorVersion <- 0
  tryCatch(
    {
      sql <- "SELECT version_number from @schema.@c_table_prefixpackage_version;"
      
      pkversion <- connectionHandler$queryDb(
        sql = sql,
        schema = schema,
        c_table_prefix = cTablePrefix
      ) %>%
        dplyr::pull(.data$versionNumber) %>%
        dplyr::first()
      
      majorVersion = strsplit(x = pkversion, split = '\\.')[[1]][1]
      minorVersion = strsplit(x = pkversion, split = '\\.')[[1]][2]
 
    },
    error = function(e) {
      # Do nothing - most likely the migration table does not exist so assume
      # v0
    }
  )
  
  if(majorVersion >= 3){
    version <- '3_0_0'
  } else{
    version <- '0'
  }
  
  return(version)
}


.getCIVersion <- function(
    connectionHandler,
    schema,
    ciTablePrefix = 'ci_'
){
  version <- 0 
  
  tryCatch(
    {
      sql <- "SELECT version_number from @schema.@ci_table_prefixpackage_version');"
      
      pkversion <- connectionHandler$queryDb(
        sql = sql,
        schema = schema,
        ci_table_prefix = ciTablePrefix
      ) %>%
        dplyr::pull(.data$versionNumber) %>%
        dplyr::first()
      
      majorVersion = strsplit(x = pkversion, split = '\\.')[[1]][1]
      minorVersion = strsplit(x = pkversion, split = '\\.')[[1]][2]
      
    },
    error = function(e) {
      # Do nothing - most likely the migration table does not exist so assume
      # v0
    }
  )
  
  return(version)
}

# the actual package version to display
.getCPackageVersion <- function(
    connectionHandler,
    schema,
    cTablePrefix = 'c_'
){

  version <- '0.0.0'
  tryCatch(
    {
      sql <- "SELECT version_number from @schema.@c_table_prefixpackage_version');"
      
      version <- connectionHandler$queryDb(
        sql = sql,
        schema = schema,
        c_table_prefix = cTablePrefix
      ) %>%
        dplyr::pull(.data$versionNumber) %>%
        dplyr::first()
      
    },
    error = function(e) {
      # Do nothing - most likely the migration table does not exist so assume
      # v0
    }
  )

  return(version)
}


#' A function to extarct the targets found in characterization
#'
#' @details
#' Specify the connectionHandler, the schema and the prefixes
#'
#' @template connectionHandler
#' @template schema
#' @template cTablePrefix
#' @template cgTablePrefix
#' @param printTimes Print the time it takes to run each query
#' @param useTte whether to determine what cohorts are used in time to event
#' @param useDcrc whether to determine what cohorts are used in dechal-rechal
#' @param useRf whether to determine what cohorts are used in risk factor
#' @param useTb whether to determine what cohorts are used in target baseline
#' @param useCs whether to determine what cohorts are used in case-series
#' 
#' @family Characterization
#' 
#' @return
#' A data.frame with the characterization target cohort ids, names and which characterization analyses the cohorts are used in.
#'
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' cohorts <- getCharacterizationTargets(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getCharacterizationTargets <- function(
    connectionHandler,
    schema,
    cTablePrefix = 'c_',
    cgTablePrefix = 'cg_',
    printTimes = FALSE,
    useTte = TRUE,
    useDcrc = TRUE,
    useRf = TRUE,
    useTb = TRUE,
    useCs = TRUE
){
  
  cVersion <- .getCVersion(
      connectionHandler = connectionHandler,
      schema = schema,
      cTablePrefix = cTablePrefix
    )

  
  first <- Sys.time()
  
  tteData <- data.frame()
  if(useTte){ 
    start <- Sys.time()
    
    # check tte normalized table with target_cohort_definition_id exists or return NULL if it does not
    normExists <- tryCatch({
      connectionHandler$queryDb(
        sql = "select * from @schema.@c_table_prefixtime_to_event_targets limit 1;",
        schema = schema,
        c_table_prefix = cTablePrefix
      )
    }, error = function(e){
      return(NULL)
      })
    
    tableOrView <- ifelse(
      is.null(normExists),
      "(select distinct target_cohort_definition_id from @schema.@c_table_prefixtime_to_event)",
      "@schema.@c_table_prefixtime_to_event_targets"
    )
    
    tteData <- tryCatch({connectionHandler$queryDb(
      sql = paste0("
      select 
           cg.cohort_name,
           tte.target_cohort_definition_id as cohort_definition_id,
           'timeToEvent' as type,
           1 as value
        from ",tableOrView," tte
        inner join 
        @schema.@cg_table_prefixcohort_definition cg
        on tte.target_cohort_definition_id = cg.cohort_definition_id
    ;"),
      schema = schema,
      cg_table_prefix = cgTablePrefix,
      c_table_prefix = cTablePrefix
    )}, error = function(e){warning(e); return(NULL)})
    
    end <- Sys.time()
    if(printTimes){
      print(paste0('extracting time_to_event targets: ', (end-start), ' ', units((end-start))))
    }
  }
  
  
  dcrcData  <- data.frame()
  if(useDcrc){
    start <- Sys.time()
    
    sql <- SqlRender::readSql(system.file(
      paste0("sql/sql_server/characterization/getCharacterizationTargetsDcrc.sql"),
      package = "OhdsiReportGenerator",
      mustWork = TRUE
    ))
    
    dcrcData <- tryCatch({connectionHandler$queryDb(
      sql = sql,
      schema = schema,
      cg_table_prefix = cgTablePrefix,
      c_table_prefix = cTablePrefix
    )}, error = function(e){warning(e); return(NULL)})
    
    end <- Sys.time()
    if(printTimes){
      print(paste0('extracting dechallenge_rechallenge targets: ',  (end-start), ' ', units((end-start))))
    }
  }
  
  rfData  <- data.frame()
  if(useRf){
    start <- Sys.time()
    
    sql <- SqlRender::readSql(system.file(
      paste0("sql/sql_server/characterization/getCharacterizationTargetsRfV", cVersion, ".sql"),
      package = "OhdsiReportGenerator",
      mustWork = TRUE
    ))
    
    rfData <- tryCatch({connectionHandler$queryDb(
      sql = sql,
      schema = schema,
      cg_table_prefix = cgTablePrefix,
      c_table_prefix = cTablePrefix
    )}, error = function(e){warning(e); return(NULL)})
    
    end <- Sys.time()
    
    if(printTimes){
      print(paste0('extracting risk factor targets: ',  (end-start), ' ', units((end-start))))
    }
    
  }
  
  tbData  <- data.frame()
  if(useTb){

    start <- Sys.time()
    
    sql <- SqlRender::readSql(system.file(
      paste0("sql/sql_server/characterization/getCharacterizationTargetsTbV", cVersion, ".sql"),
      package = "OhdsiReportGenerator",
      mustWork = TRUE
    ))
    
    tbData <- tryCatch({connectionHandler$queryDb(
      sql = sql,
      schema = schema,
      cg_table_prefix = cgTablePrefix,
      c_table_prefix = cTablePrefix
    )}, error = function(e){warning(e); return(NULL)})
    
    end <- Sys.time()
    
    if(printTimes){
      print(paste0('extracting target baseline targets: ',  (end-start), ' ', units((end-start))))
    }
 
  }
  
  csData  <- data.frame()
  if(useCs){
    start <- Sys.time()
    
    if(file.exists(system.file(
      paste0("sql/sql_server/characterization/getCharacterizationTargetsCsV", cVersion, ".sql"),
      package = "OhdsiReportGenerator"
    ))){
      
      sql <- SqlRender::readSql(system.file(
        paste0("sql/sql_server/characterization/getCharacterizationTargetsCsV", cVersion, ".sql"),
        package = "OhdsiReportGenerator",
        mustWork = TRUE
      ))
      
      csData <- tryCatch({connectionHandler$queryDb(
        sql = sql,
        schema = schema,
        cg_table_prefix = cgTablePrefix,
        c_table_prefix = cTablePrefix
      )}, error = function(e){warning(e); return(NULL)})
      
      end <- Sys.time()
      
      if(printTimes){
        print(paste0('extracting case series targets: ',  (end-start), ' ', units((end-start))))
      }
    }
    
  }
  
  start <- Sys.time()
  
  targets <- rbind(tteData, dcrcData, rfData, tbData, csData)
  if(is.null(targets)){
    message('No target data')
    end <- Sys.time()
    print(paste0('-- all extracting characterization targets took: ',  (end-first), ' ', units((end-first))))
    return(NULL)
  }
  
  targets <- targets %>%
    tidyr::pivot_wider(
      id_cols = c("cohortName", "cohortDefinitionId"), 
      names_from = "type", 
      values_from = c("value"), 
      values_fill = 0
    )
  
  end <- Sys.time()
  
  if(printTimes){
    print(paste0('pivoting data took: ',  (end-start), ' ', units((end-start))))
  }
  
  start <- Sys.time()
  # add missing types with 0 values
  colnameTypes <- c('timeToEvent','dechalRechal','riskFactors','databaseComparator') 
  
  if(sum(colnameTypes %in% colnames(targets)) != 4){
    missingCols <- colnameTypes[!colnameTypes %in% colnames(targets)]
    for(missingCol in missingCols){
      targets[missingCol] <- 0
    }
  }
  

  # pre V3
  if(!"caseSeries" %in% colnames(targets)){
    targets$caseSeries <- targets$riskFactors
  }
  
  # Add redundant columns - these are dep on each other
  targets$cohortComparator <- targets$databaseComparator
  
  end <- Sys.time()
  
  if(printTimes){
    print(paste0('processing characterization target details: ', (end-start), ' ', units((end-start))))
  }
  
  print(paste0('-- all extracting characterization targets took: ',  (end-first), ' ', units((end-first))))
  
  return(targets)
}


#' A function to extract the outcomes found in characterization
#'
#' @details
#' Specify the connectionHandler, the schema and the prefixes
#'
#' @template connectionHandler
#' @template schema
#' @template cTablePrefix
#' @template cgTablePrefix
#' @template targetId
#' @param printTimes Print the time it takes to run each query
#' @param useDcrc look for outcome in dechal-rechal results
#' @param useTte look for outcome in time-to-event results
#' @param useRf look for outcome in risk-factor results
#' @param useCs look for outcome in case series results
#' 
#' @family Characterization
#' 
#' @return
#' A data.frame with the characterization outcome cohort ids, names and which characterization analyses the cohorts are used in.
#'
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' cohorts <- getCharacterizationOutcomes(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getCharacterizationOutcomes <- function(
    connectionHandler,
    schema,
    cTablePrefix = 'c_',
    cgTablePrefix = 'cg_',
    targetId = NULL,
    printTimes = FALSE,
    useDcrc = TRUE,
    useTte = TRUE,
    useRf = TRUE,
    useCs = TRUE
){
  
  cVersion <- .getCVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cTablePrefix = cTablePrefix
  )
  
  firstStart <- Sys.time()
  
  # first check each table
  
  tteData <- data.frame()
  if(useTte){ # if user wants time to event see whether there are results
    start <- Sys.time()
    
    sql <- SqlRender::readSql(system.file(
      paste0("sql/sql_server/characterization/getCharacterizationOutcomesTte.sql"),
      package = "OhdsiReportGenerator",
      mustWork = TRUE
    ))
    
    tteData <- tryCatch({connectionHandler$queryDb(
      sql = sql,
      schema = schema,
      cg_table_prefix = cgTablePrefix,
      c_table_prefix = cTablePrefix,
      use_target = !is.null(targetId),
      target_ids = paste0(targetId, collapse = ',')
    )}, error = function(e){warning(e); return(NULL)})
    
    end <- Sys.time()
    if(printTimes){
      print(paste0('extracting time_to_event outcomes: ', (end-start), ' ', units((end-start))))
    }
  }
  
  dcrcData <- data.frame()
  if(useDcrc){ # if user wants dechal see whether there are results
    start <- Sys.time()
    
    sql <- SqlRender::readSql(system.file(
      paste0("sql/sql_server/characterization/getCharacterizationOutcomesDcrc.sql"),
      package = "OhdsiReportGenerator",
      mustWork = TRUE
    ))
    
    dcrcData <- tryCatch({connectionHandler$queryDb(
      sql = sql,
      schema = schema,
      cg_table_prefix = cgTablePrefix,
      c_table_prefix = cTablePrefix,
      use_target = !is.null(targetId),
      target_ids = paste0(targetId, collapse = ',')
    )}, error = function(e){warning(e); return(NULL)})
    
    end <- Sys.time()
    if(printTimes){
      print(paste0('extracting dechallenge_rechallenge outcomes: ', (end-start), ' ', units((end-start))))
    }
  }
  
  rfData <- data.frame()
  if(useRf){ # if user wants risk factors see whether there are results
    start <- Sys.time()
    
    sql <- SqlRender::readSql(system.file(
      paste0("sql/sql_server/characterization/getCharacterizationOutcomesRfV", cVersion, ".sql"),
      package = "OhdsiReportGenerator",
      mustWork = TRUE
    ))
    
    rfData <- tryCatch({connectionHandler$queryDb(
      sql = sql,
      schema = schema,
      cg_table_prefix = cgTablePrefix,
      c_table_prefix = cTablePrefix,
      use_target = !is.null(targetId),
      target_ids = paste0(targetId, collapse = ',')
    )}, error = function(e){warning(e); return(NULL)})
    
    end <- Sys.time()
    if(printTimes){
      print(paste0('extracting risk factor outcomes: ', (end-start), ' ', units((end-start))))
    }
  }
  
  csData <- data.frame()
  if(useCs){
    
    if(file.exists(
      system.file(
        paste0("sql/sql_server/characterization/getCharacterizationOutcomesCsV", cVersion, ".sql"),
        package = "OhdsiReportGenerator"
      )
    )){
      start <- Sys.time()
      
      sql <- SqlRender::readSql(system.file(
        paste0("sql/sql_server/characterization/getCharacterizationOutcomesCsV", cVersion, ".sql"),
        package = "OhdsiReportGenerator",
        mustWork = TRUE
      ))
      
      csData <- tryCatch({connectionHandler$queryDb(
        sql = sql,
        schema = schema,
        cg_table_prefix = cgTablePrefix,
        c_table_prefix = cTablePrefix,
        use_target = !is.null(targetId),
        target_ids = paste0(targetId, collapse = ',')
      )}, error = function(e){warning(e); return(NULL)})
      
      end <- Sys.time()
      if(printTimes){
        print(paste0('extracting case series outcomes: ', (end-start), ' ', units((end-start))))
      }
    }
  }
  
  
  start <- Sys.time()
  
  if(cVersion == 0 & nrow(rfData) > 0){
    csData <- rfData
    csData$type <- 'caseSeries'
  }
  
  outcomes <- rbind(tteData, dcrcData, rfData, csData) 
  
  if(is.null(outcomes)){
    end <- Sys.time()
    message('No outcomes found')
    print(paste0('Extracting characterization outcomes took: ', (end-firstStart), ' ', units((end-firstStart))))
    return(NULL)
  }
  
  outcomes <- outcomes %>%
    tidyr::pivot_wider(
      id_cols = c("cohortName", "cohortDefinitionId"), 
      names_from = "type", 
      values_from = c("value"), 
      values_fill = 0
    )
  
  end <- Sys.time()
  if(printTimes){
    print(paste0('pivoting characterization outcome cohort details: ', (end-start), ' ', units((end-start))))
  }
  
  
  start <- Sys.time()
  
  # add missing types with 0 values
  colnameTypes <- c('timeToEvent','dechalRechal','riskFactors', 'caseSeries') 
  if(sum(colnameTypes %in% colnames(outcomes)) != 4){
    missingCols <- colnameTypes[!colnameTypes %in% colnames(outcomes)]
    for(missingCol in missingCols){
      outcomes[missingCol] <- 0
    }
  }
  
  # get case series tar: risk_window_start/risk_window_end/start_anchor/end_anchor and outcome_washout_days
  
  if(useRf | useCs){
    
    sql <- SqlRender::readSql(system.file(
      paste0("sql/sql_server/characterization/getCharacterizationOutcomesTarsV", cVersion, ".sql"),
      package = "OhdsiReportGenerator"
    ))
    
    outcomeDetails <- tryCatch({connectionHandler$queryDb( 
      sql = sql,
      schema = schema,
      c_table_prefix = cTablePrefix,
      use_target = !is.null(targetId),
      target_ids = paste0(targetId, collapse = ',')
    ) %>% 
        dplyr::rowwise() %>%
        dplyr::mutate(
          tarName = paste0('(',.data$startAnchor, ' + ',.data$riskWindowStart , ') - (',
                           .data$endAnchor, ' + ',.data$riskWindowEnd , ')'),
          tarString = paste0(.data$riskWindowStart, '/',.data$startAnchor , '/',
                             .data$riskWindowEnd, '/',.data$endAnchor )
        ) %>%
        dplyr::select("cohortDefinitionId", "tarName", "tarString", "outcomeWashoutDays") %>%
        dplyr::group_by(.data$cohortDefinitionId) %>%
        dplyr::summarise(
          tarNames = paste0(unique(.data$tarName), collapse = ':'),
          tarStrings = paste0(unique(.data$tarString), collapse = ':'),
          outcomeWashoutDays = paste0(unique(.data$outcomeWashoutDays), collapse = ':')
        )}, error = function(e){NULL})
    
    if(!is.null(outcomeDetails)){
      outcomes <- merge(
        x = outcomes,
        y = outcomeDetails, 
        by = 'cohortDefinitionId', 
        all.x = TRUE
      )
    }
    
  }
  
  end <- Sys.time()
  if(printTimes){
    print(paste0('processing characterization outcomes and adding tars/washout: ', (end-start), ' ', units((end-start))))
  }
  
  print(paste0('Extracting characterization outcomes took: ', (end-firstStart), ' ', units((end-firstStart))))
  
  return(outcomes)
  
}


#' A function to extract the targets found in incidence
#'
#' @details
#' Specify the connectionHandler, the schema and the prefixes
#'
#' @template connectionHandler
#' @template schema
#' @template ciTablePrefix
#' @template cgTablePrefix
#' @family Characterization
#' 
#' @return
#' A data.frame with the incidence target cohort ids and names
#'
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' cohorts <- getIncidenceTargets(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getIncidenceTargets <- function(
    connectionHandler,
    schema,
    ciTablePrefix = 'ci_',
    cgTablePrefix = 'cg_'
){
  
  ciVersion <- .getCIVersion(
      connectionHandler = connectionHandler,
      ciTablePrefix = ciTablePrefix
      )
  
  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/characterization/getIncidenceTargetsV", ciVersion, ".sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
  targets <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    cg_table_prefix = cgTablePrefix,
    ci_table_prefix = ciTablePrefix
  ) %>%
    tidyr::pivot_wider(
      id_cols = c("cohortName", "cohortDefinitionId"), 
      names_from = "type", 
      values_from = c("value")
    )
  
  return(targets)
  
}


#' A function to extract the outcomes found in incidence
#'
#' @details
#' Specify the connectionHandler, the schema and the prefixes
#'
#' @template connectionHandler
#' @template schema
#' @template ciTablePrefix
#' @template cgTablePrefix
#' @template targetId
#' @family Characterization
#' 
#' @return
#' A data.frame with the incidence outcome cohort ids and names
#'
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' outcomes <- getIncidenceOutcomes(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getIncidenceOutcomes <- function(
    connectionHandler,
    schema,
    ciTablePrefix = 'ci_',
    cgTablePrefix = 'cg_',
    targetId = NULL
){
  
  ciVersion <- .getCIVersion(
    connectionHandler = connectionHandler,
    ciTablePrefix = ciTablePrefix
  )
  
  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/characterization/getIncidenceOutcomesV", ciVersion, ".sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
  outcomes <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    cg_table_prefix = cgTablePrefix,
    ci_table_prefix = ciTablePrefix,
    use_target = !is.null(targetId),
    target_id = paste0(targetId, collapse = ',')
  ) %>%
    tidyr::pivot_wider(
      id_cols = c("cohortName", "cohortDefinitionId"), 
      names_from = "type", 
      values_from = c("value")
    )
  
  return(outcomes)
  
}

#' Extract the cohort incidence result
#' @description
#' This function extracts all incidence rates across databases in the results for specified target and outcome cohorts.
#'
#' @details
#' Specify the connectionHandler, the schema and the target/outcome cohort IDs
#'
#' @template connectionHandler
#' @template schema
#' @template ciTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @template targetIds
#' @template outcomeIds
#' @family Characterization
#' @return
#' Returns a data.frame with the columns:
#' \itemize{
#'  \item{databaseName the name of the database}
#'  \item{databaseId the unique id of the database}
#'  \item{targetName the target cohort name}
#'  \item{targetId the target cohort unique identifier}
#'  \item{outcomeName the outcome name}
#'  \item{outcomeId the outcome unique identifier}
#'  \item(tar the friendly time-at-risk string)
#'  \item{cleanWindow clean windown around outcome}
#'  \item{subgroupName name for the result subgroup}
#'  \item{ageGroupName name for the result age group}
#'  \item{genderName name for the result gender group}
#'  \item{startYear name for the result start year}
#'  \item{tarStartWith time at risk start reference}
#'  \item{tarStartOffset time at risk start offset from reference}
#'  \item{tarEndWith time at risk end reference}
#'  \item{tarEndOffset time at risk end offset from reference}
#'  \item{personsAtRiskPe persons at risk per event}
#'  \item{personsAtRisk persons at risk}
#'  \item{personDaysPe person days per event}
#'  \item{personDays person days}
#'  \item{personOutcomesPe person outcome per event}
#'  \item{personOutcomes persons outcome}
#'  \item{outcomesPe number of outcome per event}
#'  \item{outcomes number of outcome}
#'  \item{incidenceProportionP100p incidence proportion per 100 persons}
#'  \item{incidenceRateP100py incidence rate per 100 person years}
#'  } 
#' 
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' ir <- getIncidenceRates(
#' connectionHandler = connectionHandler, 
#' schema = 'main'
#' )
#' 
getIncidenceRates <- function(
    connectionHandler,
    schema,
    ciTablePrefix = 'ci_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    targetIds = NULL,
    outcomeIds = NULL
){
  
  ciVersion <- .getCIVersion(
    connectionHandler = connectionHandler,
    ciTablePrefix = ciTablePrefix
  )
  
  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/characterization/getIncidenceRatesV", ciVersion, ".sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    ci_table_prefix = ciTablePrefix,
    cg_table_prefix = cgTablePrefix,
    target_id = paste0(targetIds, collapse = ','),
    use_target = !is.null(targetIds),
    outcome_id = paste0(outcomeIds, collapse = ','),
    use_outcome = !is.null(outcomeIds),
    database_table_name = databaseTable
  )
  
  if(nrow(result) > 0){
    result$incidenceProportionP100p[is.na(result$incidenceProportionP100p)] <- result$outcomes[is.na(result$incidenceProportionP100p)]/result$personsAtRisk[is.na(result$incidenceProportionP100p)]*100
    result$incidenceProportionP100p[is.na(result$incidenceProportionP100p)] <- 0
    result$incidenceRateP100py[is.na(result$incidenceRateP100py)] <- result$outcomes[is.na(result$incidenceRateP100py)]/(result$personDays[is.na(result$incidenceRateP100py)]/365)*100
    result$incidenceRateP100py[is.na(result$incidenceRateP100py)] <- 0
    result[is.na(result)] <- 'Any'
    result <- unique(result)
    
    # add friendly tar
    result$tar <- paste0('( ',result$tarStartWith, ' + ', result$tarStartOffset, ' ) - ( ',
                         result$tarEndWith, ' + ', result$tarEndOffset, ' )')
  } else{
    # add tar but using other column as length 0 this just add name
    result$tar <- result$tarStartWith
  }
  
  # change the position of tar
  result <- result %>% dplyr::relocate("tar", .after = "outcomeId")
  
  return(result)
}

#' Extract the time to event result
#' @description
#' This function extracts all time to event results across databases for specified target and outcome cohorts.
#'
#' @details
#' Specify the connectionHandler, the schema and the target/outcome cohort IDs
#'
#' @template connectionHandler
#' @template schema
#' @template cTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @template targetIds
#' @template outcomeIds
#' @family Characterization
#' @return
#' Returns a data.frame with the columns:
#' \itemize{
#'  \item{databaseName the name of the database}
#'  \item{databaseId the unique identifier of the database}
#'  \item{targetName the target cohort name}
#'  \item{targetId the target cohort unique identifier}
#'  \item{outcomeName the outcome name}
#'  \item{outcomeId the outcome unique identifier}
#'  \item{outcomeType Whether the outcome is the first or subsequent}
#'  \item{targetOutcomeType The interval that the outcome occurs}
#'  \item{timeToEvent The number of days from index}
#'  \item{numEvents The number of target cohort entries}
#'  \item{timeScale The correspondin time-scale}
#'  } 
#' 
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' tte <- getTimeToEvent(
#' connectionHandler = connectionHandler, 
#' schema = 'main'
#' )
#'  
getTimeToEvent <- function(
    connectionHandler,
    schema,
    cTablePrefix = 'c_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    targetIds = NULL,
    outcomeIds = NULL
){
  
  # add code here
  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/characterization/getTimeToEvent.sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    target_id = paste0(targetIds, collapse = ','),
    use_target = !is.null(targetIds),
    outcome_id = paste0(outcomeIds, collapse = ','),
    use_outcome = !is.null(outcomeIds),
    c_table_prefix = cTablePrefix,
    cg_table_prefix = cgTablePrefix,
    database_table = databaseTable
  )
  
  return(result)
}

#' Extract the dechallenge rechallenge results
#' @description
#' This function extracts all dechallenge rechallenge results across databases for specified target and outcome cohorts.
#'
#' @details
#' Specify the connectionHandler, the schema and the target/outcome cohort IDs
#'
#' @template connectionHandler
#' @template schema
#' @template cTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @template targetIds
#' @template outcomeIds
#' @family Characterization
#' @return
#' Returns a data.frame with the columns:
#' \itemize{
#'  \item{databaseName the name of the database}
#'  \item{databaseId the unique identifier of the database}
#'  \item{targetName the target cohort name}
#'  \item{targetId the target cohort unique identifier}
#'  \item{outcomeName the outcome name}
#'  \item{outcomeId the outcome unique identifier}
#'  \item{dechallengeStopInterval An integer specifying the how much time to add to the cohort_end when determining whether the event starts during cohort and ends after}
#'  \item{dechallengeEvaluationWindow A period of time evaluated for outcome recurrence after discontinuation of exposure, among patients with challenge outcomes}
#'  \item{numExposureEras Distinct number of exposure events (i.e. drug eras) in a given target cohort}
#'  \item{numPersonsExposed Distinct number of people exposed in target cohort. A person must have at least 1 day exposure to be included}
#'  \item{numCases Distinct number of persons in outcome cohort. A person must have at least 1 day of observation time to be included}
#'  \item{dechallengeAttempt Distinct count of people with observable time after discontinuation of the exposure era during which the challenge outcome occurred}
#'  \item{dechallengeFail Among people with challenge outcomes, the distinct number of people with outcomes during dechallengeEvaluationWindow}
#'  \item{dechallengeSuccess Among people with challenge outcomes, the distinct number of people without outcomes during the dechallengeEvaluationWindow}
#'  \item{rechallengeAttempt Number of people with a new exposure era after the occurrence of an outcome during a prior exposure era}
#'  \item{rechallengeFail Number of people with a new exposure era during which an outcome occurred, after the occurrence of an outcome during a prior exposure era}
#'  \item{rechallengeSuccess Number of people with a new exposure era during which an outcome did not occur, after the occurrence of an outcome during a prior exposure era}
#'  \item{pctDechallengeAttempt Percent of people with observable time after discontinuation of the exposure era during which the challenge outcome occurred}
#'  \item{pctDechallengeFail Among people with challenge outcomes, the percent of people without outcomes during the dechallengeEvaluationWindow}
#'  \item{pctDechallengeSuccess Among people with challenge outcomes, the percent of people with outcomes during dechallengeEvaluationWindow}
#'  \item{pctRechallengeAttempt Percent of people with a new exposure era after the occurrence of an outcome during a prior exposure era}
#'  \item{pctRechallengeFail Percent of people with a new exposure era during which an outcome did not occur, after the occurrence of an outcome during a prior exposure era}
#'  \item{pctRechallengeSuccess Percent of people with a new exposure era during which an outcome occurred, after the occurrence of an outcome during a prior exposure era}
#'  } 
#' 
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' dcrc <- getDechallengeRechallenge(
#' connectionHandler = connectionHandler, 
#' schema = 'main'
#' )
#' 
getDechallengeRechallenge <- function(
    connectionHandler,
    schema,
    cTablePrefix = 'c_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    targetIds = NULL,
    outcomeIds = NULL
){
  
  # add code here
  
  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/characterization/getDechallengeRechallenge.sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    target_id = paste0(targetIds, collapse = ','),
    use_target = !is.null(targetIds),
    outcome_id = paste0(outcomeIds, collapse = ','),
    use_outcome = !is.null(outcomeIds),
    c_table_prefix = cTablePrefix,
    cg_table_prefix = cgTablePrefix,
    database_table = databaseTable
  )
  
  return(result)
}


#' Extract the aggregate covariates for the target ids of interest
#' @description
#' This function extracts the specified covariates for the specified targets
#'
#' @details
#' Specify the connectionHandler, the schema and the target cohort IDs
#'
#' @template connectionHandler
#' @template schema
#' @template cTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @template targetIds
#' @param analysisIds The analysisIds of the covariate to restrict results to
#' @param covariateIds The covariateIds to restict results to
#' @param conceptIds The conceptIds of the covariate to restrict results to
#' @param databaseIds The databaseIds of the covariate to restrict results to
#' @family Characterization
#' @return
#' Returns a data.frame with the columns:
#' \itemize{
#'  \item{databaseName the name of the database}
#'  \item{databaseId the unique identifier of the database}
#'  \item{targetName the target cohort name}
#'  \item{targetId the target cohort unique identifier}
#'  \item{minPriorObservation the }
#'  \item{limitToFirstINDays the }
#'  \item{covariateName the }
#'  \item{covariateId the }
#'  \item{analysisId the }
#'  \item{sumValue the }
#'  \item{averageValue the }
#'  } 
#' 
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' btb <- getBinaryTargetBaseline(
#' connectionHandler = connectionHandler, 
#' schema = 'main', 
#' targetIds = 1
#' )
#'  
getBinaryTargetBaseline <- function(
    connectionHandler,
    schema,
    cTablePrefix = 'c_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    targetIds = NULL,
    analysisIds = NULL,
    covariateIds = NULL,
    conceptIds = NULL,
    databaseIds = NULL
){
  
  cVersion <- .getCVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cTablePrefix = cTablePrefix
  )
  
  if(cVersion == 0){
    message('Function not available with result version')
    return(NULL)
  }
  
  # add code here
  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/characterization/getBinaryTargetBaselineV",cVersion,".sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    target_ids = paste0(targetIds, collapse = ','),
    use_targets = !is.null(targetIds),
    covariate_ids = paste0(covariateIds, collapse = ','),
    use_covariate = !is.null(covariateIds),
    analysis_ids = paste0(analysisIds, collapse = ','),
    use_analysis = !is.null(analysisIds),
    concept_ids = paste0(conceptIds, collapse = ','),
    use_concept = !is.null(conceptIds),
    database_ids = paste0(databaseIds, collapse = ','),
    use_database = !is.null(databaseIds),
    c_table_prefix = cTablePrefix,
    cg_table_prefix = cgTablePrefix,
    database_table = databaseTable
  )
  
  return(result)
}

#' Extract the target cohort counts result
#' @description
#' This function extracts target cohort counts across databases in the results for specified target and outcome cohorts.
#'
#' @details
#' Specify the connectionHandler, the schema and the target/outcome cohort IDs
#'
#' @template connectionHandler
#' @template schema
#' @template cTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @template targetIds
#' @template outcomeIds
#' @param databaseIds A vector of database IDs to restrict to
#' 
#' @family Characterization
#' @return
#' Returns a data.frame with the columns:
#' \itemize{
#'  \item{databaseName the name of the database}
#'  \item{databaseId the unique identifier of the database}
#'  \item{targetName the target cohort name}
#'  \item{targetId the target cohort unique identifier}
#'  \item{outcomeName the outcome name}
#'  \item{outcomeId the outcome unique identifier}
#'  \item{rowCount the number of entries in the cohort}
#'  \item{personCount the number of people in the cohort}
#'  \item{withoutExcludedPersonCount the number of people in the target ignoring exclusions}
#'  \item{minPriorObservation the minimum required observation days prior to index for an entry}
#'  \item{outcomeWashoutDays patients with the outcome occurring within this number of days prior to index are excluded (NA means no exclusion)}
#'  } 
#' 
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' tc <- getCaseTargetCounts(
#' connectionHandler = connectionHandler, 
#' schema = 'main'
#' )
#' 
getCaseTargetCounts <- function(
    connectionHandler,
    schema,
    cTablePrefix = 'c_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    targetIds = NULL,
    outcomeIds = NULL,
    databaseIds = NULL
){
  
  cVersion <- .getCVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cTablePrefix = cTablePrefix
  )
  
  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/characterization/getCaseTargetCountsV", cVersion, ".sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
  
 result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    target_id = paste0(targetIds, collapse = ','),
    use_target = !is.null(targetIds),
    outcome_id = paste0(outcomeIds, collapse = ','),
    use_outcome = !is.null(outcomeIds),
    c_table_prefix = cTablePrefix,
    cg_table_prefix = cgTablePrefix,
    database_table_name = databaseTable,
    database_id = paste0("'",databaseIds,"'", collapse = ","),
    use_database = !is.null(databaseIds)
  )
  
  return(result)
}



#' Extract the outcome cohort counts result
#' @description
#' This function extracts outcome cohort counts across databases in the results for specified target and outcome cohorts.
#'
#' @details
#' Specify the connectionHandler, the schema and the target/outcome cohort IDs
#'
#' @template connectionHandler
#' @template schema
#' @template cTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @template targetIds
#' @template outcomeIds
#' @param databaseIds (optional) A vector of database IDs to restrict to
#' @param riskWindowStart (optional) A vector of time-at-risk risk window starts to restrict to
#' @param riskWindowEnd (optional) A vector of time-at-risk risk window ends to restrict to
#' @param startAnchor (optional) A vector of time-at-risk start anchors to restrict to
#' @param endAnchor (optional) A vector of time-at-risk end anchors to restrict to
#' 
#' @family Characterization
#' @return
#' Returns a data.frame with the columns:
#' \itemize{
#'  \item{databaseName the name of the database}
#'  \item{databaseId the unique identifier of the database}
#'  \item{targetName the target cohort name}
#'  \item{targetId the target cohort unique identifier}
#'  \item{outcomeName the outcome name}
#'  \item{outcomeId the outcome unique identifier}
#'  \item{rowCount the number of entries in the cohort}
#'  \item{personCount the number of people in the cohort}
#'  \item{minPriorObservation the minimum required observation days prior to index for an entry}
#'  \item{outcomeWashoutDays patients with the outcome occurring within this number of days prior to index are excluded (NA means no exclusion)}
#' \item{riskWindowStart the number of days ofset the start anchor that is the start of the time-at-risk}
#' \item{startAnchor the start anchor is either the target cohort start or cohort end date}
#' \item{riskWindowEnd the number of days ofset the end anchor that is the end of the time-at-risk}
#' \item{endAnchor the end anchor is either the target cohort start or cohort end date}
#' } 
#' 
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' cc <- getCaseCounts(
#' connectionHandler = connectionHandler, 
#' schema = 'main'
#' )
#' 
getCaseCounts <- function(
    connectionHandler,
    schema,
    cTablePrefix = 'c_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    targetIds = NULL,
    outcomeIds = NULL,
    databaseIds = NULL,
    riskWindowStart = NULL,
    riskWindowEnd = NULL,
    startAnchor = NULL,
    endAnchor = NULL
){
  
  cVersion <- .getCVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cTablePrefix = cTablePrefix
  )
  
  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/characterization/getCaseCountsV", cVersion, ".sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    c_table_prefix = cTablePrefix,
    cg_table_prefix = cgTablePrefix,
    database_table = databaseTable,
    target_id = paste0(targetIds, collapse = ','),
    use_target = !is.null(targetIds),
    outcome_id = paste0(outcomeIds, collapse = ','),
    use_outcome = !is.null(outcomeIds),
    database_id = paste0("'",databaseIds,"'", collapse = ","),
    use_database = !is.null(databaseIds),
    
    use_risk_window_start = !is.null(riskWindowStart),
    risk_window_start = paste0(riskWindowStart, collapse = ','),
    use_risk_window_end = !is.null(riskWindowEnd),
    risk_window_end = paste0(riskWindowEnd, collapse = ','),
    use_start_anchor = !is.null(startAnchor),
    start_anchor = paste0("'",startAnchor,"'", collapse = ","),
    use_end_anchor = !is.null(endAnchor),
    end_anchor = paste0("'",endAnchor,"'", collapse = ",")
  )
  
  return(result)
}


getCaseBinaryFeatures <- function(
    connectionHandler,
    schema,
    cTablePrefix = 'c_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    targetIds = NULL,
    outcomeIds = NULL,
    databaseIds = NULL,
    analysisIds = c(3), # c(8507, 8532)
    riskWindowStart = NULL,
    riskWindowEnd = NULL,
    startAnchor = NULL,
    endAnchor = NULL
){
  
  cVersion <- .getCVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cTablePrefix = cTablePrefix
  )
  
  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/characterization/getCaseBinaryFeaturesV", cVersion, ".sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))

result <- connectionHandler$queryDb(
  sql = sql,
  schema = schema,
  target_id = paste0(targetIds, collapse = ','),
  use_target = !is.null(targetIds),
  outcome_id = paste0(outcomeIds, collapse = ','),
  use_outcome = !is.null(outcomeIds),
  c_table_prefix = cTablePrefix,
  cg_table_prefix = cgTablePrefix,
  database_table = databaseTable,
  use_analysis = !is.null(analysisIds),
  analysis_ids = paste0(analysisIds, collapse = ','),
  database_id = paste0("'",databaseIds,"'", collapse = ","),
  use_database = !is.null(databaseIds),
  
  use_risk_window_start = !is.null(riskWindowStart),
  risk_window_start = paste0(riskWindowStart, collapse = ','),
  use_risk_window_end = !is.null(riskWindowEnd),
  risk_window_end = paste0(riskWindowEnd, collapse = ','),
  use_start_anchor = !is.null(startAnchor),
  start_anchor = paste0("'",startAnchor,"'", collapse = ","),
  use_end_anchor = !is.null(endAnchor),
  end_anchor = paste0("'",endAnchor,"'", collapse = ",")
)

 return(result)
}


#' Extract the binary age groups for the cases and targets
#' @description
#' This function extracts the age group feature extraction results for cases and targets corresponding to specified target and outcome cohorts.
#'
#' @details
#' Specify the connectionHandler, the schema and the target/outcome cohort IDs
#'
#' @template connectionHandler
#' @template schema
#' @template cTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @template targetId
#' @template outcomeId
#' @param type A character of 'age' or 'sex'
#' @family Characterization
#' @return
#' Returns a data.frame with the columns:
#' \itemize{
#'  \item{databaseName the name of the database}
#'  \item{databaseId the unique identifier of the database}
#'  \item{targetName the target cohort name}
#'  \item{targetId the target cohort unique identifier}
#'  \item{outcomeName the outcome name}
#'  \item{outcomeId the outcome unique identifier}
#'  \item{minPriorObservation the minimum required observation days prior to index for an entry}
#'  \item{outcomeWashoutDays patients with the outcome occurring within this number of days prior to index are excluded (NA means no exclusion)}
#' \item{riskWindowStart the number of days ofset the start anchor that is the start of the time-at-risk}
#' \item{startAnchor the start anchor is either the target cohort start or cohort end date}
#' \item{riskWindowEnd the number of days ofset the end anchor that is the end of the time-at-risk}
#' \item{endAnchor the end anchor is either the target cohort start or cohort end date}
#' \item{covariateName the name of the feature}
#' \item{sumValue the number of cases who have the feature value of 1}
#' \item{averageValue the mean feature value}
#' } 
#' 
#' @export
#' @examples
#' # example code
#' 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' ageData <- getCharacterizationDemographics(
#' connectionHandler = connectionHandler, 
#' schema = 'main'
#' )
#' 
getCharacterizationDemographics <- function(
    connectionHandler,
    schema,
    cTablePrefix = 'c_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    targetId = NULL,
    outcomeId = NULL,
    type = 'age'
){
  
  if(type == 'age'){
    analysisIds <- 3
  } else if(type == 'sex'){
    analysisIds <- 1
  } else{
    stop('Invalid type - must be age or sex')
  }
  
  cVersion <- .getCVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cTablePrefix = cTablePrefix
  )
  
  if(cVersion == 0){
    
  ageData <- getCaseBinaryFeatures(
    connectionHandler = connectionHandler, 
    schema = schema, 
    cTablePrefix = cTablePrefix,
    cgTablePrefix = cgTablePrefix,
    targetIds = targetId, 
    outcomeIds = outcomeId, 
    analysisIds = analysisIds
  )
  ageDataT <- getCaseTargetBinaryFeatures(
    connectionHandler = connectionHandler, 
    schema = schema, 
    cTablePrefix = cTablePrefix,
    cgTablePrefix = cgTablePrefix,
    targetIds = targetId, 
    outcomeIds = outcomeId, 
    analysisIds = analysisIds
  )
  
  countT <- getCaseTargetCounts(
    connectionHandler = connectionHandler, 
    schema = schema, 
    cTablePrefix = cTablePrefix,
    cgTablePrefix = cgTablePrefix,
    targetIds = targetId, 
    outcomeIds = outcomeId
  )
  
  ageDataT <- ageDataT %>% 
    dplyr::inner_join(
      y = countT %>% dplyr::select(
        "databaseName",
        "databaseId",
        "minPriorObservation",
        "outcomeWashoutDays",
        "personCount"
      ), 
      by = c("databaseName","databaseId", 'minPriorObservation', 'outcomeWashoutDays'),
      relationship = "many-to-many" #fix warning
      )
  
  ageDataT <- merge(
    ageDataT,
    unique(ageData %>% dplyr::select(
    "riskWindowStart",
    "riskWindowEnd",
    "startAnchor",
    "endAnchor"
  ))
  ) %>% dplyr::mutate(
    nonCaseCount = .data$personCount,
    nonCaseAverage = .data$sumValue/.data$personCount
  )
  
  ageData <- ageData %>%
    dplyr::rename(
      caseCount = "sumValue",
      caseAverage = "averageValue"
    )
  
  
  allData <- merge(
    x = ageData, 
    y = ageDataT,
    by = c("databaseName", "databaseId", "targetName", "targetCohortId",   
           "outcomeName", "outcomeCohortId", "minPriorObservation" ,"outcomeWashoutDays",
           "riskWindowStart", "riskWindowEnd", "startAnchor", "endAnchor",          
           "covariateId", "covariateName")
    ) %>%
    dplyr::select(-dplyr::any_of(c("sumValue", "rawSum", "rawAverage", "personCount")))
  
  # adding new column to old results
  allData <- allData %>% 
    dplyr::mutate(
      limitToFirstInNDays = 99999,
      smd = NA,
      absSmd = NA
    ) 

  } else{
    
    allData <- getBinaryRiskFactors(
      connectionHandler = connectionHandler, 
      schema = schema, 
      cTablePrefix = cTablePrefix, 
      cgTablePrefix = cgTablePrefix, 
      databaseTable = databaseTable, 
      targetId = targetId, 
      outcomeId = outcomeId,  
      analysisIds = analysisIds
      )
    
  }
  
return(allData)
}


getCaseTargetBinaryFeatures <- function(
    connectionHandler,
    schema,
    cTablePrefix = 'c_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    targetIds = NULL,
    outcomeIds = NULL,
    databaseIds = NULL,
    analysisIds = c(3) # c(8507, 8532)
){
  
  cVersion <- .getCVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cTablePrefix = cTablePrefix
  )
  
  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/characterization/getCaseTargetBinaryFeaturesV", cVersion, ".sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  

result <- connectionHandler$queryDb(
  sql = sql,
  schema = schema,
  target_id = paste0(targetIds, collapse = ','),
  use_target = !is.null(targetIds),
  outcome_id = paste0(outcomeIds, collapse = ','),
  use_outcome = !is.null(outcomeIds),
  c_table_prefix = cTablePrefix,
  cg_table_prefix = cgTablePrefix,
  database_table = databaseTable,
  use_analysis = !is.null(analysisIds),
  analysis_ids = paste0(analysisIds, collapse = ','),
  database_id = paste0("'", databaseIds, "'", collapse = ","),
  use_database = !is.null(databaseIds)
)

return(result)
}


#' Extract aggregate statistics of binary feature analysis IDs of interest for targets (ignoring excluding people with prior outcome)
#' @description
#' This function extracts the feature extraction results for targets corresponding to specified target
#'  but does not exclude any patients with the outcome during the outcome washout (so it agnostic to the outcome of interest).
#'
#' @details
#' Specify the connectionHandler, the schema and the target cohort IDs
#'
#' @template connectionHandler
#' @template schema
#' @template cTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @template targetId
#' @param databaseIds (optional) A vector of database ids to restrict to
#' @param analysisIds (optional) The feature extraction analysis ID of interest (e.g., 201 is condition)
#' @param conceptIds (optional) The feature extraction concept ID of interest to restrict to
#' @family Characterization
#' @return
#' Returns a data.frame with the columns:
#' \itemize{
#'  \item{databaseName the name of the database}
#'  \item{databaseId the unique identifier of the database}
#'  \item{targetName the target cohort name}
#'  \item{targetId the target cohort unique identifier}
#'  \item{minPriorObservation the minimum required observation days prior to index for an entry}
#'  \item{covariateId the id of the feature}
#'  \item{covariateName the name of the feature}
#'  \item{sumValue the number of target patients who have the feature value of 1 (target patients are restricted to first occurrence and require min prior obervation days)}
#'  \item{averageAvalue the fraction of target patients who have the feature value of 1 (target patients are restricted to first occurrence and require min prior obervation days)}
#' } 
#' 
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' tbf <- getTargetBinaryFeatures (
#' connectionHandler = connectionHandler, 
#' schema = 'main',
#' targetId = 1
#' )
#' 
getTargetBinaryFeatures <- function(
    connectionHandler,
    schema,
    cTablePrefix = 'c_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    targetId = NULL,
    databaseIds = NULL,
    analysisIds = NULL,
    conceptIds = NULL
){
  
  if(length(targetId) != 1){
    stop('targetId must be a single value')
  }
  
  cVersion <- .getCVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cTablePrefix = cTablePrefix
  )
  
  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/characterization/getTargetBinaryFeaturesV", cVersion, ".sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))

result <- connectionHandler$queryDb(
  sql = sql,
  schema = schema,
  target_id = targetId,
  c_table_prefix = cTablePrefix,
  cg_table_prefix = cgTablePrefix,
  database_table = databaseTable,
  use_analysis = !is.null(analysisIds),
  analysis_ids = paste0(analysisIds, collapse = ','),
  database_id = paste0("'", databaseIds, "'", collapse = ","),
  use_database = !is.null(databaseIds),
  use_concepts = !is.null(conceptIds),
  concept_ids = paste0(conceptIds, collapse = ',')
)

return(result)
}

#' A function to extract non-case and case binary characterization results
#'
#' @details
#' Specify the connectionHandler, the schema and the target/outcome cohort IDs
#'
#' @template connectionHandler
#' @template schema
#' @template cTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @template targetId
#' @template outcomeId
#' @param databaseId The database ID to restrict results to
#' @param analysisIds The feature extraction analysis ID of interest (e.g., 201 is condition)
#' @param riskWindowStart (optional) A vector of time-at-risk risk window starts to restrict to
#' @param riskWindowEnd (optional) A vector of time-at-risk risk window ends to restrict to
#' @param startAnchor (optional) A vector of time-at-risk start anchors to restrict to
#' @param endAnchor (optional) A vector of time-at-risk end anchors to restrict to

#' @family Characterization
#' 
#' @return
#' A data.frame with the characterization results for the cases and non-cases
#'
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' rf <- getBinaryRiskFactors(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main',
#'   targetId = 1, 
#'   outcomeId = 3
#' )
#' 
getBinaryRiskFactors <- function(
    connectionHandler,
    schema,
    cTablePrefix = 'c_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    targetId = NULL,
    outcomeId = NULL,
    databaseId = NULL,
    analysisIds = c(3), # TODO enable this to be NULL?
    riskWindowStart = NULL,
    riskWindowEnd = NULL,
    startAnchor = NULL,
    endAnchor = NULL
){
  
  cVersion <- .getCVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cTablePrefix = cTablePrefix
  )

  
  if(cVersion == 0){
    if(is.null(targetId)){
      stop('targetId must be entered')
    }
    if(length(targetId) > 1){
      stop('Must be single targetId')
    }
    if(is.null(outcomeId)){
      stop('outcomeId must be entered')
    }
    if(length(outcomeId) > 1){
      stop('Must be single outcomeId')
    }
  # this is the case counts per target, min prior obs,outcome,TAR, and outcome washout
  caseCounts <- getCaseCounts(
    connectionHandler = connectionHandler,
    schema = schema,
    cTablePrefix = cTablePrefix,
    cgTablePrefix = cgTablePrefix,
    databaseTable = databaseTable,
    targetIds = targetId,
    outcomeIds = outcomeId,
    databaseIds = databaseId,
    riskWindowStart = riskWindowStart,
    riskWindowEnd = riskWindowEnd,
    startAnchor = startAnchor,
    endAnchor = endAnchor
  )
  
  # target counts are agnostic to TAR
  # this gets the target and subtracts the excluded per min prior obs and outcome washout
  targetCounts <- getCaseTargetCounts(
    connectionHandler = connectionHandler,
    schema = schema,
    cTablePrefix = cTablePrefix,
    cgTablePrefix = cgTablePrefix,
    databaseTable = databaseTable,
    targetIds = targetId,
    outcomeIds = outcomeId,
    databaseIds = databaseId
  )
    
  # gets the case features per target, min prior obs,outcome,TAR, and outcome washout
  caseFeatures <- getCaseBinaryFeatures(
    connectionHandler = connectionHandler,
    schema = schema,
    cTablePrefix = cTablePrefix,
    cgTablePrefix = cgTablePrefix,
    databaseTable = databaseTable,
    targetIds = targetId,
    outcomeIds = outcomeId,
    databaseIds = databaseId,
    analysisIds = analysisIds,
    riskWindowStart = riskWindowStart,
    riskWindowEnd = riskWindowEnd,
    startAnchor = startAnchor,
    endAnchor = endAnchor
  )
  
  targetFeatures <- getCaseTargetBinaryFeatures(
    connectionHandler = connectionHandler,
    schema = schema,
    cTablePrefix = cTablePrefix,
    cgTablePrefix = cgTablePrefix,
    databaseTable = databaseTable,
    targetIds = targetId,
    outcomeIds = outcomeId,
    databaseIds = databaseId,
    analysisIds = analysisIds
  )
  
  result <- processBinaryRiskFactorFeatures(
    caseCounts = caseCounts,
    targetCounts = targetCounts,
    caseFeatures = caseFeatures,
    targetFeatures = targetFeatures
  )
  } else{
    
    sql <- SqlRender::readSql(system.file(
      paste0("sql/sql_server/characterization/getBinaryRiskFactorsV", cVersion, ".sql"),
      package = "OhdsiReportGenerator",
      mustWork = TRUE
    ))
    
    # restrict by restrictToFirstInNDays, minPriorObseration, outcomeWashoutDays and TAR?
    result <- connectionHandler$queryDb(
      sql = sql,
      schema = schema,
      target_id = paste0(targetId, collapse = ','),
      use_target = !is.null(targetId),
      outcome_id = paste0(outcomeId, collapse = ','),
      use_outcome = !is.null(outcomeId),
      c_table_prefix = cTablePrefix,
      cg_table_prefix = cgTablePrefix,
      database_table = databaseTable,
      use_analysis = !is.null(analysisIds),
      analysis_ids = paste0(analysisIds, collapse = ','),
      use_database = !is.null(databaseId),
      database_id = paste0("'",databaseId,"'", collapse = ",")
    )
    
    
  }

return(result)
}


# function that takes the counts and features and calculates the smd
processBinaryRiskFactorFeatures <- function(
    caseCounts = caseCounts,
    targetCounts = targetCounts,
    caseFeatures = caseFeatures,
    targetFeatures = targetFeatures
){
  
  if(nrow(targetCounts) == 0 | nrow(caseCounts) == 0){
    warning('No targets or outcomes')
    return(NULL)
  }
  
  allData <- c()
  
  # TODO:what if no cases?
  params <- unique(
    caseCounts %>% dplyr::select(
      "databaseName",
      "databaseId",
      "minPriorObservation",
      "outcomeWashoutDays",
      "riskWindowStart",
      "riskWindowEnd",
      "startAnchor",
      "endAnchor"
    )
  )
  
  # for each database/TAR/minPriorObservation/outcomeWashoutDays
  for(i in 1:nrow(params)){
    databaseName <- params$databaseName[i]
    databaseId <- params$databaseId[i]
    minPriorObservation <- params$minPriorObservation[i]
    outcomeWashoutDays <- params$outcomeWashoutDays[i]
    riskWindowStart <- params$riskWindowStart[i]
    riskWindowEnd <- params$riskWindowEnd[i]
    startAnchor <- params$startAnchor[i]
    endAnchor <- params$endAnchor[i]
  ## extract the case count and target count to get non-case count
  caseCount <- caseCounts %>% 
    dplyr::filter(
      .data$databaseName == !!databaseName &
      .data$databaseId == !!databaseId &
      .data$minPriorObservation == !!minPriorObservation &
        .data$outcomeWashoutDays == !!outcomeWashoutDays &
        .data$riskWindowStart == !!riskWindowStart &
        .data$riskWindowEnd == !!riskWindowEnd &
        .data$startAnchor == !!startAnchor &
        .data$endAnchor == !!endAnchor
    )
  
  targetCount <- targetCounts %>% 
    dplyr::filter(
      .data$databaseName == !!databaseName &
      .data$databaseId == !!databaseId &
      .data$minPriorObservation == !!minPriorObservation &
        .data$outcomeWashoutDays == !!outcomeWashoutDays
    )
  
  if(nrow(targetCount) > 0 & nrow(caseCount) > 0){
  
  if(caseCount$personCount > 0 & targetCount$personCount > 0){
    # only run this if the cases is >= min cell count
    nonCaseCount <- targetCount$personCount - caseCount$personCount

  ## now extract the features
  tempCases <- caseFeatures %>% 
    dplyr::filter(.data$databaseName == !!databaseName &
                    .data$databaseId == !!databaseId &
                    .data$minPriorObservation == !!minPriorObservation &
                    .data$outcomeWashoutDays == !!outcomeWashoutDays &
                    .data$riskWindowStart == !!riskWindowStart &
                    .data$riskWindowEnd == !!riskWindowEnd &
                    .data$startAnchor == !!startAnchor &
                    .data$endAnchor == !!endAnchor
                    ) %>%
    dplyr::rename(
      caseCount = "sumValue", 
      caseAverage = "averageValue"
      ) %>%
    dplyr::filter( # remove the censored counts
      .data$caseCount >= 0
    )
  
  
  tempTarget <- targetFeatures %>% 
    dplyr::filter(.data$databaseName == !!databaseName &
                    .data$databaseId == !!databaseId &
                    .data$minPriorObservation == !!minPriorObservation &
                    .data$outcomeWashoutDays == !!outcomeWashoutDays 
    ) %>%
    dplyr::filter( # remove the censored counts
      .data$sumValue >= 0
    )
  
  tempData <- tempTarget %>% 
    dplyr::left_join(
      y = tempCases, 
      by = c(
        "databaseName",
        "databaseId",
        "targetName",
        "targetCohortId",
        "outcomeName",
        "outcomeCohortId",
        "minPriorObservation",
        "outcomeWashoutDays",
        "covariateName",
        "covariateId")
      ) %>%
    tidyr::replace_na(list(
      riskWindowStart = riskWindowStart, 
      riskWindowEnd = riskWindowEnd,
      startAnchor = startAnchor,
      endAnchor = endAnchor,
      caseCount = 0,
      caseAverage = 0
      )) %>%
    dplyr::mutate(
      nonCaseCount = .data$sumValue - .data$caseCount,
      nonCaseAverage = (.data$sumValue - .data$caseCount)/!!nonCaseCount
    ) %>%
    dplyr::mutate(
      meanDiff = .data$caseAverage - .data$nonCaseAverage,
      std1 =  ifelse(!!caseCount$personCount == 0, 0 ,sqrt(((1-.data$caseAverage)^2*.data$caseCount + (-.data$caseAverage)^2*(!!caseCount$personCount - .data$caseCount))/!!caseCount$personCount)),
      std2 =  ifelse(!!nonCaseCount == 0, 0, sqrt(((1-.data$nonCaseAverage)^2*.data$nonCaseCount + (-.data$nonCaseAverage)^2*(!!nonCaseCount - .data$nonCaseCount))/!!nonCaseCount))
    ) %>% 
    dplyr::mutate(
      smd = .data$meanDiff/sqrt((.data$std1^2 + .data$std2^2)/2),
      absSmd = abs(.data$meanDiff/sqrt((.data$std1^2 + .data$std2^2)/2))
    ) 
  
  tempData <- tempData %>% 
    dplyr::mutate(
      limitToFirstInNdays = 99999
    ) %>%
    dplyr::select(
    "databaseName",
    "databaseId",
    "targetName",
    "targetCohortId",
    "outcomeName",
    "outcomeCohortId",
    "minPriorObservation",
    "limitToFirstInNdays", # added for v3 compat
    "outcomeWashoutDays",
    "riskWindowStart",
    "riskWindowEnd",
    "startAnchor",
    "endAnchor",
    "covariateName",
    "covariateId",
    "caseCount",
    "caseAverage",
    "nonCaseCount",
    "nonCaseAverage",
    "smd",
    "absSmd"
  )
  
  allData <- rbind(allData, tempData)
  }} # check for cases >= minCellCount
}
  
return(allData) 
}




#' Extract aggregate statistics of continuous feature analysis IDs of interest for targets
#' @description
#' This function extracts the continuous feature extraction results for targets corresponding to specified target cohorts.
#'
#' @details
#' Specify the connectionHandler, the schema and the target/outcome cohort IDs
#'
#' @template connectionHandler
#' @template schema
#' @template cTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @template targetIds
#' @param analysisIds The feature extraction analysis ID of interest (e.g., 201 is condition)
#' @param databaseIds (Optional) A vector of database IDs to restrict to
#'
#' @family Characterization
#' @return
#' Returns a data.frame with the columns:
#' \itemize{
#'  \item{databaseName the name of the database}
#'  \item{databaseId the unique identifier of the database}
#'  \item{targetName the target cohort name}
#'  \item{targetId the target cohort unique identifier}
#'  \item{minPriorObservation the minimum required observation days prior to index for an entry}
#'  \item{covariateName the name of the feature}
#'  \item{covariateId the id of the feature}
#'  \item{countValue the number of cases who have the feature}
#'  \item{minValue the minimum value observed for the feature}
#'  \item{maxValue the maximum value observed for the feature}
#'  \item{averageValue the mean value observed for the feature}
#'  \item{standardDeviation the standard deviation of the value observed for the feature}
#'  \item{medianValue the median value observed for the feature}
#'  \item{p10Value the 10th percentile of the value observed for the feature}
#'  \item{p25Value the 25th percentile of the value observed for the feature}
#'  \item{p75Value the 75th percentile of the value observed for the feature}
#'  \item{p90Value the 90th percentile of the value observed for the feature}
#'  
#' } 
#' 
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' tcf <- getTargetContinuousFeatures(
#' connectionHandler = connectionHandler, 
#' schema = 'main'
#' )
#' 
getTargetContinuousFeatures <- function(
    connectionHandler,
    schema,
    cTablePrefix = 'c_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    targetIds = NULL,
    analysisIds = NULL,
    databaseIds = NULL
){
  
  cVersion <- .getCVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cTablePrefix = cTablePrefix
  )
  
  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/characterization/getTargetContinuousFeaturesV", cVersion, ".sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
result <- connectionHandler$queryDb(
  sql = sql,
  schema = schema,
  target_id = paste0(targetIds, collapse = ','),
  use_target = !is.null(targetIds),
  c_table_prefix = cTablePrefix,
  cg_table_prefix = cgTablePrefix,
  database_table = databaseTable,
  use_analysis = !is.null(analysisIds),
  analysis_ids = paste0(analysisIds, collapse = ','),
  use_database = !is.null(databaseIds),
  database_id = paste0("'",databaseIds,"'", collapse = ",")
)

return(result)
}


getCaseContinuousFeatures <- function(
    connectionHandler,
    schema,
    cTablePrefix = 'c_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    targetIds = NULL,
    outcomeIds = NULL,
    analysisIds = NULL,
    databaseIds = NULL,
    riskWindowStart = NULL,
    riskWindowEnd = NULL,
    startAnchor = NULL,
    endAnchor = NULL
){
  
  
  cVersion <- .getCVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cTablePrefix = cTablePrefix
  )
  
  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/characterization/getCaseContinuousFeaturesV", cVersion, ".sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
result <- connectionHandler$queryDb(
  sql = sql,
  schema = schema,
  target_id = paste0(targetIds, collapse = ','),
  use_target = !is.null(targetIds),
  outcome_id = paste0(outcomeIds, collapse = ','),
  use_outcome = !is.null(outcomeIds),
  c_table_prefix = cTablePrefix,
  cg_table_prefix = cgTablePrefix,
  database_table = databaseTable,
  use_analysis = !is.null(analysisIds),
  analysis_ids = paste0(analysisIds, collapse = ','),
  use_database = !is.null(databaseIds),
  database_id = paste0("'",databaseIds,"'", collapse = ","),
  
  use_risk_window_start = !is.null(riskWindowStart),
  risk_window_start = paste0(riskWindowStart, collapse = ','),
  use_risk_window_end = !is.null(riskWindowEnd),
  risk_window_end = paste0(riskWindowEnd, collapse = ','),
  use_start_anchor = !is.null(startAnchor),
  start_anchor = paste0("'",startAnchor,"'", collapse = ","),
  use_end_anchor = !is.null(endAnchor),
  end_anchor = paste0("'",endAnchor,"'", collapse = ",")
)

return(result)
}




#' A function to extract non-case and case continuous characterization results
#'
#' @details
#' Specify the connectionHandler, the schema and the target/outcome cohort IDs
#'
#' @template connectionHandler
#' @template schema
#' @template cTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @template targetId
#' @template outcomeId
#' @param analysisIds The feature extraction analysis ID of interest (e.g., 201 is condition)
#' @param databaseIds (optional) A vector of database IDs to restrict to
#' @param riskWindowStart (optional) A vector of time-at-risk risk window starts to restrict to
#' @param riskWindowEnd (optional) A vector of time-at-risk risk window ends to restrict to
#' @param startAnchor (optional) A vector of time-at-risk start anchors to restrict to
#' @param endAnchor (optional) A vector of time-at-risk end anchors to restrict to
#'
#' @family Characterization
#' 
#' @return
#' A data.frame with the characterization results for the cases and non-cases
#'
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' rf <- getContinuousRiskFactors(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main',
#'   targetId = 1, 
#'   outcomeId = 3
#' )
#' 
getContinuousRiskFactors <- function(
    connectionHandler,
    schema,
    cTablePrefix = 'c_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    targetId = NULL,
    outcomeId = NULL,
    analysisIds = NULL,
    databaseIds = NULL,
    riskWindowStart = NULL,
    riskWindowEnd = NULL,
    startAnchor = NULL,
    endAnchor = NULL
){
  if(is.null(targetId)){
    stop('targetId must be entered')
  }
  if(is.null(outcomeId)){
    stop('targetId must be entered')
  }
  if(length(targetId) > 1){
    stop('Must be single targetId')
  }
  if(length(outcomeId) > 1){
    stop('Must be single outcomeId')
  }
  
  cVersion <- .getCVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cTablePrefix = cTablePrefix
  )
  
  if(cVersion == 0){
    caseFeatures <- getCaseContinuousFeatures(
      connectionHandler = connectionHandler,
      schema = schema,
      cTablePrefix = cTablePrefix,
      cgTablePrefix = cgTablePrefix,
      databaseTable = databaseTable,
      targetIds = targetId,
      outcomeIds = outcomeId,
      analysisIds = analysisIds,
      databaseIds = databaseIds,
      riskWindowStart = riskWindowStart,
      riskWindowEnd = riskWindowEnd,
      startAnchor = startAnchor,
      endAnchor = endAnchor
    )
    
    targetFeatures <- getTargetContinuousFeatures(
      connectionHandler = connectionHandler,
      schema = schema,
      cTablePrefix = cTablePrefix,
      cgTablePrefix = cgTablePrefix,
      databaseTable = databaseTable,
      targetIds = targetId,
      analysisIds = analysisIds,
      databaseIds = databaseIds
    )
    
    result <- processContinuousRiskFactorFeatures(
      caseFeatures = caseFeatures,
      targetFeatures = targetFeatures
    )
  } else{
    
    sql <- SqlRender::readSql(system.file(
      paste0("sql/sql_server/characterization/getContinuousRiskFactorsV", cVersion, ".sql"),
      package = "OhdsiReportGenerator",
      mustWork = TRUE
    ))
    
    result <- connectionHandler$queryDb(
      sql = sql,
      schema = schema,
      target_id = paste0(targetId, collapse = ','),
      use_target = !is.null(targetId),
      outcome_id = paste0(outcomeId, collapse = ','),
      use_outcome = !is.null(outcomeId),
      c_table_prefix = cTablePrefix,
      cg_table_prefix = cgTablePrefix,
      database_table = databaseTable,
      use_analysis = !is.null(analysisIds),
      analysis_ids = paste0(analysisIds, collapse = ','),
      use_database = !is.null(databaseIds),
      database_id = paste0("'",databaseIds,"'", collapse = ","),
      
      use_risk_window_start = !is.null(riskWindowStart),
      risk_window_start = paste0(riskWindowStart, collapse = ','),
      use_risk_window_end = !is.null(riskWindowEnd),
      risk_window_end = paste0(riskWindowEnd, collapse = ','),
      use_start_anchor = !is.null(startAnchor),
      start_anchor = paste0("'",startAnchor,"'", collapse = ","),
      use_end_anchor = !is.null(endAnchor),
      end_anchor = paste0("'",endAnchor,"'", collapse = ",")
    )
    
  }
  
  return(result)
}


# function that takes the counts and features and calculates the smd
processContinuousRiskFactorFeatures <- function(
    caseFeatures = caseFeatures,
    targetFeatures = targetFeatures
){
  
  # get outcomes and outcomewashout
  outcomes <- unique(caseFeatures[,c('outcomeName','outcomeCohortId','outcomeWashoutDays',
                                     'riskWindowStart', 'riskWindowEnd', 'startAnchor', 'endAnchor')])
  
  targetFeatures <- targetFeatures %>% dplyr::rename(
    "targetCountValue" = "countValue",
    "targetMinValue" = "minValue",
    "targetMaxValue" = "maxValue",
    "targetAverageValue" = "averageValue", 
    "targetStandardDeviation" = "standardDeviation",
    "targetMedianValue" = "medianValue",
    "targetP10Value" = "p10Value",
    "targetP25Value" = "p25Value",
    "targetP75Value" = "p75Value",
    "targetP90Value" = "p90Value"
  )
  
  caseFeatures <- caseFeatures %>% dplyr::rename(
    "caseCountValue" = "countValue",
    "caseMinValue" = "minValue",
    "caseMaxValue" = "maxValue",
    "caseAverageValue" = "averageValue", 
    "caseStandardDeviation" = "standardDeviation",
    "caseMedianValue" = "medianValue",
    "caseP10Value" = "p10Value",
    "caseP25Value" = "p25Value",
    "caseP75Value" = "p75Value",
    "caseP90Value" = "p90Value"
  )
  
  if(nrow(outcomes) > 0){
    allData <- c()
    for(i in 1:nrow(outcomes)){
      
      res <- merge(
        x = targetFeatures %>% 
          dplyr::filter(
            .data$targetCohortId != outcomes[i,]$outcomeCohortId 
          ), 
        y = caseFeatures %>% 
          dplyr::filter(
            .data$outcomeCohortId == outcomes[i,]$outcomeCohortId &
              .data$outcomeWashoutDays == outcomes[i,]$outcomeWashoutDays &
              .data$riskWindowStart == outcomes[i,]$riskWindowStart &
              .data$riskWindowEnd == outcomes[i,]$riskWindowEnd &
              .data$startAnchor == outcomes[i,]$startAnchor &
              .data$endAnchor == outcomes[i,]$endAnchor
          ),
        
        by = c('databaseName','databaseId','targetName','targetCohortId','minPriorObservation',
               'covariateName', 'covariateId'),
        all.x = TRUE
      ) %>%
        tidyr::replace_na(
          list(
            outcomeCohortId = outcomes[i,]$outcomeCohortId, 
            outcomeName = outcomes[i,]$outcomeName,
            outcomeWashoutDays = outcomes[i,]$outcomeWashoutDays,
            riskWindowStart = outcomes[i,]$riskWindowStart,
            riskWindowEnd = outcomes[i,]$riskWindowEnd,
            startAnchor = outcomes[i,]$startAnchor,
            endAnchor = outcomes[i,]$endAnchor
          )
        ) %>% 
        dplyr::mutate_all(~replace(., is.na(.), 0))
      
      allData <- rbind(allData, res)
    }
  
  allData <- allData %>% dplyr::mutate(
        limitToFirstInNDays = 99999,
        smd = (.data$caseAverageValue - .data$targetAverageValue)/sqrt((.data$caseStandardDeviation^2 + .data$targetStandardDeviation^2)/2),
        absSmd = abs((.data$caseAverageValue - .data$targetAverageValue)/sqrt((.data$caseStandardDeviation^2 + .data$targetStandardDeviation^2)/2))
      ) 
  
  return(allData) 
  
  } else{
    return(NULL)
  }

}


# case series data.frame
#' A function to extract case series characterization results
#'
#' @details
#' Specify the connectionHandler, the schema and the target/outcome cohort IDs
#'
#' @template connectionHandler
#' @template schema
#' @template cTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @template targetId
#' @template outcomeId
#' @param databaseIds (optional) One or more unique identifiers for the databases
#' @param riskWindowStart (optional) A riskWindowStart to restrict to
#' @param riskWindowEnd (optional) A riskWindowEnd to restrict to
#' @param startAnchor (optional) A startAnchor to restrict to
#' @param endAnchor (optional) An endAnchor to restrict to
#' @param conceptIds (optional) An conceptIds to restrict to
#' @param minVal (optional) the minimum averageVal to extract
#' @family Characterization
#' 
#' @return
#' A data.frame with the characterization case series results
#'
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' cs <- getBinaryCaseSeries(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main',
#'   targetId = 1, 
#'   outcomeId = 3
#' )
#' 
getBinaryCaseSeries <- function(
    connectionHandler,
    schema,
    cTablePrefix = 'c_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    targetId = NULL,
    outcomeId = NULL,
    databaseIds = NULL,
    riskWindowStart = NULL,
    riskWindowEnd = NULL,
    startAnchor = NULL,
    endAnchor = NULL,
    conceptIds = NULL,
    minVal = NULL
){
  if(is.null(targetId)){
    stop('targetId must be entered')
  }
  if(is.null(outcomeId)){
    stop('outcomeId must be entered')
  }
  if(length(targetId) > 1){
    stop('Must be single targetId')
  }
  if(length(outcomeId) > 1){
    stop('Must be single outcomeId')
  }
  
  cVersion <- .getCVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cTablePrefix = cTablePrefix
  )
  
  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/characterization/getBinaryCaseSeriesV", cVersion, ".sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))

result <- connectionHandler$queryDb(
  sql = sql,
  schema = schema,
  target_id = paste0(targetId, collapse = ','),
  outcome_id = paste0(outcomeId, collapse = ','),
  c_table_prefix = cTablePrefix,
  cg_table_prefix = cgTablePrefix,
  database_table = databaseTable,
  use_database = !is.null(databaseIds),
  database_ids = paste0("'",databaseIds,"'", collapse = ','),
  use_risk_window_start = !is.null(riskWindowStart),
  risk_window_start = riskWindowStart,
  use_risk_window_end = !is.null(riskWindowEnd),
  risk_window_end = riskWindowEnd,
  use_start_anchor = !is.null(startAnchor),
  start_anchor = startAnchor,
  use_end_anchor = !is.null(endAnchor),
  end_anchor = endAnchor,
  use_min_val = !is.null(minVal),
  min_val = minVal,
  use_concepts = !is.null(conceptIds),
  concept_ids = paste0(conceptIds, collapse = ',')
)
  
  return(result)
}


#' A function to extract case series continuous feature characterization results
#'
#' @details
#' Specify the connectionHandler, the schema and the target/outcome cohort IDs
#'
#' @template connectionHandler
#' @template schema
#' @template cTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @template targetId
#' @template outcomeId
#' @param databaseIds (optional) One or more unique identifiers for the databases
#' @param riskWindowStart (optional) A riskWindowStart to restrict to
#' @param riskWindowEnd (optional) A riskWindowEnd to restrict to
#' @param startAnchor (optional) A startAnchor to restrict to
#' @param endAnchor (optional) An endAnchor to restrict to
#' @family Characterization
#' 
#' @return
#' A data.frame with the characterization case series results
#'
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' cs <- getContinuousCaseSeries(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main',
#'   targetId = 1, 
#'   outcomeId = 3
#' )
#' 
getContinuousCaseSeries <- function(
    connectionHandler,
    schema,
    cTablePrefix = 'c_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    targetId = NULL,
    outcomeId = NULL,
    databaseIds = NULL,
    riskWindowStart = NULL,
    riskWindowEnd = NULL,
    startAnchor = NULL,
    endAnchor = NULL
){
  if(is.null(targetId)){
    stop('targetId must be entered')
  }
  if(is.null(outcomeId)){
    stop('targetId must be entered')
  }
  if(length(targetId) > 1){
    stop('Must be single targetId')
  }
  if(length(outcomeId) > 1){
    stop('Must be single outcomeId')
  }
  
  cVersion <- .getCVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cTablePrefix = cTablePrefix
  )
  
  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/characterization/getContinuousCaseSeriesV", cVersion, ".sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    target_id = paste0(targetId, collapse = ','),
    outcome_id = paste0(outcomeId, collapse = ','),
    c_table_prefix = cTablePrefix,
    cg_table_prefix = cgTablePrefix,
    database_table = databaseTable,
    use_database = !is.null(databaseIds),
    database_ids = paste0("'",databaseIds,"'", collapse = ','),
    use_risk_window_start = !is.null(riskWindowStart),
    risk_window_start = riskWindowStart,
    use_risk_window_end = !is.null(riskWindowEnd),
    risk_window_end = riskWindowEnd,
    use_start_anchor = !is.null(startAnchor),
    start_anchor = startAnchor,
    use_end_anchor = !is.null(endAnchor),
    end_anchor = endAnchor
  )
  
  return(result)
}

#' A function to extract cohort aggregate binary feature characterization results
#'
#' @details
#' Specify the connectionHandler, the schema and the target cohort ID and database id
#'
#' @template connectionHandler
#' @template schema
#' @template cTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @template targetIds
#' @param databaseIds (optional) One or more unique identifiers for the databases
#' @param minThreshold The minimum fraction of the cohort that must have the feature for it to be reported
#' 
#' @family Characterization
#' 
#' @return
#' A data.frame with the characterization aggregate binary features for a specific cohort and database
#'
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' binCohort <- getCharacterizationCohortBinary(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main',
#'   targetIds = 1, 
#'   databaseIds = 'eunomia'
#' )
#' 
getCharacterizationCohortBinary <- function(
    connectionHandler,
    schema,
    cTablePrefix = 'c_',
    cgTablePrefix = 'cg_', # not used 
    databaseTable = 'database_meta_data',
    targetIds = NULL,
    databaseIds = NULL,
    minThreshold = 0
){
  
  cVersion <- .getCVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cTablePrefix = cTablePrefix
  )
  
  # first get counts
  counts <- getCharacterizationCohortCounts(
    connectionHandler = connectionHandler,
    schema = schema,
    cTablePrefix = cTablePrefix,
    cgTablePrefix = cgTablePrefix,
    databaseTable = databaseTable,
    targetIds = targetIds,
    databaseIds = databaseIds
  )
  
  # add order for targets
  if(!is.null(targetIds)){
    counts <- counts %>%
      dplyr::inner_join(
        data.frame(
          cohortId = c(unique(targetIds)),
          order = 1:length(unique(targetIds))
        ), 
        by = 'cohortId') %>%
      dplyr::arrange(dplyr::desc(-1*.data$order)) %>%
      dplyr::select(-"order")
  }
  
  colRef <- counts %>%
    dplyr::mutate(id = dplyr::row_number())

  if(nrow(colRef) == 0){
    return(NULL)
  }
  
  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/characterization/getCharacterizationCohortBinaryV", cVersion, ".sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
  start <- Sys.time()
  # settings.min_characterization_mean needed?
  res <- connectionHandler$queryDb(
    sql = sql,
    use_targets = !is.null(targetIds),
    target_ids = paste0(targetIds, collapse = ','),
    use_databases = !is.null(databaseIds),
    database_ids = paste0("'",databaseIds,"'", collapse = ','),
    schema = schema,
    c_table_prefix = cTablePrefix,
    min_threshold = minThreshold
  )
  end <- Sys.time() - start 
  
  message(paste0('Extracting ', nrow(res) ,' characterization cohort rows took: ', round(end, digits = 2), ' ', units(end)))
  
  # pivot 
  res <- merge(res, colRef, by = c('cohortId','databaseId', 'minPriorObservation', 'limitToFirstInNDays'))
  
  replaceVals <- rep(list(-1*minThreshold), nrow(colRef))
  names(replaceVals) <- paste0('averageValue_', 1:nrow(colRef))
  
  # figure out the min sumVal per target/database/minobs
  replaceSumVals <- c()
  for(i in 1:nrow(colRef)){
    replaceSumVals <- c(replaceSumVals, -1*floor(minThreshold*colRef$n[i]))
  }
  replaceSumVals <- as.list(replaceSumVals)
  names(replaceSumVals) <- paste0('sumValue_', 1:nrow(colRef))
  
  res <- tidyr::pivot_wider(
    data = res, 
    id_cols = c('covariateName', 'covariateId'), 
    names_from = 'id', 
    values_from = c('sumValue', 'averageValue', 'n'), 
    values_fn = mean, 
    values_fill = NA
  ) %>%
    tidyr::replace_na(replaceVals) %>%
    tidyr::replace_na(replaceSumVals)
  
  
  # add SMD if two unique types
  # adding below check in cases when there are patients in the cohort
  # but no covariates
  twoResultsWithValues <- length(grep('sumValue', colnames(res))) == 2
    
  if(nrow(colRef) == 2 & twoResultsWithValues){
    res <- res %>% 
      dplyr::mutate(
        standardDeviation_1 = ((abs(.data$averageValue_1)-1)^2*abs(.data$sumValue_1) + (abs(.data$averageValue_1)-0)^2*(.data$n_1-abs(.data$sumValue_1)))/.data$n_1,
        standardDeviation_2 = ((abs(.data$averageValue_2)-1)^2*abs(.data$sumValue_2) + (abs(.data$averageValue_2)-0)^2*(.data$n_2-abs(.data$sumValue_2)))/.data$n_2
      ) %>%
      dplyr::mutate(
        smd = (.data$averageValue_1-.data$averageValue_2)/(sqrt((.data$standardDeviation_1^2 + .data$standardDeviation_2^2)/2))
    ) %>%
      dplyr::mutate(
        absSmd = abs(.data$smd)
      ) %>%
      dplyr::select(-"standardDeviation_1", -"standardDeviation_2")
  }
  
  # remove the ns
  res <- res %>% 
    dplyr::select(
      - colnames(res)[grep('n_', colnames(res))]
    )
  
  return(list(
    covariates = res,
    covRef = colRef
    ))
}

# function to extract continuous cohort aggregate FE
#' A function to extract cohort aggregate continuous feature characterization results
#'
#' @details
#' Specify the connectionHandler, the schema and the target cohort ID and database id
#'
#' @template connectionHandler
#' @template schema
#' @template cTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @template targetIds
#' @param databaseIds (optional) One or more unique identifiers for the databases
#' @param minThreshold The minimum fraction of the cohort that must have the feature for it to be reported
#' 
#' @family Characterization
#' 
#' @return
#' A data.frame with the characterization aggregate continuous features for a specific cohort and database
#'
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' conCohort <- getCharacterizationCohortContinuous(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main',
#'   targetIds = 1, 
#'   databaseIds = 'eunomia'
#' )
#' 
getCharacterizationCohortContinuous <- function(
    connectionHandler,
    schema,
    cTablePrefix = 'c_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    targetIds = NULL,
    databaseIds = NULL,
    minThreshold = 0
){
  
  cVersion <- .getCVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cTablePrefix = cTablePrefix
  )
  
  # first get counts
  counts <- getCharacterizationCohortCounts(
    connectionHandler = connectionHandler,
    schema = schema,
    cTablePrefix = cTablePrefix,
    cgTablePrefix = cgTablePrefix,
    databaseTable = databaseTable,
    targetIds = targetIds,
    databaseIds = databaseIds
  )
  
  # add order for targets
  if(!is.null(targetIds)){
    counts <- counts %>%
      dplyr::inner_join(
        data.frame(
          cohortId = c(unique(targetIds)),
          order = 1:length(unique(targetIds))
        ), 
        by = 'cohortId') %>%
      dplyr::arrange(dplyr::desc(-1*.data$order)) %>%
      dplyr::select(-"order")
  }
  
  colRef <- counts %>%
    dplyr::mutate(id = dplyr::row_number())
  
  if(nrow(colRef) == 0){
    return(NULL)
  }
  
  # calculate minCounts from minThreshold and N
  colRef$minCount <- colRef$n*minThreshold
  
  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/characterization/getCharCohortContinuousV", cVersion, ".sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
  # now fetch data
start <- Sys.time()

res <- connectionHandler$queryDb(
  sql = sql,
  use_targets = !is.null(targetIds),
  target_ids = paste0(targetIds, collapse = ','),
  use_databases = !is.null(databaseIds),
  database_ids = paste0("'",databaseIds,"'", collapse = ','),
  schema = schema,
  c_table_prefix = cTablePrefix,
  database_meta_table = databaseTable
)
end <- Sys.time() - start 
message(paste0('Extracting ', nrow(res) ,' continuous characterization cohort rows took: ', round(end, digits = 2), ' ', units(end)))


# pivot 
res <- res %>%
  dplyr::inner_join(
    y = colRef,
    by = c('cohortId','databaseId', 'minPriorObservation', 'limitToFirstInNDays')
      )

# remove rows where countVal < minCount
 res <- res %>%
   dplyr::filter(
     .data$countValue >= .data$minCount
   ) %>%
   dplyr::select(
     -"minCount"
   )


  res <- tidyr::pivot_wider(
    data = res, 
    id_cols = c('covariateName', 'covariateId','minPriorObservation', 'limitToFirstInNDays'), 
    names_from = 'id', 
    values_from = c('countValue', 'averageValue', 'standardDeviation', 'medianValue','minValue', 'maxValue', 'p10Value','p25Value','p75Value','p90Value'), 
    values_fn = mean, 
    values_fill = 0
  ) 

  

# add SMD if two unique types
  # adding below to fix bug where there are a small number of people in the cohort but no covs
  twoResultsWithValues <- length(grep('countValue', colnames(res))) == 2
  
if(nrow(colRef) == 2 & twoResultsWithValues){
  res <- res %>% dplyr::mutate(
    smd = (abs(.data$averageValue_1)-abs(.data$averageValue_2))/(sqrt((abs(.data$standardDeviation_1)^2 + abs(.data$standardDeviation_2)^2)/2))
  ) %>%
    dplyr::mutate(
      absSmd = abs(.data$smd)
    )
}
  

  return(list(
    covariates = res,
    covRef = colRef
  ))
}



getCharacterizationCohortCounts <- function(
    connectionHandler,
    schema,
    cTablePrefix = 'c_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    targetIds = NULL,
    databaseIds = NULL
    ){
  
  cVersion <- .getCVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cTablePrefix = cTablePrefix
  )
  
  # getting counts
  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/characterization/getCharacterizationCohortCountsV", cVersion, ".sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
  counts <- connectionHandler$queryDb(
    sql = sql,
    use_targets = !is.null(targetIds),
    target_ids = paste0(targetIds, collapse = ','),
    use_databases = !is.null(databaseIds),
    database_ids = paste0("'",databaseIds,"'", collapse = ','),
    schema = schema,
    c_table_prefix = cTablePrefix,
    cg_table_prefix = cgTablePrefix,
    database_meta_table = databaseTable
  )
  
  return(counts)
  
}


#' A function to extract the failed dechallenge-rechallenge cases
#'
#' @details
#' Specify the connectionHandler, the schema and the target/outcome cohort IDs and database id
#'
#' @template connectionHandler
#' @template schema
#' @template cTablePrefix
#' @template targetId
#' @template outcomeId
#' @param databaseId The unique identifier for the database of interest
#' @param dechallengeStopInterval (optional) The maximum number of days between the outcome start and target end for an outcome to be flagged 
#' @param dechallengeEvaluationWindow (optional) The maximum number of days after the target restarts to see whether the outcome restarts
#' 
#' @family Characterization
#' 
#' @return
#' A data.frame each failed dechallenge rechallenge exposures and outcomes
#'
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' conCohort <- getDechallengeRechallengeFails(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main',
#'   targetId = 1, 
#'   outcomeId = 3,
#'   databaseId = 'eunomia'
#' )
#' 
getDechallengeRechallengeFails <- function(
    connectionHandler,
    schema,
    cTablePrefix = 'c_',
    targetId = NULL,
    outcomeId = NULL,
    databaseId = NULL,
    dechallengeStopInterval = NULL,
    dechallengeEvaluationWindow = NULL
  ){
  
  if(length(targetId) != 1){
    stop('Must specify exactly one targetId')
  }
  if(length(outcomeId) != 1){
    stop('Must specify exactly one outcomeId')
  }
  if(length(databaseId) != 1){
    stop('Must specify exactly one databaseId')
  }
  
  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/characterization/getDechallengeRechallengeFails.sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
  result <- connectionHandler$queryDb(
    sql = sql, 
    schema = schema,
    c_table_prefix = cTablePrefix,
    target_id = targetId,
    outcome_id = outcomeId,
    database_id = databaseId,
    use_dechallenge_stop_interval = !is.null(dechallengeStopInterval),
    dechallenge_stop_interval = dechallengeStopInterval,
    use_dechallenge_evaluation_window = !is.null(dechallengeEvaluationWindow),
    dechallenge_evaluation_window = dechallengeEvaluationWindow
  )
  
  return(result)
}

test_that("incidence rates", {
  
  incidence <- getIncidenceRates(
    connectionHandler = connectionHandler, 
    schema = schema
  )
  
  testthat::expect_true(nrow(incidence) > 0)
  
  testthat::expect_true( 'incidenceProportionP100p' %in% colnames(incidence))
  testthat::expect_true( 'incidenceRateP100py' %in% colnames(incidence))
  testthat::expect_true( 'personsAtRisk' %in% colnames(incidence))
  testthat::expect_true( 'personDays' %in% colnames(incidence))
  testthat::expect_true( 'databaseName' %in% colnames(incidence))
  testthat::expect_true( 'targetName' %in% colnames(incidence))
  testthat::expect_true( 'outcomeName' %in% colnames(incidence))
})


test_that("getTimeToEvent", {
  # check results are returned
  tte <- getTimeToEvent(
    connectionHandler = connectionHandler, 
    schema = 'main'
  )
  
  testthat::expect_true(nrow(tte) > 0)
  
  testthat::expect_true( 'databaseName' %in% colnames(tte))
  testthat::expect_true( 'targetName' %in% colnames(tte))
  testthat::expect_true( 'outcomeName' %in% colnames(tte))
  testthat::expect_true( 'outcomeType' %in% colnames(tte))
  testthat::expect_true( 'targetOutcomeType' %in% colnames(tte))
  testthat::expect_true( 'timeToEvent' %in% colnames(tte))
  testthat::expect_true( 'numEvents' %in% colnames(tte))
  testthat::expect_true( 'timeScale' %in% colnames(tte))
  
  # check restriction works
  tte <- getTimeToEvent(
    connectionHandler = connectionHandler, 
    schema = 'main', 
    targetIds = 1, 
    outcomeIds = 3
  )
  testthat::expect_true(unique(tte$targetId) == 1)
  testthat::expect_true(unique(tte$outcomeId) == 3)
})

test_that("getDechallengeRechallenge", {
  # check results are returned
  result <- getDechallengeRechallenge(
    connectionHandler = connectionHandler, 
    schema = 'main'
  )
  
  testthat::expect_true(nrow(result) > 0)
  
  testthat::expect_true( 'databaseName' %in% colnames(result))
  testthat::expect_true( 'targetName' %in% colnames(result))
  testthat::expect_true( 'outcomeName' %in% colnames(result))
  testthat::expect_true( 'dechallengeStopInterval' %in% colnames(result))
  testthat::expect_true( 'dechallengeEvaluationWindow' %in% colnames(result))
  testthat::expect_true( 'numExposureEras' %in% colnames(result))
  testthat::expect_true( 'numCases' %in% colnames(result))
  testthat::expect_true( 'pctDechallengeSuccess' %in% colnames(result))
  testthat::expect_true( 'pctRechallengeFail' %in% colnames(result))
  
  # check restriction works
  result <- getDechallengeRechallenge(
    connectionHandler = connectionHandler, 
    schema = 'main', 
    targetIds = 1, 
    outcomeIds = 3
  )
  testthat::expect_true(unique(result$targetId) == 1)
  testthat::expect_true(unique(result$outcomeId) == 3)
})

test_that("getBinaryTargetBaseline", {
  
res <- getBinaryTargetBaseline(
    connectionHandler = connectionHandler,
    schema = schema,
    cTablePrefix = 'c_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    targetIds = NULL,
    analysisIds = NULL,
    covariateIds = NULL,
    conceptIds = NULL,
    databaseIds = NULL
)

testthat::expect_true(nrow(res) > 0)

res <- getBinaryTargetBaseline(
  connectionHandler = connectionHandler,
  schema = schema,
  cTablePrefix = 'c_',
  cgTablePrefix = 'cg_',
  databaseTable = 'database_meta_data',
  targetIds = 1,
  analysisIds = NULL,
  covariateIds = NULL,
  conceptIds = NULL,
  databaseIds = NULL
)

testthat::expect_true(nrow(res) > 0)
testthat::expect_true(unique(res$targetCohortId) == 1)

res <- getBinaryTargetBaseline(
  connectionHandler = connectionHandler,
  schema = schema,
  cTablePrefix = 'c_',
  cgTablePrefix = 'cg_',
  databaseTable = 'database_meta_data',
  targetIds = NULL,
  analysisIds = 3,
  covariateIds = NULL,
  conceptIds = NULL,
  databaseIds = NULL
)

testthat::expect_true(nrow(res) > 0)
testthat::expect_true(unique(res$analysisId) == 3)

})


test_that("target counts", {
  # check results are returned
  countsT <- getCaseTargetCounts(
    connectionHandler = connectionHandler, 
    schema = 'main'
  )
  
  testthat::expect_true(nrow(countsT) > 0)
  
  testthat::expect_true( 'databaseName' %in% colnames(countsT))
  testthat::expect_true( 'targetName' %in% colnames(countsT))
  testthat::expect_true( 'outcomeName' %in% colnames(countsT))
  testthat::expect_true( 'rowCount' %in% colnames(countsT))
  testthat::expect_true( 'personCount' %in% colnames(countsT))
  
  # check restriction works
  countsT2 <- getCaseTargetCounts(
    connectionHandler = connectionHandler, 
    schema = 'main', 
    targetIds = 1, 
    outcomeIds = 3
  )
  testthat::expect_true(unique(countsT2$targetId) == 1)
  testthat::expect_true(unique(countsT2$outcomeId) == 3)
})


test_that("getCaseCounts", {
  # check results are returned
  counts <- getCaseCounts(
    connectionHandler = connectionHandler, 
    schema = 'main'
  )
  
  testthat::expect_true(nrow(counts) > 0)
  
  testthat::expect_true( 'databaseName' %in% colnames(counts))
  testthat::expect_true( 'targetName' %in% colnames(counts))
  testthat::expect_true( 'outcomeName' %in% colnames(counts))
  testthat::expect_true( 'rowCount' %in% colnames(counts))
  testthat::expect_true( 'personCount' %in% colnames(counts))
  
  # check restriction works
  counts <- getCaseCounts(
    connectionHandler = connectionHandler, 
    schema = 'main', 
    targetIds = 1, 
    outcomeIds = 3
  )
  testthat::expect_true(unique(counts$targetId) == 1)
  testthat::expect_true(unique(counts$outcomeId) == 3)
})




test_that("getCharacterizationDemographics", {
  # age works
  data <- getCharacterizationDemographics(
    connectionHandler = connectionHandler, 
    schema = 'main', 
    type = 'age', 
    targetId = 1, 
    outcomeId = 3
  )
  
  testthat::expect_true(nrow(data) > 0)
  
  testthat::expect_true( 'databaseName' %in% colnames(data))
  testthat::expect_true( 'targetName' %in% colnames(data))
  testthat::expect_true( 'outcomeName' %in% colnames(data))
  testthat::expect_true( 'covariateName' %in% colnames(data))
  testthat::expect_true( 'caseCount' %in% colnames(data))
  testthat::expect_true( 'caseAverage' %in% colnames(data))
  testthat::expect_true( 'nonCaseCount' %in% colnames(data))
  testthat::expect_true( 'nonCaseAverage' %in% colnames(data))
  
  # sex works
  data <- getCharacterizationDemographics(
    connectionHandler = connectionHandler, 
    schema = 'main', 
    type = 'sex',
    targetId = 1, 
    outcomeId = 3
  )
  
  testthat::expect_true(nrow(data) > 0)
  
  testthat::expect_true( 'databaseName' %in% colnames(data))
  testthat::expect_true( 'targetName' %in% colnames(data))
  testthat::expect_true( 'outcomeName' %in% colnames(data))
  testthat::expect_true( 'covariateName' %in% colnames(data))
  testthat::expect_true( 'caseCount' %in% colnames(data))
  testthat::expect_true( 'caseAverage' %in% colnames(data))
  testthat::expect_true( 'nonCaseCount' %in% colnames(data))
  testthat::expect_true( 'nonCaseAverage' %in% colnames(data))
  
  
  # other type fails
  testthat::expect_error(
    getCharacterizationDemographics(
      connectionHandler = connectionHandler, 
      schema = 'main', 
      type = 'none'
    )
  )
  
})

test_that("getBinaryRiskFactors", {
  # age works
  data <-  getBinaryRiskFactors(
    connectionHandler = connectionHandler, 
    schema = 'main', 
    targetId = 1, 
    outcomeId = 3,
    analysisIds = c(1,3,410, 210)
  )
  
  testthat::expect_true(nrow(data) > 0)

  testthat::expect_true( 'databaseName' %in% colnames(data))
  testthat::expect_true( 'targetName' %in% colnames(data))
  testthat::expect_true( 'outcomeName' %in% colnames(data))
  testthat::expect_true( 'covariateName' %in% colnames(data))
  testthat::expect_true( 'caseCount' %in% colnames(data))
  testthat::expect_true( 'caseAverage' %in% colnames(data))
  testthat::expect_true( 'nonCaseCount' %in% colnames(data))
  testthat::expect_true( 'nonCaseAverage' %in% colnames(data))
  testthat::expect_true( 'smd' %in% colnames(data))
  
  # add code to test values sum to 1
  data <-  getBinaryRiskFactors(
    connectionHandler = connectionHandler, 
    schema = 'main', 
    targetId = 1, 
    outcomeId = 3,
    analysisIds = c(1)
  )
  
  testthat::expect_true(sum(data$nonCaseAverage) == 1)
  testthat::expect_true(sum(as.double(data$caseAverage)) == 1)
  
})


test_that("getContinuousRiskFactors", {
  # age works
  data <-  getContinuousRiskFactors(
    connectionHandler = connectionHandler, 
    schema = 'main', 
    targetId = 1, 
    outcomeId = 3
  )
  
  testthat::expect_true(nrow(data) > 0)
  
  testthat::expect_true( 'databaseName' %in% colnames(data))
  testthat::expect_true( 'targetName' %in% colnames(data))
  testthat::expect_true( 'outcomeName' %in% colnames(data))
  testthat::expect_true( 'covariateName' %in% colnames(data))
  testthat::expect_true( 'caseCountValue' %in% colnames(data))
  testthat::expect_true( 'caseAverageValue' %in% colnames(data))
  testthat::expect_true( 'targetCountValue' %in% colnames(data))
  testthat::expect_true( 'targetAverageValue' %in% colnames(data))
  testthat::expect_true( 'smd' %in% colnames(data))
  
})

test_that("getBinaryCaseSeries", {
  # age works
  data <-  getBinaryCaseSeries(
    connectionHandler = connectionHandler, 
    schema = 'main', 
    targetId = 1, 
    outcomeId = 3
  )
  
  testthat::expect_true(nrow(data) > 0)
  
  testthat::expect_true( 'databaseName' %in% colnames(data))
  testthat::expect_true( 'targetName' %in% colnames(data))
  testthat::expect_true( 'outcomeName' %in% colnames(data))
  testthat::expect_true( 'covariateName' %in% colnames(data))
  testthat::expect_true( 'minPriorObservation' %in% colnames(data))
  testthat::expect_true( 'limitToFirstInNDays' %in% colnames(data))
  testthat::expect_true( 'outcomeWashoutDays' %in% colnames(data))
  testthat::expect_true( 'casePostOutcomeDuration' %in% colnames(data))
  testthat::expect_true( 'casePreTargetDuration' %in% colnames(data))
  testthat::expect_true( 'riskWindowStart' %in% colnames(data))
  testthat::expect_true( 'startAnchor' %in% colnames(data))
  testthat::expect_true( 'riskWindowEnd' %in% colnames(data))
  testthat::expect_true( 'endAnchor' %in% colnames(data))
  testthat::expect_true( 'sumValueBefore' %in% colnames(data))
  testthat::expect_true( 'averageValueBefore' %in% colnames(data))
  testthat::expect_true( 'sumValueDuring' %in% colnames(data))
  testthat::expect_true( 'averageValueDuring' %in% colnames(data))
  testthat::expect_true( 'sumValueAfter' %in% colnames(data))
  testthat::expect_true( 'averageValueAfter' %in% colnames(data))
  
})

test_that("getContinuousCaseSeries", {
  # age works
  data <-  getContinuousCaseSeries(
    connectionHandler = connectionHandler, 
    schema = 'main', 
    targetId = 1, 
    outcomeId = 3
  )
  
  testthat::expect_true(nrow(data) > 0)
  
  testthat::expect_true( 'databaseName' %in% colnames(data))
  testthat::expect_true( 'targetName' %in% colnames(data))
  testthat::expect_true( 'outcomeName' %in% colnames(data))
  testthat::expect_true( 'covariateName' %in% colnames(data))
  testthat::expect_true( 'minPriorObservation' %in% colnames(data))
  testthat::expect_true( 'limitToFirstInNDays' %in% colnames(data))
  testthat::expect_true( 'outcomeWashoutDays' %in% colnames(data))
  testthat::expect_true( 'casePostOutcomeDuration' %in% colnames(data))
  testthat::expect_true( 'casePreTargetDuration' %in% colnames(data))
  testthat::expect_true( 'riskWindowStart' %in% colnames(data))
  testthat::expect_true( 'startAnchor' %in% colnames(data))
  testthat::expect_true( 'riskWindowEnd' %in% colnames(data))
  testthat::expect_true( 'endAnchor' %in% colnames(data))
  testthat::expect_true( 'countValueBefore' %in% colnames(data))
  testthat::expect_true( 'averageValueBefore' %in% colnames(data))
  testthat::expect_true( 'standardDeviationBefore' %in% colnames(data))
  testthat::expect_true( 'countValueDuring' %in% colnames(data))
  testthat::expect_true( 'averageValueDuring' %in% colnames(data))
  testthat::expect_true( 'standardDeviationDuring' %in% colnames(data))
  testthat::expect_true( 'countValueAfter' %in% colnames(data))
  testthat::expect_true( 'averageValueAfter' %in% colnames(data))
  testthat::expect_true( 'standardDeviationAfter' %in% colnames(data))
})


test_that("processBinaryRiskFactorFeatures with values", {
  
  caseCounts <- data.frame(
    databaseName = 'madeup',
    databaseId = 1,
    minPriorObservation = 365,
    outcomeWashoutDays = 9999,
    riskWindowStart = 1,
    riskWindowEnd = 365,
    startAnchor = 'cohort start',
    endAnchor = 'cohort start',
    personCount = c(2000)
  )
  
  caseFeatures <- data.frame(
    databaseName = rep('madeup',2),
    databaseId = 1,
    minPriorObservation = rep(365,2),
    outcomeWashoutDays = rep(9999,2),
    riskWindowStart = rep(1,2),
    riskWindowEnd = rep(365,2),
    startAnchor = rep('cohort start',2),
    endAnchor = rep('cohort start',2),
    covariateName = c('cov 1','cov 10'),
    covariateId = c(1,10),
    targetName = rep('target',2),
    targetCohortId = rep(2,2),
    outcomeName = rep('outcome',2),
    outcomeCohortId = rep(3,2),
    sumValue = c(5, 20),
    averageValue = c(5/2000, 20/2000)
  )
  
  targetCounts <- data.frame(
    databaseName = 'madeup',
    databaseId = 1,
    minPriorObservation = 365,
    outcomeWashoutDays = 9999,
    personCount = 10000
  )
  
  targetFeatures <- data.frame(
    databaseName = rep('madeup',2),
    databaseId = 1,
    minPriorObservation = rep(365,2),
    outcomeWashoutDays = rep(9999,2),
    covariateName = c('cov 1','cov 10'),
    covariateId = c(1,10),
    targetName = rep('target',2),
    targetCohortId = rep(2,2),
    outcomeName = rep('outcome',2),
    outcomeCohortId = rep(3,2),
    sumValue = c(7, 20),
    averageValue = c(5, 20)/10000
  )
  

  res <- processBinaryRiskFactorFeatures(
    caseCounts = caseCounts,
    targetCounts = targetCounts,
    caseFeatures = caseFeatures,
    targetFeatures = targetFeatures
  )
  
  testthat::expect_true(nrow(res) == 2)
  testthat::expect_true(sum(c('cov 1', 'cov 10') %in% res$covariateName) == 2)
  testthat::expect_true(res$covariateId[res$covariateName == 'cov 1'] == 1)
  testthat::expect_true(res$covariateId[res$covariateName == 'cov 10'] == 10)
  
  # check the cov 1 values
  # non case should be target minus case
  cov1Ind <- res$covariateName == 'cov 1'
  testthat::expect_true(res$caseCount[cov1Ind] == 5)
  testthat::expect_true(res$caseAverage[cov1Ind] == 5/2000)
  testthat::expect_true(res$nonCaseCount[cov1Ind] == (7-5))
  testthat::expect_true(res$nonCaseAverage[cov1Ind] == 2/(10000-2000))
  
  # check the cov 10 values
  # non case should be target minus case
  cov10Ind <- res$covariateName == 'cov 10'
  testthat::expect_true(res$caseCount[cov10Ind] == 20)
  testthat::expect_true(res$caseAverage[cov10Ind] == 20/2000)
  testthat::expect_true(res$nonCaseCount[cov10Ind] == (20-20))
  testthat::expect_true(res$nonCaseAverage[cov10Ind] == 0)
  
})


# add test for getting target count and features
# to check exclude is working correctly 

# target count - make sqlite tables
# requires: database table, cohort_counts, settings, 
#.          cohort_details, cohort_definition


test_that("getCharacterizationOutcomes", {
  
outcomes <- getCharacterizationOutcomes(
    connectionHandler = connectionHandler, 
    schema = 'main', 
    targetId = NULL
)

# check there is a result
testthat::expect_true(nrow(outcomes) > 0)

# check the columns are correct
testthat::expect_true(sum(c("cohortName", "cohortDefinitionId", "dechalRechal",
  "riskFactors", "timeToEvent", "caseSeries") %in% colnames(outcomes)) == 6)

# check results when specifying targetId
outcomes2 <- getCharacterizationOutcomes(
  connectionHandler = connectionHandler, 
  schema = 'main', 
  targetId = 1
)

testthat::expect_true(nrow(outcomes) >= nrow(outcomes2))
testthat::expect_true(nrow(outcomes2) >= 1)

# check results when specifying targetId with no outcomes
outcomes3 <- getCharacterizationOutcomes(
  connectionHandler = connectionHandler, 
  schema = 'main', 
  targetId = 3
)

testthat::expect_true(nrow(outcomes3) == 0 )


})

test_that("getIncidenceOutcomes", {
  
  outcomes <- getIncidenceOutcomes(
    connectionHandler = connectionHandler, 
    schema = 'main', 
    targetId = NULL
  )
  # check there is a result
  testthat::expect_true(nrow(outcomes) > 0)
  
  # check the columns are correct
  testthat::expect_true(sum(c("cohortName", "cohortDefinitionId", "cohortIncidence") %in% colnames(outcomes)) == 3)
  
  # check results when specifying targetId
  outcomes2 <- getIncidenceOutcomes(
    connectionHandler = connectionHandler, 
    schema = 'main', 
    targetId = 1
  )
  
  testthat::expect_true(nrow(outcomes) >= nrow(outcomes2))
  testthat::expect_true(nrow(outcomes2) >= 1)
  
  # check results when specifying targetId with no outcomes
  outcomes3 <- getIncidenceOutcomes(
    connectionHandler = connectionHandler, 
    schema = 'main', 
    targetId = 3
  )
  
  testthat::expect_true(nrow(outcomes3) == 0 )
  
})







test_that("getCharacterizationCohortBinary", {
  # check results are returned
  data <- getCharacterizationCohortBinary(
    connectionHandler = connectionHandler, 
    schema = 'main', 
    targetIds = 1
  )
  
  testthat::expect_true(nrow(data$covariates) > 0)
  testthat::expect_true(nrow(data$covRef) > 0)
  
  testthat::expect_true( 'covariateId' %in% colnames(data$covariates))
  testthat::expect_true( 'covariateName' %in% colnames(data$covariates))
  testthat::expect_true( 'sumValue_1' %in% colnames(data$covariates))
  testthat::expect_true( 'averageValue_1' %in% colnames(data$covariates))
  
  testthat::expect_true(unique(data$covRef$cohortId) == 1)
  
})


test_that("getCharacterizationCohortContinuous", {
  # check results are returned
  data <- getCharacterizationCohortContinuous(
    connectionHandler = connectionHandler, 
    schema = 'main', 
    targetIds = 1
  )
  
  testthat::expect_true(nrow(data$covariates) > 0)
  testthat::expect_true(nrow(data$covRef) > 0)
  
  testthat::expect_true( 'covariateName' %in% colnames(data$covariates))
  testthat::expect_true( 'covariateId' %in% colnames(data$covariates))
  testthat::expect_true( 'minPriorObservation' %in% colnames(data$covariates))
  testthat::expect_true( 'covariateId' %in% colnames(data$covariates))
  testthat::expect_true( 'countValue_1' %in% colnames(data$covariates))
  testthat::expect_true( 'averageValue_1' %in% colnames(data$covariates))
  
  testthat::expect_true(unique(data$covRef$cohortId) == 1)
  
})

test_that("viewIncidenceRate", {
  
  incidenceData <- getIncidenceRates(
    connectionHandler = connectionHandler , 
    schema = schema
  )
 # incidence data does not have rate values to imputing them
  incidenceData$incidenceRateP100py <- 1+sample(c(-1,1),replace = TRUE)*runif(nrow(incidenceData))
  incidenceData$incidenceProportionP100p<- 0.5+sample(c(-1,1),replace = TRUE)*runif(nrow(incidenceData))

  ageData <- getBinaryTargetBaseline(
    connectionHandler = connectionHandler, 
    schema = schema,  
    analysisIds = 3
  )

  genderData <- getBinaryTargetBaseline(
    connectionHandler = connectionHandler, 
    schema = schema,  
    analysisIds = 1
  )

  p <- viewIncidenceRate(
    incidenceData = incidenceData,
    ageData = ageData,
    genderData = genderData
    )
  testthat::expect_s3_class(p, 'gt_tbl')
})


test_that("plotAgeDistributions", {
  ageData <- getCharacterizationDemographics(
    connectionHandler = connectionHandler, 
    schema = 'main',
    targetId = 1, 
    outcomeId = 3, 
    type = 'age'
  )
  p <- plotAgeDistributions(ageData = ageData)
  testthat::expect_s3_class(p, 'ggplot')
})

test_that("plotSexDistributions", {
  sexData <- getCharacterizationDemographics(
    connectionHandler = connectionHandler, 
    schema = 'main',
    targetId = 1, 
    outcomeId = 3, 
    type = 'sex'
  )
  p <- plotSexDistributions(sexData = sexData)
  testthat::expect_s3_class(p, 'ggplot')
})

#' Extract the top N predictors per model
#' @description
#' This function extracts the top N predictors per model from the prediction results tables 
#'
#' @details
#' Specify the connectionHandler, the resultDatabaseSettings and (optionally) any targetIds or outcomeIds to restrict models to
#'
#' @template connectionHandler
#' @template schema
#' @template plpTablePrefix
#' @template cgTablePrefix
#' @param databaseTable The database table name
#' @template targetIds
#' @template outcomeIds
#' @param numberPredictors the number of predictors per model to return
#' @family Prediction
#' @return
#' Returns a data.frame with the columns: 
#' \itemize{
#'  \item{databaseName the name of the database the model was developed on}
#'  \item{tarStartDay the time-at-risk start day}
#'  \item{tarStartAnchor whether the time-at-risk start is relative to cohort start or end}
#'  \item{tarEndDay the time-at-risk end day}
#'  \item{tarEndAnchor whether the time-at-risk end is relative to cohort start or end}
#'  \item{performanceId a unique identifier for the performance}
#'  \item{covariateId the FeatureExtraction covariate identifier}
#'  \item{covariateName the name of the covariate}
#'  \item{conceptId the covariates corresponding concept or 0}
#'  \item{covariateValue the feature importance or coefficient value}
#'  \item{covariateCount how many people had the covariate}
#'  \item{covariateMean the fraction of the target population with the covariate}
#'  \item{covariateStDev the standard deviation}
#'  \item{withNoOutcomeCovariateCount the number of the target population without the outcome with the covariate}
#'  \item{withNoOutcomeCovariateMean the fraction of the target population without the outcome with the covariate}
#'  \item{withNoOutcomeCovariateStDev the covariate standard deviation of the target population without the outcome}
#'  \item{withOutcomeCovariateCount the number of the target population with the outcome with the covariate}
#'  \item{withOutcomeCovariateMean the fraction of the target population with the outcome with the covariate}
#'  \item{withOutcomeCovariateStDev the covariate standard deviation of the target population with the outcome}
#'  \item{standardizedMeanDiff the standardized mean difference comparing the target population with outcome and without the outcome}
#'  \item{rn the row number showing the covariate rank}
#' }
#'
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' topPreds <- getPredictionTopPredictors(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main',
#'   targetIds = 1,
#'   outcomeIds = 3
#' )
#' 
getPredictionTopPredictors <- function(
    connectionHandler,
    schema,
    plpTablePrefix = 'plp_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    targetIds = NULL,
    outcomeIds = NULL,
    numberPredictors = 100
){

  # get performance_id - TODO add cohort and outcome names?
  sql <- "
  select * from 
  
  (select d.cdm_source_abbreviation as database_name,
  tars.tar_start_day, tars.tar_start_anchor,
  tars.tar_end_day, tars.tar_end_anchor,
  cov.*,
  ROW_NUMBER() OVER(
  PARTITION BY p.performance_id, d.cdm_source_abbreviation,
  tars.tar_start_day, tars.tar_start_anchor,
  tars.tar_end_day, tars.tar_end_anchor
  ORDER BY ABS(cov.covariate_value) DESC
  ) AS rn
  
  from @schema.@plp_table_prefixmodel_designs md 
  
  inner join @schema.@plp_table_prefixperformances p
  on md.model_design_id = p.model_design_id
  
  inner join @schema.@plp_table_prefixcovariate_summary cov
  on p.performance_id = cov.performance_id
  
  inner join @schema.@plp_table_prefixdatabase_details dd
  on dd.database_id = p.development_database_id
  
  inner join @schema.@database_table d
  on d.database_id = dd.database_meta_data_id 
  
  inner join @schema.@plp_table_prefixtars tars
  on tars.tar_id = p.tar_id 
  
  inner join @schema.@plp_table_prefixcohorts t
  on t.cohort_id = md.target_id
  
  -- join to CG table to get target name
  
  inner join @schema.plp_cohorts o
  on o.cohort_id = md.outcome_id
  
  -- join to CG table to get outcome name
  
  {@outcome_restrict | @target_restrict}?{ where }
  {@outcome_restrict}?{o.cohort_definition_id in (@outcome_ids)}
  {@outcome_restrict & @target_restrict}?{ and } 
  {@target_restrict}?{t.cohort_definition_id in (@target_ids)}
  
  ) temp
  
  where rn <= @number_predictors;"
  
  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    plp_table_prefix = plpTablePrefix,
    #cg_table_prefix = cgTablePrefix,
    database_table = databaseTable,
    outcome_restrict = !is.null(outcomeIds),
    outcome_ids = paste0(outcomeIds, collapse = ','),
    target_restrict = !is.null(targetIds),
    target_ids = paste0(targetIds, collapse = ','),
    number_predictors = numberPredictors
  )
  
  return(result)
}


#' Extract the top N predictors across a set of models
#' @description
#' This function extracts the top N predictors across models by 
#' finding the sum of the absolute coefficient value across models.
#'
#' @details
#' Specify the connectionHandler, the resultDatabaseSettings and (optionally) any modelDesignIds to restrict to
#'
#' @template connectionHandler
#' @template schema
#' @template plpTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @param modelDesignIds One or more model design IDs to restrict to
#' @family Prediction
#' @return
#' Returns a data.frame with the columns: 
#' \itemize{
#'  \item{databaseName the name of the database the model was developed on}
#'  \item{tarStartDay the time-at-risk start day}
#'  \item{tarStartAnchor whether the time-at-risk start is relative to cohort start or end}
#'  \item{tarEndDay the time-at-risk end day}
#'  \item{tarEndAnchor whether the time-at-risk end is relative to cohort start or end}
#'  \item{covariateId the FeatureExtraction covariate identifier}
#'  \item{covariateName the name of the covariate}
#'  \item{conceptId the covariates corresponding concept or 0}
#'  \item{sumCovariateValue the total absolute feature importance or coefficient value}
#'  \item{numberOfTimesPredictive number of models that contained the covariate}
#' }
#'
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' topPreds <- getPredictionAggregateTopPredictors(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main',
#'   modelDesignIds = c(1,2,5)
#' )
#' 
getPredictionAggregateTopPredictors <- function(
    connectionHandler,
    schema,
    plpTablePrefix = 'plp_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    modelDesignIds = NULL
){
  
  sql <- "
  select 
  
  d.cdm_source_abbreviation as database_name,
  cov.covariate_id,
  cov.covariate_name,
  cov.concept_id,
  SUM(ABS(cov.covariate_value)) as sum_covariate_value,
  count(*) as number_of_times_predictive
  
  from 

  @schema.@plp_table_prefixperformances p
  
  inner join @schema.@plp_table_prefixcovariate_summary cov
  on p.performance_id = cov.performance_id
  
  inner join @schema.@plp_table_prefixdatabase_details dd
  on dd.database_id = p.development_database_id
  
  inner join @schema.@database_table d
  on d.database_id = dd.database_meta_data_id 
  
  {@model_design_restrict}?{ where p.model_design_id in (@model_design_ids)}
  
  GROUP BY 
  d.cdm_source_abbreviation,
  cov.covariate_id,
  cov.covariate_name,
  cov.concept_id
  ;"
  
  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    plp_table_prefix = plpTablePrefix,
    database_table = databaseTable,
    model_design_restrict = !is.null(modelDesignIds),
    model_design_ids = paste0(modelDesignIds, collapse = ',')
  )
  
  return(result)
}


#' A function to extarct the targets found in prediction
#'
#' @details
#' Specify the connectionHandler, the schema and the prefixes
#'
#' @template connectionHandler
#' @template schema
#' @template plpTablePrefix
#' @template cgTablePrefix
#' @family Prediction
#' 
#' @return
#' A data.frame with the prediction target cohort ids and names.
#'
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' cohorts <- getPredictionTargets(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getPredictionTargets <- function(
    connectionHandler,
    schema,
    plpTablePrefix = 'plp_',
    cgTablePrefix = 'cg_'
){
  
  sql <- "SELECT distinct 
    cohorts.cohort_name,
    cohorts.cohort_definition_id, 
    'prediction' as type,
    1 as value

       FROM
          @schema.@plp_table_prefixmodel_designs as model_designs
          inner join
        (SELECT c.cohort_id, c.cohort_definition_id, cd.cohort_name FROM @schema.@plp_table_prefixcohorts c
        inner join @schema.@cg_table_prefixcohort_definition cd
        on c.cohort_definition_id = cd.cohort_definition_id 
        and c.cohort_name = cd.cohort_name
        ) AS cohorts
        ON model_designs.target_id = cohorts.cohort_id
        ;"
  
  targets <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    plp_table_prefix = plpTablePrefix,
    cg_table_prefix = cgTablePrefix
  ) %>%
    tidyr::pivot_wider(
      id_cols = c("cohortName", "cohortDefinitionId"), 
      names_from = "type", 
      values_from = c("value")
    )
  
  return(targets)
  
}


#' A function to extract the outcomes found in prediction
#'
#' @details
#' Specify the connectionHandler, the schema and the prefixes
#'
#' @template connectionHandler
#' @template schema
#' @template plpTablePrefix
#' @template cgTablePrefix
#' @template targetId
#' @family Prediction
#' 
#' @return
#' A data.frame with the prediction outcome cohort ids and names.
#'
#' @export
#' 
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' outcomes <- getPredictionOutcomes(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getPredictionOutcomes <- function(
    connectionHandler,
    schema,
    plpTablePrefix = 'plp_',
    cgTablePrefix = 'cg_',
    targetId = NULL
){
  
  sql <- "SELECT distinct 
    cohorts.cohort_name,
    cohorts.cohort_definition_id, 
    'prediction' as type,
    1 as value

       FROM
          @schema.@plp_table_prefixmodel_designs as model_designs
          inner join
        (SELECT c.cohort_id, c.cohort_definition_id, cd.cohort_name FROM @schema.@plp_table_prefixcohorts c
        inner join @schema.@cg_table_prefixcohort_definition cd
        on c.cohort_definition_id = cd.cohort_definition_id
        and c.cohort_name = cd.cohort_name
        ) AS cohorts
        ON model_designs.outcome_id = cohorts.cohort_id
        
      {@use_target}?{ 
      
        inner join
        (SELECT distinct cohort_id
        FROM @schema.@plp_table_prefixcohorts 
        where cohort_definition_id in (@target_id)
        ) AS targets
        ON model_designs.target_id = targets.cohort_id
        
        }
        
        ;"
  
  outcomes <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    plp_table_prefix = plpTablePrefix,
    cg_table_prefix = cgTablePrefix,
    use_target = !is.null(targetId),
    target_id = paste0(targetId, collapse = ',') 
  ) %>%
    tidyr::pivot_wider(
      id_cols = c("cohortName", "cohortDefinitionId"), 
      names_from = "type", 
      values_from = c("value")
    )
  
  return(outcomes)
  
}

#' Extract a complete set of cohorts used in the prediction results
#' @description
#' This function extracts the target and outcome cohorts used to develop any model in the results
#'
#' @details
#' Specify the connectionHandler, the resultDatabaseSettings and any targetIds or outcomeIds to restrict models to
#'
#' @template connectionHandler
#' @template schema
#' @template plpTablePrefix
#' @template cgTablePrefix
#' @family Prediction
#' @return
#' Returns a data.frame with the columns: 
#' \itemize{
#'  \item{cohortId the cohort definition ID}
#'  \item{cohortName the name of the cohort}
#'  \item{type whether the cohort was used as a target or outcome cohort}
#'}
#'
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' predCohorts <- getPredictionCohorts(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getPredictionCohorts <- function(
    connectionHandler,
    schema,
    plpTablePrefix = 'plp_',
    cgTablePrefix = 'cg_'
){
  
  allres <- c()
  for(type in c('target','outcome')){
    sql <- "SELECT distinct 
    cohorts.cohort_id, 
    cohorts.cohort_name,
    '@type' as type

       FROM
          @schema.@plp_table_prefixmodel_designs as model_designs
          inner join
        (SELECT c.cohort_id, cd.cohort_name FROM @schema.@plp_table_prefixcohorts c
        inner join @schema.@cg_table_prefixcohort_definition cd
        on c.cohort_definition_id = cd.cohort_definition_id
        and c.cohort_name = cd.cohort_name
        ) AS cohorts
        ON model_designs.@type_id = cohorts.cohort_id
        ;"
    
    result <- connectionHandler$queryDb(
      sql = sql,
      schema = schema,
      plp_table_prefix = plpTablePrefix,
      cg_table_prefix = cgTablePrefix,
      type = type
    )
    
    allres <- rbind(allres, result)
    
  }
  
  allres <- unique(allres)

  return(allres)
}

#' Extract the model designs from the prediction results
#' @description
#' This function extracts the model design settings 
#'
#' @details
#' Specify the connectionHandler, the resultDatabaseSettings and (optionally) any targetIds or outcomeIds to restrict model designs to
#'
#' @template connectionHandler
#' @template schema
#' @template plpTablePrefix
#' @template cgTablePrefix
#' @template targetIds
#' @template outcomeIds
#' @param modelDesignIds (Optional) A set of model design ids to restrict to
#' @family Prediction
#' @return
#' Returns a data.frame with the columns:
#' \itemize{ 
#'  \item{modelDesignId a unique identifier in the database for the model design}
#'  \item{modelType the type of classifier or surival model}
#'  \item{developmentTargetId a unique identifier for the development target ID}
#'  \item{developmentTargetName the name of the development target cohort}
#'  \item{developmentTargetJson the json of the target cohort}
#'  \item{developmentOutcomeId a unique identifier for the development outcome ID}
#'  \item{developmentOutcomeName the name of the development outcome cohort}
#'  \item{timeAtRisk the time at risk string}
#'  \item{developmentOutcomeJson the json of the outcome cohort}
#'  \item{covariateSettingsJson the covariate settings json}
#'  \item{populationSettingsJson the population settings json}
#'  \item{tidyCovariatesSettingsJson the tidy covariate settings json}
#'  \item{plpDataSettingsJson the plp data extraction settings json}
#'  \item{featureEngineeringSettingsJson the feature engineering settings json}
#'  \item{splitSettingsJson the split settings json}
#'  \item{sampleSettingsJson the sample settings json}
#'  \item{modelSettingsJson the model settings json}
#'  }
#' 
#' @export
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' modDesign <- getPredictionModelDesigns(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getPredictionModelDesigns <- function(
    connectionHandler,
    schema,
    plpTablePrefix = 'plp_',
    cgTablePrefix = 'cg_',
    targetIds = NULL,
    outcomeIds = NULL,
    modelDesignIds = NULL
){
  
  sql <- "SELECT distinct 
          model_designs.model_design_id,
          model_settings.model_type model_type,
          targets.cohort_definition_id AS development_target_id,
          targets.cohort_name AS development_target_name,
          targets.json AS development_target_json,
          outcomes.cohort_definition_id AS development_outcome_id,
          outcomes.cohort_name AS development_outcome_name,
          outcomes.json AS development_outcome_json,
          tars.tar_start_day,
          tars.tar_start_anchor,
          tars.tar_end_day,
          tars.tar_end_anchor,
          covariate_settings.covariate_settings_json,
          population_settings.population_settings_json,
          tidy_covariates_settings.tidy_covariates_settings_json,
          plp_data_settings.plp_data_settings_json,
          feature_engineering_settings.feature_engineering_settings_json,
          split_settings.split_settings_json,
          sample_settings.sample_settings_json,
          model_settings.model_settings_json

       FROM
          @schema.@plp_table_prefixmodel_designs as model_designs

          inner join
          @schema.@plp_table_prefixmodel_settings as model_settings
          on model_designs.model_setting_id = model_settings.model_setting_id

          inner join
          @schema.@plp_table_prefixcovariate_settings as covariate_settings
          on model_designs.covariate_setting_id = covariate_settings.covariate_setting_id

          inner join
          @schema.@plp_table_prefixpopulation_settings as population_settings
          on model_designs.population_setting_id = population_settings.population_setting_id

          inner join
          @schema.@plp_table_prefixtidy_covariates_settings as tidy_covariates_settings
          on model_designs.tidy_covariates_setting_id = tidy_covariates_settings.tidy_covariates_setting_id

          inner join
          @schema.@plp_table_prefixsample_settings as sample_settings
          on model_designs.sample_setting_id = sample_settings.sample_setting_id

          inner join
          @schema.@plp_table_prefixfeature_engineering_settings as feature_engineering_settings
          on model_designs.feature_engineering_setting_id = feature_engineering_settings.feature_engineering_setting_id

          inner join
          @schema.@plp_table_prefixsplit_settings as split_settings
          on model_designs.split_setting_id = split_settings.split_setting_id

          inner join
          @schema.@plp_table_prefixplp_data_settings as plp_data_settings
          on model_designs.plp_data_setting_id = plp_data_settings.plp_data_setting_id


        LEFT JOIN
        (SELECT c.cohort_id, cd.cohort_name, cd.json, cd.cohort_definition_id
        FROM @schema.@plp_table_prefixcohorts c
        inner join @schema.@cg_table_prefixcohort_definition cd
        on c.cohort_definition_id = cd.cohort_definition_id
        and c.cohort_name = cd.cohort_name
        ) AS targets
        ON model_designs.target_id = targets.cohort_id

        LEFT JOIN
        (SELECT c.cohort_id, cd.cohort_name, cd.json, cd.cohort_definition_id
        FROM @schema.@plp_table_prefixcohorts c
        inner join @schema.@cg_table_prefixcohort_definition cd
        on c.cohort_definition_id = cd.cohort_definition_id
        and c.cohort_name = cd.cohort_name
        ) AS outcomes
        ON model_designs.outcome_id = outcomes.cohort_id

        LEFT JOIN @schema.@plp_table_prefixtars AS tars
        ON model_designs.tar_id = tars.tar_id

        where 1 = 1
        {@target_restrict} ? { and  targets.cohort_definition_id in (@target_ids) }
        {@outcome_restrict} ? { and  outcomes.cohort_definition_id in (@outcome_ids) }
        {@model_design_restrict} ? { and  model_designs.model_design_id in (@model_design_ids) }
        ;"
  
  summaryTable <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    plp_table_prefix = plpTablePrefix,
    cg_table_prefix = cgTablePrefix,
    target_restrict = !is.null(targetIds),
    target_ids = paste(targetIds, collapse = ','),
    outcome_restrict = !is.null(outcomeIds),
    outcome_ids = paste(outcomeIds, collapse = ','),
    model_design_restrict = !is.null(modelDesignIds),
    model_design_ids = paste(modelDesignIds, collapse = ','),
  )
  
  # add TAR column and remove individual TARs
  summaryTable <- addPredictionTimeAtRisk(
    result = summaryTable,
    tarColumnName = 'timeAtRisk',
    tarStartAnchor = 'tarStartAnchor',
    tarStartDay = 'tarStartDay',
    tarEndAnchor = 'tarEndAnchor',
    tarEndDay = 'tarEndDay',
    removeIndividualTarColumns = TRUE
  )
  
  summaryTable <- summaryTable %>%
    dplyr::relocate("timeAtRisk", .after = "developmentOutcomeName")
  
  return(summaryTable)
  
}

hasPlpModelNameColumn <- function(connectionHandler, schema, plpTablePrefix = "plp_") {
  sql <- "SELECT *
          FROM @schema.@plp_table_prefixmodel_settings
          WHERE 1 = 0;"

  hasColumn <- tryCatch(
    {
      res <- connectionHandler$queryDb(
        sql = sql,
        schema = schema,
        plp_table_prefix = plpTablePrefix
      )
      'modelName' %in% colnames(res)
    },
    error = function(e) FALSE
  )
  
  return(hasColumn)
}

# can be used per modelDesignId to explore specific models
# or per modelDesignId and developmentDatabaseId to find validations

#' Extract the model performances 
#' @description
#' This function extracts the model performances
#'
#' @details
#' Specify the connectionHandler, the resultDatabaseSettings and (optionally) a modelDesignId and/or developmentDatabaseId to restrict models to
#'
#' @template connectionHandler
#' @template schema
#' @template plpTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @param databaseTablePrefix A prefix to the database table, either '' or 'plp_'
#' @param modelDesignId The identifier for a model design  to restrict results to
#' @param developmentDatabaseId The identifier for the development database to restrict results to
#' @family Prediction
#' @return
#' Returns a data.frame with the columns:
#' \itemize{  
#'  \item{performanceId the unique identifier for the performance}
#'  \item{modelDesignId the unique identifier for the model design}
#'  \item{modelType the type of classifier}
#'  \item{algorithmName the model algorithm name}
#'  \item{developmentDatabaseId the unique identifier for the database used to develop the model}
#'  \item{validationDatabaseId the unique identifier for the database used to validate the model}
#'  \item{developmentTargetId the unique cohort id for the development target population}
#'  \item{developmentTargetName the name for the development target population}
#'  \item{developmentOutcomeId the unique cohort id for the development outcome}
#'  \item{developmentOutcomeName the name for the development outcome}
#'  \item{developmentDatabase the name for the database used to develop the model}
#'  \item{validationDatabase the name for the database used to validate the model}
#'  \item{validationTargetName the name for the validation target population}
#'  \item{validationOutcomeName the name for the validation outcome}
#'  \item{timeStamp the date/time when the analysis occurred}
#'  \item{auroc the test/validation AUROC value for the model}
#'  \item{auroc95lb the test/validation lower bound of the 95 percent CI AUROC value for the model}
#'  \item{auroc95ub the test/validation upper bound of the 95 percent CI AUROC value for the model}
#'  \item{calibrationInLarge the test/validation calibration in the large value for the model}
#'  \item{eStatistic the test/validation calibration e-statistic value for the model}
#'  \item{brierScore the test/validation brier value for the model}
#'  \item{auprc the test/validation discrimination AUPRC value for the model}
#'  \item{populationSize the test/validation population size used to develop the model}
#'  \item{outcomeCount the test/validation outcome count used to develop the model}
#'  \item{evalPercent the percentage of the development data used as the test set}
#'  \item{outcomePercent the outcome percent in the development data}
#'  \item{validationTimeAtRisk time at risk for the validation}
#'  \item{predictionResultType development or validation}
#'  
#'}
#' 
#' @export
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' perf <- getPredictionPerformances(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getPredictionPerformances <- function(
    connectionHandler,
    schema,
    plpTablePrefix = 'plp_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    databaseTablePrefix = '',
    modelDesignId = NULL,
    developmentDatabaseId = NULL
){
  hasModelName <- hasPlpModelNameColumn(
    connectionHandler = connectionHandler,
    schema = schema,
    plpTablePrefix = plpTablePrefix
  )
  algorithmNameSelect <- if (hasModelName) {
    "COALESCE(model_settings.model_name, models.model_type, model_settings.model_type) AS algorithm_name,"
  } else {
    "COALESCE(models.model_type, model_settings.model_type) AS algorithm_name,"
  }
  
  sql <- "SELECT distinct
       results.performance_id,
       results.model_design_id,
       model_settings.model_type,
       @algorithm_name_select
       results.development_database_id,
       results.validation_database_id,
       devtargets.cohort_definition_id AS development_target_id,
       devtargets.cohort_name AS development_target_name,
       devoutcomes.cohort_definition_id AS development_outcome_id,
       devoutcomes.cohort_name AS development_outcome_name,
       d.database_acronym AS development_database,
       v.database_acronym AS validation_database,
       targets.cohort_name AS validation_target_name,
       outcomes.cohort_name AS validation_outcome_name,
       results.execution_date_time AS time_stamp,
       tars.tar_start_day,
       tars.tar_start_anchor,
       tars.tar_end_day,
       tars.tar_end_anchor,
       aucResult.auc AS auroc,
       auclbResult.auclb AS auroc_95lb,
       aucubResult.aucub AS auroc_95ub,
       calibration_in_large_result.calibration_in_large AS calibration_in_large,
       estat.value as e_statistic,
       brier.value as brier_score,
       auprcResult.auprc AS auprc,
       nResult.population_size,
       oResult.outcome_count,
       nTest.test_size*100.0/nResult.population_size AS eval_percent,
       oResult.outcome_count*100.0/nResult.population_size AS outcome_percent,
       results.model_development AS model_development_flag

       FROM

       (select * from @schema.@plp_table_prefixperformances
        {@model_design_restrict | @development_database_id_restrict} ?{ where }
        {@model_design_restrict} ? {model_design_id in (@model_design_id)}
        {@model_design_restrict & @development_database_id_restrict} ?{ and }
        {@development_database_id_restrict} ? {development_database_id = @development_database_id}
       )

       AS results
       

    inner join @schema.@plp_table_prefixmodel_designs as model_designs
    on model_designs.model_design_id = results.model_design_id
    
    inner join @schema.@plp_table_prefixmodel_settings as model_settings
    on model_designs.model_setting_id = model_settings.model_setting_id

    LEFT JOIN
        (
          SELECT m.model_design_id, m.database_id, m.model_type
          FROM @schema.@plp_table_prefixmodels m
          INNER JOIN
            (
              SELECT model_design_id, database_id, MAX(model_id) AS model_id
              FROM @schema.@plp_table_prefixmodels
              GROUP BY model_design_id, database_id
            ) latest_models
          ON m.model_id = latest_models.model_id
        ) AS models
    ON results.model_design_id = models.model_design_id
    AND results.development_database_id = models.database_id

    LEFT JOIN
        (SELECT c.cohort_id, c.cohort_definition_id, cd.cohort_name FROM @schema.@plp_table_prefixcohorts c
        inner join @schema.@cg_table_prefixcohort_definition cd
        on c.cohort_definition_id = cd.cohort_definition_id
        and c.cohort_name = cd.cohort_name
        ) AS devtargets ON model_designs.target_id = devtargets.cohort_id

        LEFT JOIN
        (SELECT c.cohort_id, c.cohort_definition_id, cd.cohort_name FROM @schema.@plp_table_prefixcohorts c
        inner join @schema.@cg_table_prefixcohort_definition cd
        on c.cohort_definition_id = cd.cohort_definition_id
        and c.cohort_name = cd.cohort_name
        ) AS targets ON results.target_id = targets.cohort_id

        LEFT JOIN
        (SELECT c.cohort_id, c.cohort_definition_id, cd.cohort_name FROM @schema.@plp_table_prefixcohorts c
        inner join @schema.@cg_table_prefixcohort_definition cd
        on c.cohort_definition_id = cd.cohort_definition_id
        and c.cohort_name = cd.cohort_name
        ) AS outcomes ON results.outcome_id = outcomes.cohort_id

        LEFT JOIN
        (SELECT c.cohort_id, c.cohort_definition_id, cd.cohort_name FROM @schema.@plp_table_prefixcohorts c
        inner join @schema.@cg_table_prefixcohort_definition cd
        on c.cohort_definition_id = cd.cohort_definition_id
        and c.cohort_name = cd.cohort_name
        ) AS devoutcomes ON model_designs.outcome_id = devoutcomes.cohort_id


        LEFT JOIN
        (select dd.database_id, md.cdm_source_abbreviation database_acronym
         from @schema.@database_table_prefix@database_table md inner join
         @schema.@plp_table_prefixdatabase_details dd
         on md.database_id = dd.database_meta_data_id
        ) AS d ON results.development_database_id = d.database_id

        LEFT JOIN
        (select dd.database_id, md.cdm_source_abbreviation database_acronym
         from @schema.@database_table_prefix@database_table md inner join
         @schema.@plp_table_prefixdatabase_details dd
         on md.database_id = dd.database_meta_data_id
         ) AS v ON results.validation_database_id = v.database_id

        LEFT JOIN @schema.@plp_table_prefixtars AS tars ON results.tar_id = tars.tar_id
        LEFT JOIN (SELECT performance_id, value AS auc FROM @schema.@plp_table_prefixevaluation_statistics where metric = 'AUROC' and evaluation in ('Test','Validation') ) AS aucResult ON results.performance_id = aucResult.performance_id
        LEFT JOIN (SELECT performance_id, value AS auprc FROM @schema.@plp_table_prefixevaluation_statistics where metric = 'AUPRC' and evaluation in ('Test','Validation') ) AS auprcResult ON results.performance_id = auprcResult.performance_id
        LEFT JOIN (SELECT performance_id, sum(value) AS population_size FROM @schema.@plp_table_prefixevaluation_statistics where metric = 'populationSize' and evaluation in ('Train','Test','Validation') group by performance_id) AS nResult ON results.performance_id = nResult.performance_id
        LEFT JOIN (SELECT performance_id, sum(value) AS outcome_count FROM @schema.@plp_table_prefixevaluation_statistics where metric = 'outcomeCount' and evaluation in ('Train','Test','Validation') group by performance_id) AS oResult ON results.performance_id = oResult.performance_id
        
        LEFT JOIN (select performance_id, value from @schema.@plp_table_prefixEVALUATION_STATISTICS where EVALUATION in ('Test','Validation') and METRIC = 'brier score scaled') AS brier
        on brier.performance_id = results.performance_id  
        
        LEFT JOIN (select performance_id, value from @schema.@plp_table_prefixEVALUATION_STATISTICS where EVALUATION in ('Test','Validation') and METRIC = 'Eavg') AS estat
        on estat.performance_id = results.performance_id   
        
        LEFT JOIN (SELECT performance_id, value AS test_size FROM @schema.@plp_table_prefixevaluation_statistics where metric = 'populationSize' and evaluation in ('Test', 'Validation') ) AS nTest ON results.performance_id = nTest.performance_id

        LEFT JOIN (SELECT performance_id, value AS auclb FROM @schema.@plp_table_prefixevaluation_statistics where metric = '95% lower AUROC' and evaluation in ('Test','Validation') ) AS auclbResult ON results.performance_id = auclbResult.performance_id
        LEFT JOIN (SELECT performance_id, value AS aucub FROM @schema.@plp_table_prefixevaluation_statistics where metric = '95% upper AUROC' and evaluation in ('Test','Validation') ) AS aucubResult ON results.performance_id = aucubResult.performance_id
        LEFT JOIN (SELECT performance_id, value AS calibration_in_large FROM @schema.@plp_table_prefixevaluation_statistics where metric = 'calibrationInLarge intercept' and evaluation in ('Test','Validation') ) AS calibration_in_large_result ON results.performance_id = calibration_in_large_result.performance_id

    ;"
  
  
  summaryTable <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    plp_table_prefix = plpTablePrefix,
    model_design_id = paste0(modelDesignId, collapse = ','),
    model_design_restrict = !is.null(modelDesignId),
    development_database_id = developmentDatabaseId,
    development_database_id_restrict = !is.null(developmentDatabaseId),
    algorithm_name_select = algorithmNameSelect,
    database_table_prefix = databaseTablePrefix,
    database_table = databaseTable,
    cg_table_prefix = cgTablePrefix
  )
  
  
  summaryTable <- addPredictionTimeAtRisk(
    result = summaryTable,
    tarColumnName = 'validationTimeAtRisk',
    tarStartAnchor = 'tarStartAnchor',
    tarStartDay = 'tarStartDay',
    tarEndAnchor = 'tarEndAnchor',
    tarEndDay = 'tarEndDay',
    removeIndividualTarColumns = TRUE
  )
  
  summaryTable$validationTargetName <- trimws(summaryTable$validationTargetName )
  summaryTable$validationOutcomeName <- trimws(summaryTable$validationOutcomeName)
  summaryTable$validationTargetName  <- as.factor(summaryTable$validationTargetName )
  summaryTable$validationOutcomeName <- as.factor(summaryTable$validationOutcomeName)
  
  summaryTable$predictionResultType <- ifelse(summaryTable$modelDevelopmentFlag == 1, 'Development', 'Validation')
  
  summaryTable <- summaryTable %>%
    dplyr::select(-"modelDevelopmentFlag")
  
  return(summaryTable)
}


#' Extract the model performances per evaluation
#' @description
#' This function extracts the model performances per evaluation
#'
#' @details
#' Specify the connectionHandler, the resultDatabaseSettings and (optionally) a modelDesignId and/or developmentDatabaseId to restrict models to
#'
#' @template connectionHandler
#' @template schema
#' @template plpTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @param databaseTablePrefix A prefix to the database table, either '' or 'plp_'
#' @param modelDesignId The identifier for a model design  to restrict results to
#' @param developmentDatabaseId The identifier for the development database to restrict results to
#' @family Prediction
#' @return
#' Returns a data.frame with the columns:
#' \itemize{  
#'  \item{timeStamp the date/time when the analysis occurred}
#'  \item{performanceId the unique identifier for the performance}
#'  \item{modelDesignId the unique identifier for the model design}
#'  \item{modelType the type of classifier}
#'  \item{algorithmName the model algorithm name}
#'  \item{covariateName a summary name for the candidate covariates}
#'  \item{developmentDatabaseId the unique identifier for the database used to develop the model}
#'  \item{validationDatabaseId the unique identifier for the database used to validate the model}
#'  \item{developmentTargetId the unique cohort id for the development target population}
#'  \item{developmentTargetName the name for the development target population}
#'  \item{validationTargetId the id for the validation target population}
#'  \item{validationTargetName the name for the validation target population if different from development}
#'  \item{developmentOutcomeId the unique cohort id for the development outcome}
#'  \item{developmentOutcomeName the name for the development outcome}
#'  \item{validationOutcomeId the id for the validation outcome}
#'  \item{validationOutcomeName the name for the validation outcome if different from development}
#'  \item{developmentDatabase the name for the database used to develop the model}
#'  \item{validationDatabase the name for the database used to validate the model if different from development}
#'  \item{validationTarId the validation time at risk id}
#'  \item{validationTimeAtRisk the time at risk used when evaluating the model if different from development}
#'  \item{developmentTarId the development time at risk id}
#'  \item{developmentTimeAtRisk the time at risk used when developing the model}
#'  \item{evaluation The type of evaluation: Test/Train/CV/Validation}
#'  \item{populationSize the test/validation population size used to develop the model}
#'  \item{outcomeCount the test/validation outcome count used to develop the model}
#'  \item{AUROC the AUROC value for the model}
#'  \item{95 lower AUROC: the lower bound of the 95 percent CI AUROC value for the model}
#'  \item{95 upper AUROC: the upper bound of the 95 percent CI AUROC value for the model}
#'  \item{AUPRC the discrimination AUPRC value for the model}
#'  \item{brier score: the brier value for the model}
#'  \item{brier score scaled: the scaled brier value for the model}
#'  \item{Average Precision: the average precision value for the model}
#'  \item{Eavg the calibration average error e-statistic value for the model}
#'  \item{E90 the calibration 90 percent upper bound e-statistic value for the model}
#'  \item{Emax the calibration max error e-statistic value for the model}
#'  \item{calibrationInLarge mean prediction: the  calibration in the large mean predicted risk value for the model}
#'  \item{calibrationInLarge observed risk: the calibration in the large mean observed risk value for the model}
#'  \item{calibrationInLarge intercept: the calibration in the large value intercept for the model}
#'  \item{weak calibration intercept: the weak calibration intercept for the model}
#'  \item{weak calibration gradient: the weak calibration gradient for the model}
#'  \item{Hosmer Lemeshow calibration intercept: the Hosmer Lemeshow calibration intercept for the model}
#'  \item{Hosmer Lemeshow calibration gradient: the Hosmer Lemeshow calibration gradient for the model}
#'  \item{... Additional metrics that are added to PLP}
#'}
#' 
#' @export
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' perf <- getFullPredictionPerformances(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getFullPredictionPerformances <- function(
    connectionHandler,
    schema,
    plpTablePrefix = 'plp_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    databaseTablePrefix = '',
    modelDesignId = NULL,
    developmentDatabaseId = NULL
){
  hasModelName <- hasPlpModelNameColumn(
    connectionHandler = connectionHandler,
    schema = schema,
    plpTablePrefix = plpTablePrefix
  )
  algorithmNameSelect <- if (hasModelName) {
    "COALESCE(model_settings.model_name, models.model_type, model_settings.model_type) AS algorithm_name,"
  } else {
    "COALESCE(models.model_type, model_settings.model_type) AS algorithm_name,"
  }
  
  sql <- "SELECT distinct
       results.execution_date_time AS time_stamp,
       results.performance_id,
       results.model_design_id,
       
       model_settings.model_type,
       @algorithm_name_select
       
       results.development_database_id,
       results.validation_database_id,
       
       devtargets.cohort_definition_id AS development_target_id,
       devtargets.cohort_name AS development_target_name,
       targets.cohort_definition_id AS validation_target_id,
       targets.cohort_name AS validation_target_name,
       
       devoutcomes.cohort_definition_id AS development_outcome_id,
       devoutcomes.cohort_name AS development_outcome_name,
       outcomes.cohort_definition_id AS validation_outcome_id,
       outcomes.cohort_name AS validation_outcome_name,
       
       d.database_acronym AS development_database,
       v.database_acronym AS validation_database,
       
       val_tars.tar_id as validation_tar_id,
       val_tars.tar_start_day,
       val_tars.tar_start_anchor,
       val_tars.tar_end_day,
       val_tars.tar_end_anchor,
       
       dev_tars.tar_id as development_tar_id,
       dev_tars.tar_start_day as dev_tar_start_day,
       dev_tars.tar_start_anchor as dev_tar_start_anchor,
       dev_tars.tar_end_day as dev_tar_end_day,
       dev_tars.tar_end_anchor as dev_tar_end_anchor
      
       FROM

       (select * from @schema.@plp_table_prefixperformances
        {@model_design_restrict | @development_database_id_restrict} ?{ where }
        {@model_design_restrict} ? {model_design_id in (@model_design_id)}
        {@model_design_restrict & @development_database_id_restrict} ?{ and }
        {@development_database_id_restrict} ? {development_database_id = @development_database_id}
       )

       AS results

    inner join @schema.@plp_table_prefixmodel_designs as model_designs
    on model_designs.model_design_id = results.model_design_id
    
    inner join @schema.@plp_table_prefixmodel_settings as model_settings
    on model_designs.model_setting_id = model_settings.model_setting_id

    LEFT JOIN
        (
          SELECT m.model_design_id, m.database_id, m.model_type
          FROM @schema.@plp_table_prefixmodels m
          INNER JOIN
            (
              SELECT model_design_id, database_id, MAX(model_id) AS model_id
              FROM @schema.@plp_table_prefixmodels
              GROUP BY model_design_id, database_id
            ) latest_models
          ON m.model_id = latest_models.model_id
        ) AS models
    ON results.model_design_id = models.model_design_id
    AND results.development_database_id = models.database_id

    LEFT JOIN
        (SELECT c.cohort_id, c.cohort_definition_id, cd.cohort_name FROM @schema.@plp_table_prefixcohorts c
        inner join @schema.@cg_table_prefixcohort_definition cd
        on c.cohort_definition_id = cd.cohort_definition_id
        and c.cohort_name = cd.cohort_name
        ) AS devtargets ON model_designs.target_id = devtargets.cohort_id

        LEFT JOIN
        (SELECT c.cohort_id, c.cohort_definition_id, cd.cohort_name FROM @schema.@plp_table_prefixcohorts c
        inner join @schema.@cg_table_prefixcohort_definition cd
        on c.cohort_definition_id = cd.cohort_definition_id
        and c.cohort_name = cd.cohort_name
        ) AS targets ON results.target_id = targets.cohort_id

        LEFT JOIN
        (SELECT c.cohort_id, c.cohort_definition_id, cd.cohort_name FROM @schema.@plp_table_prefixcohorts c
        inner join @schema.@cg_table_prefixcohort_definition cd
        on c.cohort_definition_id = cd.cohort_definition_id
        and c.cohort_name = cd.cohort_name
        ) AS outcomes ON results.outcome_id = outcomes.cohort_id

        LEFT JOIN
        (SELECT c.cohort_id, c.cohort_definition_id, cd.cohort_name FROM @schema.@plp_table_prefixcohorts c
        inner join @schema.@cg_table_prefixcohort_definition cd
        on c.cohort_definition_id = cd.cohort_definition_id
        and c.cohort_name = cd.cohort_name
        ) AS devoutcomes ON model_designs.outcome_id = devoutcomes.cohort_id


        LEFT JOIN
        (select dd.database_id, md.cdm_source_abbreviation database_acronym
         from @schema.@database_table_prefix@database_table md inner join
         @schema.@plp_table_prefixdatabase_details dd
         on md.database_id = dd.database_meta_data_id
        ) AS d ON results.development_database_id = d.database_id

        LEFT JOIN
        (select dd.database_id, md.cdm_source_abbreviation database_acronym
         from @schema.@database_table_prefix@database_table md inner join
         @schema.@plp_table_prefixdatabase_details dd
         on md.database_id = dd.database_meta_data_id
         ) AS v ON results.validation_database_id = v.database_id

        LEFT JOIN @schema.@plp_table_prefixtars AS val_tars ON results.tar_id = val_tars.tar_id
        
        LEFT JOIN @schema.@plp_table_prefixtars AS dev_tars ON model_designs.tar_id = dev_tars.tar_id
        
    ;"
  
  
  summaryTable <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    plp_table_prefix = plpTablePrefix,
    model_design_id = paste0(modelDesignId, collapse = ','),
    model_design_restrict = !is.null(modelDesignId),
    development_database_id = developmentDatabaseId,
    development_database_id_restrict = !is.null(developmentDatabaseId),
    algorithm_name_select = algorithmNameSelect,
    database_table_prefix = databaseTablePrefix,
    database_table = databaseTable,
    cg_table_prefix = cgTablePrefix
  )
  
  # process table if it is not empty
  if(nrow(summaryTable) > 0){
    
    summaryTable <- addPredictionTimeAtRisk(
      result = summaryTable,
      tarColumnName = 'validationTimeAtRisk',
      tarStartAnchor = 'tarStartAnchor',
      tarStartDay = 'tarStartDay',
      tarEndAnchor = 'tarEndAnchor',
      tarEndDay = 'tarEndDay',
      removeIndividualTarColumns = TRUE
    )
    
    summaryTable <- addPredictionTimeAtRisk(
      result = summaryTable,
      tarColumnName = 'developmentTimeAtRisk',
      tarStartAnchor = 'devTarStartAnchor',
      tarStartDay = 'devTarStartDay',
      tarEndAnchor = 'devTarEndAnchor',
      tarEndDay = 'devTarEndDay',
      removeIndividualTarColumns = TRUE
    )

    # ?TODO remove this and edit table in OhdsiShinyModules instead
    # set valDb, T, O, TAR to '-' if it is the same as the dev
    sameT <- summaryTable$developmentTargetId == summaryTable$validationTargetId
    if(sum(sameT, na.rm = TRUE) > 0){
      summaryTable$validationTargetName[sameT] <- '-'
    }
    sameO <- summaryTable$developmentOutcomeId == summaryTable$validationOutcomeId
    if(sum(sameO, na.rm = TRUE) > 0){
      summaryTable$validationOutcomeName[sameO] <- '-'
    }
    sameDb <- summaryTable$developmentDatabase == summaryTable$validationDatabase
    if(sum(sameDb, na.rm = TRUE) > 0){
      summaryTable$validationDatabase[sameDb] <- '-'
    }
    sameTar <- summaryTable$developTimeAtRisk == summaryTable$validationTimeAtRisk
    if(sum(sameTar, na.rm = TRUE) > 0){
      summaryTable$validationTimeAtRisk[sameTar] <- '-'
    }
    
  }
  
  # add covariateName
  if(nrow(summaryTable) > 0){
    
    covariateNames <- getCovariateSummaryName(
      connectionHandler = connectionHandler,
      schema = schema,
      plpTablePrefix = plpTablePrefix,
      modelDesignIds = modelDesignId
    )
    
    summaryTable <- merge(
      x = summaryTable, 
      y = covariateNames, 
      by = 'modelDesignId'
      ) %>%
      dplyr::relocate(
        "covariateName", 
        .after = "modelType"
        )
    
  }
  
  if(length(unique(summaryTable$performanceId)) > 0){
    # now select the performances for the performanceIds
    sql <- "SELECT * FROM @schema.@plp_table_prefixevaluation_statistics
          {@restrict_performance}?{WHERE performance_id in (@performance_ids)};"
    
    performanceTable <- connectionHandler$queryDb(
      sql = sql,
      schema = schema,
      plp_table_prefix = plpTablePrefix,
      restrict_performance = !is.null(modelDesignId), # only restrict if modelDesignId is restricted
      performance_ids = paste0(unique(summaryTable$performanceId), collapse = ',')
    )
    
    performanceTable <- performanceTable %>%
      tidyr::pivot_wider(
        id_cols = c('performanceId','evaluation'), 
        names_from = "metric", 
        values_from = "value", 
        values_fill = 0
      )
    
    summaryTable <- merge(
      x = summaryTable, 
      y = performanceTable, 
      by = c('performanceId')
    )
  }

  return(summaryTable)
}

#' Extract the model design diagnostics for a specific development database
#' @description
#' This function extracts the PROBAST diagnostics 
#'
#' @details
#' Specify the connectionHandler, the resultDatabaseSettings and (optionally) a modelDesignId and threshold1_2 a threshold value to use for the PROBAST 1.2 
#'
#' @template connectionHandler
#' @template schema
#' @template plpTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @param modelDesignIds The identifier for a model design  to restrict results to
#' @param threshold1_2 A threshold for probast 1.2
#' @family Prediction
#' @return
#' Returns a data.frame with the columns:
#' \itemize{   
#'  \item{modelDesignId the unique identifier for the model design}
#'  \item{diagnosticId the unique identifier for diagnostic result}
#'  \item{developmentDatabaseName the name for the database used to develop the model}
#'  \item{developmentTargetName the name for the development target population}
#'  \item{developmentOutcomeName the name for the development outcome}
#'  \item{probast1_1 Were appropriate data sources used, e.g., cohort, RCT, or nested case-control study data?}
#'  \item{probast1_2 Were all inclusions and exclusions of paticipants appropriate?}
#'  \item{probast2_1 Were predictors defined and assessed in a similar way for all participants?}
#'  \item{probast2_2 Were predictors assessments made without knowledge of outcome data?}
#'  \item{probast2_3 All all predictors available at the time the model is intended to be used?}
#'  \item{probast3_4 Was the outcome defined and determined in a similar way for all participants?}
#'  \item{probast3_6 Was the time interval between predictor assessment and outcome determination appropriate?}
#'  \item{probast4_1 Were there a reasonable number of participants with the outcome?}
#'  }
#' 
#' @export
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' diag <- getPredictionDiagnostics(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getPredictionDiagnostics <- function(
    connectionHandler,
    schema,
    plpTablePrefix = 'plp_',
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    modelDesignIds = NULL,
    threshold1_2 = 0.9
){
  
  sql <- "SELECT distinct
          design.MODEL_DESIGN_ID,
          diagnostics.diagnostic_id,
          database.DATABASE_NAME as development_DATABASE_NAME,
          database.database_id as development_database_id,
          cohortT.COHORT_NAME development_target_name,
          design.target_id as development_target_id,
          cohortO.COHORT_NAME development_outcome_name,
          design.outcome_id as development_outcome_id,
          summary.PROBAST_ID,
          summary.RESULT_VALUE

          from
          (select * from @schema.@plp_table_prefixDIAGNOSTICS
          {@model_design_restrict} ? {where MODEL_DESIGN_ID in (@model_design_ids)}
          ) as diagnostics
          inner join
          @schema.@plp_table_prefixMODEL_DESIGNS design
          on diagnostics.MODEL_DESIGN_ID = design.MODEL_DESIGN_ID

          inner join
          @schema.@plp_table_prefixDIAGNOSTIC_SUMMARY summary
          on diagnostics.DIAGNOSTIC_ID = summary.DIAGNOSTIC_ID

          inner join
          (select dd.database_id, md.cdm_source_abbreviation as database_name
                   from @schema.@database_table md inner join
                   @schema.@plp_table_prefixdatabase_details dd
                   on md.database_id = dd.database_meta_data_id) as database
          on database.database_id = diagnostics.database_id

         inner join
          @schema.@plp_table_prefixCOHORTS cohortT
         on cohortT.cohort_id = design.target_id

          inner join
          @schema.@plp_table_prefixCOHORTS cohortO
          on cohortO.cohort_id = design.outcome_id;
  "
  
  summaryTable <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    plp_table_prefix = plpTablePrefix,
    model_design_ids = paste0(modelDesignIds, collapse = ','),
    model_design_restrict = !is.null(modelDesignIds),
    database_table = databaseTable
  )
  
  if(nrow(summaryTable)==0){
    message("No diagnostic summary")
    return(NULL)
  }
  
  # update 1.2 based on threshold
  summaryTableNew <- summaryTable[grep('1.2.',summaryTable$probastId),] %>% 
    dplyr::group_by(.data$developmentDatabaseId, .data$modelDesignId) %>%
    dplyr::mutate(
      probastId = '1.2',
      resultValue = if(sum(.data$resultValue > !!threshold1_2) == length(.data$resultValue)){'Pass'}else{'False'}
    ) %>%
    dplyr::ungroup()
  
  summaryTable <- rbind(
    summaryTable[-grep('1.2.',summaryTable$probastId),],
    unique(summaryTableNew)
  )
  
  
  # add probast text
  description <- data.frame(
    probastId = c(
      '1.1', 
      '1.2',
      '2.1',
      '2.2',
      '2.3',
      #"3.1",
      #"3.2",
      #"3.3",
      '3.4',
      '3.6',
      '4.1'
      #"4.2"
      #"4.3"
      #"4.4"
      #"4.5"
      #"4.6"
      #"4.7"
      #"4.8"
      #"4.9"
      ),
    probastDescription = c(
      'Participants: Were appropriate data sources used, e.g. cohort, RCT or nested case-control study data?',
      'Participants: Were all inclusions and exclusions of participants appropriate?',
      'Predictors: Were predictors defined and assessed in a similar way for all participants?',
      "Predictors: Were predictor assessments made without knowledge of outcome data?",
      "Predictors: Are all predictors available at the time the model is intended to be used?",
      #"Outcome: Was the outcome determined appropriately?",
      #"Outcome: Was a pre-specified or standard outcome definition used?",
      #"Outcome: Were predictors excluded from the outcome definition?",
      "Outcome: Was the outcome defined and determined in a similar way for all participants?",
      "Outcome: Was the time interval between predictor assessment and outcome determination appropriate?",
      "Design: Were there a reasonable number of participants with the outcome?"
      #"Design:  Were continuous and categorical predictors handled appropriately?",
      #4.3 Were all enrolled participants included in the analysis?
      #4.4 Were participants with missing data handled appropriately?
      #4.5 Was selection of predictors based on univariable analysis avoided? 
      #4.6 Were complexities in the data (e.g. censoring, competing risks, sampling of controls) accounted for appropriately?
      #4.7 Were relevant model performance measures evaluated appropriately? - Pass for all PLP
      #4.8 Were model overfitting and optimism in model performance accounted for? - Pass for all PLP
      #4.9 Do predictors and their assigned weights in the final model correspond to the results from multivariable analysis? - Pass for all PLP
      )
  )
  
  # add info about diagostic
  summaryTable <- merge(
    x = summaryTable, 
    y = description, 
    by = 'probastId', 
    all.x = TRUE
    )
  
  return(summaryTable)
}


# get results table for specific performance
#' Extract specific results table
#' @description
#' This function extracts the specified table
#'
#' @details
#' Specify the connectionHandler, the resultDatabaseSettings, the table of interest and (optionally) a performanceId to filter to 
#'
#' @template connectionHandler
#' @template schema
#' @template plpTablePrefix
#' @template databaseTable
#' @param table The table to extract (covariate_summary, attrition, prediction_distribution, threshold_summary, calibration_summary, evaluation_statistics or demographic_summary )
#' @param modelDesignIds (optional) restrict to the input modelDesignIds
#' @param performanceIds (optional) restrict to the input performanceIds
#' @param evaluations (optional) restrict to the type of evaluation (e.g., 'Test'/'Train'/'CV'/'Validation')
#' @family Prediction
#' @return
#' Returns a data.frame with the specified table
#' 
#' @export
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' attrition <- getPredictionPerformanceTable(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main',
#'   table = 'attrition'
#' )
#' 
getPredictionPerformanceTable <- function(
    connectionHandler,
    schema,
    plpTablePrefix = 'plp_',
    databaseTable = 'database_meta_data',
    table = 'attrition',
    modelDesignIds = NULL,
    performanceIds = NULL,
    evaluations = NULL
){
  
    sql <- "SELECT distinct
    p.model_design_id,
    p.development_database_id,
    db1.cdm_source_abbreviation as development_database_name,
    p.validation_database_id,
    db2.cdm_source_abbreviation as validation_database_name,
    p.target_id,
    p.outcome_id,
    p.tar_id,
    toi.* 
    
    FROM @schema.@plp_table_prefix@table toi 
    
    INNER JOIN @schema.@plp_table_prefixperformances p 
    ON p.performance_id = toi.performance_id 
    
    INNER JOIN
    (SELECT dd.database_id, md.cdm_source_abbreviation
     FROM @schema.@database_table md 
     INNER JOIN @schema.@plp_table_prefixdatabase_details dd
     ON md.database_id = dd.database_meta_data_id
     ) as db1
     ON db1.database_id = p.development_database_id
     
    INNER JOIN
    (SELECT dd.database_id, md.cdm_source_abbreviation
     FROM @schema.@database_table md 
     INNER JOIN @schema.@plp_table_prefixdatabase_details dd
     ON md.database_id = dd.database_meta_data_id
     ) as db2
     ON db2.database_id = p.validation_database_id
    
    
    WHERE (1=1)
    {@use_model_id}?{ and p.model_design_id in (@model_ids)}
    {@use_performance_id}?{ and p.performance_id in (@performance_ids)}
    {@use_evaluation}?{ and toi.evaluation in (@evaluations)}
    ;"
    
    result  <- connectionHandler$queryDb(
      sql = sql,
      schema = schema,
      database_table = databaseTable,
      table = table,
      use_model_id = !is.null(modelDesignIds),
      model_ids = paste0(modelDesignIds, collapse = ','),
      use_performance_id = !is.null(performanceIds),
      performance_ids = paste0(performanceIds, collapse = ','),
      use_evaluation = !is.null(evaluations),
      evaluations = paste0("'",evaluations, "'", sep = '', collapse = ','),
      plp_table_prefix = plpTablePrefix
    )
    
    return(result)
}


# get prediction covariates for model and validation settings
#' Extract covariate summary details
#' @description
#' This function extracts the covariate summary details
#'
#' @details
#' Specify the connectionHandler, the resultDatabaseSettings, the table of interest and (optionally) modelDesignIds/performanceIds to filter to 
#'
#' @template connectionHandler
#' @template schema
#' @template plpTablePrefix
#' @template cgTablePrefix
#' @template databaseTable
#' @param modelDesignIds (optional) restrict to the input modelDesignIds
#' @param performanceIds (optional) restrict to the input performanceIds
#' @family Prediction
#' @return
#' Returns a data.frame with the specified table
#' 
#' @export
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' covs <- getPredictionCovariates(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getPredictionCovariates <- function(
  connectionHandler, 
  schema, 
  plpTablePrefix = 'plp_',
  cgTablePrefix = 'cg_',
  databaseTable = 'database_meta_data',
  performanceIds = NULL,
  modelDesignIds = NULL
){
  
  sql <- "SELECT distinct
    p.model_design_id,
    p.development_database_id,
    db1.cdm_source_abbreviation as development_database_name,
    p.validation_database_id,
    db2.cdm_source_abbreviation as validation_database_name,
    targets_dev.cohort_definition_id as development_target_cohort_id,
    targets_dev.cohort_name as development_target_name,
    targets.cohort_definition_id as validation_target_cohort_id,
    targets.cohort_name as validation_target_name,
    outcomes_dev.cohort_definition_id as development_outcome_cohort_id,
    outcomes_dev.cohort_name as development_outcome_name,
    outcomes.cohort_definition_id as validation_outcome_cohort_id,
    outcomes.cohort_name as validation_outcome_name,
    t.*,
    t_dev.tar_start_anchor as development_tar_start_anchor,
    t_dev.tar_start_day as development_tar_start_day,
    t_dev.tar_end_anchor as development_tar_end_anchor,
    t_dev.tar_end_day as development_tar_end_day,
    p.plp_data_setting_id,
    ds.plp_data_settings_json,
    p.population_setting_id,
    ps.population_settings_json,
    toi.* 
    
    FROM @schema.@plp_table_prefixcovariate_summary toi 
    
    INNER JOIN @schema.@plp_table_prefixperformances p 
    ON p.performance_id = toi.performance_id 
    
    INNER JOIN @schema.@plp_table_prefixmodel_designs md 
    on md.model_design_id = p.model_design_id
    
    INNER JOIN
    (SELECT dd.database_id, md.cdm_source_abbreviation
     FROM @schema.@database_table md 
     INNER JOIN @schema.@plp_table_prefixdatabase_details dd
     ON md.database_id = dd.database_meta_data_id
     ) as db1
     ON db1.database_id = p.development_database_id
     
    INNER JOIN
    (SELECT dd.database_id, md.cdm_source_abbreviation
     FROM @schema.@database_table md 
     INNER JOIN @schema.@plp_table_prefixdatabase_details dd
     ON md.database_id = dd.database_meta_data_id
     ) as db2
     ON db2.database_id = p.validation_database_id
     
     INNER JOIN
    (SELECT c.cohort_id, c.cohort_definition_id, cd.cohort_name
     FROM @schema.@cg_table_prefixcohort_definition cd 
     INNER JOIN @schema.@plp_table_prefixcohorts c
     ON c.cohort_definition_id = cd.cohort_definition_id 
     and c.cohort_name = cd.cohort_name
     ) as targets
     ON targets.cohort_id = p.target_id
     
     INNER JOIN
    (SELECT c.cohort_id, c.cohort_definition_id, cd.cohort_name
     FROM @schema.@cg_table_prefixcohort_definition cd 
     INNER JOIN @schema.@plp_table_prefixcohorts c
     ON c.cohort_definition_id = cd.cohort_definition_id 
     and c.cohort_name = cd.cohort_name
     ) as targets_dev
     ON targets_dev.cohort_id = md.target_id
     
     INNER JOIN
    (SELECT c.cohort_id, c.cohort_definition_id, cd.cohort_name
     FROM @schema.@cg_table_prefixcohort_definition cd 
     INNER JOIN @schema.@plp_table_prefixcohorts c
     ON c.cohort_definition_id = cd.cohort_definition_id
     and c.cohort_name = cd.cohort_name
     ) as outcomes
     ON outcomes.cohort_id = p.outcome_id
     
    INNER JOIN
    (SELECT c.cohort_id, c.cohort_definition_id, cd.cohort_name
     FROM @schema.@cg_table_prefixcohort_definition cd 
     INNER JOIN @schema.@plp_table_prefixcohorts c
     ON c.cohort_definition_id = cd.cohort_definition_id
     and c.cohort_name = cd.cohort_name
     ) as outcomes_dev
     ON outcomes_dev.cohort_id = md.outcome_id
     
    INNER JOIN @schema.@plp_table_prefixtars t
    ON p.tar_id = t.tar_id
    
    INNER JOIN @schema.@plp_table_prefixtars t_dev
    ON md.tar_id = t_dev.tar_id
    
    INNER JOIN 
    @schema.@plp_table_prefixplp_data_settings ds
    ON p.plp_data_setting_id = ds.plp_data_setting_id
    
    INNER JOIN 
    @schema.@plp_table_prefixpopulation_settings ps
    ON p.population_setting_id = ps.population_setting_id
    
    WHERE (1=1)
    {@use_model_id}?{ and p.model_design_id in (@model_ids)}
    {@use_performance_id}?{ and p.performance_id in (@performance_ids)}
    ;"
  
  result  <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    plp_table_prefix = plpTablePrefix,
    cg_table_prefix = cgTablePrefix,
    database_table = databaseTable,
    use_model_id = !is.null(modelDesignIds),
    model_ids = paste0(modelDesignIds, collapse = ','),
    use_performance_id = !is.null(performanceIds),
    performance_ids = paste0(performanceIds, collapse = ','),
  )
  
  if(nrow(result) >0){
    
    result <- addPredictionTimeAtRisk(
      result = result,
      tarColumnName = 'validationTimeAtRisk',
      tarStartAnchor = 'tarStartAnchor',
      tarStartDay = 'tarStartDay',
      tarEndAnchor = 'tarEndAnchor',
      tarEndDay = 'tarEndDay',
      removeIndividualTarColumns = TRUE
    )
    
    result <- addPredictionTimeAtRisk(
      result = result,
      tarColumnName = 'developmentTimeAtRisk',
      tarStartAnchor = 'developmentTarStartAnchor',
      tarStartDay = 'developmentTarStartDay',
      tarEndAnchor = 'developmentTarEndAnchor',
      tarEndDay = 'developmentTarEndDay',
      removeIndividualTarColumns = TRUE
    )
    
    
    result <- result %>%
      dplyr::relocate("developmentTimeAtRisk", .after = "developmentOutcomeName") %>%
      dplyr::relocate("validationTimeAtRisk", .after = "validationOutcomeName")
    
  }
  
  return(result)
}

# get model lift (PPV/outcomeRate) for a given model sensitivity
#' Extract model lift at given model sensitivity
#' @description
#' This function extracts the model lift (PPV/outcomeRate)
#'
#' @details
#' Specify the connectionHandler, the resultDatabaseSettings and (optionally) modelDesignIds or performanceIds to filter to 
#'
#' @template connectionHandler
#' @template schema
#' @template plpTablePrefix
#' @param modelDesignIds (optional) restrict to the input modelDesignIds
#' @param performanceIds (optional) restrict to the input performanceIds
#' @param sensitivity (default 0.1) the lift at the threshold with the sensitivity closest to this value is return 
#' 
#' @family Prediction
#' @return
#' Returns a data.frame with the columns: modelDesignId, performanceId, evaluation, sensitivity, outcomeCount, positivePredictiveValue, outcomeRate and lift.
#' 
#' @export
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' liftsAt0p15 <- getPredictionLift(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main', 
#'   sensitivity = 0.15
#' )
#' 
getPredictionLift <- function(
    connectionHandler,
    schema,
    plpTablePrefix = 'plp_',
    modelDesignIds = NULL,
    performanceIds = NULL,
    sensitivity = 0.1
){
  
  sql <- "SELECT * from
  
  ( select 
    p.model_design_id,
    toi.performance_id,
    toi.evaluation,
    toi.sensitivity,
    toi.true_count outcome_count,
    toi.positive_predictive_value,
    toi.true_count*1.0/(toi.true_count+toi.false_count) as outcome_rate,
    toi.positive_predictive_value/(toi.true_count*1.0/(toi.true_count+toi.false_count)) as lift,
    row_number() over (partition by toi.performance_id, toi.evaluation order by abs(toi.sensitivity - @sensitivity) asc) rn 
    FROM @schema.@plp_table_prefixthreshold_summary toi 
    inner join 
    @schema.@plp_table_prefixperformances p on 
    p.performance_id = toi.performance_id 
    where 1=1
    {@use_model_id}?{ and p.model_design_id in (@model_ids)}
  {@use_performance_id}?{ and performance_id in (@performance_ids)}
  ) temp
      WHERE rn = 1
    ;"
  
  result  <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    sensitivity = sensitivity,
    use_model_id = !is.null(modelDesignIds),
    model_ids = paste0(modelDesignIds, collapse = ','),
    use_performance_id = !is.null(performanceIds),
    performance_ids = performanceIds,
    plp_table_prefix = plpTablePrefix
  )
  
  return(result)
}

#' Extract specific diagnostic table
#' @description
#' This function extracts the specified diagnostic table
#'
#' @details
#' Specify the connectionHandler, the resultDatabaseSettings, the table of interest and (optionally) a diagnosticId to filter to 
#'
#' @template connectionHandler
#' @template schema
#' @template plpTablePrefix
#' @param table The table to extract
#' @param diagnosticId (optional) restrict to the input diagnosticId
#' @family Prediction
#' @return
#' Returns a data.frame with the specified table
#' 
#' @export
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' diagPred <- getPredictionDiagnosticTable(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main',
#'   table = 'diagnostic_predictors'
#' )
#' 
getPredictionDiagnosticTable <- function(
    connectionHandler,
    schema,
    plpTablePrefix = 'plp_',
    table = 'diagnostic_participants', # 'diagnostic_participants','diagnostic_predictors','diagnostic_outcomes'
    diagnosticId = NULL
){
  
  if(!is.null(diagnosticId)){
    
    sql <- "SELECT * FROM @schema.@plp_table_prefix@table
     WHERE diagnostic_id = @diagnostic_id;"
    
    result  <- connectionHandler$queryDb(
      sql = sql,
      schema = schema,
      table = table,
      diagnostic_id = diagnosticId,
      plp_table_prefix = plpTablePrefix
    )
    
    if('design' %in% colnames(result)){
      result$parameter <- unlist(
        lapply(
          result$design,
          function(x){strsplit(x, ':')[[1]][1]}
        )
      )
      result$paramvalue <- unlist(
        lapply(
          result$design,
          function(x){gsub(' ', '', strsplit(x, ':')[[1]][2])}
        )
      )
      
      result <- result %>% dplyr::select(-"design")
    }
    
    return(result)
  }
  
}


#===== model extraction

#' Extract hyper parameters details
#' @description
#' This function extracts the hyper parameters details
#'
#' @details
#' Specify the connectionHandler, the resultDatabaseSettings, the modelDesignId and the databaseId
#'
#' @template connectionHandler
#' @template schema
#' @template plpTablePrefix
#' @param modelDesignId The identifier for a model design  to restrict to
#' @param databaseId The identifier for the development database to restrict  to
#' @family Prediction
#' @return
#' Returns a data.frame with the columns:
#' \itemize{
#'  \item{metric the hyperparameter optimization metric}
#'  \item{fold the fold in cross validation}
#'  \item{value the metric value for the fold with the specified hyperparameter combination}
#'  } 
#'  plus columns for all the hyperparameters and their values
#' 
#' 
#' @export
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' hyperParams <- getPredictionHyperParamSearch(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#'
getPredictionHyperParamSearch <- function(
    connectionHandler,
    schema,
    plpTablePrefix = 'plp_',
    modelDesignId = NULL,
    databaseId = NULL
){
  
  if(!is.null(modelDesignId) & !is.null(databaseId)){
    
    sql <- "SELECT train_details
            FROM @schema.@plp_table_prefixmodels
            WHERE database_id = @database_id
            and model_design_id = @model_design_id;"
    
    models <- connectionHandler$queryDb(
      sql = sql,
      schema = schema,
      database_id = databaseId,
      model_design_id = modelDesignId,
      plp_table_prefix = plpTablePrefix
    )
    trainDetails <- tryCatch(
      {ParallelLogger::convertJsonToSettings(models$trainDetails)}, 
      error = function(e){print(e); return(list(hyperParamSearch = data.frame()))}
    )
    
    return(trainDetails$hyperParamSearch)
  } else{
    warning('Please enter a modelDesignId and databaseId')
  }
}

#' Extract model interception (for logistic regression)
#' @description
#' This function extracts the interception value
#'
#' @details
#' Specify the connectionHandler, the resultDatabaseSettings, the modelDesignId and the databaseId
#'
#' @template connectionHandler
#' @template schema
#' @template plpTablePrefix
#' @param modelDesignId The identifier for a model design  to restrict to
#' @param databaseId The identifier for the development database to restrict  to
#' @family Prediction
#' @return
#' Returns a single value corresponding to the model intercept or NULL if not a logistic regression model
#' 
#' @export
#' @examples
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' intercepts <- getPredictionIntercept(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#'
getPredictionIntercept <- function(
    connectionHandler,
    schema,
    plpTablePrefix = 'plp_',
    modelDesignId = NULL,
    databaseId = NULL
){
  
  if(!is.null(modelDesignId) & !is.null(databaseId)){
    sql <- "SELECT intercept
          FROM @schema.@plp_table_prefixmodels
          WHERE database_id = @database_id
          and model_design_id = @model_design_id"
    
    models <- connectionHandler$queryDb(
      sql = sql,
      schema = schema,
      database_id = databaseId,
      model_design_id = modelDesignId,
      plp_table_prefix = plpTablePrefix
    )
    
    intercept <- models$intercept
    
    if(is.null(intercept)){
      return(0)
    }
    return(intercept)
  } else{
    warning('Please enter a modelDesignId and databaseId')
  }
}


# covariateSummary
getCovariateSummaryName <- function(
    connectionHandler,
    schema,
    plpTablePrefix = 'plp_',
    modelDesignIds = NULL
    ){
  
  sql <- "select distinct 
  cs.covariate_setting_id,
  cs.covariate_settings_json
  
  from 
  @schema.@plp_table_prefixcovariate_settings cs
  inner join
  @schema.@plp_table_prefixmodel_designs md
  
  on cs.covariate_setting_id = md.covariate_setting_id
  
  {@restrict_model_design_ids}?{where md.model_design_id in (@model_design_ids)}
  ;
  "
  
  covariateDetails <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    restrict_model_design_ids = !is.null(modelDesignIds),
    model_design_ids = modelDesignIds,
    plp_table_prefix = plpTablePrefix
  )
  
  covariateFunctions <- lapply(covariateDetails$covariateSettingsJson, function(x){
    ParallelLogger::convertJsonToSettings(x)
  })
  

  covariateFunctionNames <- c()
  for(i in 1:length(covariateFunctions)){
    
    isNotList <- inherits(covariateFunctions[[i]], 'covariateSettings')
    if(isNotList){
      covariateFunctions[[i]] <- list(covariateFunctions[[i]])
    }
    
    covariateFunctionNames <- c(
        covariateFunctionNames,
        paste0(lapply(covariateFunctions[[i]], getCovariateFunName), collapse = ', ')
      )
    
  }
  
  covariateDetails$covariateName <- covariateFunctionNames
  covariateDetails <- covariateDetails %>%
    dplyr::select("covariateSettingId","covariateName")
  
  # now get model design ids and the covariateFunctionNames
  sql <- "select distinct 
  md.model_design_id,
  md.covariate_setting_id
  
  from 
  @schema.@plp_table_prefixmodel_designs md
  
  {@restrict_model_design_ids}?{where md.model_design_id in (@model_design_ids)}
  ;
  "
  
  modelDesignCovariates <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    restrict_model_design_ids = !is.null(modelDesignIds),
    model_design_ids = modelDesignIds,
    plp_table_prefix = plpTablePrefix
  )
  
  modelDesignCovariates <- merge(
    x = modelDesignCovariates, 
    y = covariateDetails, 
    by = 'covariateSettingId'
    ) %>% 
    dplyr::select("modelDesignId","covariateName")
    
  return(modelDesignCovariates)
}

# helper for covariate summary name
getCovariateFunName <- function(covariateSetting){
  if(inherits(covariateSetting, 'covariateSettings')){
    func <- attributes(covariateSetting)$fun
    
    if(func == 'getDbDefaultCovariateData'){
      params <- names(covariateSetting)
      
      # check age
      age <- FALSE
      if('DemographicsAge' %in% params){
        if(covariateSetting$DemographicsAge){
          age <- TRUE
        }}
      if('DemographicsAgeGroup' %in% params){
        if(covariateSetting$DemographicsAgeGroup){
          age <- TRUE
        }}
      
      # check sec
      sex <- FALSE
      if('DemographicsGender' %in% params){
        if(covariateSetting$DemographicsGender){
          sex <- TRUE
        }}
      
      numberSettings <- length(params[!params %in% c(
        'temporal', 'temporalSequence', 'longTermStartDays',
        'mediumTermStartDays', 'shortTermStartDays', 'endDays',
        'includedCovariateConceptIds', 'addDescendantsToInclude',
        'excludedCovariateConceptIds', 'addDescendantsToExclude',
        'includedCovariateIds'
      )])
      
      func <- paste0(
        func, ' (', numberSettings, ' types)',
        ifelse(age|sex, ' inc ', ''),
        ifelse(age, 'age', ''),
        ifelse(age & sex, ' and ', ''),
        ifelse(sex, 'sex', '')
      )
      
    }
    
    return(func)
  }
}

# adding helpter for TAR consistency 
addPredictionTimeAtRisk <- function(
    result,
    tarColumnName = 'timeAtRisk',
    tarStartAnchor = 'tarStartAnchor',
    tarStartDay = 'tarStartDay',
    tarEndAnchor = 'tarEndAnchor',
    tarEndDay = 'tarEndDay',
    removeIndividualTarColumns = TRUE
){
  
  result <- result %>%
    dplyr::mutate(newCol = paste0('(',.data[[tarStartAnchor]], ' + ', .data[[tarStartDay]], ') - ',
                                                '(',.data[[tarEndAnchor]], ' + ', .data[[tarEndDay]], ')'
    )) %>%
    dplyr::rename(
      !!tarColumnName := "newCol"
    )
  
  if(removeIndividualTarColumns){
    result <- result %>%
      dplyr::select(-dplyr::any_of(c("tarStartAnchor", "tarStartDay", "tarEndAnchor", "tarEndDay")))
  }
  
  return(result)
}

test_that("getPredictionTopPredictors", {
  
  data <- getPredictionTopPredictors(
    connectionHandler = connectionHandler, 
    schema = schema
  )
  
  testthat::expect_true(nrow(data) > 0)
  
  testthat::expect_true('databaseName' %in% colnames(data))
  testthat::expect_true('covariateName' %in% colnames(data))
  testthat::expect_true('covariateValue' %in% colnames(data))
  testthat::expect_true('covariateCount' %in% colnames(data))
  testthat::expect_true('covariateMean' %in% colnames(data))
  testthat::expect_true('covariateStDev' %in% colnames(data))
  testthat::expect_true('withNoOutcomeCovariateMean' %in% colnames(data))
  testthat::expect_true('withOutcomeCovariateMean' %in% colnames(data))
  testthat::expect_true('standardizedMeanDiff' %in% colnames(data))
  testthat::expect_true('rn' %in% colnames(data))
  
  data <- getPredictionTopPredictors(
    connectionHandler = connectionHandler, 
    schema = schema, numberPredictors = 1
  )
  testthat::expect_true(max(data$rn) <= 1)
  
})

test_that("getPredictionCohorts", {
  
  data <- getPredictionCohorts(
    connectionHandler = connectionHandler, 
    schema = schema
  )
  
  testthat::expect_true(nrow(data) > 0)
  
  testthat::expect_true('cohortId' %in% colnames(data))
  testthat::expect_true('cohortName' %in% colnames(data))
  testthat::expect_true('type' %in% colnames(data))
  
})

test_that("getPredictionOutcomes", {
  
data <- getPredictionOutcomes(
  connectionHandler = connectionHandler, 
  schema = schema
)

testthat::expect_true(nrow(data) > 0)

data <- getPredictionOutcomes(
  connectionHandler = connectionHandler, 
  schema = schema, 
  targetId = c(1002,1)
)

testthat::expect_true(nrow(data) > 0)

# not a target in the prediction results
data <- getPredictionOutcomes(
  connectionHandler = connectionHandler, 
  schema = schema, 
  targetId = 1
)

testthat::expect_true(nrow(data) == 0)
  
})

test_that("getPredictionModelDesigns", {
  
  data <- getPredictionModelDesigns(
    connectionHandler = connectionHandler, 
    schema = schema
  )
  
  testthat::expect_true(nrow(data) > 0)
  
  testthat::expect_true('modelDesignId' %in% colnames(data))
  testthat::expect_true('modelType' %in% colnames(data))
  testthat::expect_true('developmentTargetName' %in% colnames(data))
  testthat::expect_true('developmentOutcomeName' %in% colnames(data))
  testthat::expect_true('timeAtRisk' %in% colnames(data))
  testthat::expect_true('covariateSettingsJson' %in% colnames(data))
  testthat::expect_true('populationSettingsJson' %in% colnames(data))
  
  data <- getPredictionModelDesigns(
    connectionHandler = connectionHandler, 
    schema = schema, 
    targetIds = 1002, 
    outcomeIds = 3
  )
  
  testthat::expect_true(nrow(data) > 0)
  testthat::expect_true(max(data$developmentTargetId) == 1002)
  testthat::expect_true(max(data$developmentOutcomeId) == 3)
  
})


test_that("getPredictionPerformances", {
  
  data <- getPredictionPerformances(
    connectionHandler = connectionHandler, 
    schema = schema
  )
  
  testthat::expect_true(nrow(data) > 0)
  
  testthat::expect_true('performanceId' %in% colnames(data))
  testthat::expect_true('modelDesignId' %in% colnames(data))
  testthat::expect_true('algorithmName' %in% colnames(data))
  testthat::expect_true('developmentTargetName' %in% colnames(data))
  testthat::expect_true('developmentOutcomeName' %in% colnames(data))
  testthat::expect_true('validationTargetName' %in% colnames(data))
  testthat::expect_true('validationOutcomeName' %in% colnames(data))
  testthat::expect_true('validationTimeAtRisk' %in% colnames(data))
  testthat::expect_true('timeStamp' %in% colnames(data))
  testthat::expect_true('auroc' %in% colnames(data))
  testthat::expect_true('auroc95lb' %in% colnames(data))
  testthat::expect_true('auroc95ub' %in% colnames(data))
  testthat::expect_true('calibrationInLarge' %in% colnames(data))
  testthat::expect_true('eStatistic' %in% colnames(data))
  testthat::expect_true('brierScore' %in% colnames(data))
  testthat::expect_true('auprc' %in% colnames(data))
  testthat::expect_true('populationSize' %in% colnames(data))
  testthat::expect_true('outcomeCount' %in% colnames(data))
  testthat::expect_true('evalPercent' %in% colnames(data))
  testthat::expect_true('outcomePercent' %in% colnames(data))
  testthat::expect_true(any(!is.na(data$algorithmName)))
  
  data <- getPredictionPerformances(
    connectionHandler = connectionHandler, 
    schema = schema, 
    modelDesignId = 1
  )
  
  testthat::expect_true(nrow(data) > 0)
  testthat::expect_true(max(data$modelDesignId) == 1)
  
})

test_that("getFullPredictionPerformances", {
  
  data <- getFullPredictionPerformances(
    connectionHandler = connectionHandler, 
    schema = schema
  )
  
  testthat::expect_true(nrow(data) > 0)
  
  testthat::expect_true('performanceId' %in% colnames(data))
  testthat::expect_true('modelDesignId' %in% colnames(data))
  testthat::expect_true('algorithmName' %in% colnames(data))
  testthat::expect_true('developmentTargetName' %in% colnames(data))
  testthat::expect_true('developmentOutcomeName' %in% colnames(data))
  testthat::expect_true('validationTargetName' %in% colnames(data))
  testthat::expect_true('validationOutcomeName' %in% colnames(data))
  testthat::expect_true('validationTimeAtRisk' %in% colnames(data))
  testthat::expect_true('timeStamp' %in% colnames(data))
  testthat::expect_true('modelType' %in% colnames(data))
  testthat::expect_true('evaluation' %in% colnames(data))
  testthat::expect_true('AUROC' %in% colnames(data))
  testthat::expect_true('95% lower AUROC' %in% colnames(data))
  testthat::expect_true('95% upper AUROC' %in% colnames(data))
  testthat::expect_true('calibrationInLarge mean prediction' %in% colnames(data))
  testthat::expect_true('Eavg' %in% colnames(data))
  testthat::expect_true('brier score' %in% colnames(data))
  testthat::expect_true('AUPRC' %in% colnames(data))
  testthat::expect_true('populationSize' %in% colnames(data))
  testthat::expect_true('outcomeCount' %in% colnames(data))
  testthat::expect_true(any(!is.na(data$algorithmName)))
  
  data <- getFullPredictionPerformances(
    connectionHandler = connectionHandler, 
    schema = schema, 
    modelDesignId = 1
  )
  
  testthat::expect_true(nrow(data) > 0)
  testthat::expect_true(max(data$modelDesignId) == 1)
  
})

test_that("getPredictionPerformances", {
  
  data <- getPredictionDiagnostics(
    connectionHandler = connectionHandler, 
    schema = schema
  )
  
  testthat::expect_true(nrow(data) > 0)
  
  testthat::expect_true('diagnosticId' %in% colnames(data))
  testthat::expect_true('modelDesignId' %in% colnames(data))
  testthat::expect_true('developmentDatabaseName' %in% colnames(data))
  testthat::expect_true('developmentTargetName' %in% colnames(data))
  testthat::expect_true('developmentOutcomeName' %in% colnames(data))
  testthat::expect_true('probastId' %in% colnames(data))
  testthat::expect_true('probastDescription' %in% colnames(data))

  data <- getPredictionDiagnostics(
    connectionHandler = connectionHandler, 
    schema = schema, 
    modelDesignId = 1
  )
  
  testthat::expect_true(nrow(data) > 0)
  testthat::expect_true(max(data$modelDesignId) == 1)
  
})


test_that("getPredictionPerformances", {
  
  data <- getPredictionPerformanceTable(
    connectionHandler = connectionHandler, 
    schema = schema, 
    table = 'evaluation_statistics', 
    performanceId = 1
  )
  
  testthat::expect_true(nrow(data) > 0)
  
  testthat::expect_true('performanceId' %in% colnames(data))
  testthat::expect_true('evaluation' %in% colnames(data))
  testthat::expect_true('metric' %in% colnames(data))
  testthat::expect_true('value' %in% colnames(data))
  testthat::expect_true(max(data$performanceId) == 1)
  
})

test_that("getPredictionDiagnosticTable", {
  
  data <- getPredictionDiagnosticTable(
    connectionHandler = connectionHandler, 
    schema = schema, 
    table = 'diagnostic_participants', 
    diagnosticId = 1
  )
  
  testthat::expect_true(nrow(data) > 0)
  
  testthat::expect_true('diagnosticId' %in% colnames(data))
  testthat::expect_true('metric' %in% colnames(data))
  testthat::expect_true('value' %in% colnames(data))
  testthat::expect_true('probastId' %in% colnames(data))
  testthat::expect_true('parameter' %in% colnames(data))
  testthat::expect_true('paramvalue' %in% colnames(data))
  
  testthat::expect_true(max(data$diagnosticId) == 1)
  
})

test_that("getPredictionHyperParamSearch", {
  
  data <- getPredictionHyperParamSearch(
    connectionHandler = connectionHandler, 
    schema = schema, 
    modelDesignId = 1,
    databaseId = 1
  )
  
  testthat::expect_true(nrow(data) > 0)
  
  # Does this differ by classifier?
  testthat::expect_true('metric' %in% colnames(data))
  testthat::expect_true('fold' %in% colnames(data))
  testthat::expect_true('value' %in% colnames(data))
  
})


test_that("getPredictionIntercept", {
  
  data <- getPredictionIntercept(
    connectionHandler = connectionHandler, 
    schema = schema, 
    modelDesignId = 1,
    databaseId = 1
  )
  
  testthat::expect_true(is.numeric(data))
  
})



test_that("getPredictionCovariates ", {
  
  alldata <- getPredictionCovariates (
    connectionHandler = connectionHandler, 
    schema = schema
  )
  
  testthat::expect_true(nrow(alldata) > 0)
  
  filterdata <- getPredictionCovariates (
    connectionHandler = connectionHandler, 
    schema = schema, 
    modelDesignId = 1,
    performanceIds = NULL
  )
  testthat::expect_true(unique(filterdata$modelDesignId) == 1)
  
  filterdata <- getPredictionCovariates (
    connectionHandler = connectionHandler, 
    schema = schema, 
    modelDesignId = NULL,
    performanceIds = 1
  )
  testthat::expect_true(unique(filterdata$performanceId) == 1)
  
})


test_that("addPredictionTimeAtRisk works ", {
  
dres <- data.frame(
  test = 1:4,
  tarStartAnchor = c('start', 'end', 'start', 'start'),
  tarStartDay =  rep(0,4),
  tarEndAnchor = c('start', 'end', 'end', 'end'),
  tarEndDay = rep(30,4)
)
res <- addPredictionTimeAtRisk(
  result = dres, 
  tarColumnName = 'newtar'
  )

testthat::expect_true('newtar' %in% colnames(res))
testthat::expect_true(!'tarStartAnchor' %in% colnames(res))
testthat::expect_true(!'tarStartDay' %in% colnames(res))
testthat::expect_true(!'tarEndAnchor' %in% colnames(res))
testthat::expect_true(!'tarEndDay' %in% colnames(res))

# check tar 1 is correct
testthat::expect_true(res$newtar[1] == '(start + 0) - (start + 30)')


dres <- data.frame(
  test = 1:4,
  rTarStartAnchor = c('start', 'end', 'start', 'start'),
  rTarStartDay =  rep(0,4),
  rTarEndAnchor = c('start', 'end', 'end', 'end'),
  rTarEndDay = rep(30,4)
)
res <- addPredictionTimeAtRisk(
  result = dres, 
  tarColumnName = 'tar', 
  tarStartAnchor = 'rTarStartAnchor', 
  tarStartDay = 'rTarStartDay', 
  tarEndAnchor = 'rTarEndAnchor', 
  tarEndDay = 'rTarEndDay', 
  removeIndividualTarColumns = FALSE
  )

testthat::expect_true('tar' %in% colnames(res))
testthat::expect_true('rTarStartAnchor' %in% colnames(res))
testthat::expect_true('rTarStartDay' %in% colnames(res))
testthat::expect_true('rTarEndAnchor' %in% colnames(res))
testthat::expect_true('rTarEndDay' %in% colnames(res))

})

#' createPredictionReport
#'
#' @description
#' Generates a report for a given prediction model design
#'
#' @details
#' Specify the connection handler to the result database, the schema name
#' and the modelDesignId of interest to generate a html report summarizing the
#' performance of models developed across databases.
#' 
#' @param connectionHandler The connection handler to the results database
#' @param schema The result database schema
#' @param plpTablePrefix The prediction table prefix
#' @param databaseTablePrefix The database table name e.g., database_meta_data
#' @param cgTablePrefix The cohort generator table prefix
#' @param modelDesignId The model design ID of interest
#' @param output The folder name where main.html will be save to
#' @param intermediatesDir The work directory for rmarkdown
#' @param outputFormat the type of outcome html_document or html_fragment
#' 
#' @return
#' An named R list with the elements 'standard' and 'source'
#' 
#' @family Reporting
#'
#' @export
#' 
createPredictionReport <- function(
    connectionHandler, 
    schema,
    plpTablePrefix,
    databaseTablePrefix = plpTablePrefix,
    cgTablePrefix = plpTablePrefix,
    modelDesignId,
    output,
    intermediatesDir = file.path(tempdir(), 'plp-prot'),
    outputFormat = "html_document" # NULL
){
  
  protocolLoc <- system.file(
    'templates',
    'patient-level-prediction-document', 
    "main.Rmd", 
    package = "OhdsiReportGenerator"
    )
  
  if(!dir.exists(intermediatesDir)){
    dir.create(intermediatesDir)
  }
  
  rmarkdown::render(
    output_format = outputFormat,
    input = protocolLoc, 
    intermediates_dir = intermediatesDir,
    output_dir = output, 
    params = list(
      connectionHandler = connectionHandler,
      resultSchema = schema, 
      myTableAppend = plpTablePrefix,
      modelDesignIds = modelDesignId,
      databaseTableAppend = databaseTablePrefix,
      cohortTableAppend = cgTablePrefix
    )
  )
  
  
}

test_that("create prediction report", {
  
  skip('skip prediction report due to odd linux error')
  
  testthat::expect_true(!file.exists(file.path(tempdir(), 'main.html')))
  
  createPredictionReport(
    connectionHandler, 
    schema = schema,
    plpTablePrefix = 'plp_',
    modelDesignId = 1,
    output = tempdir(),
    intermediatesDir = file.path(tempdir(), 'plp-prot'), 
    outputFormat = "html_document"
  )
  
  testthat::expect_true(file.exists(file.path(tempdir(), 'main.html')))
  # now remove the file
  file.remove(file.path(tempdir(), 'main.html'))
  
})

#' Extracts the different analyses ran for each target and event cohorts
#' @description
#' This function extracts analysis ids, events cohorts, and databases per target from treatment patterns
#'
#' @details
#' Specify the connectionHandler and the schema
#'
#' @template connectionHandler
#' @template schema
#' @template tpTablePrefix
#' @family TreatmentPatterns
#' @return
#' Returns a data.frame with the columns:
#' \itemize{
#'  \item{databaseName a concatinated string of all the database names ran for that analysis}
#'  \item{databaseId a concatinated string of all the database ids ran for that analysis}
#'  \item{analysisId the analysis ids for the treament patterns run}
#'  \item{targetCohortName the target cohort name}
#'  \item{targetCohortId the target cohort unique identifier}
#'  \item{eventCohortList a concatinated string of all the event cohort names ran for that target}
#'  \item{exitCohortList a concatinated string of all the exit cohort names ran for that target}
#'  }
#'
#' @export
#'
#' @examples
#' conDet <- getExampleConnectionDetails()
#'
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#'
#' cohortAnalysis <- getAnalysisCohorts(
#'   connectionHandler = connectionHandler,
#'   schema = "main"
#' )
getAnalysisCohorts <- function(
  connectionHandler,
  schema,
  tpTablePrefix = "tp_"
) {
  sql <- "SELECT
      d.CDM_SOURCE_ABBREVIATION AS database_name,
      d.database_id AS database_id,
      t.analysis_id,
      t.cohort_name,
      t.cohort_id,
      t.type
    FROM @schema.@tp_table_prefixanalysis_cohorts t
    Inner JOIN @schema.@tp_table_prefixcdm_source_info d
    ON t.analysis_id = d.analysis_id"

  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    tp_table_prefix = tpTablePrefix,
  ) 

  targets <- result %>%
    dplyr::filter(.data$type == "target") %>%
    dplyr::rename(targetCohortId = "cohortId",
                  targetCohortName = "cohortName") %>%
    dplyr::select("analysisId", "targetCohortId", "targetCohortName") %>%
    dplyr::distinct()
  
  events <- result %>%
    dplyr::filter(.data$type == "event") %>%
    dplyr::select("analysisId", "cohortId", "cohortName") %>%
    dplyr::distinct() %>%
    dplyr::group_by(.data$analysisId) %>%
    dplyr::summarise(eventCohortList = paste(.data$cohortName, collapse = ", "), .groups = "drop")
  
  exits <- result %>%
    dplyr::filter(.data$type == "exit") %>%
    dplyr::select("analysisId", "cohortId", "cohortName") %>%
    dplyr::distinct() %>%
    dplyr::group_by(.data$analysisId) %>%
    dplyr::summarise(exitCohortList = paste(.data$cohortName, collapse = ", "), .groups = "drop")
  
  databases <- result %>%
    dplyr::select("analysisId", "databaseId", "databaseName") %>%
    dplyr::distinct() %>%
    dplyr::group_by(.data$analysisId) %>%
    dplyr::summarise(
      databaseId = paste(.data$databaseId, collapse = ", "),
      databaseName = paste(.data$databaseName, collapse = ", "),
      .groups = "drop"
    )
  
  final <- targets %>%
    dplyr::left_join(events, by = "analysisId") %>%
    dplyr::left_join(exits, by = "analysisId") %>%
    dplyr::left_join(databases, by = "analysisId")
  
  return(final)
}


#' Extracts treatment pathways
#' @description
#' This function extracts results pathways for specified analysis ids and target cohorts
#'
#' @details
#' Specify the connectionHandler and the schema
#'
#' @template connectionHandler
#' @template schema
#' @template tpTablePrefix
#' @template databaseTable
#' @param age (optional) a string representing an age bucket to restrict (e.g., "0-17", "18-34", "65+")
#' @param sex (optional) A string "male" or "female" to restrict
#' @param indexYear (optional) A string with a four-digit year to restrict
#' @param analysisIds (optional) A vector of analysis ids to restrict to
#' @param databaseIds (optional) A vector of database ids to restrict to
#' @param databaseNames (optional) A vector of database Names to restrict to
#' @param targetIds (optional) A vector of target cohort ids to restrict to
#' @family TreatmentPatterns
#' @return
#' Returns a data.frame with the columns:
#' \itemize{
#'  \item{databaseName the name of the database}
#'  \item{databaseId the unique identifier of the database}
#'  \item{analysisId the unique identifier of a treament patterns run}
#'  \item{targetCohortName the target cohort name}
#'  \item{targetCohortId the target cohort unique identifier}
#'  \item{pathway a string representing the progression of events for a target. Use '-' to separate sequential steps and '+' for combination of events at that step}
#'  \item{freq the count of pathway occurance}
#'  \item{age the stratifyed pathways for age}
#'  \item{indexYear the stratifyed pathways for index year}
#'  \item{sex the stratifyed pathways for sex}

#'  }
#'
#' @export
#'
#' @examples
#' conDet <- getExampleConnectionDetails()
#'
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#'
#' tp <- getTreatmentPathways(
#'   connectionHandler = connectionHandler,
#'   schema = "main"
#' )
getTreatmentPathways <- function(
  connectionHandler,
  schema,
  tpTablePrefix = "tp_",
  databaseTable = "database_meta_data",
  age = "all",
  sex = "all",
  indexYear = "all",
  analysisIds = NULL,
  databaseIds = NULL,
  databaseNames = NULL,
  targetIds = NULL
) {
  sql <- "SELECT
    d.CDM_SOURCE_ABBREVIATION AS database_name,
    t.database_id,
    t.analysis_id,
    t.target_cohort_name,
    t.target_cohort_id,
    t.freq,
    t.pathway,
    t.age,
    t.index_year,
    t.sex
  FROM @schema.@tp_table_prefixtreatment_pathways t
  INNER JOIN
  @schema.@database_table d
  ON t.database_id = d.database_id 
  WHERE
    t.age = @age AND 
    t.index_year = @index_year AND
    t.sex = @sex
    {@use_targets}?{AND t.target_cohort_id IN (@target_ids) }
    {@use_database_id}?{AND t.database_id IN (@database_id) }
    {@use_analysis}?{and t.analysis_id IN (@analysis_ids)}
    {@use_database_name}?{and d.CDM_SOURCE_ABBREVIATION IN (@database_name)}
  "

  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    tp_table_prefix = tpTablePrefix,
    database_table = databaseTable,
    use_targets = !is.null(targetIds),
    target_ids = paste0(targetIds, collapse = ","),
    use_analysis = !is.null(analysisIds),
    analysis_ids = paste0(analysisIds, collapse = ","),
    use_database_id = !is.null(databaseIds),
    database_id = paste0("'", databaseIds, "'", collapse = ","),
    use_database_name = !is.null(databaseNames),
    database_name = paste0("'", databaseNames, "'", collapse = ","),
    age = paste0("'", age, "'"),
    sex = paste0("'", sex, "'"),
    index_year = paste0("'", indexYear, "'")
  )

  return(result)
}

#' Extracts summary of event duration
#' @description
#' This function extracts results summary stats of event duration for specified analysis ids and target cohorts
#'
#' @details
#' Specify the connectionHandler, the schema, and the analysisIds
#'
#' @template connectionHandler
#' @template schema
#' @template tpTablePrefix
#' @template databaseTable
#' @param analysisIds A vector of analysis ids to restrict to
#' @param databaseIds (optional) A vector of database ids to restrict to
#' @param databaseNames (optional) A vector of database Names to restrict to
#' @param targetIds (optional) A vector of target cohort ids to restrict to
#' @family TreatmentPatterns
#' @return
#' Returns a data.frame with the columns:
#' \itemize{
#'  \item{databaseName the name of the database}
#'  \item{databaseId the unique identifier of the database}
#'  \item{analysisId the unique identifier of a treament patterns run}
#'  \item{targetCohortId the target cohort unique identifier}
#'  \item{targetCohortName the target cohort name}
#'  \item{eventName a string representing an events for a target. Uses '+' for combination of event cohorts}
#'  \item{rank the step number of event occurance}
#'  \item{eventCount the count of event occurance at rank}
#'  \item{durationAverage the average duration of event}
#'  \item{durationMax the maximum duration of event}
#'  \item{durationMin the minimum duration of event}
#'  \item{durationMedian the median duration of event}
#'  \item{p25Value the 25th percentile for duration of event}
#'  \item{p75Value the 75th percentile for duration of event}
#'  \item{standardDeviation the standard deviation for duration of event}
#'  }
#'
#' @export
#'
#' @examples
#' conDet <- getExampleConnectionDetails()
#'
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#'
#' ed <- getEventDuration(
#'   connectionHandler = connectionHandler,
#'   schema = "main",
#'   analysisIds = c(1)
#' )
getEventDuration <- function(
  connectionHandler,
  schema,
  analysisIds,
  tpTablePrefix = "tp_",
  databaseTable = "database_meta_data",
  databaseIds = NULL,
  databaseNames = NULL,
  targetIds = NULL
) {
  sql <- "SELECT
    d.CDM_SOURCE_ABBREVIATION AS database_name,
    t.database_id,
    t.analysis_id,
    t.target_cohort_id,
    t.target_cohort_name,
    t.event_name,
    t.line AS rank,
    t.event_count,
    t.duration_average,
    t.duration_max,
    t.duration_min,
    t.duration_median,
    t.duration_q_1 as p_25_value,
    t.duration_q_2 as p_75_value,
    t.duration_sd as standard_deviation
  FROM @schema.@tp_table_prefixsummary_event_duration t
  INNER JOIN
  @schema.@database_table d
  ON t.database_id = d.database_id 
  WHERE t.analysis_id IN (@analysis_ids)
  {@use_targets}?{AND t.target_cohort_id IN (@target_ids) }
  {@use_database_id}?{AND t.database_id IN (@database_id) }
  {@use_database_name}?{and d.CDM_SOURCE_ABBREVIATION IN (@database_name)}
  "

  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    tp_table_prefix = tpTablePrefix,
    database_table = databaseTable,
    use_targets = !is.null(targetIds),
    target_ids = paste0(targetIds, collapse = ","),
    analysis_ids = paste0(analysisIds, collapse = ","),
    use_database_id = !is.null(databaseIds),
    database_id = paste0("'", databaseIds, "'", collapse = ","),
    use_database_name = !is.null(databaseNames),
    database_name = paste0("'", databaseNames, "'", collapse = ","),
  )

  return(result)
}

test_that("cohorts used in treatment patterns analysis", {
  analysisCohorts <- getAnalysisCohorts(
    connectionHandler = connectionHandler,
    schema = "main"
  )
  
  testthat::expect_true("databaseName" %in% colnames(analysisCohorts))
  testthat::expect_true("databaseId" %in% colnames(analysisCohorts))
  testthat::expect_true("analysisId" %in% colnames(analysisCohorts))
  testthat::expect_true("targetCohortName" %in% colnames(analysisCohorts))
  testthat::expect_true("targetCohortId" %in% colnames(analysisCohorts))
  testthat::expect_true("eventCohortList" %in% colnames(analysisCohorts))
  testthat::expect_true("exitCohortList" %in% colnames(analysisCohorts))
})

test_that("get pathways", {
  pathways <- getTreatmentPathways(
    connectionHandler = connectionHandler,
    schema = "main",
    analysisIds = c(2),
    targetIds = c(11),
    databaseIds = c("388020256"),
    databaseNames = c("Synthea")
  )

  testthat::expect_true("databaseName" %in% colnames(pathways))
  testthat::expect_true("databaseId" %in% colnames(pathways))
  testthat::expect_true("analysisId" %in% colnames(pathways))
  testthat::expect_true("targetCohortName" %in% colnames(pathways))
  testthat::expect_true("targetCohortId" %in% colnames(pathways))
  testthat::expect_true("pathway" %in% colnames(pathways))
  testthat::expect_true("freq" %in% colnames(pathways))
  testthat::expect_true("age" %in% colnames(pathways))
  testthat::expect_true("indexYear" %in% colnames(pathways))
  testthat::expect_true("sex" %in% colnames(pathways))
  testthat::expect_true(all(pathways$analysisId == 2))
})

test_that("get event duration", {
  eventDuration <- getEventDuration(
    connectionHandler = connectionHandler,
    schema = "main",
    analysisIds = c(1, 2)
  )


  testthat::expect_true("databaseName" %in% colnames(eventDuration))
  testthat::expect_true("databaseId" %in% colnames(eventDuration))
  testthat::expect_true("analysisId" %in% colnames(eventDuration))
  testthat::expect_true("targetCohortName" %in% colnames(eventDuration))
  testthat::expect_true("targetCohortId" %in% colnames(eventDuration))
  testthat::expect_true("eventName" %in% colnames(eventDuration))
  testthat::expect_true("rank" %in% colnames(eventDuration))
  testthat::expect_true("eventCount" %in% colnames(eventDuration))
  testthat::expect_true("durationAverage" %in% colnames(eventDuration))
  testthat::expect_true("durationMax" %in% colnames(eventDuration))
  testthat::expect_true("durationMin" %in% colnames(eventDuration))
  testthat::expect_true("durationMedian" %in% colnames(eventDuration))
  testthat::expect_true("p25Value" %in% colnames(eventDuration))
  testthat::expect_true("p75Value" %in% colnames(eventDuration))
  testthat::expect_true("standardDeviation" %in% colnames(eventDuration))
})

#' generateSummaryPredictionReport
#'
#' @description
#' Generates a summary report for a given targets and outcomes
#'
#' @details
#' Specify the connection handler to the result database, the schema name
#' and the cohortId of interest to generate a html report summarizing the
#' performance of prediction models in the database.
#' 
#' @param connectionHandler The connection handler to the results database
#' @param schema The result database schema
#' @param targetIds The target cohort IDs of interest
#' @param outcomeIds The outcome cohort IDs of interest
#' @param plpTablePrefix The prediction table prefix
#' @param databaseTablePrefix The database table name e.g., database_meta_data
#' @param cgTablePrefix The cohort generator table prefix
#' @param outputFolder The folder name where file will be save to
#' @param outputFileName The file name of the saved report
#' @param intermediatesDir The work directory for rmarkdown
#' @param overwrite whether to overwrite any existing file at the outputFolder/outputFileName
#' 
#' @return
#' A html file is created with the summary report
#' 
#' @family Reporting
#'
#' @export
#' 
generateSummaryPredictionReport <- function(
    connectionHandler,
    schema,
    targetIds = NULL,
    outcomeIds = NULL,
    plpTablePrefix = 'plp_',
    databaseTablePrefix = '',
    cgTablePrefix = 'cg_',
    outputFolder,
    outputFileName = 'plp-summary.html',
    intermediatesDir = file.path(tempdir(), 'plp-prot'),
    overwrite = FALSE
){
  
  # check ResultModelManager is installed as it is used
  # in quarto code
  rlang::check_installed("ResultModelManager")
  
  templateLoc <- system.file(
    'templates',
    'summary-templates', 
    package = "OhdsiReportGenerator"
  )
  
  if(!dir.exists(file.path(intermediatesDir,'prediction-summary','prediction-questions'))){
    dir.create(file.path(intermediatesDir,'prediction-summary','prediction-questions'), recursive = TRUE)
  }
  
  filesOfInt <- c(
    dir(templateLoc, pattern = '.qmd', recursive = TRUE)
  )
  file.copy(
    from = file.path(templateLoc, filesOfInt), 
    to = file.path(file.path(intermediatesDir, 'prediction-summary'), filesOfInt), 
    overwrite = TRUE
  )

  quarto::quarto_render(
    input = file.path(intermediatesDir,'prediction-summary', 'prediction-summary.qmd'), 
    execute_params = list(
      dbms = connectionHandler$connectionDetails$dbms,
      server = connectionHandler$connectionDetails$server(),
      username = connectionHandler$connectionDetails$user(),
      password = connectionHandler$connectionDetails$password(),
      schema = schema,
      targetIds = targetIds,
      outcomeIds = outcomeIds,
      plpTablePrefix = plpTablePrefix,
      databaseTablePrefix = databaseTablePrefix,
      cgTablePrefix = cgTablePrefix
    )
  )
  
  if(!dir.exists(outputFolder)){
    dir.create(outputFolder, recursive = TRUE)
  }
  file.copy(
    from = file.path(intermediatesDir, 'prediction-summary', 'prediction-summary.html'), 
    to = file.path(outputFolder, outputFileName), 
    overwrite = overwrite
  )
  
  
}

#' View the Incidence Rates
#' @description
#' Creates a table with the incidence rates and optionally demographics
#'
#' @details
#' Input the incidence rate data (and optionally demographic data)
#'
#' @param incidenceData The data extracted using 'getIncidenceRates'
#' @param ageData The data extracted using 'getBinaryTargetBaseline' with analysisIds = 3 
#' @param genderData The data extracted using 'getBinaryTargetBaseline' with analysisIds = 1
#' @param stratification Pick either overall/age/sex/year to specify whether to view the overall rates or stratified by age/sex/year
#' @param maxAgeSampleSize When creating the age distributions this is the max age vector length to create
#' 
#' @family Characterization
#' @return
#' Returns a gt table that displays the incidence rates
#' 
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' schema <- 'main'
#' 
#' incidenceData <- getIncidenceRates(
#'   connectionHandler = connectionHandler , 
#'   schema = schema
#'   )
#'   
#'   # incidence data does not have rate values to imputing them
#'   incidenceData$incidenceRateP100py <- 1 +
#'     sample(c(-1,1),replace = TRUE)*runif(nrow(incidenceData))
#'   incidenceData$incidenceProportionP100p <- 0.5 +
#'     sample(c(-1,1),replace = TRUE)*runif(nrow(incidenceData))
#'   
#'  ageData <- getBinaryTargetBaseline(
#'   connectionHandler = connectionHandler, 
#'   schema = schema,  
#'   analysisIds = 3
#'  )
#'  
#'  genderData <- getBinaryTargetBaseline(
#'   connectionHandler = connectionHandler, 
#'   schema = schema,  
#'   analysisIds = 1
#'  )
#' 
#' viewIncidenceRate(
#'   incidenceData = incidenceData,
#'   ageData = ageData,
#'   genderData = genderData
#'   )
#'   
#' 
viewIncidenceRate <- function(
  incidenceData,
  ageData = NULL, 
  genderData = NULL, 
  stratification = 'overall', 
  maxAgeSampleSize = 5000
  ){
  
  if(!stratification %in% c('overall', 'age', 'sex',' year')){
    message("Please use one of 'overall', 'age', 'sex',' year' for stratification")
    return(invisible(NULL))
  }
  
  x <- filterCiGroup(
    incidenceData = incidenceData, 
    stratification = stratification
    )
  
  addAge <- FALSE
  if(!is.null(ageData) & stratification == 'overall'){
    if(ageData$analysisId[1] ==3){
      addAge <- TRUE
    }
  }  
  
  if(addAge){
    
    # added below to sample sumValue if too large
    
    cohortSizes <- ageData %>%
      dplyr::group_by(
        .data$databaseId, .data$targetCohortId
      ) %>%
      dplyr::summarise(
        n = sum(.data$sumValue)
      ) %>%
      dplyr::rowwise() %>%
      dplyr::mutate(
        scale = dplyr::case_when(.data$n/!!maxAgeSampleSize < 1 ~ 1,
                                 .data$n/!!maxAgeSampleSize >= 1 ~ .data$n/!!maxAgeSampleSize,
                                 )
      ) %>%
      dplyr::select("databaseId", "targetCohortId", "scale")
    
    # scale mod then join to ageData below and do size = floor(.data$sumValue/.data$scale)
    
    ageData <- ageData %>%
      dplyr::left_join(cohortSizes, by = c("databaseId", "targetCohortId")) %>%
      dplyr::rowwise() %>%
      dplyr::mutate(
        databaseId = as.character(.data$databaseId),
        ageMin = strsplit(x = gsub('age group:  ', '', .data$covariateName), split = ' -  ')[[1]][1],
        ageMax = strsplit(x = gsub('age group:  ', '', .data$covariateName), split = ' -  ')[[1]][2],
      ) %>%
      dplyr::mutate(
        ageList = list(sample(.data$ageMin:.data$ageMax, size = ceiling(.data$sumValue/.data$scale), replace = TRUE))
      ) %>%
      dplyr::group_by(
        .data$databaseId, .data$targetCohortId
      ) %>%
      dplyr::summarise(
        ageList = list(do.call(what = 'c', c(.data$ageList)))
      ) %>%
      dplyr::rename(
        targetId = "targetCohortId"
      )
    
    x <- x %>%
      dplyr::left_join(
        y = ageData, 
        by = c("databaseId","targetId")
          )
  } else{
    x <- x %>%
      dplyr::mutate(
        ageList = list(0)
      )
  }


addGender <- FALSE
if(!is.null(genderData) & stratification == 'overall'){
  if(genderData$analysisId[1] == 1){
    addGender <- TRUE
  }
}  
  
  if(addGender){
  
      totals <- genderData %>%
      dplyr::group_by(.data$databaseId, .data$targetCohortId) %>%
      dplyr::summarise(
        total = sum(.data$sumValue)
      )
      
    genderData <- genderData %>%
      dplyr::filter(.data$covariateName == 'gender = MALE') %>%
      dplyr::left_join(totals, by = c('databaseId','targetCohortId')) %>%
      dplyr::mutate(
        databaseId = as.character(.data$databaseId),
        malePercentage = .data$sumValue/.data$total*100
      ) %>%
      dplyr::rename(
        targetId = "targetCohortId"
      ) %>%
      dplyr::select(
        "databaseId", "targetId", "malePercentage"
      )
      
    
    x <- x %>%
      dplyr::left_join(
        y = genderData, 
        by = c("databaseId","targetId")
      ) %>%
      dplyr::rename(
        malePerc = "malePercentage"
      )
  } else{
    x <- x %>%
      dplyr::mutate(
        malePerc = NA
      )
  }
  
  if(stratification == 'overall'){
    x <- x %>% 
      dplyr::mutate(
        type = ''
      )
  } else if(stratification == 'age'){
    x <- x %>% 
      dplyr::mutate(
        type = .data[['ageGroupName']]
      )
  } else if(stratification == 'year'){
    x <- x %>% 
      dplyr::mutate(
        type = .data[['startYear']]
      )
  } else if(stratification == 'sex'){
    x <- x %>% 
      dplyr::mutate(
        type = .data[['genderName']]
      )
  }
  
 nicetbl <- x %>%
    dplyr::mutate(
      analysis = paste0(.data$outcomeName, ' ( wo ',.data$cleanWindow ,') during ', .data$tar),
      targetNameWithId = paste(.data$targetName, ' (',.data$targetId ,')'),
      rate = .data$incidenceRateP100py,
      proportion = .data$incidenceProportionP100p,
      meanFollowUp = .data$personDays/.data$personsAtRisk
      ) %>%
    dplyr::select("targetNameWithId","analysis", "databaseName","rate","incidenceRateP100py",
                  "proportion","incidenceProportionP100p", "personsAtRisk" , "meanFollowUp",
                  "outcomes", "type", "ageList", "malePerc") %>%
    dplyr::arrange(.data$targetNameWithId,.data$type) %>%
    dplyr::group_by(.data$type, .data$analysis) |>
    gt::gt() %>%
    gtExtras::gt_plt_bar(
      column = "incidenceRateP100py",
      color = "MidnightBlue", 
    ) %>%
    gtExtras::gt_plt_bar(
      column = "incidenceProportionP100p",
      color = "MidnightBlue"
    ) %>%
    gtExtras::gt_plt_dist(
      column = "ageList",
      type = "density", 
      same_limit = TRUE 
     ) %>%
    gt::fmt_number(
      columns = 'malePerc', 
      decimals = 1
      ) %>%
    gt::fmt_number(
      columns = c('rate', 'proportion'), 
      decimals = 3
    ) %>%
   gt::fmt_number(
     columns = c('meanFollowUp'),
     sep_mark = ',',
     decimals = 0
   ) %>%
   gt::fmt(
     columns = c('personsAtRisk', 'outcomes'),
     fns = function(x){ 
       x <- sprintf("%.0f", x)
       if(sum(x < 0) > 0){ 
         x[x < 0] <- paste('<', abs(x[x < 0]))
       }
       return(x)
     }
   ) %>%
    gt::cols_label(
      incidenceProportionP100p =  gt::md("**Proportion per 100 persons**"),
      incidenceRateP100py =  gt::md("**Rate per 100 person years**"),
      databaseName = gt::md("**Database**"),
      rate = "",
      proportion = "",
      personsAtRisk = gt::md("**N**"),
      meanFollowUp = gt::md("**Mean Follow-up Days**"),
      outcomes = gt::md("**Outcomes**"),
      ageList = gt::md("**Age**"),
      malePerc = gt::md("**Male %**"),
      targetNameWithId = ""
    ) %>%
    gt::tab_style(
      style = gt::cell_fill(color = "GoldenRod"),
      locations = gt::cells_row_groups(groups = gt::everything()) # Targets specific group
    ) %>%
    gt::tab_style(
      style = gt::cell_text(color = "MidnightBlue", weight = "bold"),
      locations = gt::cells_body(columns = c("rate", "proportion"))
    )
  
  if(!addGender | !addAge){
    nicetbl <- nicetbl %>%
      gt::cols_hide(
        columns = c("ageList","malePerc")[c(!addAge, !addGender)]
          )
  }

  return(nicetbl)
}


filterCiGroup <- function(
    incidenceData,
    stratification
    ){
  
  noAge <- incidenceData$ageGroupName == 'Any'
  noGen <- incidenceData$genderName == 'Any'
  noYear <- incidenceData$startYear == 'Any'
  
  if(stratification == 'overall'){
    include <- (noAge & noGen & noYear) 
  } else if(stratification == 'age'){
    include <- (!noAge & noGen & noYear)
  } else if(stratification == 'sex'){
    include <- (noAge & !noGen & noYear)
  } else if(stratification == 'year'){
    include <- (noAge & noGen & !noYear)
  } else{
    warning('Invalid stratification')
    return(NULL)
  }
  
  return(incidenceData[include,])
}


#' Plots the age distributions using the binary age groups
#' @description
#' Creates bar charts for the target and case age groups.
#'
#' @details
#' Input the data returned from 'getCharacterizationDemographics(type = 'age')' and the time-at-risk
#'
#' @param ageData The age data extracted using 'getCharacterizationDemographics(type = 'age')'
#' @param riskWindowStart The time at risk window start
#' @param riskWindowEnd The time at risk window end
#' @param startAnchor The anchor for the time at risk start
#' @param endAnchor The anchor for the time at risk end
#' 
#' @family Characterization
#' @return
#' Returns a ggplot with the distributions
#' 
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' ageData <- getCharacterizationDemographics(
#' connectionHandler = connectionHandler, 
#' schema = 'main',
#' targetId = 1, 
#' outcomeId = 3, 
#' type = 'age'
#' )
#' 
#' plotAgeDistributions(ageData = ageData)
#' 
plotAgeDistributions <- function(
    ageData,
    riskWindowStart = '1',
    riskWindowEnd = '365', 
    startAnchor = 'cohort start', 
    endAnchor = 'cohort start'
){
  # TODO add input checks
  
  ageData <- ageData %>%
    dplyr::filter(
      .data$riskWindowStart %in% c(NA,riskWindowStart) &
      .data$riskWindowEnd %in% c(NA,riskWindowEnd) &
        .data$startAnchor %in% c(NA,startAnchor) &
        .data$endAnchor %in% c(NA, endAnchor)
      )
  
  if(nrow(ageData) == 0){
    return() # empty plot?
  } 
  # TODO add input checks
  
  
  # filter to Target and Cases and remove censored
  ageData <- rbind(
    ageData %>% 
      dplyr::filter(.data$caseCount > 0) %>%
      dplyr::select("covariateName","caseAverage", 
                    "startAnchor","endAnchor", 
                    "riskWindowStart","riskWindowEnd",
                    "databaseName",
                    "targetName", "outcomeName") %>%
      dplyr::rename(averageValue = "caseAverage") %>%
      dplyr::mutate(cohortType = 'Cases'),
    ageData %>% 
      dplyr::filter(.data$nonCaseCount > 0) %>%
      dplyr::select("covariateName","nonCaseAverage",
                    "startAnchor","endAnchor", 
                    "riskWindowStart","riskWindowEnd",
                    "databaseName",
                    "targetName", "outcomeName") %>%
      dplyr::rename(averageValue = "nonCaseAverage") %>%
      dplyr::mutate(
        averageValue = -1*.data$averageValue,
        cohortType = 'Non-cases')
  )
  
ageData$tar <- addTar(ageData)

ageData$averageValue <- as.double(ageData$averageValue)

# order the age group
covNames <- unique(ageData$covariateName)
covOrder <- as.double(unlist(lapply(strsplit(covNames, '-  '), function(x) x[2])))
ageData$covariateName <- factor(
  x = ageData$covariateName, 
  levels = covNames[order(covOrder)]
)

result <- ggplot2::ggplot(
  data = ageData,
  ggplot2::aes(
    x = .data$averageValue,
    y = .data$covariateName,
    fill = formatCohortType(.data$cohortType)
  )
) +
  ggplot2::geom_col() +
  ggplot2::scale_x_continuous(
    breaks  = c(-1,-0.5, 0, 0.5, 1),
    labels = abs(c(-1,-0.5, 0, 0.5,  1))
  ) +
  ggplot2::facet_grid(
    cols = ggplot2::vars(.data$databaseName), 
    rows = ggplot2::vars(.data$targetName, .data$outcomeName)
  ) +
  ggplot2::theme(
    legend.title=ggplot2::element_blank()
    ) +
  ggplot2::labs(
    y = "Variable", 
    x = "Frequency"
  )

return(result)
}

#' Plots the sex distributions using the sex features
#' @description
#' Creates bar charts for the target and case sex.
#'
#' @details
#' Input the data returned from 'getCharacterizationDemographics(type = 'sex')' and the time-at-risk
#'
#' @param sexData The sex data extracted using 'getCharacterizationDemographics(type = 'sex')'
#' @param riskWindowStart The time at risk window start
#' @param riskWindowEnd The time at risk window end
#' @param startAnchor The anchor for the time at risk start
#' @param endAnchor The anchor for the time at risk end
#' 
#' @family Characterization
#' @return
#' Returns a ggplot with the distributions
#' 
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' sexData <- getCharacterizationDemographics(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main',
#'   targetId = 1, 
#'   outcomeId = 3, 
#'   type = 'sex'
#' )
#' plotSexDistributions(sexData = sexData)
#' 
plotSexDistributions <- function(
    sexData,
    riskWindowStart = '1',
    riskWindowEnd = '365', 
    startAnchor = 'cohort start', 
    endAnchor = 'cohort start'
    ){
  # TODO add input checks
  
  sexData <- sexData %>% 
    dplyr::filter(
      .data$riskWindowStart %in% c(NA,riskWindowStart) &
      .data$riskWindowEnd %in% c(NA,riskWindowEnd) &
      .data$startAnchor %in% c(NA,startAnchor) &
      .data$endAnchor %in% c(NA, endAnchor)
  )
  
  if(nrow(sexData) == 0){
    return() # empty plot?
  } 
  
  # filter to Target and Cases and remove censored
  sexData <- rbind(
    sexData %>% 
      dplyr::filter(.data$caseCount > 0) %>%
      dplyr::select("covariateName","caseAverage", 
                    "startAnchor","endAnchor", 
                    "riskWindowStart","riskWindowEnd",
                    "databaseName", 
                    "targetName", "outcomeName") %>%
      dplyr::rename(averageValue = "caseAverage") %>%
      dplyr::mutate(cohortType = 'Cases'),
    sexData %>% 
      dplyr::filter(.data$nonCaseCount > 0) %>%
      dplyr::select("covariateName","nonCaseAverage",
                    "startAnchor","endAnchor", 
                    "riskWindowStart","riskWindowEnd",
                    "databaseName",
                    "targetName", "outcomeName") %>%
      dplyr::rename(averageValue = "nonCaseAverage") %>%
      dplyr::mutate(
        averageValue = -1*.data$averageValue,
        cohortType = 'Non-cases')
  )
  sexData$averageValue <- as.double(sexData$averageValue)
  
  sexData$tar <- addTar(sexData)
  
  result <- ggplot2::ggplot(
    data = sexData,
    ggplot2::aes(
      x = .data$averageValue,
      y = .data$covariateName,
      fill =  formatCohortType(.data$cohortType)
    )
  ) +
    ggplot2::geom_col() +
    ggplot2::scale_x_continuous(
      breaks  = c(-1,-0.5, 0, 0.5, 1),
      labels = abs(c(-1,-0.5, 0, 0.5,  1))
    ) +
    ggplot2::facet_grid(
      cols = ggplot2::vars(.data$databaseName),
      rows = ggplot2::vars(.data$targetName, .data$outcomeName)
    ) +
    ggplot2::theme(
      legend.title=ggplot2::element_blank()
      ) +
    ggplot2::labs(
      y = "Variable", 
      x = "Frequency"
    )
  
  return(result)
}

# create connection handler for tests to example result database
conDet <- OhdsiReportGenerator:::getExampleConnectionDetails()
connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

schema <- 'main'

# CRAN failed when find_quarto() errored, so only run quarto if this
# does not fail
quartoInstalled <- tryCatch({quarto:::find_quarto()}, error = function(e) return(""))
skipQuarto <- quartoInstalled == ""

#' Extract the target cohorts and where they are used in the analyses.
#' @description
#' This function extracts the target cohorts, the number of subjects/entries and where the cohort was used.
#'
#' @details
#' Specify the connectionHandler, the schema and the table prefixes
#'
#' @template connectionHandler
#' @template schema
#' @template cgTablePrefix
#' @template cTablePrefix
#' @template ciTablePrefix
#' @template cmTablePrefix
#' @template sccsTablePrefix
#' @template plpTablePrefix
#' @template databaseTable
#' @param getIncidenceInclusion Whether to check useage of the cohort in incidence
#' @param getCharacterizationInclusion Whether to check useage of the cohort in characterization
#' @param getPredictionInclusion Whether to check useage of the cohort in prediction
#' @param getCohortMethodInclusion Whether to check useage of the cohort in cohort method
#' @param getSccsInclusion Whether to check useage of the cohort in SCCS
#' @param printTimes Whether to print how long each query took 
#' @family helper
#' @return
#' Returns a data.frame with the columns: 
#' \itemize{
#'  \item{cohortId the number id for the target cohort}
#'  \item{cohortName the name of the cohort}
#'  \item{subsetParent the number id of the parent cohort}
#'  \item{subsetDefinitionId the number id of the subset}
#'  \item{subsetDefinitionJson the json of the subset}
#'  \item{subsetCohortIds the ids of any cohorts that are restricted to by the subset logic}
#'  \item{numDatabase number of databases with the cohort}
#'  \item{databaseString all the names of the databases with the cohort}
#'  \item{databaseCount all the names of the databases with the cohort and their sizes}
#'  \item{minSubjectCount number of subjects in databases with lowest count}
#'  \item{maxSubjectCount number of subjects in databases with highest count}
#'  \item{minEntryCount number of entries in databases with lowest count}
#'  \item{maxEntryCount number of entries in databases with highest count}
#'  \item{cohortIncidence whether the cohort was used in cohort incidence}
#'  \item{databaseComparator whether the cohort was used in database comparator}
#'  \item{cohortComparator whether the cohort was used in cohort comparator}
#'  \item{dechalRechal whether the cohort was used in dechallenge rechallenge}
#'  \item{riskFactors whether the cohort was used in risk factors}
#'  \item{caseSeries whether the cohort was used in case series analysis}
#'  \item{timeToEvent whether the cohort was used in time to event}
#'  \item{prediction whether the cohort was used in prediction}
#'  \item{cohortMethod whether the cohort was used in cohort method}
#'  \item{selfControlledCaseSeries whether the cohort was used in self controlled case series}
#' }
#'
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' targetTable <- getTargetTable(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getTargetTable <- function(
  connectionHandler, 
  schema,
  cgTablePrefix = 'cg_',
  cTablePrefix = 'c_',
  ciTablePrefix = 'ci_',
  cmTablePrefix = 'cm_',
  sccsTablePrefix = 'sccs_',
  plpTablePrefix = 'plp_',
  databaseTable = 'database_meta_data',
  getIncidenceInclusion = TRUE,
  getCharacterizationInclusion = TRUE,
  getPredictionInclusion = TRUE,
  getCohortMethodInclusion = TRUE,
  getSccsInclusion = TRUE,
  printTimes = FALSE
){
  
  start <- Sys.time()
  firstStart <- start
  
  cohorts <- getCohortDefinitions(
    connectionHandler = connectionHandler,
    schema = schema
  ) %>%
    dplyr::select("cohortDefinitionId", "cohortName", "subsetParent", "subsetDefinitionId", "subsetDefinitionJson") %>%
    dplyr::rename(cohortId = "cohortDefinitionId")
  
  end <- Sys.time()
  if(printTimes){
    print(paste0('extracting target cohorts: ', (end-start), ' ', units((end-start))))
  }
  start <- Sys.time()
  
  # TODO: process subsetDefinitionJson
  cohorts$subsetCohortId <- as.double(unlist(lapply(cohorts$subsetDefinitionJson, function(json){extractSubsetCohorts(json)})))
  
  # find parents names
  parents <- cohorts %>% 
    dplyr::filter(.data$cohortId == .data$subsetParent) %>%
    dplyr::select("cohortId", "cohortName") %>%
    dplyr::rename(
      parentName = "cohortName",
      subsetParent = "cohortId"
    )
  # add the parent name to cohorts
  cohorts <- merge(cohorts, parents, by = 'subsetParent', all.x = TRUE)
  
  end <- Sys.time()
  if(printTimes){
    print(paste0('processing target cohorts: ', (end-start), ' ', units((end-start))))
  }
  start <- Sys.time()
  
  sql <- "select 
  dt.cdm_source_abbreviation as database_name,
  cc.*
  from @schema.@cg_prefixcohort_count cc inner join
  @schema.@database_table dt on cc.database_id = dt.database_id;"
  counts <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    cg_prefix = cgTablePrefix,
    database_table = databaseTable
    ) %>% 
    dplyr::group_by(.data$cohortId) %>%
    dplyr::summarise(
      numDatabase = length(unique(.data$databaseName)),
      databaseString = paste0(unique(.data$databaseName), collapse = ', '),
      databaseIdString = paste0(unique(.data$databaseId), collapse = ', '),
      databaseStringCount = paste0(unique(paste0(.data$databaseName,' (', .data$cohortSubjects ,')')), collapse = ', '),
      minSubjectCount = min(.data$cohortSubjects, na.rm = TRUE),
      maxSubjectCount = max(.data$cohortSubjects, na.rm = TRUE),
      minEntryCount = min(.data$cohortEntries, na.rm = TRUE),
      maxEntryCount = max(.data$cohortEntries, na.rm = TRUE)
    )
  
  cohortCounts <- merge(cohorts, counts, by = 'cohortId')
  
  end <- Sys.time()
  if(printTimes){
    print(paste0('extracting target cohort counts: ', (end-start), ' ', units((end-start))))
  }
  start <- Sys.time()
  
  # now find whether it is a target for each analysis
  
  if(getIncidenceInclusion){
    inc <- tryCatch(getIncidenceTargets(
      connectionHandler = connectionHandler,
      schema = schema,
      cgTablePrefix = cgTablePrefix,
      ciTablePrefix = ciTablePrefix
    ), error = function(e){return(NULL)})
    if(!is.null(inc)){
      cohortCounts <- merge(
        x = cohortCounts, 
        y = inc, 
        by.x = c('cohortId','cohortName'),
        by.y = c('cohortDefinitionId','cohortName'),
        all.x = TRUE
      )
    }
    
    end <- Sys.time()
    if(printTimes){
      print(paste0('extracting incidence targets: ', (end-start), ' ', units((end-start))))
    }
    start <- Sys.time()
    
  }
  
  if(getCharacterizationInclusion){
    char <- tryCatch(getCharacterizationTargets(
      connectionHandler = connectionHandler,
      schema = schema,
      cgTablePrefix = cgTablePrefix,
      cTablePrefix = cTablePrefix,
      printTimes = printTimes
    ), error = function(e){return(NULL)})
    if(!is.null(char)){
      cohortCounts <- merge(
        x = cohortCounts, 
        y = char, 
        by.x = c('cohortId','cohortName'),
        by.y = c('cohortDefinitionId','cohortName'),
        all.x = TRUE
      )
    }
    
    start <- Sys.time() # reset time if char called
  }
  
  if(getPredictionInclusion){
    pred <- tryCatch(getPredictionTargets(
      connectionHandler = connectionHandler,
      schema = schema,
      cgTablePrefix = cgTablePrefix,
      plpTablePrefix = plpTablePrefix
    ), error = function(e){return(NULL)})
    if(!is.null(pred)){
      cohortCounts <- merge(
        x = cohortCounts, 
        y = pred, 
        by.x = c('cohortId','cohortName'),
        by.y = c('cohortDefinitionId','cohortName'),
        all.x = TRUE
      )
    }
    
    end <- Sys.time()
    if(printTimes){
      print(paste0('extracting prediction target cohorts: ', (end-start), ' ', units((end-start))))
    }
    start <- Sys.time()
    
  }
  
  if(getCohortMethodInclusion){
    cm <- tryCatch(getCmTargets(
      connectionHandler = connectionHandler,
      schema = schema,
      cgTablePrefix = cgTablePrefix,
      cmTablePrefix = cmTablePrefix
    ), error = function(e){return(NULL)})
    if(!is.null(cm)){
      cohortCounts <- merge(
        x = cohortCounts, 
        y = cm, 
        by.x = c('cohortId','cohortName'),
        by.y = c('cohortDefinitionId','cohortName'),
        all.x = TRUE
      )
    }
    
    end <- Sys.time()
    if(printTimes){
      print(paste0('extracting cohort method target cohorts: ', (end-start), ' ', units((end-start))))
    }
    start <- Sys.time()
    
  }
  
  if(getSccsInclusion){
    sccs <- tryCatch(getSccsTargets(
      connectionHandler = connectionHandler,
      schema = schema,
      cgTablePrefix = cgTablePrefix,
      sccsTablePrefix = sccsTablePrefix
    ), error = function(e){return(NULL)})
    if(!is.null(sccs)){
      cohortCounts <- merge(
        x = cohortCounts, 
        y = sccs, 
        by.x = c('cohortId','cohortName'),
        by.y = c('cohortDefinitionId','cohortName'),
        all.x = TRUE
      )
    }
    
    end <- Sys.time()
    if(printTimes){
      print(paste0('extracting sscs target cohorts: ', (end-start), ' ', units((end-start))))
    }
    start <- Sys.time()
  }
  
  if(sum(is.na(cohortCounts)) !=0){
    cohortCounts[is.na(cohortCounts)] <- 0
  }
  
  # add missing analyses (if they failed) with all 0 columns?
  colnameTypes <- c('timeToEvent','dechalRechal','riskFactors','databaseComparator',
                    'cohortComparator', 'caseSeries',
                    'cohortMethod', 'selfControlledCaseSeries', 'prediction',
                    'cohortIncidence') 

  if(sum(colnameTypes %in% colnames(cohortCounts)) != length(colnameTypes)){
    missingCols <- colnameTypes[!colnameTypes %in% colnames(cohortCounts)]
    for(missingCol in missingCols){
      cohortCounts[missingCol] <- 0
    }
  }
  
  # TODO: reorder columns to match docs 
  
  
  # order by names
  cohortCounts <- cohortCounts %>% 
    dplyr::arrange(.data$parentName, .data$cohortName)
  
  end <- Sys.time()
  if(printTimes){
    print(paste0('final target cohort processing: ', (end-start), ' ', units((end-start))))
  }
  
  # report total time
  print(paste0('-- Total time for extarcting target table: ', (end-firstStart), ' ', units((end-firstStart))))
  
return(cohortCounts)
}



#' Extract the outcome cohorts and where they are used in the analyses.
#' @description
#' This function extracts the outcome cohorts, the number of subjects/entries and where the cohort was used.
#'
#' @details
#' Specify the connectionHandler, the schema and the table prefixes
#'
#' @template connectionHandler
#' @template schema
#' @template cgTablePrefix
#' @template cTablePrefix
#' @template ciTablePrefix
#' @template cmTablePrefix
#' @template sccsTablePrefix
#' @template plpTablePrefix
#' @template databaseTable
#' @template targetId
#' @param getIncidenceInclusion Whether to check usage of the cohort in incidence
#' @param getCharacterizationInclusion Whether to check usage of the cohort in characterization
#' @param getPredictionInclusion Whether to check usage of the cohort in prediction
#' @param getCohortMethodInclusion Whether to check usage of the cohort in cohort method
#' @param getSccsInclusion Whether to check usage of the cohort in SCCS
#' @param printTimes whether to print the time it takes to run each SQL query
#' @family helper
#' @return
#' Returns a data.frame with the columns: 
#' \itemize{
#'  \item{cohortId the number id for the target cohort}
#'  \item{cohortName the name of the cohort}
#'  \item{subsetParent the number id of the parent cohort}
#'  \item{subsetDefinitionId the number id of the subset}
#'  \item{numDatabase number of databases with the cohort}
#'  \item{databaseString all the names of the databases with the cohort}
#'  \item{databaseIdString all the ids of the databases with the cohort}
#'  \item{databaseStringCount all the names of the databases with the cohort plus their counts}
#'  \item{databaseCount all the names of the databases with the cohort and their sizes}
#'  \item{minSubjectCount number of subjects in databases with lowest count}
#'  \item{maxSubjectCount number of subjects in databases with highest count}
#'  \item{minEntryCount number of entries in databases with lowest count}
#'  \item{maxEntryCount number of entries in databases with highest count}
#'  \item{cohortIncidence whether the cohort was used in cohort incidence}
#'  \item{dechalRechal whether the cohort was used in dechallenge rechallenge}
#'  \item{riskFactors whether the cohort was used in risk factors}
#'  \item{caseSeries whether the cohort was used in case series analysis}
#'  \item{timeToEvent whether the cohort was used in time to event}
#'  \item{prediction whether the cohort was used in prediction}
#'  \item{cohortMethod whether the cohort was used in cohort method}
#'  \item{selfControlledCaseSeries whether the cohort was used in self controlled case series}
#' }
#'
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' outcomeTable <- getOutcomeTable(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getOutcomeTable <- function(
    connectionHandler, 
    schema,
    cgTablePrefix = 'cg_',
    cTablePrefix = 'c_',
    ciTablePrefix = 'ci_',
    cmTablePrefix = 'cm_',
    sccsTablePrefix = 'sccs_',
    plpTablePrefix = 'plp_',
    databaseTable = 'database_meta_data',
    targetId = NULL,
    getIncidenceInclusion = TRUE,
    getCharacterizationInclusion = TRUE,
    getPredictionInclusion = TRUE,
    getCohortMethodInclusion = TRUE,
    getSccsInclusion = TRUE,
    printTimes = FALSE
){
  
  start <- Sys.time()
  firstStart <- start
  
  cohorts <- getCohortDefinitions(
    connectionHandler = connectionHandler,
    schema = schema
  ) %>%
    dplyr::select("cohortDefinitionId", "cohortName", "subsetParent", "subsetDefinitionId") %>%
    dplyr::rename(cohortId = "cohortDefinitionId")
  
  end <- Sys.time()
  if(printTimes){
    print(paste0('extracting outcome cohorts: ', (end-start), ' ', units((end-start))))
  }
  start <- Sys.time()
  
  # find parents names
  parents <- cohorts %>% 
    dplyr::filter(.data$cohortId == .data$subsetParent) %>%
    dplyr::select("cohortId", "cohortName") %>%
    dplyr::rename(
      parentName = "cohortName",
      subsetParent = "cohortId"
    )
  # add the parent name to cohorts
  cohorts <- merge(cohorts, parents, by = 'subsetParent', all.x = TRUE)
  
  end <- Sys.time()
  if(printTimes){
    print(paste0('processing outcome parent cohorts: ', (end-start), ' ', units((end-start))))
  }
  start <- Sys.time()
  
  sql <- "select 
  dt.cdm_source_abbreviation as database_name,
  cc.*
  from @schema.@cg_prefixcohort_count cc inner join
  @schema.@database_table dt on cc.database_id = dt.database_id;"
  counts <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    cg_prefix = cgTablePrefix,
    database_table = databaseTable
  ) %>% 
    dplyr::group_by(.data$cohortId) %>%
    dplyr::summarise(
      numDatabase = length(unique(.data$databaseName)),
      databaseString = paste0(unique(.data$databaseName), collapse = ', '),
      databaseIdString = paste0(unique(.data$databaseId), collapse = ', '),
      databaseStringCount = paste0(unique(paste0(.data$databaseName,' (', .data$cohortSubjects ,')')), collapse = ', '),
      minSubjectCount = min(.data$cohortSubjects, na.rm = TRUE),
      maxSubjectCount = max(.data$cohortSubjects, na.rm = TRUE),
      minEntryCount = max(.data$cohortEntries, na.rm = TRUE),
      maxEntryCount = max(.data$cohortEntries, na.rm = TRUE)
    )
  
  cohortCounts <- merge(cohorts, counts, by = 'cohortId')
  
  end <- Sys.time()
  if(printTimes){
    print(paste0('adding outcome cohort counts: ', (end-start), ' ', units((end-start))))
  }
  
  # now find whether it is a target for each analysis
  
  if(getIncidenceInclusion){
    start <- Sys.time()
    inc <- tryCatch(getIncidenceOutcomes(
      connectionHandler = connectionHandler,
      schema = schema,
      cgTablePrefix = cgTablePrefix,
      ciTablePrefix = ciTablePrefix, 
      targetId = targetId
    ), error = function(e){return(NULL)})
    if(!is.null(inc)){
      cohortCounts <- merge(
        x = cohortCounts, 
        y = inc, 
        by.x = c('cohortId','cohortName'),
        by.y = c('cohortDefinitionId','cohortName'),
        all.x = TRUE
      )
    }
    
    end <- Sys.time()
    if(printTimes){
      print(paste0('finding incidence outcomes: ', (end-start), ' ', units((end-start))))
    }
  }
  
  if(getCharacterizationInclusion){
    char <- tryCatch(getCharacterizationOutcomes(
      connectionHandler = connectionHandler,
      schema = schema,
      cgTablePrefix = cgTablePrefix,
      cTablePrefix = cTablePrefix, 
      targetId = targetId, 
      printTimes = printTimes
    ), error = function(e){return(NULL)})
    if(!is.null(char)){
      cohortCounts <- merge(
        x = cohortCounts, 
        y = char, 
        by.x = c('cohortId','cohortName'),
        by.y = c('cohortDefinitionId','cohortName'),
        all.x = TRUE
      )
    }
  }

  
  if(getPredictionInclusion){
    start <- Sys.time()
    pred <- tryCatch(getPredictionOutcomes(
      connectionHandler = connectionHandler,
      schema = schema,
      cgTablePrefix = cgTablePrefix,
      plpTablePrefix = plpTablePrefix, 
      targetId = targetId
    ), error = function(e){return(NULL)})
    if(!is.null(pred)){
      cohortCounts <- merge(
        x = cohortCounts, 
        y = pred, 
        by.x = c('cohortId','cohortName'),
        by.y = c('cohortDefinitionId','cohortName'),
        all.x = TRUE
      )
    }
    
    end <- Sys.time()
    if(printTimes){
      print(paste0('extracting prediction cohorts: ', (end-start), ' ', units((end-start))))
    }
  }
  
  if(getCohortMethodInclusion){
    start <- Sys.time()
    cm <- tryCatch(getCmOutcomes(
      connectionHandler = connectionHandler,
      schema = schema,
      cgTablePrefix = cgTablePrefix,
      cmTablePrefix = cmTablePrefix, 
      targetId = targetId
    ), error = function(e){return(NULL)})
    if(!is.null(cm)){
      cohortCounts <- merge(
        x = cohortCounts, 
        y = cm, 
        by.x = c('cohortId','cohortName'),
        by.y = c('cohortDefinitionId','cohortName'),
        all.x = TRUE
      )
    }
    
    end <- Sys.time()
    if(printTimes){
      print(paste0('extracting cohort method cohorts: ', (end-start), ' ', units((end-start))))
    }
  }
  
  if(getSccsInclusion){
    start <- Sys.time()
    sccs <- tryCatch(getSccsOutcomes(
      connectionHandler = connectionHandler,
      schema = schema,
      cgTablePrefix = cgTablePrefix,
      sccsTablePrefix = sccsTablePrefix, 
      targetId = targetId
    ), error = function(e){return(NULL)})
    if(!is.null(sccs)){
      cohortCounts <- merge(
        x = cohortCounts, 
        y = sccs, 
        by.x = c('cohortId','cohortName'),
        by.y = c('cohortDefinitionId','cohortName'),
        all.x = TRUE
      )
    }
    
    end <- Sys.time()
    if(printTimes){
      print(paste0('extracting sccs cohorts: ', (end-start), ' ', units((end-start))))
    }
  }
  
  start <- Sys.time()
  
  if(sum(is.na(cohortCounts)) !=0){
    cohortCounts[is.na(cohortCounts)] <- 0
  }
  
  # add missing analyses (if they failed) with all 0 columns?
  colnameTypes <- c('timeToEvent','dechalRechal','riskFactors','caseSeries',
                    'cohortMethod', 'selfControlledCaseSeries', 'prediction',
                    'cohortIncidence') 
  
  if(sum(colnameTypes %in% colnames(cohortCounts)) != length(colnameTypes)){
    missingCols <- colnameTypes[!colnameTypes %in% colnames(cohortCounts)]
    for(missingCol in missingCols){
      cohortCounts[missingCol] <- 0
    }
  }
  
  # remove columns that are 0 for all as these are not outcomes
  removeInd <- apply(cohortCounts[,colnameTypes],1, sum) == 0
  cohortCounts <- cohortCounts[!removeInd,]
  
  # TODO: reorder columns to match docs 
  
  
  # order by names
  cohortCounts <- cohortCounts %>% 
    dplyr::arrange(.data$parentName, .data$cohortName)
  
  end <- Sys.time()
  if(printTimes){
    print(paste0('final processing of outcome cohorts: ', (end-start), ' ', units((end-start))))
  }
  
  # report total time
  print(paste0('-- Total time for extarcting outcome table: ', (end-firstStart), ' ', units((end-firstStart))))
  
  return(cohortCounts)
}




extractSubsetCohorts <- function(json){
  
  if(is.na(json)){
    return("")
  }
  
  if(is.null(json)){
    return("")
  }
  
  
  subsetDefs <- lapply(json, function(x) ParallelLogger::convertJsonToSettings(x))
  
  subsetOps <- lapply(subsetDefs, function(x){
    x$subsetOperators
  })
  
  # remove name and extract cohortIds when subsetType == "CohortSubsetOperator"
  subsetUnique <- subsetOps
  subsetCohorts <- c()
  for(sind in 1:length(subsetUnique)){
    for(sind2 in 1:length(subsetUnique[[sind]])){
      if(subsetUnique[[sind]][[sind2]]$subsetType == 'CohortSubsetOperator'){
        subsetCohorts <- c(subsetCohorts,subsetUnique[[sind]][[sind2]]$cohortIds)
      }
    }
  }
  subsetCohorts <- paste0(unique(subsetCohorts), collapse = '')
  
  return(subsetCohorts)
  
}

test_that("getTargetTable", {
  
  targetTable <- getTargetTable(
    connectionHandler, 
    schema
    )
  
  testthat::expect_true(nrow(targetTable) > 0)
  
  testthat::expect_true("cohortId" %in% colnames(targetTable))
  testthat::expect_true("cohortName" %in% colnames(targetTable))
  testthat::expect_true("subsetParent" %in% colnames(targetTable))
  testthat::expect_true("subsetDefinitionId" %in% colnames(targetTable))
  testthat::expect_true("numDatabase" %in% colnames(targetTable))
  testthat::expect_true("databaseString" %in% colnames(targetTable))
  testthat::expect_true("minSubjectCount" %in% colnames(targetTable))
  testthat::expect_true("maxSubjectCount" %in% colnames(targetTable))
  
  testthat::expect_true("databaseStringCount" %in% colnames(targetTable))
  
  allAnalyses <-   c('timeToEvent','dechalRechal','riskFactors','databaseComparator',
                    'cohortMethod', 'selfControlledCaseSeries', 'prediction',
                    'cohortIncidence') 
  testthat::expect_true(sum(allAnalyses %in% colnames(targetTable)) == length(allAnalyses))
  
})


test_that("getOutcomeTable", {
  
outcomes <- getOutcomeTable(
    connectionHandler  = connectionHandler, 
    schema = schema,
    targetId = NULL
)

testthat::expect_true(nrow(outcomes) > 0)

testthat::expect_true("cohortId" %in% colnames(outcomes))
testthat::expect_true("cohortName" %in% colnames(outcomes))
testthat::expect_true("subsetParent" %in% colnames(outcomes))
testthat::expect_true("parentName" %in% colnames(outcomes))
testthat::expect_true("subsetDefinitionId" %in% colnames(outcomes))
testthat::expect_true("numDatabase" %in% colnames(outcomes))
testthat::expect_true("databaseString" %in% colnames(outcomes))
testthat::expect_true("databaseStringCount" %in% colnames(outcomes))

testthat::expect_true(sum(
  c('cohortIncidence', 'dechalRechal', 'riskFactors',
    'timeToEvent', 'caseSeries', 'prediction', 'cohortMethod', 
    'selfControlledCaseSeries')
  %in% colnames(outcomes)) == 8)


})

#' Plots the cohort method results for one analysis
#' @description
#' Creates nice cohort method plots 
#'
#' @details
#' Input the cohort method data 
#'
#' @param cmData The cohort method data 
#' @param cmDiagnostics (optional) The cohort method diagnostic data
#' @param cmMeta (optional) The cohort method evidence synthesis data
#' @param cohortNames A data.frame with columns cohortId and cohortName
#' @param includeCounts Whether to include the target/comp size and event counts
#' @param selectedAnalysisId The analysis ID of interest to plot
#' 
#' @family Estimation
#' @return
#' Returns a ggplot with the estimates
#' 
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' cmEst <- getCMEstimation(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main',
#'   targetIds = 1002,
#'   outcomeIds = 3
#' )
#' plotCmEstimates(
#'   cmData = cmEst, 
#'   cmMeta = NULL, 
#'   selectedAnalysisId = 1
#' )
#' 
plotCmEstimates <- function (
    cmData,
    cmDiagnostics = NULL,
    cmMeta = NULL,
    cohortNames = NULL,
    includeCounts = TRUE,
    selectedAnalysisId = NULL
)
{
  if(!is.null(selectedAnalysisId)){
    cmDataAll <- cmData %>%
      dplyr::filter(.data$analysisId == !!selectedAnalysisId)
  } else{
    cmDataAll <- cmData
  }
  
  # Add in diagnostics 
  diagColumns <- c("MDRR","EASE","SMD","EQUIPOISE")
  
  if(!is.null(cmDiagnostics)){
    
    cmDataAll <- cmDataAll %>%
      dplyr::inner_join(
        y = cmDiagnostics %>%
          dplyr::select("databaseId","analysisId", "targetId",
                        "comparatorId", "outcomeId", "indicationId",
                        "mdrrDiagnostic", "easeDiagnostic",
                        "sharedBalanceDiagnostic",
                        "equipoiseDiagnostic") %>%
          dplyr::rename(
            MDRR = "mdrrDiagnostic",
            EASE = "easeDiagnostic",
            SMD = "sharedBalanceDiagnostic",
            EQUIPOISE = "equipoiseDiagnostic"
          ),
        by = c("databaseId","analysisId", "targetId",
               "comparatorId", "outcomeId", "indicationId")
      )
    
  }
  
  if(!is.null(cohortNames)){
    # add code to change target, comp and outcome names
    cmDataAll <- cmDataAll %>%
      dplyr::left_join(
        cohortNames %>% dplyr::rename(
          targetId = "cohortId",
          targetNameFriendly = "cohortName"
        ),
        by = "targetId"
      ) %>%
      dplyr::left_join(
        cohortNames %>% dplyr::rename(
          outcomeId = "cohortId",
          outcomeNameFriendly = "cohortName"
        ),
        by = "outcomeId"
      ) %>%
      dplyr::left_join(
        cohortNames %>% dplyr::rename(
          comparatorId = "cohortId",
          comparatorNameFriendly = "cohortName"
        ),
        by = "comparatorId"
      ) %>%
      dplyr::mutate(
        targetName = dplyr::coalesce(.data$targetNameFriendly, .data$targetName),
        outcomeName = dplyr::coalesce(.data$outcomeNameFriendly, .data$outcomeName),
        comparatorName = dplyr::coalesce(.data$comparatorNameFriendly, .data$comparatorName)
      )
  }
  
  if(!is.null(cmMeta)){
    
    if(!is.null(selectedAnalysisId)){
      cmMetaAll <- cmMeta %>%
        dplyr::filter(.data$analysisId == !!selectedAnalysisId)
    } else{
      cmMetaAll <- cmMeta
    }
    
    if(!is.null(cohortNames)){
      cmMetaAll <- cmMetaAll %>%
        dplyr::left_join(
          cohortNames %>% dplyr::rename(
            targetId = "cohortId",
            targetNameFriendly = "cohortName"
          ),
          by = "targetId"
        ) %>%
        dplyr::left_join(
          cohortNames %>% dplyr::rename(
            outcomeId = "cohortId",
            outcomeNameFriendly = "cohortName"
          ),
          by = "outcomeId"
        ) %>%
        dplyr::left_join(
          cohortNames %>% dplyr::rename(
            comparatorId = "cohortId",
            comparatorNameFriendly = "cohortName"
          ),
          by = "comparatorId"
        ) %>%
        dplyr::mutate(
          targetName = dplyr::coalesce(.data$targetNameFriendly, .data$targetName),
          outcomeName = dplyr::coalesce(.data$outcomeNameFriendly, .data$outcomeName),
          comparatorName = dplyr::coalesce(.data$comparatorNameFriendly, .data$comparatorName)
        )
    }
  }
  
  ton <- cmDataAll %>%
    dplyr::select("targetId","outcomeId", "indicationId",
                  "targetName","outcomeName", "indicationName") %>%
    dplyr::distinct()
  
  plotList <- list()
  length(plotList) <- nrow(ton)
  names(plotList) <- paste0(ton$targetName,'-', ton$outcomeName, '-', ton$indicationName)
  
  for(ind in 1:nrow(ton)){
    
    # filter the cmDataAll to specific T/O/I
    cmDataTon <- cmDataAll %>%
      dplyr::filter(.data$targetId == !!ton$targetId[ind] &
                    .data$outcomeId == !!ton$outcomeId[ind] &
                    .data$indicationId == !!ton$indicationId[ind]
      ) %>%
      dplyr::mutate(  
        order = 2,
        summary = FALSE
        )
    
    # create the heading rows
    headings <- rbind(
      cmDataTon %>% 
        dplyr::select("description", "comparatorName") %>%
        dplyr::distinct() %>%
        dplyr::mutate(
          databaseName = substr(.data$comparatorName,1,50),
          unblindForEvidenceSynthesis = NA,
          MDRR = NA,
          EASE = NA,
          SMD = NA,
          EQUIPOISE = NA,
          calibratedRr  = NA,
          calibratedCi95Lb  = NA,
          calibratedCi95Ub  = NA,
          targetSubjects  = NA,
          targetDays  = NA,
          targetOutcomes = NA,
          comparatorSubjects = NA,
          comparatorDays = NA,
          comparatorOutcomes = NA,
          unblind = NA,
          order = 1,
          summary = FALSE
        ),
      cmDataTon %>% 
        dplyr::select("description", "comparatorName") %>%
        dplyr::distinct() %>%
        dplyr::mutate(
          databaseName = substr(.data$description,1,50),
          unblindForEvidenceSynthesis = NA,
          MDRR = NA,
          EASE = NA,
          SMD = NA,
          EQUIPOISE = NA,
          calibratedRr  = NA,
          calibratedCi95Lb  = NA,
          calibratedCi95Ub  = NA,
          targetSubjects  = NA,
          targetDays  = NA,
          targetOutcomes = NA,
          comparatorSubjects = NA,
          comparatorDays = NA,
          comparatorOutcomes = NA,
          unblind = NA,
          order = 1.2,
          summary = FALSE
        )
    )
    
    # add the header rows to the cm data
    plotData <- dplyr::bind_rows(
      cmDataTon %>%
        dplyr::mutate(
          databaseName = paste0('   ', .data$databaseName)
        ) %>%
        dplyr::select(
          "databaseName",
          "comparatorName",
          "description",
          "order",
          "summary",
          
          "targetSubjects",
          "targetOutcomes",
          "comparatorSubjects",
          "comparatorOutcomes",
          
          "calibratedRr",
          "calibratedCi95Lb", 
          "calibratedCi95Ub",
          "unblindForEvidenceSynthesis",
          dplyr::any_of(diagColumns)
        ),
      headings %>%
        dplyr::select(
          "databaseName",
          "comparatorName",
          "description",
          "order",
          "summary",
          
          "targetSubjects",
          "targetOutcomes",
          "comparatorSubjects",
          "comparatorOutcomes",
          
          "calibratedRr",
          "calibratedCi95Lb", 
          "calibratedCi95Ub",
          "unblindForEvidenceSynthesis"
        )
    )
    
    # add the meta rows (if input)
    if (!is.null(cmMeta)) {
      
      # create three rows - the main meta, tau row below and then prediction interval
      metaRows <- rbind(
        cmMetaAll %>%
          dplyr::filter(.data$targetId == !!ton$targetId[ind] &
                          .data$outcomeId == !!ton$outcomeId[ind] &
                          .data$indicationId == !!ton$indicationId[ind]
          ) %>%
          dplyr::mutate(
            order = 3,
            summary = TRUE
          ) %>%
          dplyr::select(
            "databaseName",
            "comparatorName",
            "description",
            "order",
            "summary",
            
            "targetSubjects",
            "targetOutcomes",
            "comparatorSubjects",
            "comparatorOutcomes",
            
            "calibratedRr",
            "calibratedCi95Lb", 
            "calibratedCi95Ub"
          ),
        
        cmMetaAll %>%
          dplyr::filter(.data$targetId == !!ton$targetId[ind] &
                          .data$outcomeId == !!ton$outcomeId[ind] &
                          .data$indicationId == !!ton$indicationId[ind]
          ) %>% 
          dplyr::mutate(
          databaseName = paste0('tau = ',sprintf("%.3f", .data$tau)),
          order = 3.1,
          summary = FALSE,
          
          targetSubjects = NA,
          targetOutcomes = NA,
          comparatorSubjects = NA,
          comparatorOutcomes = NA,
          
          calibratedRr = NA,
          calibratedCi95Lb = NA,
          calibratedCi95Ub = NA
        ) %>%
          dplyr::select(
            "databaseName",
            "comparatorName",
            "description",
            "order",
            "summary",
            
            "targetSubjects",
            "targetOutcomes",
            "comparatorSubjects",
            "comparatorOutcomes",
            
            "calibratedRr",
            "calibratedCi95Lb", 
            "calibratedCi95Ub"
          ),
      
        cmMetaAll %>%
          dplyr::filter(.data$targetId == !!ton$targetId[ind] &
                          .data$outcomeId == !!ton$outcomeId[ind] &
                          .data$indicationId == !!ton$indicationId[ind]
          ) %>%
        dplyr::mutate(
          databaseName = "Prediction Interval",
          order = 3.2,
          summary = TRUE,
          
          targetSubjects = NA,
          targetOutcomes = NA,
          comparatorSubjects = NA,
          comparatorOutcomes = NA,
          
          calibratedRr = NA,
          calibratedCi95Lb = .data$calibratedPi95Lb,
          calibratedCi95Ub = .data$calibratedPi95Ub
        ) %>%
          dplyr::select(
            "databaseName",
            "comparatorName",
            "description",
            "order",
            "summary",
            
            "targetSubjects",
            "targetOutcomes",
            "comparatorSubjects",
            "comparatorOutcomes",
            
            "calibratedRr",
            "calibratedCi95Lb", 
            "calibratedCi95Ub"
          )
      )
      
      
      # add the rows to the main
      plotData <- dplyr::bind_rows(plotData,metaRows)
    }
    
    # if diagnostics are missing add them:
    if(sum(diagColumns %in% colnames(plotData)) != length(diagColumns)){
      missCols <- diagColumns[!diagColumns %in% colnames(plotData)]
      plotData[missCols] <- NA
    }
    
  
    # Now the code to do the plot
    plotData <- plotData %>%
      dplyr::arrange(.data$comparatorName, .data$description, .data$order)
    
    plotTable <- plotData %>%
      dplyr::mutate(
        `Study` = dplyr::case_when(
          is.na(.data$unblindForEvidenceSynthesis) ~ .data$databaseName, 
          .data$unblindForEvidenceSynthesis == 1 ~ paste0(.data$databaseName, "*"),
          .data$unblindForEvidenceSynthesis == 0 ~ .data$databaseName
        ),
        ` ` = strrep(" ", 50), # spacer column used for CI drawing
        `RR (95% CI)` = dplyr::case_when(
          !is.na(.data$calibratedRr) ~ sprintf("%.2f (%.2f, %.2f)", .data$calibratedRr, .data$calibratedCi95Lb, .data$calibratedCi95Ub),
          is.na(.data$calibratedRr) & !is.na(.data$calibratedCi95Lb) ~ sprintf("(%0.2f, %0.2f)", .data$calibratedCi95Lb, .data$calibratedCi95Ub),
          TRUE ~ ""
        ),
        
        `Target\n    N` = ifelse(
          is.na(.data$targetSubjects),
          "",
          format(.data$targetSubjects, big.mark = ",")
        ),
        `Target\nevents` = ifelse(
          is.na(.data$targetOutcomes),
          "",
          format(.data$targetOutcomes, big.mark = ",")
        ),
        `Comparator\n      N` = ifelse(
          is.na(.data$comparatorSubjects),
          "",
          format(.data$comparatorSubjects, big.mark = ",")
        ),
        `Comparator\n    events` = ifelse(
          is.na(.data$comparatorOutcomes),
          "",
          format(.data$comparatorOutcomes, big.mark = ",")
        ),
        
        `MDRR` = dplyr::case_when(
          is.na(.data$MDRR) ~ "", 
          .data$MDRR == 'PASS' ~ "\u2714",
          .data$MDRR == 'FAIL' ~ "\u2718",
          .data$MDRR == 'NOT EVALUATED' ~ "?"
        ), 
        `EASE` = dplyr::case_when(
          is.na(.data$EASE) ~ "", 
          .data$EASE == 'PASS' ~ "\u2714",
          .data$EASE == 'FAIL' ~ "\u2718",
          .data$EASE == 'NOT EVALUATED' ~ "?"
        ),
        `SMD` = dplyr::case_when(
          is.na(.data$SMD) ~ "", 
          .data$SMD == 'PASS' ~ "\u2714",
          .data$SMD == 'FAIL' ~ "\u2718",
          .data$SMD == 'NOT EVALUATED' ~ "?"
        ),
        `EQUIP\nOISE` = dplyr::case_when(
          is.na(.data$EQUIPOISE) ~ "", 
          .data$EQUIPOISE == 'PASS' ~ "\u2714",
          .data$EQUIPOISE == 'FAIL' ~ "\u2718",
          .data$EQUIPOISE == 'NOT EVALUATED' ~ "?"
        )
      ) %>%
      dplyr::select(-"EQUIPOISE")
    
    isPi <- grepl("Prediction Interval", plotTable$Study, fixed = TRUE)
    isMeta <- grepl("Bayesian", plotTable$Study, fixed = TRUE)
    isSummaryPlot <- isMeta & !isPi
    
    
    # For rows with PI only (no point estimate), use geometric midpoint for plotting
    estForPlot <- ifelse(
      is.na(plotTable$calibratedRr) &
        !is.na(plotTable$calibratedCi95Lb) &
        !is.na(plotTable$calibratedCi95Ub),
      sqrt(plotTable$calibratedCi95Lb * plotTable$calibratedCi95Ub),
      plotTable$calibratedRr
    )
    
    # Keep log-scale limits data-driven but stable
    finiteBounds <- c(plotTable$calibratedCi95Lb, plotTable$calibratedCi95Ub)
    finiteBounds <- finiteBounds[is.finite(finiteBounds) & finiteBounds > 0]
    xMin <- max(min(finiteBounds, na.rm = TRUE) * 0.8, 0.1)
    xMax <- max(finiteBounds, na.rm = TRUE) * 1.2
    
    tm <- forestploter::forest_theme(
      base_size = 10,
      refline_col = "#555555",
      ci_col = "#1F78B4",
      summary_col = "#D95F02",
      footnote_col = "#555555"
    )
    
    sizes <- ifelse(isPi, 0.001, 0.4)
    
    if(!is.null(cmDiagnostics)){
      plotDiagCols <- c('MDRR',"EASE", "SMD", "EQUIP\nOISE")
    } else{
      plotDiagCols <- c()
    }
    
    if(includeCounts){
      countCols <- c("Target\n    N", "Target\nevents", "Comparator\n      N", "Comparator\n    events")
    } else{
      countCols <- c()
    }
    
    columsToInclude <- c("Study", " ", "RR (95% CI)", countCols, plotDiagCols)
    
    p <- forestploter::forest(
      plotTable[, columsToInclude],
      est = estForPlot,
      lower = plotTable$calibratedCi95Lb,
      upper = plotTable$calibratedCi95Ub,
      sizes = sizes,
      ci_column = 2,
      is_summary = isSummaryPlot,
      ref_line = 1,
      x_trans = "log",
      xlim = c(0.1, 10),
      vert_line = c(0.25, 0.5, 2 , 4),
      vert_line_lty = "dashed",
      vert_line_col = "grey50", 
      ticks_at = c(0.1, 0.5, 1, 2, 10),
      arrow_lab = c("Favours target", "Favours comparator"),
      theme = tm, 
      footnote = "* database used in Baysian analysis"
    )
    

    # Shade from each meta-analysis row through its prediction-interval row
    metaRows <- which(grepl("Bayesian", plotTable$Study, fixed = TRUE))
    piRows <- which(grepl("Prediction Interval", plotTable$Study, fixed = TRUE))
    
    # Light blue transparent fill
    shadeFill <- grDevices::adjustcolor("#ADD8E6", alpha.f = 0.5)
    
    for (i in seq_len(min(length(metaRows), length(piRows)))) {
      blockRows <- metaRows[i]:piRows[i]
      p <- forestploter::edit_plot(
        p,
        row = blockRows,
        col = 1:(3+length(countCols)+length(plotDiagCols)),                  # all displayed table/CI columns
        part = "body",
        which = "background",
        gp = grid::gpar(fill = shadeFill, col = NA)
      )
    }
    
    # add bold text for summary
    p <- forestploter::edit_plot(
      p,
      row = which(isMeta | isPi),
      col = 1:(3+length(countCols)),        # adjust upper bound to match your total text column count
      part = "body",
      which = "text",
      gp = grid::gpar(fontface = "bold")
    ) 
    
    # edit the diagnostics columns to make headings small and enable ticks/crosses
    if(length(plotDiagCols) > 0){
      p <- p %>%
        forestploter::edit_plot(
        col = (1+3+length(countCols)):(3+length(countCols)+length(plotDiagCols)),
        part = "header",
        which = "text",
        gp = grid::gpar(fontsize = 5)
      ) %>%
      forestploter::edit_plot(
        col = (1+3+length(countCols)):(3+length(countCols)+length(plotDiagCols)),
        part = "body",
        which = "text",
        gp = grid::gpar(fontfamily = "Arial Unicode MS")
      )
    }
    
    # changing color for tick/cross
    for(diagnostic in c('MDRR', 'EASE', "SMD", "EQUIP\nOISE")){
      
      if(diagnostic %in% columsToInclude){
        colInd <- which(columsToInclude %in% diagnostic)
        passRows <- which(plotTable[diagnostic] == "\u2714")
        failRows <- which(plotTable[diagnostic] == "\u2718")
        
        if (length(passRows) > 0) {
          p <- forestploter::edit_plot(
            p,
            row = passRows,
            col = colInd,
            part = "body",
            which = "text",
            gp = grid::gpar(col = "#2E8B57", fontface = "bold")  # green
          )
        }
        
        if (length(failRows) > 0) {
          p <- forestploter::edit_plot(
            p,
            row = failRows,
            col = colInd,
            part = "body",
            which = "text",
            gp = grid::gpar(col = "#CC0000", fontface = "bold")  # red
          )
        }
      }
      
    }
    
      plotList[[ind]] <- p
  }
  
  return(plotList)
}


#' Plots the self controlled case series results for one analysis
#' @description
#' Creates nice self controlled case series plots 
#'
#' @details
#' Input the self controlled case series data 
#'
#' @param sccsData The self controlled case series data 
#' @param sccsDiagnostics (optional) The self controlled case series diagnostic data
#' @param sccsMeta (optional) The self controlled case series evidence synthesis data
#' @param includeCounts Whether to include count on the plot
#' @param selectedAnalysisId The analysis ID of interest to plot
#' 
#' @family Estimation
#' @return
#' Returns a ggplot with the estimates
#' 
#' @export
#' @examples
#' 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' sccsEst <- getSccsEstimation(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main',
#'   targetIds = 1,
#'   outcomeIds = 3
#' )
#' plotSccsEstimates(
#'   sccsData = sccsEst, 
#'   sccsMeta = NULL, 
#'   selectedAnalysisId = 1
#' )
#' 
plotSccsEstimates <- function(
    sccsData, 
    sccsDiagnostics = NULL,
    sccsMeta = NULL, 
    includeCounts = TRUE,
    selectedAnalysisId
    )
{
  
  # replace indicationId NA with 0
  sccsData <- sccsData %>%
    dplyr::mutate(
      indicationId = dplyr::case_when(
        is.na(.data$indicationId) ~ 0,
        !is.na(.data$indicationId) ~ .data$indicationId
      )
    )
  if(!is.null(sccsDiagnostics)){
    sccsDiagnostics <- sccsDiagnostics %>%
      dplyr::mutate(
        indicationId = dplyr::case_when(
          is.na(.data$indicationId) ~ 0,
          !is.na(.data$indicationId) ~ .data$indicationId
        )
      )
  }
  if(!is.null(sccsMeta)){
    sccsMeta <- sccsMeta %>%
      dplyr::mutate(
        indicationId = dplyr::case_when(
          is.na(.data$indicationId) ~ 0,
          !is.na(.data$indicationId) ~ .data$indicationId
        )
      )
  }
  
  if(!is.null(selectedAnalysisId)){
    sccsDataAll <- sccsData %>%
      dplyr::filter(.data$analysisId == !!selectedAnalysisId)
  } else{
    sccsDataAll <- sccsData
  }
  
  # Add in diagnostics 
  diagColumns <- c("MDRR","EASE","TIME", 
                   "EXPOSURE", "OBS",
                   "RARE")
  
  if(!is.null(sccsDiagnostics)){
    
    sccsDataAll <- sccsDataAll %>%
      dplyr::inner_join(
        y = sccsDiagnostics %>%
          dplyr::select("databaseId","analysisId", "targetId",
                        "outcomeId", "indicationId", "covariateName",
                        "mdrrDiagnostic", "easeDiagnostic",
                        "timeStabilityDiagnostic",
                        "eventExposureDiagnostic",
                        "eventObservationDiagnostic",
                        "rareOutcomeDiagnostic") %>%
          dplyr::rename(
            MDRR = "mdrrDiagnostic",
            EASE = "easeDiagnostic",
            TIME = "timeStabilityDiagnostic",
            EXPOSURE = "eventExposureDiagnostic",
            OBS = "eventObservationDiagnostic",
            RARE = "rareOutcomeDiagnostic"
          ),
        by = c("databaseId","analysisId", "targetId",
                "outcomeId", "indicationId", "covariateName")
      )
    
  }
  
  if(!is.null(sccsMeta)){
    
    if(!is.null(selectedAnalysisId)){
      sccsMetaAll <- sccsMeta %>%
        dplyr::filter(.data$analysisId == !!selectedAnalysisId)
    } else{
      sccsMetaAll <- sccsMeta
    }
  
  }
  
  ton <- sccsDataAll %>%
    dplyr::select("targetId","outcomeId", "indicationId",
                  "targetName","outcomeName", "indicationName") %>%
    dplyr::distinct()
  
  plotList <- list()
  length(plotList) <- nrow(ton)
  names(plotList) <- paste0(ton$targetName,'-', ton$outcomeName, '-', ton$indicationName)
  
  for(ind in 1:nrow(ton)){
    
    # filter the cmDataAll to specific T/O/I
    sccsDataTon <- sccsDataAll %>%
      dplyr::filter(.data$targetId == !!ton$targetId[ind] &
                      .data$outcomeId == !!ton$outcomeId[ind] &
                      .data$indicationId == !!ton$indicationId[ind]
      ) %>%
      dplyr::mutate(  
        order = 2,
        summary = FALSE
      )
    
    # create the heading rows
    headings <- rbind(
      sccsDataTon %>% 
        dplyr::select("description", "covariateName") %>%
        dplyr::distinct() %>%
        dplyr::mutate(
          databaseName = substr(.data$covariateName,1,50),
          unblindForEvidenceSynthesis = NA,
          
          MDRR = NA,
          EASE = NA,
          TIME = NA,
          EXPOSURE = NA,
          OBS = NA,
          RARE = NA,
          
          calibratedRr  = NA,
          calibratedCi95Lb  = NA,
          calibratedCi95Ub  = NA,
          
          covariateSubjects  = NA,
          covariateDays  = NA,
          covariateOutcomes = NA,
          outcomeSubjects = NA,
          outcomeEvents = NA,
          
          unblind = NA,
          order = 1,
          summary = FALSE
        ),
      sccsDataTon %>% 
        dplyr::select("description", "covariateName") %>%
        dplyr::distinct() %>%
        dplyr::mutate(
          databaseName = substr(.data$description,1,50),
          unblindForEvidenceSynthesis = NA,
          
          MDRR = NA,
          EASE = NA,
          TIME = NA,
          EXPOSURE = NA,
          OBS = NA,
          RARE = NA,
          
          calibratedRr  = NA,
          calibratedCi95Lb  = NA,
          calibratedCi95Ub  = NA,
          
          covariateSubjects  = NA,
          covariateDays  = NA,
          covariateOutcomes = NA,
          outcomeSubjects = NA,
          outcomeEvents = NA,
          
          unblind = NA,
          order = 1.2,
          summary = FALSE
        )
    )
    
    # add the header rows to the cm data
    plotData <- dplyr::bind_rows(
      sccsDataTon %>%
        dplyr::mutate(
          databaseName = paste0('   ', .data$databaseName)
        ) %>%
        dplyr::select(
          "databaseName",
          "covariateName",
          "description",
          "order",
          "summary",
          
          "covariateSubjects",
          "covariateDays",
          "covariateOutcomes",
          "outcomeSubjects",
          "outcomeEvents",
          
          "calibratedRr",
          "calibratedCi95Lb", 
          "calibratedCi95Ub",
          "unblindForEvidenceSynthesis",
          dplyr::any_of(diagColumns)
        ),
      headings %>%
        dplyr::select(
          "databaseName",
          "covariateName",
          "description",
          "order",
          "summary",
          
          "covariateSubjects",
          "covariateDays",
          "covariateOutcomes",
          "outcomeSubjects",
          "outcomeEvents",
          
          "calibratedRr",
          "calibratedCi95Lb", 
          "calibratedCi95Ub",
          "unblindForEvidenceSynthesis"
        )
    )
    
    # add the meta rows (if input)
    if (!is.null(sccsMeta)) {
      
      # create three rows - the main meta, tau row below and then prediction interval
      metaRows <- rbind(
        sccsMetaAll %>%
          dplyr::filter(.data$targetId == !!ton$targetId[ind] &
                          .data$outcomeId == !!ton$outcomeId[ind] &
                          .data$indicationId == !!ton$indicationId[ind]
          ) %>%
          dplyr::mutate(
            order = 3,
            summary = TRUE
          ) %>%
          dplyr::select(
            "databaseName",
            "covariateName",
            "description",
            "order",
            "summary",
            
            "covariateSubjects",
            "covariateDays",
            "covariateOutcomes",
            "outcomeSubjects",
            "outcomeEvents",
            
            "calibratedRr",
            "calibratedCi95Lb", 
            "calibratedCi95Ub"
          ),
        
        sccsMetaAll %>%
          dplyr::filter(.data$targetId == !!ton$targetId[ind] &
                          .data$outcomeId == !!ton$outcomeId[ind] &
                          .data$indicationId == !!ton$indicationId[ind]
          ) %>% 
          dplyr::mutate(
            databaseName = paste0('tau = ',sprintf("%.3f", .data$tau)),
            order = 3.1,
            summary = FALSE,
            
            covariateSubjects = NA,
            covariateDays = NA,
            covariateOutcomes = NA,
            outcomeSubjects = NA,
            outcomeEvents = NA,
            
            calibratedRr = NA,
            calibratedCi95Lb = NA,
            calibratedCi95Ub = NA
          ) %>%
          dplyr::select(
            "databaseName",
            "covariateName",
            "description",
            "order",
            "summary",
            
            "covariateSubjects",
            "covariateDays",
            "covariateOutcomes",
            "outcomeSubjects",
            "outcomeEvents",
            
            "calibratedRr",
            "calibratedCi95Lb", 
            "calibratedCi95Ub"
          ),
        
        sccsMetaAll %>%
          dplyr::filter(.data$targetId == !!ton$targetId[ind] &
                          .data$outcomeId == !!ton$outcomeId[ind] &
                          .data$indicationId == !!ton$indicationId[ind]
          ) %>%
          dplyr::mutate(
            databaseName = "Prediction Interval",
            order = 3.2,
            summary = TRUE,
            
            covariateSubjects = NA,
            covariateDays = NA,
            covariateOutcomes = NA,
            outcomeSubjects = NA,
            outcomeEvents = NA,
            
            calibratedRr = NA,
            calibratedCi95Lb = .data$calibratedPi95Lb,
            calibratedCi95Ub = .data$calibratedPi95Ub
          ) %>%
          dplyr::select(
            "databaseName",
            "covariateName",
            "description",
            "order",
            "summary",
            
            "covariateSubjects",
            "covariateDays",
            "covariateOutcomes",
            "outcomeSubjects",
            "outcomeEvents",
            
            "calibratedRr",
            "calibratedCi95Lb", 
            "calibratedCi95Ub"
          )
      ) %>%
        dplyr::mutate(
          calibratedCi95Lb = as.double(.data$calibratedCi95Lb),
          calibratedCi95Ub = as.double(.data$calibratedCi95Ub)
        )
      
      
      # add the rows to the main
      plotData <- dplyr::bind_rows(plotData,metaRows)
    }
    
    # if diagnostics are missing add them:
    if(sum(diagColumns %in% colnames(plotData)) != length(diagColumns)){
      missCols <- diagColumns[!diagColumns %in% colnames(plotData)]
      plotData[missCols] <- NA
    }
    
    
    # Now the code to do the plot
    plotData <- plotData %>%
      dplyr::arrange(.data$covariateName, .data$description, .data$order)
    
    plotTable <- plotData %>%
      dplyr::mutate(
        `Study` = dplyr::case_when(
          is.na(.data$unblindForEvidenceSynthesis) ~ .data$databaseName, 
          .data$unblindForEvidenceSynthesis == 1 ~ paste0(.data$databaseName, "*"),
          .data$unblindForEvidenceSynthesis == 0 ~ .data$databaseName
        ),
        ` ` = strrep(" ", 50), # spacer column used for CI drawing
        `HR (95% CI)` = dplyr::case_when(
          !is.na(.data$calibratedRr) ~ sprintf("%.2f (%.2f, %.2f)", .data$calibratedRr, .data$calibratedCi95Lb, .data$calibratedCi95Ub),
          is.na(.data$calibratedRr) & !is.na(.data$calibratedCi95Lb) ~ sprintf("(%0.2f, %0.2f)", .data$calibratedCi95Lb, .data$calibratedCi95Ub),
          TRUE ~ ""
        ),
        
        `Target\n    N` = ifelse(
          is.na(.data$covariateSubjects),
          "",
          format(.data$covariateSubjects, big.mark = ",")
        ),
        `Target\nevents` = ifelse(
          is.na(.data$covariateOutcomes),
          "",
          format(.data$covariateOutcomes, big.mark = ",")
        ),
        `Outcome\nSubjects` = ifelse(
          is.na(.data$outcomeSubjects),
          "",
          format(.data$outcomeSubjects, big.mark = ",")
        ),
        `Outcome\nEvents` = ifelse(
          is.na(.data$outcomeEvents),
          "",
          format(.data$outcomeEvents, big.mark = ",")
        ),
        
        `MDRR` = dplyr::case_when(
          is.na(.data$MDRR) ~ "", 
          .data$MDRR == 'PASS' ~ "\u2714",
          .data$MDRR == 'FAIL' ~ "\u2718",
          .data$MDRR == 'NOT EVALUATED' ~ "?"
        ), 
        `EASE` = dplyr::case_when(
          is.na(.data$EASE) ~ "", 
          .data$EASE == 'PASS' ~ "\u2714",
          .data$EASE == 'FAIL' ~ "\u2718",
          .data$EASE == 'NOT EVALUATED' ~ "?"
        ),
        `TIME` = dplyr::case_when(
          is.na(.data$TIME) ~ "", 
          .data$TIME == 'PASS' ~ "\u2714",
          .data$TIME == 'FAIL' ~ "\u2718",
          .data$TIME == 'NOT EVALUATED' ~ "?"
        ),
        `EXPO\nSURE` = dplyr::case_when(
          is.na(.data$EXPOSURE) ~ "", 
          .data$EXPOSURE == 'PASS' ~ "\u2714",
          .data$EXPOSURE == 'FAIL' ~ "\u2718",
          .data$EXPOSURE == 'NOT EVALUATED' ~ "?"
        ),
        `OBS` = dplyr::case_when(
          is.na(.data$OBS) ~ "", 
          .data$OBS == 'PASS' ~ "\u2714",
          .data$OBS == 'FAIL' ~ "\u2718",
          .data$OBS == 'NOT EVALUATED' ~ "?"
        ),
        `RARE` = dplyr::case_when(
          is.na(.data$RARE) ~ "", 
          .data$RARE == 'PASS' ~ "\u2714",
          .data$RARE == 'FAIL' ~ "\u2718",
          .data$RARE == 'NOT EVALUATED' ~ "?"
        )
      ) %>%
      dplyr::select(-"EXPOSURE")
    
    isPi <- grepl("Prediction Interval", plotTable$Study, fixed = TRUE)
    isMeta <- grepl("Bayesian", plotTable$Study, fixed = TRUE)
    isSummaryPlot <- isMeta & !isPi
    
    
    # For rows with PI only (no point estimate), use geometric midpoint for plotting
    estForPlot <- ifelse(
      is.na(plotTable$calibratedRr) &
        !is.na(plotTable$calibratedCi95Lb) &
        !is.na(plotTable$calibratedCi95Ub),
      sqrt(plotTable$calibratedCi95Lb * plotTable$calibratedCi95Ub),
      plotTable$calibratedRr
    )
    
    # Keep log-scale limits data-driven but stable
    finiteBounds <- c(plotTable$calibratedCi95Lb, plotTable$calibratedCi95Ub)
    finiteBounds <- finiteBounds[is.finite(finiteBounds) & finiteBounds > 0]
    xMin <- max(min(finiteBounds, na.rm = TRUE) * 0.8, 0.1)
    xMax <- max(finiteBounds, na.rm = TRUE) * 1.2
    
    tm <- forestploter::forest_theme(
      base_size = 10,
      refline_col = "#555555",
      ci_col = "#1F78B4",
      summary_col = "#D95F02",
      footnote_col = "#555555"
    )
    
    sizes <- ifelse(isPi, 0.001, 0.4)
    
    if(!is.null(sccsDiagnostics)){
      plotDiagCols <- c('MDRR',"EASE", "TIME", "EXPO\nSURE","OBS", "RARE")
    } else{
      plotDiagCols <- c()
    }
    
    if(includeCounts){
      countCols <- c("Target\n    N", "Target\nevents", "Outcome\nSubjects", "Outcome\nEvents")
    } else{
      countCols <- c()
    }
    
    columsToInclude <- c("Study", " ", "HR (95% CI)", countCols, plotDiagCols)
    
    p <- forestploter::forest(
      plotTable[, columsToInclude],
      est = estForPlot,
      lower = as.numeric(plotTable$calibratedCi95Lb),
      upper = as.numeric(plotTable$calibratedCi95Ub),
      sizes = sizes,
      ci_column = 2,
      is_summary = isSummaryPlot,
      ref_line = 1,
      x_trans = "log",
      xlim = c(0.1, 10),
      vert_line = c(0.25, 0.5, 2 , 4),
      vert_line_lty = "dashed",
      vert_line_col = "grey50", 
      ticks_at = c(0.1, 0.5, 1, 2, 10),
      arrow_lab = c("Favours target", "Favours comparator"),
      theme = tm, 
      footnote = "* database used in Baysian analysis"
    )
    
    
    # Shade from each meta-analysis row through its prediction-interval row
    metaRows <- which(grepl("Bayesian", plotTable$Study, fixed = TRUE))
    piRows <- which(grepl("Prediction Interval", plotTable$Study, fixed = TRUE))
    
    # Light blue transparent fill
    shadeFill <- grDevices::adjustcolor("#ADD8E6", alpha.f = 0.5)
    
    for (i in seq_len(min(length(metaRows), length(piRows)))) {
      blockRows <- metaRows[i]:piRows[i]
      p <- forestploter::edit_plot(
        p,
        row = blockRows,
        col = 1:(3+length(countCols)+length(plotDiagCols)),                  # all displayed table/CI columns
        part = "body",
        which = "background",
        gp = grid::gpar(fill = shadeFill, col = NA)
      )
    }
    
    # add bold text for summary
    p <- forestploter::edit_plot(
      p,
      row = which(isMeta | isPi),
      col = 1:(3+length(countCols)),        # adjust upper bound to match your total text column count
      part = "body",
      which = "text",
      gp = grid::gpar(fontface = "bold")
    ) 
    
    # edit the diagnostics columns to make headings small and enable ticks/crosses
    if(length(plotDiagCols) > 0){
      p <- p %>%
        forestploter::edit_plot(
          col = (1+3+length(countCols)):(3+length(countCols)+length(plotDiagCols)),
          part = "header",
          which = "text",
          gp = grid::gpar(fontsize = 5)
        ) %>%
        forestploter::edit_plot(
          col = (1+3+length(countCols)):(3+length(countCols)+length(plotDiagCols)),
          part = "body",
          which = "text",
          gp = grid::gpar(fontfamily = "Arial Unicode MS")
        )
    }
    
    # changing color for tick/cross
    for(diagnostic in c('MDRR',"EASE", "TIME", "EXPO\nSURE","OBS", "RARE")){
      
      if(diagnostic %in% columsToInclude){
        colInd <- which(columsToInclude %in% diagnostic)
        passRows <- which(plotTable[diagnostic] == "\u2714")
        failRows <- which(plotTable[diagnostic] == "\u2718")
        
        if (length(passRows) > 0) {
          p <- forestploter::edit_plot(
            p,
            row = passRows,
            col = colInd,
            part = "body",
            which = "text",
            gp = grid::gpar(col = "#2E8B57", fontface = "bold")  # green
          )
        }
        
        if (length(failRows) > 0) {
          p <- forestploter::edit_plot(
            p,
            row = failRows,
            col = colInd,
            part = "body",
            which = "text",
            gp = grid::gpar(col = "#CC0000", fontface = "bold")  # red
          )
        }
      }
      
    }
    
    plotList[[ind]] <- p
  }
  
  return(plotList)
}

# Copyright 2023 Observational Health Data Sciences and Informatics
#
# This file is part of OhdsiReportGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' create a connection detail for an example OHDSI results database
#'
#' @description
#' This returns an object of class `ConnectionDetails` that lets you connect via `DatabaseConnector::connect()` to the example result database.
#'
#' @details
#' Finds the location of the example result database in the package and calls `DatabaseConnector::createConnectionDetails` to create a `ConnectionDetails` object for connecting to the database. 
#' 
#' @param exdir a directory to unzip the example result data into.  Default is tempdir().
#' 
#' @return
#' An object of class `ConnectionDetails` with the details to connect to the example OHDSI result database
#'
#' @family helper
#'
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
getExampleConnectionDetails <- function(exdir = tempdir()) {
  
  # unzip the data - it is compressed to save space
  utils::unzip(
    zipfile = system.file('exampledata','results.sqlite.zip', package = 'OhdsiReportGenerator'), 
    exdir = exdir
  )
  
  server <- file.path(exdir, 'results.sqlite')
  cd <- DatabaseConnector::createConnectionDetails(
    dbms = "sqlite", 
    server = server
    )
  return(cd)
}

#' removeSpaces
#'
#' @description
#' Removes spaces and replaces with under scroll
#'
#' @details
#' Removes spaces and replaces with under scroll
#' 
#' @param x A string
#' @return
#' A string without spaces
#' 
#' @family helper
#' 
#' @examples 
#' removeSpaces(' made up.   string')
#'
#' @export
removeSpaces <- function(x){
  return(gsub(' ', '_', gsub("[[:punct:]]", "", x)))
}

#' addTarColumn
#'
#' @description
#' Finds the four TAR columns and creates a new column called tar that pastes the columns
#' into a nice string
#'
#' @details
#' Create a friendly single tar column
#' 
#' @param data The data.frame with the individual TAR columns that you want to 
#'              combine into one column
#' @return
#' The data data.frame object with the tar column added if seperate TAR columns are found 
#' 
#' @family helper
#' 
#' @examples 
#' addTarColumn(data.frame(
#' tarStartWith = 'cohort start',
#' tarStartOffset = 1,
#' tarEndWith = 'cohort start',
#' tarEndOffset = 0
#' ))
#'
#' @export
addTarColumn <- function(
    data
){
  
    if(sum(c("tarStartWith","tarStartOffset","tarEndWith","tarEndOffset") %in%
           colnames(data)) == 4){
      message('Found CI TAR columns')
      
      if(nrow(data) > 0){
        data$tar <- paste0(
          '(',data$tarStartWith,' + ',data$tarStartOffset, ') - ',
          '(',data$tarEndWith,' + ', data$tarEndOffset, ')'
        )
      } else{
        data$tar <- data$tarStartWith
      }
    }
  
  if(sum(c("tarStartDay","tarStartAnchor","tarEndDay","tarEndAnchor") %in%
         colnames(data)) == 4){
    message('Found PLP TAR columns')
    if( nrow(data) > 0){
      data$tar <- paste0(
        '(',data$tarStartAnchor,' + ',data$tarStartDay, ') - ',
        '(',data$tarEndAnchor,' + ', data$tarEndDay, ')'
      )
    } else{
      data$tar <- data$tarStartAnchor
    }
  }
    
    if(sum(c("startAnchor","riskWindowStart","endAnchor","riskWindowEnd") %in%
           colnames(data)) == 4){
      message('Found CM TAR columns')
      if( nrow(data) > 0){
      data$tar <- paste0(
        '(',data$startAnchor,' + ',data$riskWindowStart, ') - ',
        '(',data$endAnchor,' + ', data$riskWindowEnd, ')'
      )
      } else{
        data$tar <- data$startAnchor
      }
    }
  
  return(data)
}

#' formatBinaryCovariateName
#'
#' @description
#' Removes the long part of the covariate name to make it friendly 
#'
#' @details
#' Makes the covariateName more friendly and shorter
#' 
#' @param data The data.frame with the covariateName column
#' @return
#' The data data.frame object with the ovariateName column changed to be more friendly
#' 
#' @family helper
#' 
#' @examples 
#' formatBinaryCovariateName(data.frame(
#' covariateName = c("fdfgfgf: dgdgff","made up test")
#' ))
#'
#' @export
formatBinaryCovariateName <- function(data){
  if('covariateName' %in% colnames(data) & nrow(data) > 0 ){
    
    #riskFactorsBinary$covariateName[nchar(riskFactorsBinary$covariateName) < 25]
    
    tempNames <- data$covariateName
    for(val in c('ethnicity = ','gender = ','race = ','age group: ')){
      rfIds <- grep(val, tempNames)
      if(length(rfIds) > 0){
        tempNames[rfIds] <- gsub(val, ': ', tempNames[rfIds])
      }
    }
    
    data$covariateName <- unlist(lapply(tempNames, function(x) ifelse(length(strsplit(x,':')[[1]])>=2, strsplit(x,': ')[[1]][2], 'Unknown covariate')))
  } 
  
  # add conceptId to the name
  if('covariateId' %in% colnames(data)){
    if(sum(is.null(data$covariateId)) > 0){
      data$covariateId[is.null(data$covariateId)] <- 0
    }
  data$covariateName <- paste0(data$covariateName, ' (', floor(data$covariateId/1000) ,')')
  }
  
  return(data)
}

formatCohortType <- function(
    cohortType
){
  x <- rep('No outcome', length(cohortType))
  x[cohortType == 'Cases'] <- 'outcome'
  
  return(x)
}

getTars <- function(
    data,
    tarColumnNames = c("tarStartWith","tarStartOffset","tarEndWith","tarEndOffset")
    ){
  tar <- data %>% 
    dplyr::select(dplyr::all_of(tarColumnNames)) %>%
    tidyr::drop_na()
  
  tar <- unique(tar)
  tar <- lapply(
    X = 1:nrow(tar), 
    FUN = function(i){as.list(tar[i,])}
    )
  return(tar)
}

addTar <- function(data){
  result <- paste0(
    data$riskWindowStart,
    data$riskWindowEnd, 
    data$startAnchor, 
    data$endAnchor
  )
  
  return(result)
}



getAnalyses <- function(
    server,
    username,
    password,
    dbms,
    schema
){
  
  connectionDetails <- DatabaseConnector::createConnectionDetails(
    dbms = dbms, 
    user = username, 
    password = password, 
    server = server 
  )
  
  connection <- DatabaseConnector::connect(
    connectionDetails = connectionDetails
    )
  on.exit(DatabaseConnector::disconnect(connection))
  
  tables <- DatabaseConnector::getTableNames(
    connection = connection, 
    databaseSchema = schema 
  )
  
  resultsRun <- unique(
    unlist(
      lapply(strsplit(tables, '_'), function(x) x[[1]][1])
    )
  )
  
  # TODO replace this with the resultDatabaseSettings values
  analyses <- data.frame(
    prefix = c('cd','cg','cm', 'sccs', 'plp', 'c', 'ci'),
    name = c('cohort diagnostics', 'cohort generator',
             'cohort method', 'self controlled case series',
             'patient level prediction', 
             'characterization', 'cohort incidence')
  )
  
  return(analyses[analyses$prefix %in% resultsRun,])
}


# TODO remove or have an input for the name to type?
getDbs <- function(
    schema,
    connectionHandler,
    dbDetails = data.frame(
      cdmSourceAbbreviation = c(
        "AMBULATORY EMR", "IBM CCAE", "German DA",
        "JMDC", "Optum EHR", "OPTUM Extended SES", "IBM MDCD",
        "IBM MDCR"
      ),
      type = c('us ehr', 'us claims', 'non-us claims',
               "non-us claims", 'us ehr', 'us claims', 'us claims',
               'us claims')
    )
){
  
  res <- connectionHandler$queryDb(
    "select CDM_SOURCE_ABBREVIATION from @schema.database_meta_data;",
    schema = schema
  )
  dbs <- merge(res, dbDetails)$type
  
  types <- lapply(unique(dbs), function(type){sum(dbs == type)})
  names(types) <- unique(dbs)
  
  return(types)
}


#' output a nicely formatted html table
#'
#' @description
#' This returns a html table with the input data
#'
#' @details
#' Input the data that you want to be shown via a dark html table 
#' 
#' @param data A data.frame containing data of interest to show via a table
#' @param caption A caption for the table
#' @param position The position for the table if used within a quarto document.  
#'        This is the "real" or say floating position for the latex table environment. 
#'        The kable only puts tables in a table environment when a caption is provided. 
#'        That is also the reason why your tables will be floating around if you specify 
#'        captions for your table. Possible choices are h (here), t (top, default), 
#'        b (bottom) and p (on a dedicated page).
#' 
#' @return
#' An object of class `knitr_kable` that will show the data via a nice html table
#'
#' @family helper
#'
#' @export
#' @examples 
#' kableDark(
#' data = data.frame(a=1,b=4), 
#' caption = 'A made up table to demonstrate this function',
#' position = 'h'
#' )
#' 
kableDark <- function(data, caption = NULL, position = NULL){
  result <- data %>%
    kableExtra::kbl(
      caption = caption, 
      position = position
    ) %>%
    kableExtra::kable_material_dark(
      lightable_options = 'hover'
    )
  return(result)
}

test_that("getExampleConnectionDetails", {
  
  con <- getExampleConnectionDetails()
  testthat::expect_s3_class(con, 'ConnectionDetails')
  
})

test_that("removeSpaces", {
  testthat::expect_true(removeSpaces(x = 'test ') == 'test_')
  testthat::expect_true(removeSpaces(x = 't e s t ') == 't_e_s_t_')
})

test_that("formatCohortType", {
  
  x <- formatCohortType(
    cohortType = c('Target', 'Target', 'Cases')
    )
  testthat::expect_true(length(x) == 3)
  testthat::expect_true(sum(x == 'outcome') == 1)
  testthat::expect_true(sum(x == 'No outcome') == 2)
  testthat::expect_true(x[3] == 'outcome')
  
})


test_that("getTars", {
  
  x <- getTars(
    data = data.frame(
      madeUp = c(1,2,'dfdf'),
      tarStartWith = c('cohort start', 'cohort end', 'cohort start'),
      tarStartOffset = c(1,90,30),
      tarEndWith = c('cohort start', 'cohort end', 'cohort start'),
      tarEndOffset = c(2,91,23)
    ), 
    tarColumnNames = c("tarStartWith","tarStartOffset","tarEndWith","tarEndOffset")
  )
  
  testthat::expect_true(length(x) == 3)
  testthat::expect_true(x[[1]]$tarStartWith == 'cohort start')
  testthat::expect_true(x[[1]]$tarStartOffset == 1)
  testthat::expect_true(x[[1]]$tarEndWith == 'cohort start')
  testthat::expect_true(x[[1]]$tarEndOffset == 2)
  testthat::expect_true(x[[2]]$tarStartWith == 'cohort end')
  testthat::expect_true(x[[2]]$tarStartOffset == 90)
  testthat::expect_true(x[[2]]$tarEndWith == 'cohort end')
  testthat::expect_true(x[[2]]$tarEndOffset == 91)
  testthat::expect_true(x[[3]]$tarStartWith == 'cohort start')
  testthat::expect_true(x[[3]]$tarStartOffset == 30)
  testthat::expect_true(x[[3]]$tarEndWith == 'cohort start')
  testthat::expect_true(x[[3]]$tarEndOffset == 23)
})

test_that("getTars", {
  
  x <- addTar(
    data = data.frame(
      madeUp = c(1,2,'dfdf'),
      riskWindowStart = c('cohort start', 'cohort end', 'cohort start'),
      startAnchor = c(1,90,30),
      riskWindowEnd = c('cohort start', 'cohort end', 'cohort start'),
      endAnchor = c(2,91,23)
    )
  )
  
  testthat::expect_true(x[1] == 'cohort startcohort start12')
  testthat::expect_true(x[2] == 'cohort endcohort end9091')
  testthat::expect_true(x[3] == 'cohort startcohort start3023')
})

test_that("getAnalyses", {
  
  x <- getAnalyses(
    server = conDet$server(),
    username = conDet$user(),
    password = conDet$password(),
    dbms = conDet$dbms,
    schema = schema
  )
  
  testthat::expect_true(sum(colnames(x) %in% c('prefix', 'name')) == 2)
  testthat::expect_true(sum(x$prefix %in% c('cd', 'cg', 'cm', 'sccs', 'plp', 'c', 'ci')) == 7)
  testthat::expect_true(nrow(x) == 7)
})

# TODO finish this
test_that("getDbs", {
  
  x <- OhdsiReportGenerator:::getDbs(
    schema = schema, 
    connectionHandler = connectionHandler,
    dbDetails = data.frame(
      CDM_SOURCE_ABBREVIATION = c(
        "eunomia"
      ),
      type = c('synthetic')
  )
  )
  
})

test_that("kableDark", {
result <- kableDark( data = data.frame(a=1,b=4), 
           caption = 'A made up table to demonstrate this function',
           position = 'h')

testthat::expect_true(inherits(result, 'knitr_kable'))
})

test_that("addTarColumn", {
  
  # check it works when the three valid columns are there
  # settings 1:
  data = data.frame(
    tarStartWith = 'cohort start',
    tarStartOffset = 34,
    tarEndWith = 'cohort start',
    tarEndOffset = 99
  )
res <- addTarColumn(data)
testthat::expect_true(ncol(res) == 5)
testthat::expect_true('tar' %in% colnames(res))
testthat::expect_true(res$tar == "(cohort start + 34) - (cohort start + 99)") 
testthat::expect_true(sum(colnames(data) %in% colnames(res)) == 4)

# settings 2:
data = data.frame(
  tarStartAnchor = 'cohort start',
  tarStartDay = 34,
  tarEndAnchor = 'cohort start',
  tarEndDay = 99
)
res <- addTarColumn(data)
testthat::expect_true(ncol(res) == 5)
testthat::expect_true('tar' %in% colnames(res))
testthat::expect_true(res$tar == "(cohort start + 34) - (cohort start + 99)") 
testthat::expect_true(sum(colnames(data) %in% colnames(res)) == 4)

# settings 3:
data = data.frame(
  startAnchor = 'cohort start',
  riskWindowStart = 34,
  endAnchor = 'cohort start',
  riskWindowEnd = 99
)
res <- addTarColumn(
  data
)
testthat::expect_true(ncol(res) == 5)
testthat::expect_true('tar' %in% colnames(res))
testthat::expect_true(res$tar == "(cohort start + 34) - (cohort start + 99)") 
testthat::expect_true(sum(colnames(data) %in% colnames(res)) == 4)

# Now check it does not add for invalid columns
data = data.frame(
  startAnchor = 'cohort start',
  riskWindowStart = 34,
  endAnchor = 'cohort start',
  tarEndDay = 99
)
res <- addTarColumn(data)
testthat::expect_true(!'tar' %in% colnames(res))
testthat::expect_true(sum(colnames(data) %in% colnames(res)) == 4)

})



test_that("formatBinaryCovariateName", {
  
  # check ethnicity
  data <- data.frame(
    covariateName = c('ethnicity = madeup'),
    covariateId = 1
  )
  newData <- formatBinaryCovariateName(data)
  testthat::expect_true(newData$covariateName == "madeup (0)")
  
  # check race
  data <- data.frame(
    covariateName = c('race = madeup'),
    covariateId = 1
  )
  newData <- formatBinaryCovariateName(data)
  testthat::expect_true(newData$covariateName == "madeup (0)")
  
  # check gender
  data <- data.frame(
    covariateName = c('gender = madeup'),
    covariateId = 1002
  )
  newData <- formatBinaryCovariateName(data)
  testthat::expect_true(newData$covariateName == "madeup (1)")
  
  # check age
  data <- data.frame(
    covariateName = c('age group: 0-4'),
    covariateId = 10001
  )
  newData <- formatBinaryCovariateName(data)
  testthat::expect_true(newData$covariateName == "0-4 (10)")
  
  # check condition and covariateId
  data <- data.frame(
    covariateName = c('condition occurrence -10 to 0 days prior: madeup'),
    covariateId = 123210
  )
  newData <- formatBinaryCovariateName(data)
  testthat::expect_true(newData$covariateName == "madeup (123)")
  
  # check it works when no covariateId
  data <- data.frame(
    covariateName = c('condition occurrence -10 to 0 days prior: madeup')
  )
  newData <- formatBinaryCovariateName(data)
  testthat::expect_true(newData$covariateName == "madeup")
  
  
})

.getCgVersion <- function(
    connectionHandler,
    schema,
    cgTablePrefix = 'cg_'
){
  version <- 0 # Default to v0
  tryCatch(
    {
      sql <- "
      SELECT MAX(migration_order) max_migration_order
      FROM @schema.@cg_table_prefixmigration
      ;"
      
      maxMigrationOrder <- connectionHandler$queryDb(
        sql = sql,
        schema = schema,
        cg_table_prefix = cgTablePrefix
      ) %>%
        dplyr::pull(maxMigrationOrder) %>%
        dplyr::first()
      version <- switch(
        as.character(dplyr::case_when(
          is.na(maxMigrationOrder) ~ "v0",
          maxMigrationOrder < 3 ~ "v0",
          maxMigrationOrder == 3 ~ "v1",
          maxMigrationOrder > 3 ~ "v1.1"
        )),
        "v0" = 0,
        "v1" = 1,
        "v1.1" = 1.1
      )
    },
    error = function(e) {
      # Do nothing - most likely the migration table does not exist so assume
      # CohortGenerator v0
    }
  )
  return(version)
}

#' Extract the cohort definition details
#' @description
#' This function extracts all cohort definitions for the targets of interest.
#'
#' @details
#' Specify the connectionHandler, the schema and the target cohort IDs
#'
#' @template connectionHandler
#' @template schema
#' @template cgTablePrefix
#' @template targetIds
#' @family Cohorts
#' @return
#' Returns a data.frame with the cohort details
#' 
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' cohortDef <- getCohortDefinitions(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getCohortDefinitions <- function(
    connectionHandler,
    schema,
    cgTablePrefix = 'cg_',
    targetIds = NULL
){
  cgVersion <- round(
      .getCgVersion(
      connectionHandler = connectionHandler,
      schema = schema,
      cgTablePrefix = cgTablePrefix
    )
  )
  
  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/cohort/getCohortDefinitionsV", cgVersion, ".sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))

  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    cg_table_prefix = cgTablePrefix,
    restrict_to_targets = !is.null(targetIds),
    target_ids = paste0(targetIds, collapse = ',')
  )
  
  return(result)
}

#' Extract the cohort parents and children cohorts (cohorts derieved from the parent cohort)
#' @description
#' This function lets you split the cohort data.frame into the parents and the children per parent.
#'
#' @details
#' Finds the parent cohorts and children cohorts
#'
#' @param cohort The data.frame extracted using `getCohortDefinitions()` 
#' @family Cohorts
#' @return
#' Returns a list containing parents: a named vector of all the parent cohorts and cohortList: a list 
#' the same length as the parent vector with the first element containing all the children
#' of the first parent cohort, the second element containing the children of the second parent, etc.
#' 
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' cohortDef <- getCohortDefinitions(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
#' parents <- processCohorts(cohortDef)
#' 
processCohorts <- function(cohort){
  
  parentCodes <- unique(cohort$subsetParent)
  
  cohortList <- list()
  for(parentCode in parentCodes){
    cohortList[[length(cohortList)+1]] <- cohort %>% 
      dplyr::filter(.data$subsetParent == !! parentCode)
  }
  names(cohortList) <- parentCodes
  
  names(parentCodes) <- sapply(parentCodes, 
                               function(x){
                                 cohort$cohortName[cohort$cohortDefinitionId == x]
                                 }
                               )
  
  return(
    list(
      parents = parentCodes,
      cohortList = cohortList
    )
  )
}

# TODO - find which analyses each cohort is used and whether target or outcome


#' Extract the cohort subset definition details
#' @description
#' This function extracts all cohort subset definitions for the subsets of interest.
#'
#' @details
#' Specify the connectionHandler, the schema and the subset IDs
#'
#' @template connectionHandler
#' @template schema
#' @template cgTablePrefix
#' @param subsetIds A vector of subset cohort ids or NULL
#' @family Cohorts
#' @return
#' Returns a data.frame with the cohort subset details
#' 
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' subsetDef <- getCohortSubsetDefinitions(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getCohortSubsetDefinitions <- function(
    connectionHandler,
    schema,
    cgTablePrefix = 'cg_',
    subsetIds = NULL
){
  
  sql <- SqlRender::readSql(system.file(
    "sql/sql_server/cohort/getCohortSubsetDefinitions.sql",
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
  result <- tryCatch({connectionHandler$queryDb(
    sql = sql, 
    schema = schema,
    cg_table_prefix = cgTablePrefix,
    use_subsets = !is.null(subsetIds),
    subset_id = paste0(subsetIds, collapse = ',')
  )}, 
  error = function(e){print(e); return(NULL)}
  )
  
  return(result)
}



#' Extract the cohort inclusion stats
#' @description
#' This function extracts all cohort inclusion stats for the cohorts of interest.
#'
#' @details
#' Specify the connectionHandler, the schema and the cohort IDs
#'
#' @template connectionHandler
#' @template schema
#' @template cgTablePrefix
#' @template databaseTable
#' @param cohortIds Optionally a list of cohortIds to restrict to
#' @family Cohorts
#' @return
#' Returns a data.frame with the cohort inclusion stats
#' 
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' cohortInclsuionsStats <- getCohortInclusionStats(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getCohortInclusionStats <- function(
    connectionHandler,
    schema,
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    cohortIds = NULL
) {
  
  sql <- SqlRender::readSql(system.file(
    "sql/sql_server/cohort/getCohortInclusionStats.sql",
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    cg_table_prefix = cgTablePrefix,
    database_table = databaseTable,
    use_cohort_id = !is.null(cohortIds),
    cohort_definition_ids = paste0(cohortIds, collapse = ',')
  )
  
  return(result)
}



#' Extract the cohort inclusion rules
#' @description
#' This function extracts all cohort inclusion rules for the cohorts of interest.
#'
#' @details
#' Specify the connectionHandler, the schema and the cohort IDs
#'
#' @template connectionHandler
#' @template schema
#' @template cgTablePrefix
#' @param cohortIds Optionally a list of cohortIds to restrict to
#' @family Cohorts
#' @return
#' Returns a data.frame with the cohort inclusion rules
#' 
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' cohortInclsuionsRules <- getCohortInclusionRules(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getCohortInclusionRules <- function(
    connectionHandler,
    schema,
    cgTablePrefix = 'cg_',
    cohortIds = NULL
) {
  
  sql <- SqlRender::readSql(system.file(
    "sql/sql_server/cohort/getCohortInclusionRules.sql",
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    cg_table_prefix = cgTablePrefix,
    use_cohort_id = !is.null(cohortIds),
    cohort_definition_ids = paste0(cohortIds, collapse = ',')
  )
  
  return(result)
}



#' Extract the cohort inclusion summary
#' @description
#' This function extracts all cohort inclusion summary for the cohorts of interest.
#'
#' @details
#' Specify the connectionHandler, the schema and the cohort IDs
#'
#' @template connectionHandler
#' @template schema
#' @template cgTablePrefix
#' @template databaseTable
#' @param cohortIds Optionally a list of cohortIds to restrict to
#' @family Cohorts
#' @return
#' Returns a data.frame with the cohort inclusion rules
#' 
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' cohortInclsuionsSummary <- getCohortInclusionSummary(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getCohortInclusionSummary <- function(
    connectionHandler,
    schema,
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    cohortIds = NULL
) {
  
  sql <- SqlRender::readSql(system.file(
    "sql/sql_server/cohort/getCohortInclusionSummary.sql",
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    cg_table_prefix = cgTablePrefix,
    database_table = databaseTable,
    use_cohort_id = !is.null(cohortIds),
    cohort_definition_ids = paste0(cohortIds, collapse = ',')
  )
  
  return(result)
}




#' Extract the cohort meta
#' @description
#' This function extracts all cohort meta for the cohorts of interest.
#'
#' @details
#' Specify the connectionHandler, the schema and the cohort IDs
#'
#' @template connectionHandler
#' @template schema
#' @template cgTablePrefix
#' @template databaseTable
#' @param cohortIds Optionally a list of cohortIds to restrict to
#' @family Cohorts
#' @return
#' Returns a data.frame with the cohort inclusion rules
#' 
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' cohortMeta <- getCohortMeta(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getCohortMeta <- function(
    connectionHandler,
    schema,
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    cohortIds = NULL
) {
  cgVersion <- round(
    .getCgVersion(
      connectionHandler = connectionHandler,
      schema = schema,
      cgTablePrefix = cgTablePrefix
    )
  )

  sql <- SqlRender::readSql(system.file(
    paste0("sql/sql_server/cohort/getCohortMetaV", cgVersion, ".sql"),
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
  result <- tryCatch({connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    cg_table_prefix = cgTablePrefix,
    database_table = databaseTable,
    use_cohort_id = !is.null(cohortIds),
    cohort_definition_ids = paste0(cohortIds, collapse = ',')
  )}, error = function(e){
    print(e);
    return(NULL)
  })
  
  return(result)
}




#' Extract the cohort counts
#' @description
#' This function extracts all cohort counts for the cohorts of interest.
#'
#' @details
#' Specify the connectionHandler, the schema and the cohort IDs
#'
#' @template connectionHandler
#' @template schema
#' @template cgTablePrefix
#' @template databaseTable
#' @param cohortIds Optionally a list of cohortIds to restrict to
#' @family Cohorts
#' @return
#' Returns a data.frame with the cohort inclusion rules
#' 
#' @export
#' @examples 
#' conDet <- getExampleConnectionDetails()
#' 
#' connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
#' 
#' cohortMeta <- getCohortCounts(
#'   connectionHandler = connectionHandler, 
#'   schema = 'main'
#' )
#' 
getCohortCounts <- function(
    connectionHandler,
    schema,
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    cohortIds = NULL
) {
  
  sql <- SqlRender::readSql(system.file(
    "sql/sql_server/cohort/getCohortCounts.sql",
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  
  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    cg_table_prefix = cgTablePrefix,
    database_table = databaseTable,
    use_cohort_id = !is.null(cohortIds),
    cohort_definition_ids = paste0(cohortIds, collapse = ',')
  )
  
  return(result)
}



#' Get cohort attrition
#'
#' Retrieves attrition information for specified cohorts from the database.
#'
#' @param connectionHandler A connection handler object.
#' @param schema The database schema name.
#' @param cgTablePrefix Prefix for cohort generator tables. Default is 'cg_'.
#' @param databaseTable Name of the database metadata table. Default is 'database_meta_data'.
#' @param cohortIds Optional vector of cohort IDs to filter.
#' @family Cohorts
#' @return A data.frame with attrition details for each cohort.
#' @export
getCohortAttrition <- function(
    connectionHandler,
    schema,
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    cohortIds = NULL
) {
  cgVersion <- .getCgVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cgTablePrefix = cgTablePrefix
  )

  if (cgVersion < 1.1) {
    warning("Cohort attrition information is only available for CohortGenerator v1.1 or higher.")
    return(NULL)
  }
 
  sql <- SqlRender::readSql(system.file(
    "sql/sql_server/cohort/getCohortAttrition.sql",
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    cg_table_prefix = cgTablePrefix,
    database_table = databaseTable,
    use_cohort_id = !is.null(cohortIds),
    cohort_definition_ids = paste0(cohortIds, collapse = ',')
  )
  return(result)
}

#' Get cohort subset attrition
#'
#' Retrieves attrition information for specified cohort subsets from the database.
#'
#' @param connectionHandler A connection handler object.
#' @param schema The database schema name.
#' @param cgTablePrefix Prefix for cohort generator tables. Default is 'cg_'.
#' @param databaseTable Name of the database metadata table. Default is 'database_meta_data'.
#' @param cohortIds Optional vector of cohort IDs to filter.
#' @family Cohorts
#' @return A data.frame with attrition details for each cohort subset.
#' @export
getCohortSubsetAttrition <- function(
    connectionHandler,
    schema,
    cgTablePrefix = 'cg_',
    databaseTable = 'database_meta_data',
    cohortIds = NULL
) {
  cgVersion <- .getCgVersion(
    connectionHandler = connectionHandler,
    schema = schema,
    cgTablePrefix = cgTablePrefix
  )

  if (cgVersion < 1.1) {
    warning("Cohort subset attrition information is only available for CohortGenerator v1.1 or higher.")
    return(NULL)
  }
 
  sql <- SqlRender::readSql(system.file(
    "sql/sql_server/cohort/getCohortSubsetAttrition.sql",
    package = "OhdsiReportGenerator",
    mustWork = TRUE
  ))
  result <- connectionHandler$queryDb(
    sql = sql,
    schema = schema,
    cg_table_prefix = cgTablePrefix,
    database_table = databaseTable,
    use_cohort_id = !is.null(cohortIds),
    cohort_definition_ids = paste0(cohortIds, collapse = ',')
  )
  return(result)
}




#' Function processes the cohortDefinitions object ready for use in the main quarto report.
#'
#' @details
#' Function processes the cohortDefinitions object by adding friendly names for 
#' specified parent cohorts, determines which cohorts are nested in the specified
#' indication cohort ids and ...
#' @family Cohorts
#'
#' @param cohortDefinitions The output of `getCohortDefinitions()`
#' @param friendlyCohortIds a vector of parent cohort ids that you want to rename
#' @param friendlyCohortNames a vector of new names for the friendlyCohortIds
#' @param restrictTargetToIndications whether to restrict the results to cohorts nested in certain indications
#' @param indicationIds The indication ids of interest for restrictTargetToIndications
#'                
#' @return
#' A cohortDefinitions object with extra columns: friendlyName, 
#'
#' @export
processCohortDefinitionsForQuarto <- function(
  cohortDefinitions,
  friendlyCohortIds,
  friendlyCohortNames,
  restrictTargetToIndications,
  indicationIds
){
  
  # 1) add the friendly names 
  if(!is.null(friendlyCohortIds)){
    cohortDefinitions <- 
      merge(
        cohortDefinitions,
        data.frame(
          cohortId = friendlyCohortIds,
          friendlyName = friendlyCohortNames
        ), 
        by.x = 'subsetParent', 
        by.y = 'cohortId', 
        all.x = TRUE 
      )
    cohortDefinitions$friendlyName[is.na(cohortDefinitions$friendlyName)] <- cohortDefinitions$cohortName[is.na(cohortDefinitions$friendlyName)]
  } else{
    cohortDefinitions$friendlyName <- cohortDefinitions$cohortName
  }
  
  # 2) add friendly subset logic text
  # add subset text for the subsetDefinitionJson
  cohortDefinitions$subsetText <- unlist(
    lapply(
      X = cohortDefinitions$subsetDefinitionJson, 
      FUN = function(x){getSubsetText(
        subsetDefinitionJson = x, 
        cohortDefinitions = cohortDefinitions
      )}
    ))
  
  # 3) add indicationOfInt if required
  if(restrictTargetToIndications){
    cohortDefinitions$indicationOfInt <- unlist(lapply(
      X = cohortDefinitions$subsetDefinitionJson, 
      FUN = function(x){
        subsetNestedInIds(subsetDefinitionJson = x, nestIds = indicationIds)
        }))
  }
  
  return(cohortDefinitions)
  
}

#' Function to figure out the target, comparator, outcome and indication ids of interest
#' for the quarto report based on the user inputs
#'
#' @details
#' This function finds the targets, comparators, indications and outcomes of interest based
#' on the user inputs for quarto report generation.
#' @family Cohorts
#'
#' @template connectionHandler
#' @template schema
#' @param cohortDefinitions The output of `processCohortDefinitionsForQuarto()`
#' @param targetId a parent target id
#' @param outcomeIds a vector of outcome ids
#' @param comparatorIds NULL or a vector of comparator parent ids
#' @param restrictTargetToIndications whether to restrict the target ids to cohorts nested in indicationIds
#' @param indicationIds The indication ids of interest for restrictTargetToIndications
#' @param includeCohortMethod If TRUE, when comparatorIds is NULL all comparators included in CohortMethod are included in the report
#'                
#' @return
#' A list of targetIdsOfInterest that is a vector of cohortIds that are targets of interest to include in the report,
#' comparatorIds a vector of cohortIds that are comparators, 
#' comparatorIdsOfInterest a vector of cohortIds that are comparators of interest, 
#' outcomeIdsOfInterest  a vector of cohortIds that are outcomes of interest and
#' indicationIdsOfInterest  a vector of cohortIds that are indications of interest.
#'
#' @export
restrictCohortDefinitionsForQuarto <- function(
    connectionHandler,
    schema,
    cohortDefinitions,
    targetId, # from param
    outcomeIds, # from param
    comparatorIds, # from param
    restrictTargetToIndications,
    indicationIds, # from param
    includeCohortMethod
    ){
  
  # if not restrictTargetToIndications then include all children
  #==================
  # TARGETS
  #==================
  if(restrictTargetToIndications){
    
    targetsOfInterest <- cohortDefinitions %>%
      dplyr::filter(
        ((.data$subsetParent %in% !!targetId) & .data$indicationOfInt) 
      )
    
  } else{
    
    # take all children of targetId
    targetsOfInterest <- cohortDefinitions %>%
      dplyr::filter(
        .data$subsetParent %in% !!targetId
      )
    
  }
  
  targetIdsOfInterest <- targetsOfInterest$cohortDefinitionId
  

  #==================
  # COMPARATORS
  #==================
# if comparators is NULL then include all that are found in CM in results
if(is.null(comparatorIds)){
  if(includeCohortMethod){
    newComps <- getCmDiagnosticsData(
      connectionHandler = connectionHandler,
      schema = schema, 
      targetIds = targetIdsOfInterest, 
      outcomeIds = outcomeIds
    )
    
    # get all the parents
    comparatorIds <- unique(cohortDefinitions$subsetParent[cohortDefinitions$cohortDefinitionId %in% unique(newComps$comparatorId)])
    
  }
}
  
  if(restrictTargetToIndications){
    
    comparatorOfInterest <- cohortDefinitions %>%
      dplyr::filter(
        ((.data$subsetParent %in% !!comparatorIds) & .data$indicationOfInt)
      ) 
    
  } else{
    
    # take all children of targetId
    comparatorOfInterest <- cohortDefinitions %>%
      dplyr::filter(
        .data$subsetParent %in% !!comparatorIds
      )
  }

comparatorIdsOfInterest <-  comparatorOfInterest$cohortDefinitionId

#==================
# OUTCOMES
#==================
outcomeOfInterest <- cohortDefinitions %>%
  dplyr::filter(
    (.data$cohortDefinitionId %in% !!outcomeIds)
  )
outcomeIdsOfInterest <- outcomeOfInterest$cohortDefinitionId

#==================
# INDICATIONS - take the children of all indication ids
#==================
if(is.numeric(indicationIds)){
  indicationOfInterest <- cohortDefinitions %>%
    dplyr::filter(
      (.data$subsetParent %in% !!indicationIds)
    )
  indicationIdsOfInterest <- indicationOfInterest$cohortDefinitionId
} else{
  indicationIdsOfInterest <- NULL
}


return(
  list(
    targetIdsOfInterest = targetIdsOfInterest,
    comparatorIds = comparatorIds,
    comparatorIdsOfInterest = comparatorIdsOfInterest,
    outcomeIdsOfInterest = outcomeIdsOfInterest,
    indicationIdsOfInterest = indicationIdsOfInterest
  )
)
}


subsetNestedInIds <- function(subsetDefinitionJson = NULL, nestIds){ 
  if(is.na(subsetDefinitionJson)){
    return(FALSE)
  }
  if(is.null(subsetDefinitionJson)){
    return(FALSE)
  }
  
  sub <- ParallelLogger::convertJsonToSettings(as.character(subsetDefinitionJson))
  result <- any(unlist(lapply(sub$subsetOperators, function(x){
    
    if(x$subsetType == "CohortSubsetOperator"){
      if(!x$negate){
        return(any(x$cohortIds %in% nestIds))
      }
    } 
    return(FALSE)
    
  })))
  
}


#' Function that converts a subsetDefinitionJson into text description
#'
#' @details
#' The function takes a subsetDefinitionJson and converts it into friendly text describing the 
#' subset logic
#' @family Cohorts
#'
#' @param subsetDefinitionJson The subset logic json
#' @param cohortDefinitions A data.frame with the columns cohortDefinitionId, cohortName and optionally friendlyName that will
#'                          be used to know the friendly cohort name for any subsetting that nests in other cohorts
#'                
#' 
#' @return
#' A text string describing the subsetting 
#'
#' @export
getSubsetText <- function(subsetDefinitionJson, cohortDefinitions){
  
  if(is.null(subsetDefinitionJson)){
    return('')
  } 
  if(is.na(subsetDefinitionJson)){
    return('')
  }
  
  text <- paste0(unlist(lapply(X = ParallelLogger::convertJsonToSettings(subsetDefinitionJson)$subsetOperators, 
                       FUN = function(subsetOperator){
                         getSubsetOperatorsText(
                           subsetOperator = subsetOperator, 
                           cohortDefinitions = cohortDefinitions
                         )
                       }
                         )), 
         collapse = ' ')
  
  return(text)
}

# a helper for getSubsetText
getSubsetOperatorsText <- function(subsetOperator, cohortDefinitions){
  
  if(is.null(cohortDefinitions$friendlyName)){
    cohortDefinitions$friendlyName <- cohortDefinitions$cohortName
  }
  
  if(subsetOperator$subsetType == "CohortSubsetOperator"){
    
    if(sum(cohortDefinitions$cohortDefinitionId %in% subsetOperator$cohortIds) != 0){
      name <- cohortDefinitions$friendlyName[cohortDefinitions$cohortDefinitionId %in% subsetOperator$cohortIds]
    } else{
      name <- paste('id', subsetOperator$cohortIds)
    }
    
    txt <- paste0(ifelse(subsetOperator$negate, 'Not in','In'), ' cohorts ', 
                  paste0(name,
                         collapse = ifelse(subsetOperator$cohortCombinationOperator == 'all', ' and ', ' or ')
                  ))
    return(paste0(txt, '.'))
  }
  
  if(subsetOperator$subsetType == "DemographicSubsetOperator"){
    gender <- ''
    if(sum(c(8507,8532) %in% subsetOperator$gender) == 2){
      gender = ''
    } else if( 8507 %in% subsetOperator$gender){
      gender = 'Resticted to males.'
    } else if( 8532 %in% subsetOperator$gender){
      gender = 'Resticted to females.'
    }
    return(paste0('Aged between ', subsetOperator$ageMin, ' to ', subsetOperator$ageMax, '.', 
                  gender
    ))
  }
  
  if(subsetOperator$subsetType == "LimitSubsetOperator"){
    txt <- ''
    joiner <- ' and '
    if(!is.null(subsetOperator$calendarStartDate)){
      txt <-  paste0(txt,paste0(
        'Restict to exposures occurring after ', subsetOperator$calendarStartDate, 
        ifelse(!is.null(subsetOperator$calendarEndDate), paste0(joiner,' before ', subsetOperator$calendarEndDate), '')
      ))
    } else{
      if(!is.null(subsetOperator$calendarEndDate)){
        txt <-  paste0(txt,paste0('Restict to exposures occurring before ', subsetOperator$calendarEndDate))
      }
    }
    
    if(subsetOperator$priorTime > 0){
      txt <- paste0(txt, ifelse(txt == '', 'Requiring ',paste0(joiner, ' requiring ')) , subsetOperator$priorTime, ' days observation prior to index')
    }
    
    if(subsetOperator$followUpTime > 0){
      txt <- paste0(txt, ifelse(txt == '', 'Requiring ',paste0(joiner, ' requiring ')), subsetOperator$followUpTime, ' days follow up post index')
    }
    
    if(subsetOperator$limitTo == 'firstEver'){
      txt <- paste0(txt, ifelse(txt == '', 'Limit ', ' and limit '), 'to first exposure')
    }
    
    txt <- paste0(txt, '.')
    return(txt)
  }
  
}

test_that("getCohortDefinitions", {
  
  cohorts <- getCohortDefinitions(
    connectionHandler = connectionHandler,
    schema = schema
  )
  
  testthat::expect_true(nrow(cohorts) > 0)
  
  testthat::expect_true('cohortDefinitionId' %in% colnames(cohorts))
  testthat::expect_true('cohortName' %in% colnames(cohorts))
  testthat::expect_true('description' %in% colnames(cohorts))
  testthat::expect_true('json' %in% colnames(cohorts))
  testthat::expect_true('sqlCommand' %in% colnames(cohorts))
  testthat::expect_true('subsetParent' %in% colnames(cohorts))
  testthat::expect_true('isSubset' %in% colnames(cohorts))
  testthat::expect_true('subsetDefinitionId' %in% colnames(cohorts))
  
})

test_that("processCohorts", {
  
  cohorts <- getCohortDefinitions(
    connectionHandler = connectionHandler,
    schema = schema
  )

  parents <- processCohorts(cohorts)
  
  # check correct number of parents
  testthat::expect_true(sum(is.na(cohorts$subsetDefinitionId)) == length(parents$parents))
  
  # check all cohorts are in the parents list
  testthat::expect_true(nrow(do.call('rbind', parents$cohortList)) == nrow(cohorts))
  
})


test_that("getCohortSubsetDefinitions", {
  
  subsetDef <- getCohortSubsetDefinitions(
    connectionHandler,
    schema
    )
  
  # make sure results are returned
  testthat::expect_true(nrow(subsetDef) > 0)
  
  firstJson <- ParallelLogger::convertJsonToSettings(subsetDef$json[1])
  
  # check key entries
  testthat::expect_true('name' %in% names(firstJson))
  testthat::expect_true('definitionId' %in% names(firstJson))
  testthat::expect_true('subsetOperators' %in% names(firstJson))
  
})


test_that("getCohortInclusionStats", {
  
  result <- getCohortInclusionStats(
    connectionHandler,
    schema
  )
  
  # check key entries
  testthat::expect_true('databaseId' %in% colnames(result))
  testthat::expect_true('databaseName' %in% colnames(result))
  testthat::expect_true('cohortDefinitionId' %in% colnames(result))
  testthat::expect_true('cohortName' %in% colnames(result))
  testthat::expect_true('inclusionRuleMask' %in% colnames(result))
  testthat::expect_true('personCount' %in% colnames(result))
  testthat::expect_true('modeId' %in% colnames(result))
})


test_that("getCohortInclusionRules", {
  
  result <- getCohortInclusionRules(
    connectionHandler,
    schema
  )
  
  # check key entries
  testthat::expect_true('cohortDefinitionId' %in% colnames(result))
  testthat::expect_true('cohortName' %in% colnames(result))
  testthat::expect_true('ruleName' %in% colnames(result))
  testthat::expect_true('ruleSequence' %in% colnames(result))
})

test_that("getCohortInclusionSummary", {
  
  result <- getCohortInclusionSummary(
    connectionHandler,
    schema
  )
  
  # check key entries
  testthat::expect_true('cohortDefinitionId' %in% colnames(result))
  testthat::expect_true('cohortName' %in% colnames(result))
  testthat::expect_true('baseCount' %in% colnames(result))
  testthat::expect_true('finalCount' %in% colnames(result))
  testthat::expect_true('modeId' %in% colnames(result))
  testthat::expect_true('databaseName' %in% colnames(result))
  testthat::expect_true('databaseId' %in% colnames(result))
})


test_that("getCohortMeta", {
  
  result <- getCohortMeta(
    connectionHandler,
    schema
  )
  
  # check key entries
  testthat::expect_true('cohortId' %in% colnames(result))
  testthat::expect_true('cohortName' %in% colnames(result))
  testthat::expect_true('generationStatus' %in% colnames(result))
  testthat::expect_true('startTime' %in% colnames(result))
  testthat::expect_true('endTime' %in% colnames(result))
  testthat::expect_true('databaseName' %in% colnames(result))
  testthat::expect_true('databaseId' %in% colnames(result))
})


test_that("getCohortCounts", {
  
  result <- getCohortCounts(
    connectionHandler,
    schema
  )
  
  # check key entries
  testthat::expect_true('cohortId' %in% colnames(result))
  testthat::expect_true('cohortName' %in% colnames(result))
  testthat::expect_true('cohortEntries' %in% colnames(result))
  testthat::expect_true('cohortSubjects' %in% colnames(result))
  testthat::expect_true('databaseName' %in% colnames(result))
  testthat::expect_true('databaseId' %in% colnames(result))
})

test_that("getCohortAttrition", {
  
  result <- getCohortAttrition(
    connectionHandler,
    schema
  )

    # check key entries
  testthat::expect_true('databaseId' %in% colnames(result))
  testthat::expect_true('databaseName' %in% colnames(result))
  testthat::expect_true('cohortDefinitionId' %in% colnames(result))
  testthat::expect_true('modeId' %in% colnames(result))
  testthat::expect_true('cohortEntry' %in% colnames(result))
  testthat::expect_true('ruleSequence' %in% colnames(result))
  testthat::expect_true('ruleName' %in% colnames(result))
  testthat::expect_true('personCount' %in% colnames(result))  
})

test_that("getCohortSubsetAttrition", {
  
  result <- getCohortSubsetAttrition(
    connectionHandler,
    schema
  )

    # check key entries
  testthat::expect_true('databaseId' %in% colnames(result))
  testthat::expect_true('databaseName' %in% colnames(result))
  testthat::expect_true('cohortDefinitionId' %in% colnames(result))
  testthat::expect_true('subsetDefinitionId' %in% colnames(result))
  testthat::expect_true('subsetParentId' %in% colnames(result))
  testthat::expect_true('modeId' %in% colnames(result))
  testthat::expect_true('cohortEntry' %in% colnames(result))
  testthat::expect_true('operatorSequence' %in% colnames(result))
  testthat::expect_true('operatorName' %in% colnames(result))
  testthat::expect_true('operatorType' %in% colnames(result))
  testthat::expect_true('countValue' %in% colnames(result))  
})

test_that("getCohortAttrition and getCohortSubsetAttrition throw warnings when CG version < 1.1", {
  # Override the .getCgVersion function to 
  # return 1 for this test
  original <- get(".getCgVersion", envir = asNamespace("OhdsiReportGenerator"))
  assignInNamespace(
    ".getCgVersion",
    function(connectionHandler, schema, ...) 1,
    ns = "OhdsiReportGenerator"
  )
  on.exit({
    assignInNamespace(".getCgVersion", original, ns = "OhdsiReportGenerator")
  })
  
  testthat::expect_warning(
    getCohortAttrition(
      connectionHandler,
      schema
    )
  )

  testthat::expect_warning(
    getCohortSubsetAttrition(
      connectionHandler,
      schema
    )
  )
})



test_that("test processCohortDefinitionsForQuarto", {

  # nest to cohort 10
  sub1Json <- '{
  "name": "madeup",
  "definitionId": 1,
  "subsetOperators": [
    {
      "name": "made 1",
      "subsetType": "CohortSubsetOperator",
      "cohortIds": [10],
      "cohortCombinationOperator": "all",
      "negate": false,
      "windows": []
    }
  ],
  "packageVersion": "1.0.2",
  "identifierExpression": "targetId * 1000 + definitionId",
  "operatorNameConcatString": ", ",
  "subsetCohortNameTemplate": "@baseCohortName - @subsetDefinitionName"
}' 
  
  sub2Json <- '
  {
  "name": "madeup 2",
  "definitionId": 2,
  "subsetOperators": [
    {
      "name": "aged 18+",
      "subsetType": "DemographicSubsetOperator",
      "ageMin": 18,
      "ageMax": 99999
    }
  ],
  "packageVersion": "1.0.2",
  "identifierExpression": "targetId * 1000 + definitionId",
  "operatorNameConcatString": ", ",
  "subsetCohortNameTemplate": "@baseCohortName - @subsetDefinitionName"
} 
  '
  
  sub3Json <- '
  {
  "name": "madeup 3",
  "definitionId": 3,
  "subsetOperators": [
    {
      "name": "aged 18+",
      "subsetType": "DemographicSubsetOperator",
      "ageMin": 18,
      "ageMax": 99999
    },
  {
      "name": "made 1",
      "subsetType": "CohortSubsetOperator",
      "cohortIds": [5],
      "cohortCombinationOperator": "all",
      "negate": false,
      "windows": []
    }
  ],
  "packageVersion": "1.0.2",
  "identifierExpression": "targetId * 1000 + definitionId",
  "operatorNameConcatString": ", ",
  "subsetCohortNameTemplate": "@baseCohortName - @subsetDefinitionName"
} 
  '
  
  cohortDef <- data.frame(
    cohortDefinitionId = c(1,2,3,4,5),
    cohortName = c(1,2,3,4,5), 
    subsetParent = c(1,1,1,2,5),
    isSubset = c(0,1,1,1,0),
    subsetDefinitionId = c(NA,1,2,3,NA),
    subsetDefinitionJson = c(NA, sub1Json, sub2Json, sub3Json, NA)
  )
  
  cohortDefinitionsPro <- processCohortDefinitionsForQuarto(
    cohortDefinitions = cohortDef,
    friendlyCohortIds = c(1,2,3,4,5),
    friendlyCohortNames = paste0('cohort ', 1:5),
    restrictTargetToIndications = FALSE,
    indicationIds = 10
  )
  
  testthat::expect_true(sum(colnames(cohortDef) %in% colnames(cohortDefinitionsPro) ) == ncol(cohortDef))
  testthat::expect_true(nrow(cohortDef) == nrow(cohortDefinitionsPro))
  testthat::expect_true('friendlyName' %in% colnames(cohortDefinitionsPro))
  testthat::expect_true('subsetText' %in% colnames(cohortDefinitionsPro))
  
  testthat::expect_true(cohortDefinitionsPro$subsetText[1] == "")
  testthat::expect_true(cohortDefinitionsPro$subsetText[5] == "")
  
  testthat::expect_true(cohortDefinitionsPro$subsetText[2] == "In cohorts id 10.")
  testthat::expect_true(cohortDefinitionsPro$subsetText[3] == "Aged between 18 to 99999.")
  testthat::expect_true(cohortDefinitionsPro$subsetText[4] == "Aged between 18 to 99999. In cohorts cohort 5.")
  
  # now check when restrictTargetToIndications is set to TRUE
  cohortDefinitionsPro <- processCohortDefinitionsForQuarto(
    cohortDefinitions = cohortDef,
    friendlyCohortIds = c(1,2,3,4,5),
    friendlyCohortNames = paste0('cohort ', 1:5),
    restrictTargetToIndications = TRUE,
    indicationIds = c(5,10)
  )
  testthat::expect_true(sum(colnames(cohortDef) %in% colnames(cohortDefinitionsPro) ) == ncol(cohortDef))
  testthat::expect_true(nrow(cohortDef) == nrow(cohortDefinitionsPro))
  testthat::expect_true('friendlyName' %in% colnames(cohortDefinitionsPro))
  testthat::expect_true('subsetText' %in% colnames(cohortDefinitionsPro))
  testthat::expect_true('indicationOfInt' %in% colnames(cohortDefinitionsPro))
  
  testthat::expect_true(cohortDefinitionsPro$subsetText[2] == "In cohorts id 10.")
  testthat::expect_true(cohortDefinitionsPro$subsetText[3] == "Aged between 18 to 99999.")
  testthat::expect_true(cohortDefinitionsPro$subsetText[4] == "Aged between 18 to 99999. In cohorts cohort 5.")
  
  testthat::expect_true(cohortDefinitionsPro$indicationOfInt[1] == FALSE)
  testthat::expect_true(cohortDefinitionsPro$indicationOfInt[2] == TRUE)
  testthat::expect_true(cohortDefinitionsPro$indicationOfInt[3] == FALSE)
  testthat::expect_true(cohortDefinitionsPro$indicationOfInt[4] == TRUE)
  testthat::expect_true(cohortDefinitionsPro$indicationOfInt[5] == FALSE)
  
})


test_that("test restrictCohortDefinitionsForQuarto", {
  
  sub1Json <- '{
  "name": "madeup",
  "definitionId": 1,
  "subsetOperators": [
    {
      "name": "made 1",
      "subsetType": "CohortSubsetOperator",
      "cohortIds": [10],
      "cohortCombinationOperator": "all",
      "negate": false,
      "windows": []
    }
  ],
  "packageVersion": "1.0.2",
  "identifierExpression": "targetId * 1000 + definitionId",
  "operatorNameConcatString": ", ",
  "subsetCohortNameTemplate": "@baseCohortName - @subsetDefinitionName"
}' 
  
  sub2Json <- '
  {
  "name": "madeup 2",
  "definitionId": 2,
  "subsetOperators": [
    {
      "name": "aged 18+",
      "subsetType": "DemographicSubsetOperator",
      "ageMin": 18,
      "ageMax": 99999
    }
  ],
  "packageVersion": "1.0.2",
  "identifierExpression": "targetId * 1000 + definitionId",
  "operatorNameConcatString": ", ",
  "subsetCohortNameTemplate": "@baseCohortName - @subsetDefinitionName"
} 
  '
  
  sub3Json <- '
  {
  "name": "madeup 3",
  "definitionId": 3,
  "subsetOperators": [
    {
      "name": "aged 18+",
      "subsetType": "DemographicSubsetOperator",
      "ageMin": 18,
      "ageMax": 99999
    },
  {
      "name": "made 1",
      "subsetType": "CohortSubsetOperator",
      "cohortIds": [5],
      "cohortCombinationOperator": "all",
      "negate": false,
      "windows": []
    }
  ],
  "packageVersion": "1.0.2",
  "identifierExpression": "targetId * 1000 + definitionId",
  "operatorNameConcatString": ", ",
  "subsetCohortNameTemplate": "@baseCohortName - @subsetDefinitionName"
} 
  '
  
  cohortDef <- data.frame(
    cohortDefinitionId = c(1,2,3,4,5,6,10),
    cohortName = c(1,2,3,4,5,6,10), 
    subsetParent = c(1,1,1,4,5,4,10),
    isSubset = c(0,1,1,1,0,1,0),
    subsetDefinitionId = c(NA,1,2,3,NA,1,NA),
    subsetDefinitionJson = c(NA, sub1Json, sub2Json, sub3Json, NA,sub1Json,NA)
  )
  cohortDefinitionsPro <- processCohortDefinitionsForQuarto(
    cohortDefinitions = cohortDef,
    friendlyCohortIds = c(1,2,3,4,5),
    friendlyCohortNames = paste0('cohort ', 1:5),
    restrictTargetToIndications = TRUE,
    indicationIds = c(10)
  )
  
  cohorts <- restrictCohortDefinitionsForQuarto(
    cohortDefinitions = cohortDefinitionsPro, 
    targetId = 1, 
    outcomeIds = 5, 
    comparatorIds = 4, 
    includeCohortMethod = FALSE, 
    indicationIds = NULL, 
    restrictTargetToIndications = FALSE
  )
  
  testthat::expect_true(sum(c(1,2,3) %in% cohorts$targetIdsOfInterest) == 3)
  testthat::expect_true(length(cohorts$targetIdsOfInterest) == 3)
  testthat::expect_true(cohorts$outcomeIdsOfInterest == 5)
  testthat::expect_true(sum(c(4,6) %in% cohorts$comparatorIdsOfInterest) == 2)
  testthat::expect_true(length(cohorts$comparatorIdsOfInterest) == 2)
  testthat::expect_true(is.null(cohorts$indicationIdsOfInterest))
  
  
  # now with indications
  cohorts <- restrictCohortDefinitionsForQuarto(
    cohortDefinitions = cohortDefinitionsPro, 
    targetId = 1, 
    outcomeIds = 5, 
    comparatorIds = 4, 
    includeCohortMethod = FALSE, 
    indicationIds = 10, 
    restrictTargetToIndications = FALSE
  )
  
  testthat::expect_true(sum(c(1,2,3) %in% cohorts$targetIdsOfInterest) == 3)
  testthat::expect_true(length(cohorts$targetIdsOfInterest) == 3)
  testthat::expect_true(cohorts$outcomeIdsOfInterest == 5)
  testthat::expect_true(sum(c(4,6) %in% cohorts$comparatorIdsOfInterest) == 2)
  testthat::expect_true(length(cohorts$comparatorIdsOfInterest) == 2)
  testthat::expect_true(cohorts$indicationIdsOfInterest == 10)
  
  
  # now test restricting target to indication 10
  cohorts <- restrictCohortDefinitionsForQuarto(
    cohortDefinitions = cohortDefinitionsPro, 
    targetId = 1, 
    outcomeIds = 5, 
    comparatorIds = 4, 
    includeCohortMethod = FALSE, 
    indicationIds = 10, 
    restrictTargetToIndications = TRUE
  )
  testthat::expect_true(cohorts$targetIdsOfInterest == 2)
  testthat::expect_true(cohorts$outcomeIdsOfInterest == 5)
  testthat::expect_true(cohorts$comparatorIdsOfInterest == 6)
  testthat::expect_true(cohorts$indicationIdsOfInterest == 10)
  
})

#' generateFullReport
#'
#' @description
#' Generates a full report from a Strategus analysis
#'
#' @details
#' Specify the connection details to the result database and the schema name
#' to generate the full report.
#' 
#' @param server The server containing the result database
#' @param username The username for an account that can access the result database
#' @param password The password for an account that can access the result database
#' @param dbms The dbms used to access the result database
#' @param resultsSchema The result database schema
#' @param targetId The cohort definition id for the target cohort
#' @param outcomeIds The cohort definition id for the outcome
#' @param comparatorIds (optional) The cohort definition id for any comparator cohorts.  If NULL the report will find and include all possible comparators in the results if includeCohortMethod is TRUE. 
#' @param indicationIds The cohort definition id for any indication cohorts to show in characterization. If no indication use NULL. 
#' @param restrictTargetToIndications If you only want the results for targets that are nested by the indicationIds set this to TRUE otherwise results for all children of the targetId will be generated.
#' @param cohortNames Friendly names for any cohort used in the study
#' @param cohortIds  The corresponding Ids for the cohortNames
#' @param includeCI Whether to include the cohort incidence slides
#' @param includeCharacterization Whether to include the characterization slides
#' @param includeCohortMethod Whether to include the cohort method slides
#' @param includeSccs Whether to include the self controlled case series slides
#' @param includePrediction Whether to include the patient level prediction slides
#' @param webAPI The ATLAS web API to use for the characterization index breakdown (set to NULL to not include)
#' @param authMethod The authorization method for the webAPI
#' @param webApiUsername The username for the webAPI authorization
#' @param webApiPassword The password for the webAPI authorization
#' @param outputLocation The file location and name to save the protocol 
#' @param outputName The name of the html protocol that is created
#' @param intermediateDir The work directory for quarto
#' @param pathToDriver Path to a folder containing the JDBC driver JAR files.
#' 
#' @return
#' An html document containing the full results for the target, comparators, indications and outcomes specified.
#' 
#' @family Reporting
#'
#' @export
#' 
generateFullReport <- function(
    server,
    username,
    password,
    dbms,
    resultsSchema = NULL,
    targetId = 1,
    outcomeIds = 3,
    comparatorIds = 2,
    indicationIds = NULL,
    restrictTargetToIndications = FALSE,
    cohortNames = c('target name','outcome name', 'comp name'),
    cohortIds = c(1,3,2),
    includeCI = TRUE,
    includeCharacterization = TRUE,
    includeCohortMethod = TRUE,
    includeSccs = TRUE,
    includePrediction = TRUE,
    webAPI = NULL,
    authMethod = NULL,
    webApiUsername = NULL, 
    webApiPassword = NULL,
    outputLocation,
    outputName = paste0('full_report_', gsub(':', '_',gsub(' ','_',as.character(date()))),'.html'),
    intermediateDir = tempdir(),
    pathToDriver = Sys.getenv("DATABASECONNECTOR_JAR_FOLDER")
){
  
  # check ResultModelManager is installed as it is used
  # in quarto code
  rlang::check_installed("ResultModelManager")
  rlang::check_installed("htmltools")
  
  if(missing(outputLocation)){
    stop('Must enter location for outputLocation')
  }
  
  # add code for gt?
  test <- gt::gt
  
  # add dummy code for CirceR as that is used in quarto
  # so needs to be in Description and used in the R folder
  CirceROptions <- CirceR::createGenerateOptions()
  
  templateLoc <- system.file(
    'templates','full-report', 
    package = "OhdsiReportGenerator"
  )
  
  if(!dir.exists(file.path(intermediateDir, 'full-report'))){
    dir.create(file.path(intermediateDir, 'full-report'), recursive = TRUE)
  }
  
  # add the char folder
  if(!dir.exists(file.path(intermediateDir, 'full-report', 'characterization'))){
    dir.create(file.path(intermediateDir, 'full-report','characterization'), recursive = TRUE)
  }
  
  filesOfInt <- c(
    dir(templateLoc, pattern = '.qmd', recursive = TRUE)
  )
  
  file.copy(
    from = file.path(templateLoc, filesOfInt), 
    to = file.path(file.path(intermediateDir, 'full-report'), filesOfInt)
  )
  
  quarto::quarto_render(
    input = file.path(intermediateDir, 'full-report', "main_template.qmd"), 
    execute_params = list(
      server = server,
      username = username,
      password = password,
      dbms = dbms,
      schema = resultsSchema,
      targetId = targetId,
      outcomeIds = outcomeIds,
      indicationIds = indicationIds,
      restrictTargetToIndications = restrictTargetToIndications,
      comparatorIds = comparatorIds,
      cohortIds = cohortIds,
      cohortNames = cohortNames,
      includeCI = includeCI,
      includeCharacterization = includeCharacterization,
      includeCohortMethod = includeCohortMethod,
      includeSccs = includeSccs,
      includePrediction = includePrediction,
      webAPI = webAPI,
      authMethod = authMethod,
      webApiUsername = webApiUsername, 
      webApiPassword = webApiPassword,
      pathToDriver = pathToDriver
    )
  )
  
  # now move html output to output location
  if(!dir.exists(outputLocation)){
    dir.create(outputLocation, recursive = TRUE)
  }
  file.copy(
    from = file.path(intermediateDir, 'full-report', 'main_template.html'), 
    to = file.path(outputLocation, outputName)
  )
  
  return(file.path(outputLocation, outputName))
}

test_that("generateFullReport", {
  
  #if skipQuarto skip this on CRAN
  if(skipQuarto){
    testthat::skip_on_cran()
  }
  
  savLoc <- file.path(tempdir(), 'example2')
  if(!dir.exists(savLoc)){
    dir.create(savLoc)
  }
  
  generateFullReport(
    server = conDet$server(),
    username = conDet$user(),
    password = conDet$password(),
    dbms = conDet$dbms,
    resultsSchema = schema,
    
    targetId = 1,
    outcomeIds = 3,
    comparatorIds = 2,
    indicationIds = NULL,
    restrictTargetToIndications = FALSE,
    cohortNames = c('target','outcome','comp'),
    cohortIds = c(1,3,2),
    includeCI = TRUE,
    includeCharacterization = TRUE,
    includeCohortMethod = TRUE,
    includeSccs = FALSE,
    includePrediction = FALSE,
    webAPI = NULL, 
    authMethod = NULL, 
    outputLocation = savLoc,
    outputName = paste0('full_report_', gsub(':', '_',gsub(' ','_',as.character(date()))),'.html'),
    intermediateDir = tempdir()
  )
  
  # ensure report is generated
  testthat::expect_true(length(dir(savLoc, pattern = '.html', full.names = TRUE)) > 0)
  
  # clean up and remove file
  file.remove(dir(savLoc, pattern = '.html', full.names = TRUE))
  
})

# Causal Analyses {.break-slides}

## Risk Estimation Methods

::: {style="font-size: 50%;"}

### Comparative Cohort Design (Cohort Method)
Comparison of outcomes between two or more treatment cohorts, while controlling for other factors that may influence the results (here: via propensity score matching).

![Illustration of CohortMethod Design](pictures/cohort_method.png)

### Self-Controlled Case Series (SCCS)
Comparison of outcome incidence between time periods at risk (here: based on treatment exposure) and non-exposed periods; individuals serve as their own controls. 

![Illustration of SCCS design](pictures/sccs.png)

:::

---
output: html_document
---

```{r, echo=FALSE, results='asis'}
  
sql <- "select dm.cdm_source_abbreviation as DATABASE_NAME, 
a.PROBAST_ID, a.result_value 
from @result_schema.@my_table_appenddiagnostic_summary as a 
inner join @result_schema.@my_table_appenddiagnostics as b
on a.DIAGNOSTIC_ID = b.DIAGNOSTIC_ID
inner join @result_schema.@my_table_appenddatabase_details as c
on c.database_id = b.database_id
inner join @result_schema.@database_table_appenddatabase_meta_data as dm
on c.database_meta_data_id = dm.database_id
where b.model_design_id = @model_design_id
;"  
  
  res <- connectionHandler$queryDb(
    sql = sql, 
    result_schema = params$resultSchema,
    my_table_append = params$myTableAppend,
    database_table_append = params$databaseTableAppend,
    model_design_id = modelDesignId
  )
  
  if(nrow(res) >0 ){
  # pivot to get databases as columns 
  res <- tidyr::pivot_wider(
    data = res, 
    id_cols = 'probastId', 
    names_from = 'databaseName', 
    values_from = 'resultValue'
    )
  } else{
    
    res <- data.frame(
      probastId = c('1.1', '1.2.2', '1.2.4', '2.1', '2.2', '2.3','3.1','3.2',
                    '3.3', '3.4', '3.5', '3.6', '4.1'),
      none = c('Pass', rep('No diagnostics run', 2), 'Pass', rep('No diagnostics run', 5),
               'Pass', rep('No diagnostics run', 3) )
    )
    
  }


  probastDefs <- data.frame(
    rbind(
     c('1.1','Participants', 'Were appropriate data sources used, e.g. cohort, RCT or nested case-control study data?', 'PatientLevelPrediction uses a cohort design'),
     
     c('1.2.2','Participants','Were all inclusions and exclusions of participants appropriate?', "Check the demographic differences between inclusion criteria in popualtion settings and no additional inclusions (1 = similar, 0 = disimilar)"),
     
     c('1.2.4','Participants','Were all inclusions and exclusions of participants appropriate?', "Check the demographic differences between inclusion criteria in popualtion settings and no additional inclusions (1 = similar, 0 = disimilar)"),
     
     c('2.1','Predictors','Were predictors defined and assessed in a similar way for all participants?', "PatientLevelPrediction uses standardized feature extraction to consistently engineer the predictors"),
     
      c('2.2','Predictors','Were predictor assessments made without knowledge of outcome data?', "Rule check is the last date used to engineer predictors before the time-at-risk start? (if it is then pass)"),
     
   c('2.3','Predictors','Are all predictors available at the time the model is intended to be used?', "This fails if the last date used to engineer predictors is after index but this requires the user to think about their data and design to determine whether this passes"),

    c('3.1','Outcome','Was the outcome determined appropriately?', "This needs to be check via cohort diagnostic of the outcome cohort"),  
   
  c('3.2','Outcome','Was a pre-specified or standard outcome definition used?', "This passes if the outcome was from the OHDSI phenotype library (needs to be manually confirmed)"),

  c('3.3','Outcome','Were predictors excluded from the outcome definition?', "Check Kaplan Meier plots to see whether outcomes occur close to index (which may mean predictors and outcome overlap) or not"),
   
  c('3.4','Outcome','Was the outcome defined and determined in a similar way for all participants?', "PatientLevelPrediction determines the outcome based on an outcome phenotype, so the same logic is used for all patients."),   
  
  c('3.5','Outcome','Was the outcome determined without knowledge of predictor information?', "PatientLevelPrediction framework requires defining the outcome independently of the covariates - manually confirm ... "),
  
   c('3.6','Outcome','Was the time interval between predictor assessment and outcome determination appropriate?', "Rule: does the time-at-risk start on or after target index?  If so, pass."), 
  
   c('4.1','Design','Were there a reasonable number of participants with the outcome?', "Rule: If less than 100 fail.")
     
     
    )
    
  )
  
  names(probastDefs) <- c('probastId', 'probast category', 'consideration', 'explanation')
  row.names(probastDefs) <- NULL
  
  res <- merge(probastDefs, res, by = 'probastId', all.x = TRUE)

```
  
## Diagnostic

The PatientLevelPrediction objective assessment of PROBAST risk of bias:

```{r, echo=FALSE, result = 'asis'}
  knitr::kable(x = res, caption = 'Diagnostic results')
```

# References
Austin PC (2009). Balance diagnostics for comparing the distribution of baseline covariates between treatment groups in propensity-score matched samples. Stat Med. 2009 Nov 10;28(25):3083-107. doi: 10.1002/sim.3697. PMID: 19757444; PMCID: PMC3472075.

Schuemie MJ (2016), Hripcsak G, Ryan PB, Madigan D, Suchard MA. Robust empirical calibration of p-values using observational data. Statistics in medicine. 2016;35(22):3883-3888.

Schuemie MJ (2018), Hripcsak G, Ryan PB, Madigan D, Suchard MA. Empirical confidence interval calibration for population-level effect estimation studies in observational healthcare data. Proceedings of the National Academy of Sciences of the United States of America. 2018;115(11):2571-2577.

## Visualizations

The following visualizations will can be viewed per model:

```{r, echo=FALSE, result = 'asis'}
modelEvaluation <- data.frame(rbind(
    c("ROC Plot", "The ROC plot plots the sensitivity against 1-specificity on the test set. The plot shows how well the model is able to discriminate between the people with the outcome and those without. The dashed diagonal line is the performance of a model that randomly assigns predictions. The higher the area under the ROC plot the better the discrimination of the model."),
    c("Calibration Plot", "The calibration plot shows how close the predicted risk is to the observed risk. The diagonal dashed line thus indicates a perfectly calibrated model. The ten (or fewer) dots represent the mean predicted values for each quantile plotted against the observed fraction of people in that quantile who had the outcome (observed fraction). The straight black line is the linear regression using these 10 plotted quantile mean predicted vs observed fraction points. The two blue straight lines represented the 95% lower and upper confidence intervals of the slope of the fitted line."),
    c("Smooth Calibration Plot", "Similar to the traditional calibration shown above the Smooth Calibration plot shows the relationship between predicted and observed risk. the major difference is that the smooth fit allows for a more fine grained examination of this. Whereas the traditional plot will be heavily influenced by the areas with the highest density of data the smooth plot will provide the same information for this region as well as a more accurate interpretation of areas with lower density. the plot also contains information on the distribution of the outcomes relative to predicted risk.  However the increased information game comes at a computational cost. It is recommended to use the traditional plot for examination and then to produce the smooth plot for final versions."),
    c("Prediction Distribution Plots", "The preference distribution plots are the preference score distributions corresponding to i) people in the test set with the outcome (red) and ii) people in the test set without the outcome (blue)."),
    c("Box Plots", "The prediction distribution boxplots are box plots for the predicted risks of the people in the test set with the outcome (class 1: blue) and without the outcome (class 0: red)."),
    c("Test-Train Similarity Plot", "The test-train similarity is presented by plotting the mean covariate values in the train set against those in the test set for people with and without the outcome."),
    c("Variable Scatter Plot", "The variable scatter plot shows the mean covariate value for the people with the outcome against the mean covariate value for the people without the outcome. The size and color of the dots correspond to the importance of the covariates in the trained model (size of beta) and its direction (sign of beta with green meaning positive and red meaning negative), respectively."),
    c("Precision Recall Plot", "The precision-recall curve is valuable for dataset with a high imbalance between the size of the positive and negative class. It shows the tradeoff between precision and recall for different threshold. High precision relates to a low false positive rate, and high recall relates to a low false negative rate. High scores for both show that the classifier is returning accurate results (high precision), as well as returning a majority of all positive results (high recall). A high area under the curve represents both high recall and high precision."),
    c("Demographic Summary Plot", "This plot shows for females and males the expected and observed risk in different age groups together with a confidence area.")
  ))
  names(modelEvaluation) <- c("Plot","Description")
  row.names(modelEvaluation) <- NULL
  data <- modelEvaluation[order(modelEvaluation$Plot),]
  knitr::kable(x = data, caption = 'Standard Plots')
  
```
  

---
output: html_document
---
  
### Analysis

The model that will be trained is a ``r attr(modelSettings$param,'settings')$name `` that uses the `PatientLevelPrediction` function ``r modelSettings$fitFunction `` to fit the model.

```{r, echo=FALSE}
if(modelSettings$fitFunction == "fitCyclopsModel"){
 parameters <- data.frame(
   name = names(modelSettings$param),
   value = unlist(
     lapply(modelSettings$param, function(x) paste(names(x), x, collapse = '-', sep=':'))
   )
 )
 
 settings <- data.frame(
   name = names(attr(modelSettings$param,"settings")),
   value = unlist(
     lapply(
       attr(modelSettings$param,"settings"), 
       function(x) paste0(names(x), x, collapse = ':', sep=' ')
            )
     )
 )
 row.names(settings) <- NULL
} else{
   parameters <- 
       do.call('rbind', lapply(
       modelSettings$param, 
       function(x){
         unlist(lapply(x, function(x) paste0(x, sep=' ', collapse=':')))
       })
     )
 
 settings <- data.frame(
   name = names(attr(modelSettings$param,"settings")),
   value = unlist(
     lapply(
       attr(modelSettings$param,"settings"), 
       function(x) paste0(names(x), x, collapse = '-', sep='')
            )
     )
 )
 row.names(settings) <- NULL
}
```

**Cross-validation settings**

The cross validation settings are to use ``r splitSettings$nfold `` folds in the training data that are partitioned using the ``r attr(splitSettings,"fun") `` function and consist of ``r splitSettings$train*100 `` \% of the complete data.  The seed used for splitting the data is ``r splitSettings$seed ``.

**Hyper-parameter search**

The hyper-parameters investigated while fitting the model are listed below.  The combination of hyper-parameters that obtains the highest AUROC value in the training data via cross validation will be uses in the final model.

```{r, echo=FALSE, results='asis'}
print(knitr::kable(x = parameters, caption = paste('Hyper-parameters combinations searched to fit the model')))
```

**Other settings**

The other settings used to fit the model, such as seeds used for reproducibility, are:

```{r, echo=FALSE, results='asis'}
print(knitr::kable(x = settings, caption = paste('Other model fitting settings')))
```

**Internal validation**

The model will be assessed internally using a test set that consists of ``r splitSettings$test*100`` \% of the complete data.

### Index breakdown

```{r}
#| label: characterization_index_target
#| echo: false
#| results: 'asis'

# check ROhdsiWebApi installed and skip otherwise
if(system.file(package = "ROhdsiWebApi") != ""){

# TARGET
  
targetIds <- cohortDefinitions %>%
  dplyr::filter(.data$subsetParent == params$targetId) %>%
  dplyr::arrange(.data$order, .data$cohortDefinitionId) %>%
  dplyr::select("cohortDefinitionId") 
targetIds <- targetIds$cohortDefinitionId

for(i in 1:length(params$outcomeIds)){
  jsonList <- ParallelLogger::convertJsonToSettings(json = cohortDefinitions$json[cohortDefinitions$cohortDefinitionId == params$outcomeIds[i]])
  
  if(!is.null(params$authMethod)){
  ROhdsiWebApi::authorizeWebApi(
    baseUrl = params$webAPI, 
    authMethod = params$authMethod, 
    webApiUsername = params$webApiUsername, 
    webApiPassword = params$webApiPassword
  )
  }
  
  conceptsInOutcome <- c()
  if(length(jsonList$ConceptSets)>0){
    for(jListInd in 1:length(jsonList$ConceptSets)){
      conceptsInOutcome <- unique(c(conceptsInOutcome, ROhdsiWebApi::resolveConceptSet(
        conceptSetDefinition = jsonList$ConceptSets[[jListInd]], 
        baseUrl = params$webAPI
      )))
    }
  }

  for(j in 1:length(targetIds)){
    
    res <- OhdsiReportGenerator::getBinaryCaseSeries(
      connectionHandler = connectionHandler, 
      schema = params$schema, 
      targetId = targetIds[j], 
      outcomeId = params$outcomeIds[i],
      conceptIds = unique(conceptsInOutcome),
      minVal = 0.01
    )

        if(nrow(res)>0){
    
    res <- OhdsiReportGenerator::addTarColumn(res) 
    
    # make the covariateName nicer
    res <- OhdsiReportGenerator::formatBinaryCovariateName(res)
    
    counts <- OhdsiReportGenerator::getCaseCounts(
      connectionHandler = connectionHandler, 
      schema = params$schema, 
      targetIds = targetIds[j], 
      outcomeIds = params$outcomeIds[i] 
    ) 
    counts <- OhdsiReportGenerator::addTarColumn(counts) %>%
      dplyr::select("databaseName", "personCount", "tar")
    
    index <- res %>% 
      dplyr::filter(.data$type == 'During') %>%
      dplyr::select("databaseName", "targetCohortId", "outcomeCohortId",
                    "covariateName", 
                    "averageValue", "tar") %>% 
      dplyr::inner_join(counts, by = c("databaseName", "tar"))
    
    aggreggateIndex <- index %>% 
      dplyr::group_by(.data$targetCohortId, .data$outcomeCohortId, .data$covariateName, .data$tar) %>%
      dplyr::summarise(
        averageValue = mean(.data$averageValue, na.rm = TRUE),
        personCount = 0
      ) %>%
      dplyr::mutate(
        databaseName = 'all'
      )
    
    # add the combined that will be used to order table
    index <- dplyr::bind_rows(index, aggreggateIndex)
    
  tempCd <- cohortDefinitions[,c("friendlyName","subsetText","cohortDefinitionId")]
colnames(tempCd) <- paste0(colnames(tempCd), 'Target')
index <- merge(index, tempCd, by.x = 'targetCohortId', by.y = 'cohortDefinitionIdTarget')
tempCd <- cohortDefinitions[,c("friendlyName","subsetText","cohortDefinitionId")]
colnames(tempCd) <- paste0(colnames(tempCd), 'Outcome')
index <- merge(index, tempCd, by.x = 'outcomeCohortId', by.y = 'cohortDefinitionIdOutcome')

ibOutcome <- unique(index$friendlyNameOutcome)

# now we add the N to the databaseName we need to loop over TAR
for(ibTar in unique(index$tar)){

indexTar <- tidyr::pivot_wider(
  data = index %>%
    dplyr::filter(.data$tar == !!ibTar) %>% 
    dplyr::select(-"tar") %>%
    dplyr::mutate(
      databaseName = paste0(.data$databaseName, ' (N = ', 
                            ifelse(.data$personCount >= 0 ,
                                   .data$personCount,
                                   paste0('<',abs(.data$personCount))
                                   ),')')
      ) 
    , 
  names_from = "databaseName", 
  #values_from = c("sumValue","averageValue"), 
  values_from = c("averageValue"), 
  id_cols = c("covariateName","friendlyNameTarget","subsetTextTarget")
) %>%
    dplyr::arrange(-1*.data$`all (N = 0)`) %>%
  dplyr::select(-"all (N = 0)") %>%
  dplyr::group_by(
    .data$friendlyNameTarget,.data$subsetTextTarget
  )

# display

boxName <- paste0('Frequency of ',ibOutcome,' concept codes when the patient exposed to ', indexTar$friendlyNameTarget[1], ' ', indexTar$subsetTextTarget[1],' has the outcome during ', ibTar)

  cat('\n\n::: {.callout-note collapse="true"}')
  cat(paste0("\n# ",boxName, '\n\n'))
      
print(htmltools::tagList(
  indexTar  %>%  
    gt::gt() %>%
    gt::fmt_percent(
      columns = colnames(indexTar)[! colnames(indexTar) %in% c("covariateName","friendlyNameTarget","subsetTextTarget", "friendlyNameOutcome", "tar")],
      decimals = 2
    ) %>%
    gt::cols_label(
      covariateName = "Covariate"
    ) %>%
    gt::tab_options(
      heading.align = "left",
      data_row.padding = gt::px(7),
      column_labels.font.size = gt::px(12),
      column_labels.background.color = "#EB1800",
      column_labels.font.weight = "bold",
      row_group.background.color = "#D3D3D3",
      row_group.font.weight = "bold"
    ) %>%
    gt::tab_style(
      style = gt::cell_text(
        color = "darkgrey",
        font = gt::google_font("Source Sans Pro"),
        transform = "uppercase"
      ),
      locations = gt::cells_column_labels(gt::everything())
    ) %>%
    gt::tab_header(
      title = paste0('Frequency of ',ibOutcome,' concept codes when the patient in the target cohort has the outcome during ', ibTar)
    )
))

cat('\n\n:::\n\n')

} # end tar loop
  
    } # if res has results
  
  }


}

}

```

## Comments

::: {style="font-size: 70%;"}

- There were several potential limitations to this analysis, which concern the provenance, purpose, and use of the very diverse datasets considered for this analysis:

- Our analysis observed data from those subjects who presented for medical attention; those who did not seek medical attention could not be included and may affect the metrics in this study. Those with milder disease may also not be presented or be underrepresented.

- There might be coding inaccuracies in the recording of diagnose and procedures, as the data were originally not collected for research purposes; the databases might, e.g., not accurately capture all events and event dates.

- Furthermore, findings in uninsured populations, populations with restricted healthcare access, or other countries and healthcare systems may differ from the findings in this project. 

:::

---
title: "Using Observational Health Data Sciences and Informatics (OHDSI) Report Generator"
author: "Jenna Reps, Anthony Sena"
date: "`r Sys.Date() `"
output: html_document
vignette: >
  %\VignetteIndexEntry{guide}
  %\VignetteEngine{knitr::knitr}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
library(dplyr)
```

# Using the OhdsiReportGenerator Package

This package enables users to extract characterization, estimation and prediction results from the OHDSI results (see [result model](https://ohdsi.github.io/Strategus/results-schema/index.html)) generated via R packages such as `PatientLevelPrediction`, `Characterization`, `CohortMethod` and `SelfControlledCaseSeries`.

## Example Result Database

This package provides users with an example OHDSI result database.  This can be accessed via `getExampleConnectionDetails()` and contains the results of running a demo analysis using the Eunomia R package sample OMOP CDM data.    

```{r echo=TRUE}
library(OhdsiReportGenerator)
# create a connection details object to 
connectionDetails <- getExampleConnectionDetails()

# create a connection handler to the results
library(ResultModelManager)
ConnectionHandler <- ResultModelManager::ConnectionHandler$new(connectionDetails)

```

## Extracting Cohorts

The cohort details can be extracted using the function `getCohortDefinitions` that requires the user to input the connection handler, schema, table prefix used by CohortGenerator (default is 'cg\_') and target cohort ids to restrict to.

The example results are in a sqlite database and use the schema 'main'.

```{r echo=TRUE}

cohorts <- getCohortDefinitions(
    connectionHandler = ConnectionHandler,
    schema = 'main',
    cgTablePrefix = 'cg_',
    targetIds = NULL
)

knitr::kable(
  x = cohorts %>% 
    dplyr::select(-"json", -"sqlCommand"), 
  caption = 'The data.frame extracted containing the cohort details minus the json and sqlCommand columns.'
  )

```

You can process the cohorts definitions to extract the parent cohorts and the children for each parent cohort.  

```{r echo=TRUE}
if(nrow(cohorts) > 0){
processedCohorts <- processCohorts(cohorts)

knitr::kable(
  x = data.frame(
    parentId = processedCohorts$parents,
    parentName= names(processedCohorts$parents)
    ), 
  caption = 'The parent cohorts.'
  )

knitr::kable(
  x = processedCohorts$cohortList[[1]] %>% 
    dplyr::select(-"json", -"sqlCommand"),
  caption = 'The children/subset cohorts for the first parent cohort.'
  )
}

```

For example, if you created a cohort corresponding to all patients exposed to drug A with an id of 1 and then created 'children' subset cohorts corresponding to patients exposed to drug A for indication X with an id 1001 and patients exposed to drug A aged 60+ with a cohort id 1002, then `getCohortDefinition()` would extract all three of these cohorts and `processCohorts()` would identify cohort 1 as the parent and cohorts 1,1001,1002 as the children/subset of cohort 1.

You can also extract the subset logic for a subset id:

```{r echo=TRUE}

subsets <- getCohortSubsetDefinitions(
    connectionHandler = ConnectionHandler,
    schema = 'main',
    cgTablePrefix = 'cg_'
)

knitr::kable(
  x = subsets, 
  caption = 'The subset cohort logic used in the analysis.'
  )

```


## Extracting Characterization Results

The characterization analysis contains time to event results, dechallenge-rechallenge, risk factor and cohort incidence results.

The time-to-event results tell you how often a outcome occurs relative to some target cohort index date on a 1-day, 30-day and 365-day scale.

```{r echo=TRUE}

tte <- getTimeToEvent(
    connectionHandler = ConnectionHandler,
    schema = 'main'
)

knitr::kable(
  x = tte %>% dplyr::filter(.data$timeScale == 'per 365-day'), 
  caption = 'Example time-to-event results for the 365-day scale.'
  )

```

The dechallenge-rechallenge analysis shows you how often an outcome occurs during some target exposure and then how often the target exposure is stopped shortly after the outcome and whether the outcome re-occurs when the target exposure restarts:

```{r echo=TRUE}

drc <- getDechallengeRechallenge(
    connectionHandler = ConnectionHandler,
    schema = 'main'
)

knitr::kable(
  x = drc, 
  caption = 'Example dechallenge-rechallenge results.'
  )

```

It is also possible to find the incidence rates (where we restrict to all ages, genders and start years):

```{r echo=TRUE}

ir <- getIncidenceRates(
    connectionHandler = ConnectionHandler,
    schema = 'main', 
    targetIds = 1
)

knitr::kable(
  x = ir %>% dplyr::filter(
    subgroupName == 'All' &
    ageGroupName == 'Any' &	
    genderName == 'Any' &	
    startYear == 'Any'
  ), 
  caption = 'Example incidence rate results.'
  )

```

Finally, it is possible to get risk factors (associations between features and the occurrence of the outcome during a time-at-risk) using `getBinaryRiskFactors` to identify the binary features:

```{r echo=TRUE}

rf <- getBinaryRiskFactors(
    connectionHandler = ConnectionHandler,
    schema = 'main', 
    targetId = 1, 
    outcomeId = 3
)

knitr::kable(
  x = rf, 
  caption = 'Example risk factors for binary features results.'
  )

```

and `getContinuousRiskFactors` for the continuous features:

```{r echo=TRUE}

rf <- getContinuousRiskFactors(
    connectionHandler = ConnectionHandler,
    schema = 'main', 
    targetId = 1, 
    outcomeId = 3
)

knitr::kable(
  x = rf, 
  caption = 'Example risk factors for continuous features results.'
  )

```

## Extracting Prediction Results

The model designs used to develop prediction models in an analysis can be extracted using `getPredictionModelDesigns`:

```{r echo=TRUE}

modelDesigns <- getPredictionModelDesigns(
    connectionHandler = ConnectionHandler,
    schema = 'main', 
    targetIds = 1002, 
    outcomeIds = 3
)

knitr::kable(
  x = modelDesigns %>%
    dplyr::select(
      "modelDesignId",	
      "modelType",
      "developmentTargetId",
      "developmentTargetName",
      "developmentOutcomeId",	
      "developmentOutcomeName",	
      "timeAtRisk"
    ), 
  caption = 'Example model designs for the prediction results.'
  )

```

Once you know the modelDesignId of interest, you can then extract the model performance:

```{r echo=TRUE}

perform <- getFullPredictionPerformances(
    connectionHandler = ConnectionHandler,
    schema = 'main', 
    modelDesignId = 1
)

knitr::kable(
  x = perform, 
  caption = 'Example performance for the prediction results.'
  )

```

As well as the top predictors, in this case we will return the top 5:

```{r echo=TRUE}

top5 <- getPredictionTopPredictors(
    connectionHandler = ConnectionHandler,
    schema = 'main', 
    targetIds = 1002, 
    outcomeIds = 3,
    numberPredictors = 5
)

knitr::kable(
  x = top5 , 
  caption = 'Example top five predictors.'
  )

```

You can also select certain results such as the attrition:

```{r echo=TRUE}

attrition <- getPredictionPerformanceTable(
    connectionHandler = ConnectionHandler,
    schema = 'main', 
    table = 'attrition',
    performanceId = 1
)

knitr::kable(
  x = attrition , 
  caption = 'Example prediction attrition for performance 1.'
  )

```


## Extracting Estimation Results

It is possible to extract cohort method and self controlled case series results and make nice plots.

To extract all the cohort method estimations for target id 1002 and outcome 3: 
```{r echo=TRUE}

cmEst <- getCMEstimation(
    connectionHandler = ConnectionHandler,
    schema = 'main', 
    targetIds = 1002, 
    comparatorIds = 2002,
    outcomeIds = 3
)

knitr::kable(
  x = cmEst , 
  caption = 'Example cohort method estimation results.'
  )

```

You can also extract any meta-analysis estimate for cohort method:

```{r echo=TRUE}

cmMe <- getCmMetaEstimation(
    connectionHandler = ConnectionHandler,
    schema = 'main'
)

knitr::kable(
  x = cmMe , 
  caption = 'Example cohort method meta analysis estimation results.'
  )

```

and you can create a nice plot to show the results:

```{r echo=TRUE}

plotCmEstimates(
  cmData = cmEst, 
  cmMeta = NULL,
  cohortNames = data.frame(
    cohortName = c('Example Target', 'Example Comp'),
    cohortId= c(1002,2002)
  ),
  selectedAnalysisId = 1
)


```

---
output: html_document
---


```{r results='asis', echo=FALSE}

sql <- "select distinct 
               p.development_database_id as dbval,
               dm.cdm_source_abbreviation as dbname
      from 
      @result_schema.@my_table_appendperformances p
      inner join @result_schema.@my_table_appenddatabase_details d
      on d.database_id = p.development_database_id
      inner join @result_schema.@database_table_appenddatabase_meta_data dm
      on d.database_meta_data_id = dm.database_id
      where p.model_design_id = @model_design_id 
      and p.model_development = 1 
      ;
    "
 
  res <- connectionHandler$queryDb(
    sql = sql, 
    result_schema = params$resultSchema,
    my_table_append = params$myTableAppend,
    database_table_append = params$databaseTableAppend,
    model_design_id = params$modelDesignIds[i]
  )
  
  # get all the databases with models
  devDatabases <- res$dbval

    if(length(devDatabases)>0){
    for (databaseInd in 1:length(devDatabases)) {
      devDatabase <- devDatabases[databaseInd]
      
      cat('\n## ', paste0(res$dbname[databaseInd], ' model results'), '\n')
      
      internalRes <- knitr::knit_child("plp-results-internal.Rmd", quiet = TRUE, envir = environment())
      cat(internalRes, sep = '\n')
      
      externalRes <- knitr::knit_child("plp-results-external.Rmd", quiet = TRUE, envir = environment())
      cat(externalRes  , sep = '\n')
      

    }
    }

```

# Cohorts

```{r}
#| label: cohort definitions target heading
#| echo: false
#| results: 'asis'

cat(paste0('\n\n## Target ',targetName, ' {#sec-target}\n\n'))

cat(paste0('> Full name: ',targets$cohortName[targets$cohortDefinitionId == targetId]))

cat(paste0('\n\n::: {.callout-note title="Cohort Definition"}'))
cat(paste0('\n\n### Cohort Definition\n\n'))

json <- targets$json[targets$cohortDefinitionId == params$targetId]

markdown <- tryCatch(
  {CirceR::cohortPrintFriendly(json)}, 
  error = function(e) return('')
)

cat(markdown)
cat('\n\n:::')

# if subset is not null then add that here
if(FALSE){ # figure out why this causes issue
#if(!is.null(cmSubsetIds)){
  
  for(i in 1:length(cmSubsetIds)){
    json <- subsets$json[subsets$subsetDefinitionId == cmSubsetIds[i]]

  cat("\n\n::: {.callout-note title='Subset'}\n")
  cat(paste0('\n\n### Subset ', i ,' used in CM'))
  
  cat('\n\n')
  cat(cmSubsetIds[i])
  cat(' Add json parsed code here')
  cat('\n\n')
  cat('\n\n:::\n\n')
  
  }
}
  
  if(!is.null(sccsSubsetIds)){
   
  for(i in 1:length(sccsSubsetIds)){
    json <- subsets$json[subsets$subsetDefinitionId == sccsSubsetIds[i]]

  cat("\n\n::: {.callout-note title='Subset'}")
  cat(paste0('\n\n### Subset ', i ,' used in SCCS'))
  
  cat('\n\n')
  cat('Add json parsed code here')
  cat('\n\n:::')
  
  }
}

```



```{r}
#| label: outcomes
#| echo: false
#| results: 'asis'
# Outcome

for(i in 1:nrow(outcomes)){
  cat(paste0('\n\n## Outcome ',outcomes$cohortFriendlyName[i], ' {#sec-',OhdsiReportGenerator::removeSpaces(outcomes$cohortFriendlyName[i]),'}\n\n'))
  
  cat(paste0("\n\n> Full name: ", outcomes$cohortName[i]))
  cat('\n\n::: {.callout-note title="Cohort Definition"}')
  cat('\n\n### Cohort Definition \n')
  
  json <- outcomes$json[i]
  
  markdown <- tryCatch(
    {CirceR::cohortPrintFriendly(json)}, 
    error = function(e) return('')
  )
  cat(markdown)
  cat('\n\n:::')
}

```


```{r}
#| label: indication
#| echo: false
#| results: 'asis'

if(!is.null(indications)){
  # Indications

  for(i in 1:nrow(indications)){
    cat(paste0('\n\n## Indication ',indications$cohortFriendlyName[i], ' {#sec-',OhdsiReportGenerator::removeSpaces(indications$cohortFriendlyName[i]),'}\n\n'))
  
  cat(paste0("\n\n> Full name: ", indications$cohortName[i]))
  cat('\n\n::: {.callout-note title="Cohort Definition"}')
  cat("\n\n### Cohort Definition\n\n")
  
  json <- indications$json[i]
  
  markdown <- tryCatch(
    {CirceR::cohortPrintFriendly(json)}, 
    error = function(e) return('')
  )
  cat(markdown)
  cat('\n\n:::')
  }
}
```



```{r}
#| label: comparators
#| echo: false
#| results: 'asis'

if(!is.null(comparators)){
# Comparator

for(i in 1:nrow(unique(comparators))){
  cat(paste0('\n\n## Comparator ',comparators$cohortFriendlyName[i], ' {#sec-',OhdsiReportGenerator::removeSpaces(comparators$cohortFriendlyName[i]),'}\n\n'))
  
  cat(paste0('\n\n> Full name:', comparators$cohortName[i]))
  
  cat('\n\n::: {.callout-note title="Cohort Definition"}')
  cat('\n\n### Cohort Definition\n')
  
  json <- comparators$json[i]
  
  markdown <- tryCatch(
    {CirceR::cohortPrintFriendly(json)}, 
    error = function(e) return('')
  )
  cat(markdown)
  cat('\n\n:::')
  
  # add same subset stuff as target has here?
}
}

```

## {{title}}

:::: {.columns}
::: {.column width="70%"}

:::: {style="font-size: 50%;"}

### Background: 
{{trigger}}

### General safety question
{{safetyQuestion}}

### ASSURE request/objective: 
{{objective}}

:::: 

:::

::: {.column width="30%"}

:::: {style="font-size: 50%;"}
- Assure lead: {{lead}}
- Assure team: {{team}}
- Date: {{date}}
::::

:::

::::

::: footer

Analysis information and ASSURE team details.

:::

```{r}
#| echo: false
#| results: 'asis'
#| fig-width: 12
#| fig-height: 10

 predictors <- OhdsiReportGenerator::getPredictionAggregateTopPredictors(
    connectionHandler = connectionHandler, 
    schema = schema,
    plpTablePrefix = params$plpTablePrefix,
    databaseTable = 'database_meta_data',
    cgTablePrefix = params$cgTablePrefix, 
    #databaseTablePrefix = params$databaseTablePrefix,   
    modelDesignIds = modelDesignOfIntCohorts$modelDesignId
    )
  
cat(paste0('\n\n### ', TA))
cat('\n\n')

if(nrow(predictors) > 0 ){

predictors$covariateName <- gsub(
  'cohort during day -9999 through -1 days relative to index:', 
  '',
  predictors$covariateName
  )
predictors$covariateName <- gsub(
  'cohort during day -365 through 0 days relative to index:', 
  '',
  predictors$covariateName
  )

# total across databases for order
predictorsOrder <- predictors %>%
  dplyr::group_by(.data$covariateName) %>%
  dplyr::summarise(
    N = sum(.data$numberOfTimesPredictive)
  )

predictors$covariateName <- factor(
  predictors$covariateName, 
  levels = predictorsOrder$covariateName[order(predictorsOrder$N)]
    )

ggplot2::ggplot(
  data = predictors, 
  mapping = ggplot2::aes(x=.data$covariateName, y=.data$numberOfTimesPredictive)
  ) +
  ggplot2::geom_bar(stat='identity', na.rm=TRUE) +
  ggplot2::coord_flip() +
  ggplot2::facet_grid(. ~ .data$databaseName, scales = 'free_x') +
  ggplot2::theme(
    axis.title.y = ggplot2::element_blank(),
    axis.text = ggplot2::element_text(size=6)
    #,strip.text.y.right = ggplot2::element_text(angle = 0)
    )

}
cat('\n\n')
  
```

```{r}
#| label: q2_get_data_{{TA}}
#| echo: false
#| results: 'asis'


modelPerf <- OhdsiReportGenerator::getPredictionPerformances(
  connectionHandler = connectionHandler, 
  schema = schema, 
  plpTablePrefix = params$plpTablePrefix, 
  cgTablePrefix = params$cgTablePrefix, 
  databaseTable = 'database_meta_data', 
  databaseTablePrefix = params$databaseTablePrefix, 
  modelDesignId = modelDesignIdsOfInt$modelDesignId, 
  developmentDatabaseId = NULL
    )


```

```{r}
#| label: question_2_plot_{{TA}}
#| fig-cap: '{{TA}}'
#| fig-width: 12
#| echo: false
#| lightbox:
#|   group: outcome_dist
#|   description: The distribution of outcome counts

if(nrow(modelPerf) > 0){
    dataPlot <- modelPerf %>%
    ggplot2::ggplot(ggplot2::aes(x=.data$outcomeCount)) +
      ggplot2::geom_histogram(binwidth = 25, fill="#0000FF") + 
      ggplot2::geom_density(
        fill="grey", 
        color="#e9ecef", 
        alpha=0.8, 
        na.rm=TRUE
        )
    
    data <- suppressWarnings(ggplot2::ggplot_build(dataPlot))
    data <- data.frame(data$data[[1]],stringsAsFactors = FALSE)
    
    # cut the x-axis at 10,000 as this will be small and 
    # makes it easier to view
    data <- data[data$x < 10000, ]
    
    ggplot2::ggplot(data = data, ggplot2::aes(.data$x,.data$y))+
     ggplot2::geom_line(na.rm=TRUE) + 
     ggplot2::geom_area(
       data = subset(data,data$x<100),
       ggplot2::aes(.data$x,.data$y),
       fill = "darkred", 
       alpha = 0.7, 
       na.rm=TRUE
       ) +
     ggplot2::geom_area(
       data = subset(data,data$x>100 & data$x<1000),
       ggplot2::aes(.data$x,.data$y),
       fill = "red", 
       alpha = 0.3, 
       na.rm=TRUE
       ) +
    suppressWarnings(
      ggplot2::geom_area(
        data = subset(data,data$x>1000),
        ggplot2::aes(.data$x,.data$y),
        fill = "grey", 
        alpha = 0.7, 
        na.rm=TRUE
      )
    ) +
    ggplot2::geom_vline(
      xintercept = 100, 
      linetype="dotted", 
      color = "black", 
      na.rm=TRUE
      ) +
    ggplot2::geom_vline(
      xintercept = 1000, 
      linetype="dotted", 
      color = "black", 
      na.rm=TRUE) +
      ggplot2::labs(
        y ="Density", 
        x = "Patients with outcome"
        )
    
    }
```


::: {.callout-important icon=false}
## <100 outcomes
```{r}
#| label: q2_box1_{{TA}}
#| echo: false
#| results: 'asis'
if(nrow(modelPerf) > 0){
cat(paste0(round(sum(modelPerf$outcomeCount < 100, na.rm = TRUE)/length(modelPerf$outcomeCount)*100, digits = 1), 
'% of the time.')
)} else{
  cat('No models')
}
```
:::

::: {.callout-important icon=false}
## 1000+ outcomes
```{r}
#| label: q2_box2_{{TA}}
#| echo: false
#| results: 'asis'
if(nrow(modelPerf) > 0){
cat(paste0(round(sum(modelPerf$outcomeCount >= 1000, na.rm = TRUE)/length(modelPerf$outcomeCount)*100, digits = 1), 
'% of the time.')
)} else{
  cat('No models')
}
```
:::

---
title: "ASSURE Full Report"
format:
  html:
    page-layout: full
    toc-location: right
    grid:
      sidebar-width: 0px
      body-width: 1200px
self-contained: true
toc: true
toc-depth: 4
toc-title: "Contents"
number-sections: true
prefer-html: true
execute:
  echo: false
  warning: false
params: 
  server: NULL
  password: NULL
  username: NULL
  dbms: "postgresql"
  schema: 'main'
  targetId: 1
  outcomeIds: 3
  indicationIds: NULL
  restrictTargetToIndications: false
  comparatorIds: NULL
  cohortIds: !expr "c(1,3,2)"
  cohortNames: !expr "c('target', 'outcome', 'comp')"
  includeCI: true
  includeCharacterization: true
  includeCohortMethod: true
  includeSccs: true
  includePrediction: true
  numberPredictors: 25
  excludeText: NULL
  timeRestrictionText: NULL
  drugGap: 30
  webAPI: NULL
  authMethod: NULL
  webApiUsername: NULL 
  webApiPassword: NULL
  pathToDriver: !expr "Sys.getenv('DATABASECONNECTOR_JAR_FOLDER')"
---
  
```{r}
#| label: configuration
#| echo: false
#| hide: true

library(dplyr)

# note: the targetName and outcomeName below currently requires
# specifying friendly names
targetName <- params$cohortNames[params$cohortIds == params$targetId]
outcomeName <- paste0(params$cohortNames[params$cohortIds %in% params$outcomeIds], collapse = '/')

restrictTargetToIndications <- params$restrictTargetToIndications
indicationIds <- params$indicationIds

#====================================
# first create connection details
#====================================

connectionDetails <- DatabaseConnector::createConnectionDetails(
  dbms = params$dbms,
  user = params$username, 
  password = params$password,
  server = params$server, 
  pathToDriver =  params$pathToDriver
)
#====================================

#====================================
# Figure out what sections to include in report
#====================================

# get prefixes - this only works with standard prefixes
con <- DatabaseConnector::connect(
  connectionDetails = connectionDetails
  )
tableNames <- DatabaseConnector::getTableNames(
  connection = con,
  databaseSchema = params$schema
)
DatabaseConnector::disconnect(con)

prefixes <- unique(unlist(lapply(tableNames, function(x) strsplit(x,'_')[[1]][1])))

includeCI <- 'ci' %in% prefixes & params$includeCI
includeCharacterization <- 'c' %in% prefixes & params$includeCharacterization
includeCohortMethod <- 'cm' %in% prefixes & params$includeCohortMethod 
includeSccs <- 'sccs' %in% prefixes & params$includeSccs 
includePrediction <- 'plp' %in% prefixes & params$includePrediction 
#====================================

#====================================
# create connection handler
#====================================
connectionHandler <- ResultModelManager::ConnectionHandler$new(
  connectionDetails = connectionDetails 
)
#====================================


# ===================================
# Get Cohorts
# ===================================

cohortDefinitions <- OhdsiReportGenerator::getCohortDefinitions(
  connectionHandler = connectionHandler, 
  schema = params$schema 
)

# code to process the cohortDefinitions - add friendly names etc...
cohortDefinitions <- OhdsiReportGenerator::processCohortDefinitionsForQuarto(
  cohortDefinitions = cohortDefinitions,
  friendlyCohortIds = params$cohortIds,
  friendlyCohortNames = params$cohortNames,
  restrictTargetToIndications = restrictTargetToIndications,
  indicationIds = params$indicationIds
)

# ===================================

# find the cohort ids of interest to report
restrictedCohorts <- OhdsiReportGenerator::restrictCohortDefinitionsForQuarto(
  connectionHandler = connectionHandler,
  schema = params$schema,
  cohortDefinitions = cohortDefinitions,
  targetId = params$targetId,
  outcomeIds = params$outcomeIds,
  comparatorIds = params$comparatorIds,
  restrictTargetToIndications = restrictTargetToIndications,
  indicationIds = params$indicationIds,
  includeCohortMethod = includeCohortMethod
)
targetIdsOfInterest <- restrictedCohorts$targetIdsOfInterest # includes children
comparatorIds <- restrictedCohorts$comparatorIds # may get replaced by CM comps if NULL
comparatorIdsOfInterest <- restrictedCohorts$comparatorIdsOfInterest # includes children
outcomeIdsOfInterest <- restrictedCohorts$outcomeIdsOfInterest
indicationIdsOfInterest <- restrictedCohorts$indicationIdsOfInterest


# for each  comps in CM get the name
comparatorText <- paste0(unique(cohortDefinitions$friendlyName[cohortDefinitions$cohortDefinitionId %in% comparatorIds]), collapse = '/')

# ===================================

# add order column
orderDf <- data.frame(
  subsetParent = c(
    params$targetId,
    params$outcomeIds, 
    params$indicationIds,
    comparatorIds
  ),
  order = 1:length(c(
    params$targetId,
    params$outcomeIds, 
    params$indicationIds,
    comparatorIds
  ))
)

cohortDefinitions <- 
  merge(
    cohortDefinitions,
    orderDf, 
    by.x = 'subsetParent', 
    by.y = 'subsetParent', 
    all.x = TRUE
  )

#nNegativeControls <- 100 # remove?



```

```{r}
#| label: summary
#| echo: false
#| results: 'asis'

cat(
  knitr::knit_child('summary.qmd', quiet = TRUE, envir = parent.frame()),
  sep = '\n'
)

```

```{r}
#| label: data
#| echo: false
#| results: 'asis'

cat(
  knitr::knit_child('data.qmd', quiet = TRUE, envir = parent.frame()),
  sep = '\n'
)

```

```{r}
#| label: definitions
#| echo: false
#| results: 'asis'

cat(
  knitr::knit_child('definitions.qmd', quiet = TRUE, envir = parent.frame()),
  sep = '\n'
)

```


```{r}
#| label: cohort_incidence
#| echo: false
#| results: 'asis'

if(includeCI){
  cat(
  knitr::knit_child('cohort_incidence.qmd', quiet = TRUE, envir = parent.frame()),
  sep = '\n'
)
}
```



```{r}
#| label: characterization
#| echo: false
#| results: 'asis'

if(includeCharacterization){
  # ===================================
result <- knitr::knit_child(text = knitr::knit_expand("characterization.qmd", quiet = TRUE, envir = environment()), quiet = TRUE)
cat(result, sep = '\n')
}
```


```{r}
#| label: causal_inference
#| echo: false
#| results: 'asis'

if(includeSccs || includeCohortMethod){
if(includeCohortMethod){
  # ===================================
  # Get CohortMethod
  # ===================================
  
  
  # TODO: restrict to comparatorIdsOfInterest?
  cohortMethodDiagnostics <- OhdsiReportGenerator::getCmDiagnosticsData(
    connectionHandler = connectionHandler, 
    schema = params$schema, 
    targetIds = unique(c(targetIdsOfInterest, indicationIdsOfInterest)),
    outcomeIds = outcomeIdsOfInterest
  )
  
  if(nrow(cohortMethodDiagnostics) > 0){
    if(!is.null(params$comparatorIds)){
    cohortMethodDiagnostics <- cohortMethodDiagnostics %>%
      dplyr::filter(.data$comparatorId %in% !!comparatorIdsOfInterest)
    }
  }
  
  if(nrow(cohortMethodDiagnostics) == 0){
    includeCohortMethod <- FALSE
  } else {
  
  # TODO: restrict to comparatorIdsOfInterest?
  cmResults <- OhdsiReportGenerator::getCMEstimation(
    connectionHandler = connectionHandler, 
    schema = params$schema, 
    targetIds = unique(c(targetIdsOfInterest, indicationIdsOfInterest)),
    outcomeIds = outcomeIdsOfInterest
  )
  
    if(nrow(cmResults) > 0){
    if(!is.null(params$comparatorIds)){
    cmResults <- cmResults %>%
      dplyr::filter(.data$comparatorId %in% !!comparatorIdsOfInterest)
    }
  }
  
  # TODO: restrict to comparatorIdsOfInterest?
  # return NULL if the evidence synth tables do not exist
  cohortMethodES <- tryCatch({OhdsiReportGenerator::getCmMetaEstimation(
    connectionHandler = connectionHandler, 
    schema = params$schema, 
    targetIds = unique(c(targetIdsOfInterest, indicationIdsOfInterest)),
    outcomeIds = outcomeIdsOfInterest
  )}, error = function(e){return(invisible(NULL))})
  
  if(!is.null(cohortMethodES)){
    if(nrow(cohortMethodES) > 0){
      if(!is.null(params$comparatorIds)){
      cohortMethodES <- cohortMethodES %>%
       dplyr::filter(.data$comparatorId %in% !!comparatorIdsOfInterest)
      }
    }
  }
  
  cmTargets <- unique(cohortMethodDiagnostics$targetId)
  cmComparators <- unique(cohortMethodDiagnostics$comparatorId)
  cmOutcomes <- unique(cohortMethodDiagnostics$outcomeId)
  cmDesc <- unique(cohortMethodDiagnostics$description) # can we extract this?
  }
  # ===================================
}

if(includeSccs){
  # ===================================
  # Get Sccs
  # ===================================
  
  sccsDiagnostics <- OhdsiReportGenerator::getSccsDiagnosticsData(
    connectionHandler = connectionHandler, 
    schema = params$schema,
    targetIds = unique(c(params$targetId, comparatorIds)), 
    outcomeIds = outcomeIdsOfInterest
  ) 
  
  # filter to the indications of interest
  if(restrictTargetToIndications){
    sccsDiagnostics <- sccsDiagnostics %>%
      dplyr::filter(ifelse(is.na(.data$indicationId), "", .data$indicationId)  %in% indicationIdsOfInterest)
  }
  
  if(nrow(sccsDiagnostics)  == 0){
    includeSccs <- FALSE
  } else {
  
  sccsResults <- OhdsiReportGenerator::getSccsEstimation(
    connectionHandler = connectionHandler, 
    schema = params$schema,
    targetIds = unique(c(params$targetId, comparatorIds)), 
    outcomeIds = outcomeIdsOfInterest
  )
  
    # filter to the indications of interest
  if(restrictTargetToIndications){
    sccsResults <- sccsResults %>%
      dplyr::filter(ifelse(is.na(.data$indicationId), "", .data$indicationId)  %in% indicationIdsOfInterest)
  }
  
  # return NULL if the evidence synth tables do not exist
  sccsES <- tryCatch({OhdsiReportGenerator::getSccsMetaEstimation(
    connectionHandler = connectionHandler, 
    schema = params$schema,
    targetIds = unique(c(params$targetId, comparatorIds)), 
    outcomeIds = outcomeIdsOfInterest
  )}, error = function(e){return(invisible(NULL))})
  
  if(restrictTargetToIndications){
    if(!is.null(sccsES)){
    sccsES <- sccsES %>%
      dplyr::filter(ifelse(is.na(.data$indicationId), "", .data$indicationId)  %in% indicationIdsOfInterest)
    }
  }
  
  sccsTargets <- unique(sccsDiagnostics$targetId)
  sccsOutcomes <- unique(sccsDiagnostics$outcomeId)
  sccsDesc <- unique(sccsDiagnostics$description)
}
  # ===================================
}
  
result <- knitr::knit_child(text = knitr::knit_expand("causal_inference.qmd", quiet = TRUE, envir = environment()), quiet = TRUE)
cat(result, sep = '\n')
}
```


```{r}
#| label: prediction
#| echo: false
#| results: 'asis'

if(includePrediction){
  
result <- knitr::knit_child(text = knitr::knit_expand("prediction.qmd", quiet = TRUE, envir = environment()), quiet = TRUE)
cat(result, sep = '\n')

}
```


```{r}
#| label: references
#| echo: false
#| results: 'asis'

result <- knitr::knit_child(text = knitr::knit_expand("references.qmd", quiet = TRUE, envir = environment()), quiet = TRUE)
cat(result, sep = '\n')
```

```{r}
#| label: q1_get_data_{{TA}}
#| echo: false
#| results: 'asis'

# How many models did we attempt?
cat(paste0('\n\n### ', TA))

# extract number of models per TA
# number of models with AUROC > 0.7

modelPerf  <- OhdsiReportGenerator::getPredictionPerformances(
  connectionHandler = connectionHandler, 
  schema = schema, 
  plpTablePrefix = params$plpTablePrefix, 
  cgTablePrefix = params$cgTablePrefix, 
  databaseTable = 'database_meta_data', 
  databaseTablePrefix = params$databaseTablePrefix, 
  modelDesignId = modelDesignIdsOfInt$modelDesignId, 
  developmentDatabaseId = NULL
  )

cat(
paste("\n\n::: {.callout-important icon=false}
\n\n## No. Models Developed \n\n",nrow(modelPerf),"\n\n:::"
)
)

cat(
paste("\n\n::: {.callout-important icon=false}
\n\n## No. with Sufficient Outcomes (100+) \n\n",sum(modelPerf$outcomeCount >= 100, na.rm = TRUE),"\n\n:::"
)
)

cat(
paste("\n\n::: {.callout-important icon=false}
\n\n## No. with Sufficient Performance (AUROC 0.7+) \n\n",sum(modelPerf$auroc >= 0.7, na.rm = TRUE),"\n\n:::"
)
)
```

```{r}
#| echo: false
#| results: 'asis'

# ===================================
# Get Databases
# ===================================

# need to add that function
##databases <- OhdsiReportGenerator::getDatabases()
databases <- connectionHandler$queryDb(
  "SELECT cdm_source_name, cdm_source_abbreviation, source_description, SOURCE_RELEASE_DATE FROM @schema.database_meta_data;",
  schema = params$schema
)

nDatabases <- nrow(databases)

# add the database names into a friendly string
databaseText <- databases$cdmSourceAbbreviation[1]
if(nDatabases > 1){
  if(nDatabases > 2){
    databaseText <- paste0(databases$cdmSourceAbbreviation[-length(databases$cdmSourceName)], collapse = ', ')
  }
  databaseText <- paste0(databaseText, ' and ', databases$cdmSourceAbbreviation[length(databases$cdmSourceAbbreviation)])
}

cat('\n# Epidemiology Data\n')

cat(paste0("Assessments were conducted to determine the feasibility of retrospective real world (RWD) analyses of ", outcomeName, " while exposed to ", targetName ," across a global research network comprised of 12 healthcare databases. The research network included electronic medical record databases, insurance claims databases, and primary care/general practitioner databases, representing a broad range of patient populations (privately insured employees or patients with limited income). Cross-network analyses are enabled by mapping data sources to the Observational Medical Outcomes Partnership (OMOP) Common Data Model (CDM). The OMOP CDM maps source data on drug exposures and outcomes to RxNorm and the Systematized Nomenclature of Medicine Clinical Terms (SNOMED CT), respectively.")) 

cat(paste0("\nThe data feasibiliy determines whether ", targetName ," and ", outcomeName, " are recorded in each database.  Additional diagnostics are applied during each analysis that may result in less databases being included in specific analyses."))

cat(paste0(nDatabases," databases in total passed initial feasibility tests with sufficient sample sizes, including ", databaseText,"."))

cat(gt::as_raw_html(
  databases %>% 
    dplyr::select("cdmSourceAbbreviation", "cdmSourceName",  "sourceDescription","sourceReleaseDate") %>%
    gt::gt() %>%
    gt::cols_label(
      cdmSourceAbbreviation = 'Database',
      cdmSourceName = "Full Database Name", 
      sourceDescription = "Database Description",
      sourceReleaseDate = "Release Date"
    ) %>%
    gt::tab_options(
      heading.align = "left",
      data_row.padding = gt::px(7),
      column_labels.font.size = gt::px(12),
      column_labels.background.color = "#EB1800",
      column_labels.font.weight = "bold",
      row_group.background.color = "#D3D3D3",
      row_group.font.weight = "bold"
    ) %>%
    gt::tab_style(
      style = gt::cell_text(
        color = "darkgrey",
        font = gt::google_font("Source Sans Pro"),
        transform = "uppercase"
      ),
      locations = gt::cells_column_labels(gt::everything())
    ) %>%
    gt::tab_header(
      title = "Databases that pass database diagostic feasibility.", 
      subtitle = "Note: not all of these databases may pass analyses diagnostics."
    )
))


```

---
output: html_document
---
  
### Predictors

**Candidate predictors**
The predictors are determined using data recorded relative to the target cohort index date.  The settings `endDays` specifies the number of days relative to the target index that is the latest point in time for determining the covariates.  For example, an `endDays` of -1 means that no data recorded on index or after are used by the covariates (the covariates only use data recorded up to index-1 day).

```{r, echo=FALSE, results = 'asis'}

if(class(covariateSettings) == 'covariateSettings'){
  covariateSettings <- list(covariateSettings)
}

cat('\n There are ', length(covariateSettings), ' covariate settings. \n')

for(i in 1:length(covariateSettings)){
  
   cat('\n Covariate Setting', i, '\n')
  
  dataC <- data.frame(
    name = c(
      'function',
      names(lapply(covariateSettings[[i]], function(x) paste(x, collapse = '-', sep='-')))), 
    value = c(
      attr(covariateSettings[[i]],"fun"), 
      unlist(lapply(covariateSettings[[i]], function(x) paste(x, collapse = '-', sep='-'))
      ))
  )
  row.names(dataC) <- NULL

print(knitr::kable(x = dataC, caption = paste('covariate setting ', i)))
  cat('\n \n')
}
```

**Feature Engineering**
```{r, echo=FALSE, results = 'asis'}

if(class(featureEngineeringSettings) == 'featureEngineeringSettings'){
  featureEngineeringSettings <- list(featureEngineeringSettings)
}

if(!is.null(attr(featureEngineeringSettings[[1]],"fun"))){
  if(attr(featureEngineeringSettings[[1]],"fun") != 'sameData'){
  
    cat('\n There are ', length(featureEngineeringSettings), ' feature engineering settings. \n')

    for(i in 1:length(featureEngineeringSettings)){
  
      cat('\n The function ', attr(featureEngineeringSettings[[i]],"fun"), ' with inputs: \n')

      feData <- data.frame(
        name = names(featureEngineeringSettings), 
        value = unlist(lapply(featureEngineeringSettings, function(x) paste(x, sep = '', collapse='-')))
        )
  
      print(knitr::kable(x = feData , caption = paste('feature engineering setting ', i)))
      cat('\n \n')
  
    }

  } else{
  cat('\n None \n')  
  }
} else{
  cat('\n None \n')  
}


```

**Pre-processing**

The following pre-processing were applied:


```{r, echo=FALSE, results = 'asis'}

if(preprocessSettings$normalize){
  cat('\n - The data were normalized using the formula: value/maxValue . \n')
}

if(preprocessSettings$minFraction > 0){
  cat('\n - Candidate predictors occuring in less than ', preprocessSettings$minFraction*100,'\\% of patients in the target population were removed. \n')
}

if(preprocessSettings$removeRedundancy){
  cat('\n - Candidate predictors that were redundant (completely correlated with another predictor) were removed.  When two predictors are completely correlated, the most common predictor is removed.  For example, if 60\\% of the target population were male and 40\\% were female, then the male gender predictor would be removed since it is redundant and more common. \n')
}

```

```{r}
#| echo: false
#| results: 'asis'
#| fig-width: 12

# extract from prediction threshold table

lift <- list()
for(sensitivity in c(0.1,0.2,0.3,0.4,0.5)){
  i <- length(lift) +1
lift[[i]] <- OhdsiReportGenerator::getPredictionLift(
    connectionHandler = connectionHandler, 
    schema = schema, 
    modelDesignIds = modelDesignOfIntCohorts$modelDesignId,
    sensitivity = sensitivity, 
    plpTablePrefix = params$plpTablePrefix
)

lift[[i]] <- lift[[i]] %>%
  dplyr::filter(
    .data$evaluation == 'Test      ',
    abs((.data$sensitivity - sensitivity)) < 0.05
  ) 
lift[[i]]$type <- paste0('@ ', sensitivity, ' sensitivity')

}

lift <- do.call(what = 'rbind', args = lift)

cat(paste0('\n\n### ', TA))
cat('\n\n')

if(nrow(lift) > 0){

ggplot2::ggplot(
  data = lift, ggplot2::aes(
    x = .data$outcomeRate, 
    y = .data$positivePredictiveValue
    )
  ) +
  ggplot2::geom_abline(
    intercept = 0, 
    slope = 1, 
    color="red",
    show.legend = FALSE
    ) +
  ggplot2::geom_point(na.rm=TRUE) + 
  ggplot2::facet_grid(. ~ .data$type, scales = 'free_x')

}

cat('\n\n')
  
```

```{r}
#| echo: false
#| results: 'asis'

cohortIds <- demoList[[demoListInd]]
cohortIdsDemo <- unique(targetBinaryOfInterest %>% 
                          dplyr::filter(.data$targetCohortId %in% !!cohortIds) %>%
                          dplyr::select("targetCohortId","minPriorObservation"))

if(nrow(cohortIdsDemo) > 0){
  cat(paste0('\n\n#### ',names(demoList)[demoListInd],' \n\n'))
  for(tId in 1:nrow(cohortIdsDemo)){
    # add the target age/sex/race/ethnicity
    dataC <- targetContinuousOfInterest %>% 
      dplyr::filter(.data$targetCohortId == !!cohortIdsDemo$targetCohortId[tId]) %>%
      dplyr::filter(.data$minPriorObservation == !!cohortIdsDemo$minPriorObservation[tId]) %>%
      dplyr::select(-c("p10Value", "p25Value", "p75Value", "p90Value")) %>%
      tidyr::pivot_longer(
        cols = c("minValue","maxValue", "averageValue", "standardDeviation", "medianValue",
        )
      ) %>%
      dplyr:: mutate(
        covariateName = paste0(.data$covariateName, ' (', .data$name, ')'),
        averageValue = .data$value
      ) %>%
      dplyr::select(-dplyr::any_of(c("name", "value"))) %>%
      dplyr::rowwise() %>%
      dplyr:: mutate(
        type = if(.data$covariateId == 1002){
          'Age'
        } else if(.data$covariateId == 1008){
          'Prior Observation'
        } else if(.data$covariateId == 1009){
          'Post Observation'
        } else if(.data$covariateId == 1010){
          'Between Observation'
        }
      ) %>%
      dplyr::select("databaseName","covariateName","averageValue", "type") 
    
    
    dataB <- targetBinaryOfInterest %>% 
      dplyr::filter(.data$targetCohortId == !!cohortIdsDemo$targetCohortId[tId]) %>%
      dplyr::filter(.data$minPriorObservation == !!cohortIdsDemo$minPriorObservation[tId]) %>%
      dplyr::select("databaseName","covariateName","averageValue")
    
    dataB$type <- rep('Age Group', nrow(dataB))
    genderId <- grep('gender = ', dataB$covariateName)
    if(length(genderId) > 0){
      dataB$type[genderId] <- 'Gender'
    }
    raceId <- grep('race = ', dataB$covariateName)
    if(length(raceId) > 0){
      dataB$type[raceId] <- 'Race'
    }
    ethnicityId <- grep('ethnicity = ', dataB$covariateName)
    if(length(ethnicityId ) > 0){
      dataB$type[ethnicityId ] <- 'Ethnicity'
    }
    
    # group by type and order by name
    data <- dataC %>%
      dplyr::bind_rows(dataB) 
    
    databaseCols <- unique(data$databaseName)
    
    data <- tidyr::pivot_wider(
      data = data,
      id_cols =  c("covariateName", "type"),
      names_from = "databaseName", 
      values_from = "averageValue"
    ) %>%
      dplyr::group_by(.data$type) %>%
      dplyr::arrange(.data$covariateName)
    
  boxName <- paste0(cohortDefinitions$friendlyName[cohortDefinitions$cohortDefinitionId == cohortIdsDemo$targetCohortId[tId]], ' - ' , cohortDefinitions$subsetText[cohortDefinitions$cohortDefinitionId == cohortIdsDemo$targetCohortId[tId]], ' with min prior obs ', cohortIdsDemo$minPriorObservation[tId], ' days')
    
  cat('\n\n::: {.callout-note collapse="true"}')
  cat(paste0("\n# ",boxName, '\n\n'))
    
    # display
    print(htmltools::tagList(
      data %>%  
        gt::gt() %>%
        gt::fmt_number(
          columns = databaseCols, # defined above 
          decimals = 2
        ) %>%
        gt::cols_label(
          covariateName = "Covariate"
        ) %>%
        gt::tab_options(
          heading.align = "left",
          data_row.padding = gt::px(7),
          column_labels.font.size = gt::px(12),
          column_labels.background.color = "#EB1800",
          column_labels.font.weight = "bold",
          row_group.background.color = "#D3D3D3",
          row_group.font.weight = "bold"
        ) %>%
        gt::tab_style(
          style = gt::cell_text(
            color = "darkgrey",
            font = gt::google_font("Source Sans Pro"),
            transform = "uppercase"
          ),
          locations = gt::cells_column_labels(gt::everything())
        ) %>%
        gt::tab_header(
          title = paste0(cohortDefinitions$friendlyName[cohortDefinitions$cohortDefinitionId == cohortIdsDemo$targetCohortId[tId]], ' - ' , cohortDefinitions$subsetText[cohortDefinitions$cohortDefinitionId == cohortIdsDemo$targetCohortId[tId]], ' with min prior obs ', cohortIdsDemo$minPriorObservation[tId], ' days')
        )
    ))
    
    cat('\n\n:::\n\n')
    
  }
}

```

---
title: "Prediction Summary Report"
lightbox: true
title-block-banner: red
description: "Information about the prediction models for a specified set of target cohorts."
date: today
format: 
  html:
    code-fold: true
    embed-resources: true
    page-layout: full
params:
  targetIds: !expr c(7795, 16916, 7788, 13947,7813,18770,
              7820, 10640, 7812, 15645, 7826, 3376,
              7811, 10626, 7815, 10617, 7810, 10682,
              7827, 10630, 7792, 16927, 16960)
  outcomeIds: !expr NULL         
  schema: 'main'
  server: ''
  username: !expr NULL
  password: !expr NULL
  dbms: 'sqlite'
  cgTablePrefix: 'cg_'
  plpTablePrefix: 'plp_'
  databaseTablePrefix: ''
engine: knitr
---

```{r}
#| label: setup
#| echo: false
#| hide: true
#| output: false

library(dplyr)

targetIds <- params$targetIds
outcomeIds <- params$outcomeIds
schema <- params$schema

# add connection here to use for the children
connectionDetails <- DatabaseConnector::createConnectionDetails(
  dbms = params$dbms, 
  user = params$username, 
  password = params$password, 
  server = params$server
)

connectionHandler <- ResultModelManager::ConnectionHandler$new(
  connectionDetails = connectionDetails
)


# load questions

predictionQuestLoc <- system.file('templates/summary-templates/prediction-questions.csv', package = 'OhdsiReportGenerator')
predictionQuest <- read.csv(file = predictionQuestLoc)


# get model designIdsOfInt
modelDesignIdsOfInt <- OhdsiReportGenerator::getPredictionModelDesigns(
  connectionHandler = connectionHandler, 
  schema = schema, 
  plpTablePrefix = params$plpTablePrefix, 
  cgTablePrefix = params$cgTablePrefix,
  targetIds = targetIds, 
  outcomeIds = outcomeIds
    )

jsons <- modelDesignIdsOfInt$covariateSettingsJson

jsons <- lapply(jsons, function(json) ParallelLogger::convertJsonToSettings(json))

# restrict to age/sex only covariateSetting
covSingle <- unlist(lapply(jsons, function(json){length(json) == 1 } ))
onlyAgeSex <- unlist(lapply(jsons, function(json){
  sum(names(json[[1]]) %in% c("temporal","temporalSequence","DemographicsGender", "DemographicsAgeGroup", "longTermStartDays", "mediumTermStartDays", "shortTermStartDays", "endDays", "includedCovariateConceptIds", "addDescendantsToInclude", "excludedCovariateConceptIds", "addDescendantsToExclude", "includedCovariateIds")) == length(json[[1]])
} ))

ageSexInd <- covSingle & onlyAgeSex
modelDesignIdsOfIntAgeSexOnly <- modelDesignIdsOfInt[ageSexInd,]

# restrict to cohort feature covariateSetting  
  cohortCovInd <- unlist(lapply(jsons, function(json){"getDbCohortBasedCovariatesData" %in% unlist(lapply(json, function(cset) attr(cset,"fun")))} ))
modelDesignOfIntCohorts <- modelDesignIdsOfInt[cohortCovInd,]

```

```{r}
#| label: questions
#| echo: false
#| results: 'asis'

for(i in 1:nrow(predictionQuest)){
cat(paste0('\n\n## [',predictionQuest$question[i],']{style="color:grey;"}'))
cat(paste0('\n\n[', predictionQuest$description[i] ,']{style="color:red;font-size: small;"}'))  
cat('\n\n:::: {.columns}')
# show table/plot per TA
for(TA in 'Selected Targets'){
cat(paste0('\n\n::: {.column width="',100,'%"}'))

  if(file.exists(paste0("prediction-questions/", predictionQuest$fileName[i]))){
output <- knitr::knit_child(
  text = knitr::knit_expand(
  paste0("prediction-questions/", predictionQuest$fileName[i]), 
  quiet = TRUE
  ),quiet = TRUE)
cat(output, sep = '\n')
  } else{
    cat(paste0('\n\n### ', TA))
    cat('\n No question code \n')
    cat(paste0('\n ',getwd(),' \n'))
    cat(paste0('\n ',paste0("prediction-questions/", predictionQuest$fileName[i]),' \n'))
}
  
cat('\n\n:::')
}

cat('\n\n::::')
}
  

```

---
output: html_document
---

```{r get model design, echo=FALSE, results='asis'}

modelDesign <- OhdsiReportGenerator::getPredictionModelDesigns(
  connectionHandler = params$connectionHandler,
  schema = params$resultSchema, 
  plpTablePrefix = params$myTableAppend, 
  cgTablePrefix = params$cohortTableAppend#, 
  #modelDesignId = params$modelDesignIds[i] # need to add this as optional input
  ) %>% 
  dplyr::filter(.data$modelDesignId == !!params$modelDesignIds[i])


target <- list(
  name = modelDesign$developmentTargetName,
  id = modelDesign$developmentTargetId,
  expression = modelDesign$developmentTargetJson
  )
outcome <- list(
  name = modelDesign$developmentOutcomeName,
  id = modelDesign$developmentOutcomeId,
  expression = modelDesign$developmentOutcomeJson
)

populationSettings <- ParallelLogger::convertJsonToSettings(modelDesign$populationSettingsJson)
covariateSettings <- ParallelLogger::convertJsonToSettings(modelDesign$covariateSettingsJson)
featureEngineeringSettings <- ParallelLogger::convertJsonToSettings(modelDesign$featureEngineeringSettingsJson)
preprocessSettings <- ParallelLogger::convertJsonToSettings(modelDesign$tidyCovariatesSettingsJson)
modelSettings <- ParallelLogger::convertJsonToSettings(modelDesign$modelSettingsJson)
sampleSettings <- ParallelLogger::convertJsonToSettings(modelDesign$sampleSettingsJson)
restrictPlpDataSettings <- ParallelLogger::convertJsonToSettings(modelDesign$plpDataSettingsJson)
splitSettings <- ParallelLogger::convertJsonToSettings(modelDesign$splitSettingsJson)
 
```



```{r print model heading, echo=FALSE, results='asis'}
cat('\n# Model ', params$modelDesignIds[i], '\n')
```


## Model Design

```{r participants, child = "plp-participants.Rmd"}
```

```{r predictors, child = "plp-predictors.Rmd"}
```

```{r outcome, child = "plp-outcome.Rmd"}
```

```{r analysis, child = "plp-analysis.Rmd"}
```

## PatientLevelPrediction outputs

The following will be output per model design and database combination via an interactive shiny app:

```{r, echo=FALSE, result = 'asis'}
modelEvaluation <- data.frame(rbind(
    c("Prediction Model", "A model that can predict the risk of the outcome for any new patient in the target population using their features."),
    
    c("Performance Evaluation", "Summary statistics for the model's discrimination [AUROC (95% CI), AUPRC (95% CI)], calibration [calibration-in-the-large, calibration intercept, calibration slope, e-statistic] and dataset details (number of patients in the target population and number of outcome).  This is calculated for the test set, the train set and the cross-validation set."), 
    
    c("Covariate Summary","A table summarizing how common each feature was in the target population, the test set and the train set for patients with and without the outcome.  The standardized mean difference comparing the frequency of the features for patients who experienced the outcome during the time-at-risk vs those who did not.")
    
    )
  )
  names(modelEvaluation) <- c("Item","Description")
  row.names(modelEvaluation) <- NULL
  data <- modelEvaluation[order(modelEvaluation$Item),]
  knitr::kable(x = data, caption = 'Standard Output of Prediction')
  
```
  

---
title: "ASSURE Analysis"
format:
  revealjs: 
    self-contained: true
    scrollable: true
    theme: [default, presentation_style.scss]
    title-slide-attributes:
      data-background-color: "#EB1700"
    footer: "ASSURE Analysis"
    slide-number: true
    show-slide-number: all
    menu:
      side: left
      numbers: true
params: 
  server: NULL
  password: NULL
  username: NULL
  dbms: NULL
  resultsSchema: NULL
  dbDetails: NULL
  lead: NULL
  team: NULL
  trigger: NULL
  safetyQuestion: NULL
  objective: NULL
  topline1: NULL
  topline2: NULL
  topline3: NULL
  date: "!expr: as.character(Sys.Date())"
  targetId: NULL
  outcomeIds: NULL
  cohortNames: NULL
  cohortIds: NULL
  covariateIds: NULL
  details: NULL
  evaluationText: NULL
  includeCI: true
  includeCharacterization: true
  includeCM: true
  includeSCCS: true
  includePLP: true
  pathToDriver: !expr "Sys.getenv('DATABASECONNECTOR_JAR_FOLDER')"
---

```{r}
#| label: setup
#| echo: false
#| hide: true

defaultPageSize <- 10

library(reactable)
library(dplyr)
resultsSchema <- params$resultsSchema
server <- params$server
username <- params$username
password <- params$password
dbms <- params$dbms

# connect
connectionDetails <- DatabaseConnector::createConnectionDetails(
  server = server,
  user = username,
  password = password,
  dbms = dbms,
  pathToDriver =  params$pathToDriver
)
connectionHandler <- ResultModelManager::ConnectionHandler$new(
  connectionDetails = connectionDetails
  )

# get all the cohorts

cohortFriendlyNames <- data.frame(
  cohortFriendlyName = 'madeup',
  cohortId = -1
)
if(!is.null(params$cohortNames)){
  cohortFriendlyNames = data.frame(
    cohortFriendlyName = unlist(params$cohortNames),
    cohortDefinitionId = unlist(params$cohortIds)
  )
}

cohorts <- OhdsiReportGenerator::getCohortDefinitions(
  connectionHandler = connectionHandler, 
  schema = resultsSchema
  )

# add cohortFriendlyName to the cohorts
cohorts <- merge(cohorts, cohortFriendlyNames, by = 'cohortDefinitionId', all.x = TRUE)
cohorts$cohortFriendlyName[is.na(cohorts$cohortFriendlyName)] <- cohorts$cohortName[is.na(cohorts$cohortFriendlyName)]

# get the target and all children in each analyses
# return data.frame with targetId, analyses, targetName, friendlyName, ...
targetId <- unlist(params$targetId)
# get all the children cohorts of the target
targets <- cohorts %>%
  filter(
    .data$subsetParent == !!targetId
  )

outcomes <- cohorts %>%
  filter(
    .data$subsetParent %in% unlist(!!params$outcomeIds)
  )
# used by prediction
outcomeIds <- outcomes$cohortDefinitionId


# get candidate indictor cohorts:
# there will always be a subset for CM so dont need to check empty
subsets <- OhdsiReportGenerator::getCohortSubsetDefinitions(
  connectionHandler = connectionHandler, 
  schema = resultsSchema, 
  subsetIds = unique(na.omit(targets$subsetDefinitionId))
)

subsetCohorts <- lapply(1:nrow(subsets), function(i){
  unlist(lapply(ParallelLogger::convertJsonToSettings(subsets$json[i])$subsetOperators,
               function(x){
                 if(x$subsetType == "CohortSubsetOperator") 
                   if(!x$negate){
                    return(x$cohortIds)
                   }
               }
               ))})

indications <- NULL
if(length(unlist(subsetCohorts))>0){
  indications <- cohorts %>% filter(
    .data$cohortDefinitionId %in% unlist(subsetCohorts)
  )
}


# figure out comparators - what if no cm?
comparators <- NULL
if(params$includeCM){
  cmDiag <- OhdsiReportGenerator::getCmDiagnosticsData(
    connectionHandler = connectionHandler, 
    schema = resultsSchema, 
    targetIds = targets$cohortDefinitionId, 
    outcomeIds = outcomes$cohortDefinitionId
    )
  
  comparatorIds <- unique(cmDiag$comparatorId)
  comparatorParent <- unique(cohorts$subsetParent[cohorts$cohortDefinitionId %in% comparatorIds])
  
  # restrict to parent comps
  comparators <- cohorts %>%
  filter(
    .data$cohortDefinitionId %in% !!comparatorParent
  )
  
}

# get databases
if(!is.null(params$dbDetails)){
dbs <- OhdsiReportGenerator:::getDbs(
  schema = resultsSchema,
  connectionHandler = connectionHandler, 
  dbDetails = params$dbDetails
)} else{
 dbs <- OhdsiReportGenerator:::getDbs(
  schema = resultsSchema,
  connectionHandler = connectionHandler
) 
}

# If this first initialization isn't run, tabs won't print
#reactable::reactable(data.frame(Study = "Multiple comparators and/or multiple outcomes."))

```

```{r}
#| label: background
#| echo: false
#| results: 'asis'

targetName <- cohorts$cohortFriendlyName[which(cohorts$cohortDefinitionId == params$targetId)]
outcomeNames <- cohorts$cohortFriendlyName[which(cohorts$cohortDefinitionId %in% params$outcomeIds)]

# add these to the first slide
title <- paste0(targetName , ' and ', paste0(outcomeNames, collapse = '/'))
lead <- params$lead
team <- params$team
date <- params$date

trigger <- params$trigger
safetyQuestion <- params$safetyQuestion
objective <- params$objective

result <- knitr::knit_child(text = knitr::knit_expand("background.qmd", quiet = TRUE, envir = environment()), quiet = TRUE, envir = environment())
cat(result, sep = '\n')
```

```{r}
#| label: topline
#| echo: false
#| results: 'asis'

# add code to auto generate the topline via LLM?

result <- knitr::knit_child(text = knitr::knit_expand("topline.qmd", quiet = TRUE, envir = environment()), quiet = TRUE)
cat(result, sep = '\n')
```

```{r}
#| label: overview
#| echo: false
#| results: 'asis'

studyPeriod <- params$details$studyPeriod
restrictions <- params$details$restrictions

result <- knitr::knit_child("intro_summary.qmd", quiet = TRUE, envir = environment())
cat(result, sep = '\n')

```

```{r}
#| label: databases
#| echo: false
#| results: 'asis'
result <- knitr::knit_child("databases.qmd", quiet = TRUE, envir = environment())
cat(result, sep = '\n')
```


```{r}
#| label: characterization
#| echo: false
#| results: 'asis'

# age, gender, top 10 covs

if(params$includeCharacterization){
result <- knitr::knit_child("characterization.qmd", quiet = TRUE, envir = environment())
cat(result, sep = '\n')
}

```

```{r}
#| label: cohort incidence
#| echo: false
#| results: 'asis'
#| 
#incidence for target, comp and indications
if(params$includeCI){

  cat('\n# Cohort Incidence {.break-slides} \n\n')
  #cat('{{< pagebreak >}}')

# figure out which targets are in char
ciTargetIds <- connectionHandler$queryDb("select distinct target_cohort_definition_id from @schema.ci_target_def;", schema = resultsSchema)

# targets
targetsCi <- targets %>% dplyr::filter(
.data$cohortDefinitionId %in% !!ciTargetIds$targetCohortDefinitionId
)

if(nrow(targetsCi)>0 & nrow(outcomes)>0){
heading <- 'Targets'
ciIds <- targetsCi$cohortDefinitionId

res <- knitr::knit_child(text = knitr::knit_expand("cohort_incidence_template.qmd", quiet = TRUE, envir = environment()),quiet = TRUE)
cat(res, sep = '\n')
}

# indications
if(!is.null(indications)){
indicationCis <- cohorts %>%
  filter(
    .data$subsetParent %in% !!indications$cohortDefinitionId
  ) %>%
 filter(
    .data$cohortDefinitionId %in% !!ciTargetIds$targetCohortDefinitionId
  )

if(nrow(indicationCis)>0 & nrow(outcomes)>0){
heading <- 'Indications'
ciIds <- indicationCis$cohortDefinitionId

res <- knitr::knit_child(text = knitr::knit_expand("cohort_incidence_template.qmd", quiet = TRUE, envir = environment()),quiet = TRUE)
cat(res, sep = '\n')
}
}

# comparators
if(!is.null(comparators)){
comparatorCis <- cohorts %>%
  filter(
    .data$subsetParent %in% !!comparators$cohortDefinitionId
  ) %>%
 filter(
    .data$cohortDefinitionId %in% !!ciTargetIds$targetCohortDefinitionId
  )

heading <- 'Comparators'
ciIds <- comparatorCis$cohortDefinitionId

res <- knitr::knit_child(text = knitr::knit_expand("cohort_incidence_template.qmd", quiet = TRUE, envir = environment()),quiet = TRUE)
cat(res, sep = '\n')
}

}
```

```{r}
#| label: methods_theory
#| echo: false
#| results: 'asis'

if(params$includeCM | params$includeSCCS){
result <- knitr::knit_child("methods_theory.qmd", quiet = TRUE, envir = environment())
cat(result, sep = '\n')
}

```


```{r}
#| label: causal_inference
#| echo: false
#| results: 'asis'

cmSubsetIds <- NULL
if(params$includeCM){
result <- knitr::knit_child("cohort_method.qmd", quiet = TRUE, envir = environment())
cat(result, sep = '\n')
}

sccsSubsetIds <- NULL
if(params$includeSCCS){
result <- knitr::knit_child("sccs.qmd", quiet = TRUE, envir = environment())
cat(result, sep = '\n')
# create sccsSubsetIds
}

```
  
```{r}
#| label: prediction
#| echo: false
#| results: 'asis'

if(params$includePLP){
result <- knitr::knit_child("prediction.qmd", quiet = TRUE, envir = environment())
cat(result, sep = '\n')
}

```
   
  
  
# Conclusions {.break-slides}
  
```{r}
#| label: executive summary
#| echo: false
#| results: 'asis'

# can potentially automate this part - summary or CI, summary of CM/SCCS
evaluationText <- params$evaluationText

cat('## Executive Summary\n\n')
cat(paste(unlist(lapply(
  evaluationText, 
  function(txt){
    paste0('\n* ', txt)
  }
)), collapse = '\n'))


```

```{r}
#| label: comment
#| echo: false
#| results: 'asis'

# Fixed text
result <- knitr::knit_child("comment.qmd", quiet = TRUE, envir = environment())
cat(result, sep = '\n')

```


# Supplementary {.break-slides}

```{r}
#| label: cohort_definitions
#| echo: false
#| results: 'asis'

# creates the target details, comparator details, indication details and outcome details.
result <- knitr::knit_child("cohort_definitions.qmd", quiet = TRUE, envir = environment())
cat(result, sep = '\n')

```

## Preformance Metrics

The following are the standardized metrics used to evaluate patient-level prediction models.

```{r, echo=FALSE, result = 'asis'}
modelEvaluation <- data.frame(rbind(
    c("AUROC", "Discrimination", "The Area Under the Receiver Operating Characteristic curve is a common discrimination metric.  It corresponds to the probability that a randomly selected patient with the outcome is assigned a higher risk by the model than a randomly selected patients without the outcome."),
    
    c("AUPRC", "Discrimination", "The Area Under the Precision Recall curve is a common discrimination metric that provides a useful insight when the outcome is rare."), 
    
    c("Calibration-in-the-large", "Calibration" , "A measure of how well the predicted risk matches the observed risk on average.  It compares the mean predicted risk for the whole popualtion (the model's predicted risk) with the observed risk (the true risk)"),
    
  c("E-statistic", "Calibration" , "A measure corresponding to the average difference between the true risk and the predicted risk.")
    
    )
  )
  names(modelEvaluation) <- c("Name", "Type","Description")
  row.names(modelEvaluation) <- NULL
  data <- modelEvaluation[order(modelEvaluation$Type),]
  knitr::kable(x = data, caption = 'Prediction Metrics')
  
```
  

# Prediction

## Method

The patient level prediction can identify who is a risk of having the outcome during some time-at-risk within the population at risk (target population).  The common metrics are:

1. Discrimination (AUC and AUPRC) that indicate how well the model ranks patients based on their risk.  An AUC of 1 means all the patients who will have the outcome are predicted a higher risk than those who will not have the outcome during the time-at-risk.  An AUC of 0.5 means the model is predicting randomly.

2. Calibration (eStatistic, calibrationInLarge) that indicate how accurate the predicted risk is.  For example, if a model is well calibrated and predicts that 10 patients have a 10% risk, then one of those patients should have the outcome (1 in 10 is 10% risk).  The calibration in the large compares the mean predicted risk with the mean observed risk, if the model is well calibrated these values should be equal and the calibration in the large ratio (mean predicted risk/ mean observed risk) should be 1.

3. Accuracy (Brier score) that measures the prediction error such as the sum of the squared difference between the predicted risk and true label for each patient.

A good model should have a high AUC (ideally >= 0.7) and be well calibrated.  However, it is also important to consider the sensitivity and positive prediction value at different thresholds, as these are the key metrics that show how well the model is likely to perform when used to guide an intervention.

## Results

```{r}
#| echo: false
#| results: 'asis'


# TODO - restrict to T/O of interest?
performances <- OhdsiReportGenerator::getPredictionPerformances(
    connectionHandler = connectionHandler, 
    schema = params$schema
)

if(nrow(performances) > 0){
cat('\n\n### Performance Summary\n\n')
  
  for(databaseName in unique(performances$developmentDatabase)){
  cat('\n\n::: {.callout-note collapse="true"}')
  cat(paste0("\n# All models developed in ",databaseName,"\n\n"))
  
  print(htmltools::tagList(
    performances %>%  
      dplyr::filter(.data$developmentTargetId %in% targetIdsOfInterest) %>%
      dplyr::filter(.data$developmentOutcomeId %in% outcomeIdsOfInterest) %>%
      dplyr::filter(.data$developmentDatabase == !!databaseName) %>%
      dplyr::select(
        -"developmentDatabaseId", -"developmentTargetId", 
        -"developmentOutcomeId", -"developmentDatabase",
        -"validationDatabaseId", -"predictionResultType"
        ) %>%
      dplyr::group_by(.data$developmentTargetName) %>%
      gt::gt() %>%
      gt::fmt_number(
        columns = c("auroc", "auroc95lb", "auroc95ub",
                    "calibrationInLarge", "eStatistic",
                    "brierScore", "auprc",
                    "evalPercent", "outcomePercent"), 
        decimals = 2
      ) %>%
      gt::fmt_integer(
        columns = c("populationSize", "outcomeCount")
      ) %>%
    gt::tab_options(
      heading.align = "left",
      data_row.padding = gt::px(7),
      column_labels.font.size = gt::px(12),
      column_labels.background.color = "#EB1800",
      column_labels.font.weight = "bold",
      row_group.background.color = "#D3D3D3",
      row_group.font.weight = "bold"
    ) %>%
    gt::tab_style(
      style = gt::cell_text(
        color = "darkgrey",
        font = gt::google_font("Source Sans Pro"),
        transform = "uppercase"
      ),
      locations = gt::cells_column_labels(gt::everything())
    ) %>%
    gt::tab_header(
      title = databaseName
    )
  ))
  
  cat('\n\n:::\n\n')
  }
  
}

```


```{r}
#| echo: false
#| results: 'asis'

numberPredictors <- 100


# loop over outcomes?

topPreds <-  OhdsiReportGenerator::getPredictionTopPredictors(
    connectionHandler = connectionHandler, 
    schema = params$schema, 
    plpTablePrefix = 'plp_',
    cgTablePrefix = 'cg_',
    targetIds = targetIdsOfInterest,
    outcomeIds = outcomeIdsOfInterest,
    numberPredictors = numberPredictors
)

if(nrow(topPreds) > 0 ){
  topPreds <- OhdsiReportGenerator::addTarColumn(topPreds)
  topPreds <- OhdsiReportGenerator::formatBinaryCovariateName(topPreds)
  
  cat('\n\n### Top Predictors\n\n')
  
  for(databaseName in unique(topPreds$databaseName)){
    
  cat('\n\n::: {.callout-note collapse="true"}')
  cat(paste0("\n# Top ",numberPredictors," predictors in ",databaseName,"\n\n"))
  
  print(htmltools::tagList(
    topPreds %>%  
      dplyr::filter(.data$databaseName == !!databaseName) %>%
      dplyr::arrange(.data$rn) %>%
      dplyr::select(
        "tar", "performanceId",
        "covariateName", "covariateCount", "withNoOutcomeCovariateMean",
        "withOutcomeCovariateMean", "standardizedMeanDiff"
        ) %>%
      dplyr::group_by(.data$tar, .data$performanceId) %>%
      gt::gt() %>%
      gt::fmt_number(
        columns = "standardizedMeanDiff", 
        decimals = 2
      ) %>%
      gt::fmt_integer(
        columns = "covariateCount"
      ) %>%
      gt::fmt_percent(
        columns = c("withNoOutcomeCovariateMean","withOutcomeCovariateMean"),
        decimals = 2
    ) %>%
    gt::cols_label(
      covariateName = "Covariate"
    ) %>%
    gt::tab_options(
      heading.align = "left",
      data_row.padding = gt::px(7),
      column_labels.font.size = gt::px(12),
      column_labels.background.color = "#EB1800",
      column_labels.font.weight = "bold",
      row_group.background.color = "#D3D3D3",
      row_group.font.weight = "bold"
    ) %>%
    gt::tab_style(
      style = gt::cell_text(
        color = "darkgrey",
        font = gt::google_font("Source Sans Pro"),
        transform = "uppercase"
      ),
      locations = gt::cells_column_labels(gt::everything())
    ) %>%
    gt::tab_header(
      title = databaseName
    )
  ))
  
  cat('\n\n:::\n\n')
  
  }
  
  
}



```

## {{heading}}

```{r}
#| echo: false
#| results: 'asis'
#| 
# add text for outcome tabs
cat("\n::: {style='font-size: 50%;'}\n\n")
cat('\n\n:::: {.panel-tabset .nav-pills}\n')
for(outcomeInd in 1:nrow(outcomes)){
  outcomeId <- outcomes$cohortDefinitionId[outcomeInd]
  outcomeName <- outcomes$cohortFriendlyName[outcomeInd]
  
incidence <- OhdsiReportGenerator::getIncidenceRates(
    connectionHandler = connectionHandler,
    schema = resultsSchema,
    targetIds = ciIds,
    outcomeIds = outcomeId
)

if(nrow(incidence) > 0 ){
  # add outcome tab
  cat(paste0('\n\n### ',outcomeName, ' \n\n'))

tars <- OhdsiReportGenerator:::getTars(
  data = incidence
  )

# add text for tabs
cat('\n::: {.panel-tabset .nav-pills}\n')
for(tar in tars){
  
  idata <- incidence %>% 
    dplyr::filter(
      .data$ageGroupName == 'Any' &
      .data$genderName == 'Any' &
      .data$startYear == 'Any' &
      .data$tarStartWith == tar$tarStartWith  &
      .data$tarStartOffset == tar$tarStartOffset &
      .data$tarEndWith == tar$tarEndWith &
      .data$tarEndOffset ==  tar$tarEndOffset
    ) %>%
    dplyr::select(
      'databaseName',
      'targetName',
      'targetId',
      'cleanWindow',
      #'ageGroupName',
      #'genderName',
      #'startYear',
      'personsAtRisk',
      'personDays',
      'personOutcomes',
      'outcomes',
      'incidenceProportionP100p',
      'incidenceRateP100py'
    )
  
  # if there is data
  if( nrow(idata) > 0 ){
        cat(paste0('\n\n#### TAR: ',paste0('(',tar$tarStartWith, " + ", tar$tarStartOffset, ') - (', tar$tarEndWith, " + ", tar$tarEndOffset, ')'),'\n\n'))
    
    OhdsiReportGenerator::printReactable(
  data = idata, 
columns = list(
  targetName = reactable::colDef(minWidth = 300),
  personOutcomes = reactable::colDef(
    cell = function(value) {
    if (value == -5) "< 5" else value
  }
    ),
  outcomes = reactable::colDef(
    cell = function(value) {
    if (value == -5) "< 5" else value
  }
    ),
  incidenceProportionP100p = reactable::colDef(
    cell = function(value) {
    if (value < 0 ){paste0("<",round(abs(value), digits = 3))}else{round(value, digits = 3)}
  }
    ),
  incidenceRateP100py = reactable::colDef(
    cell = function(value) {
    if (value < 0 ){paste0("<",round(abs(value), digits = 3))}else{round(value, digits = 3)}
  }
    )
),
 highlight = TRUE, 
 striped = TRUE, 
 defaultPageSize = defaultPageSize, 
searchable = TRUE, 
filterable = TRUE
)
cat("\n")
}

  # end inner tab
}

cat('\n\n:::')

}
}
# end outer tab
cat('\n\n::::\n\n')

cat('\n\n:::\n\n')
```

---
output: html_document
---
  
### Outcome

The outcome ``r outcome$name`` occurring between target ``r populationSettings$startAnchor `` + ``r populationSettings$riskWindowStart`` days and target ``r populationSettings$endAnchor`` + ``r populationSettings$riskWindowEnd `` days is defined as:

```{r results='asis', echo=FALSE}
markdown <- tryCatch(
  {CirceR::cohortPrintFriendly(as.character(outcome$expression))}, 
  error = function(e) return('')
)
markdown <- gsub('###', '####',markdown)
cat(markdown)
```

---
  output: html_document
---

```{r results='asis', echo=FALSE}
cat('\n ',paste0('Validation on ', result$database), '\n')
```

#### Attrition

```{r results='asis', echo=FALSE}
data <- result$attrition %>%
  dplyr::select(-"performanceId")
  
  kableDark(data = data)
```

#### Summary

```{r results='asis', echo=FALSE}
data <- result$evaluation_statistics %>%
  dplyr::select(-"performanceId") %>%
  tidyr::pivot_wider(
    id_cols =  "metric",  #.data$metric, 
    names_from = "evaluation", #.data$evaluation, 
    values_from = "value" #.data$value
    ) 
kableDark(data = data)
```

#### Plots

add roc and smooth cal plots here
  
#### Covariates

```{r results='asis', echo=FALSE}
data <- result$covariate_summary %>%
  dplyr::select(
    c(
    "covariateName", 
    "covariateValue",
    "covariateCount",
    "withNoOutcomeCovariateMean",
    "withOutcomeCovariateMean",
    "standardizedMeanDiff"
    )
    )
kableDark(data = data)
```
  

```{r}
#| echo: false
#| results: 'asis'
#| fig-width: 12

demo <- OhdsiReportGenerator::getPredictionPerformanceTable(
  connectionHandler = connectionHandler,
  schema = schema,
  table = 'demographic_summary',
  modelDesignIds = modelDesignOfIntCohorts$modelDesignId, 
  plpTablePrefix = params$plpTablePrefix
)

demo <- demo %>%
  dplyr::filter(
    .data$personCountAtRisk >= 100,
    .data$evaluation == 'Test      '
    ) %>%
  dplyr::mutate(
    error = abs(.data$averagePredictedProbability - .data$personCountWithOutcome/.data$personCountAtRisk)
  )

cat(paste0('\n\n### ', TA))
cat('\n\n')

if(nrow(demo) > 0 ){
demo$ageGroup <- gsub('Age group: ','', demo$ageGroup)
ageorder <- unlist(lapply(unique(demo$ageGroup), function(x) strsplit(x, '-')[[1]][1]))

demo$ageGroup <- factor(
  x = demo$ageGroup, 
  levels = unique(demo$ageGroup)[order(-as.double(ageorder))]
)

# plot error by ageGroup, genGroup and databaseName
suppressWarnings(
ggplot2::ggplot(
  data = demo, 
  mapping = ggplot2::aes(
    x = .data$ageGroup, 
    y = .data$error
    )
  ) + 
  ggplot2::geom_boxplot(na.rm=TRUE, outliers =  FALSE) +
  ggplot2::coord_flip(
    ylim = c(0, quantile(demo$error, probs = 0.99, na.rm = TRUE))
    ) +
  ggplot2::facet_grid(. ~ .data$genGroup) + 
  ggplot2::theme(legend.position="bottom")
)
  
}

cat('\n\n')
  
```

# Characterization

## Method

```{r}
#| echo: false
#| results: 'asis'
if(!is.null(params$webAPI)){
val <- paste0("\n\nThe index breakdown section shows how often each outcome concept id occurred for the patients in ",targetName," with ",outcomeName," during the time-at-risk after exposure.  We restrict to concept ids that occurs for one percent of the popualtion or more.  This gives insight into how the outcome is being observed in the data.\n\n")
cat(val)
}
```

The demographics (age/gender), prior observation and post observation will be calculated for the patients in `r targetName`, for the indications, the outcome `r outcomeName` and the comparators: `r comparatorText`.

The risk factors section shows the demographics, medication or comorbidities that are associated to having the `r outcomeName` during the time-at-risk after `r targetName` exposure.  We present the top 50 covariates that are associated to having `r outcomeName` during the time-at-risk for the patients exposed to `r targetName`.

## Results
```{r}
#| label: index_breakdown
#| echo: false
#| results: 'asis'

if(!is.null(params$webAPI)){
result <- knitr::knit_child(text = knitr::knit_expand("characterization/index_breakdown.qmd", quiet = TRUE, envir = environment()), quiet = TRUE)
cat(result, sep = '\n')
}
```

```{r}
#| label: demo
#| echo: false
#| results: 'asis'


  # TODO add tte?
  if(FALSE){
    tte <- OhdsiReportGenerator::getTimeToEvent(
      connectionHandler = connectionHandler, 
      schema = params$schema, 
      targetIds = unique(c(targetIdsOfInterest, comparatorIdsOfInterest, indicationIdsOfInterest)), 
    outcomeIds = outcomeIdsOfInterest
    )
  }
  
  targetContinuousOfInterest <- OhdsiReportGenerator::getTargetContinuousFeatures(
    connectionHandler = connectionHandler, 
    schema = params$schema, 
    targetIds = unique(c(targetIdsOfInterest, comparatorIdsOfInterest, indicationIdsOfInterest)), 
    analysisIds = c(2,8,9,10)
  )
  
  targetBinaryOfInterest <- do.call('rbind', lapply(unique(c(targetIdsOfInterest, comparatorIdsOfInterest, indicationIdsOfInterest)), function(x){
  OhdsiReportGenerator::getTargetBinaryFeatures(
    connectionHandler = connectionHandler, 
    schema = params$schema, 
    targetId = x, 
    analysisIds = c(1,2,3,4,5)
)}))
  
# run this per type - 
demoList <- list(
  `Exposure Target` = targetIdsOfInterest,
  `Outcome/s` = outcomeIdsOfInterest,
  `Indication/s` = ifelse(is.null(indicationIdsOfInterest), NA, indicationIdsOfInterest), # what about null?
  `Comparator/s` = comparatorIdsOfInterest
)

cat('\n\n### Index Demographics\n\n')
for(demoListInd in 1:length(demoList)){
  result <- knitr::knit_child(
    text = knitr::knit_expand(
      file = "characterization/demographics.qmd", 
      quiet = TRUE, 
      envir = environment()), 
    quiet = TRUE
    )
cat(result, sep = '\n')
}
```

```{r}
#| label: risk_factors
#| echo: false
#| results: 'asis'

result <- knitr::knit_child(text = knitr::knit_expand("characterization/risk_factors.qmd", quiet = TRUE, envir = environment()), quiet = TRUE)
#result <- knitr::knit_child("characterization/risk_factors.qmd", quiet = TRUE)
cat(result, sep = '\n')
```

# Cohort Incidence {.break-slides}

::: {style='font-size: 50%;'}

```{r}
#| label: cohort incidence
#| echo: false
#| results: 'asis'

# figure out which targets are in char
ciTargetIds <- connectionHandler$queryDb("select distinct target_cohort_definition_id from @schema.ci_target_def;", schema = resultsSchema)

# targets
targetsCi <- targets %>% filter(
.data$cohortDefinitionId %in% !!ciTargetIds$targetCohortDefinitionId
)

if(nrow(targetsCi)>0 & nrow(outcomes)>0){
heading <- 'Targets'
ciIds <- targetsCi$cohortDefinitionId

result <- knitr::knit_child(text = knitr::knit_expand("cohort_incidence_template.qmd", quiet = TRUE, envir = environment()), quiet = TRUE, envir = environment())
cat(result, sep = '\n')
}

# indications
if(F){
#if(!is.null(indications)){
indicationCis <- cohorts %>%
  filter(
    .data$subsetParent %in% !!indications$cohortDefinitionId
  ) %>%
 filter(
    .data$subsetParent %in% !!ciTargetIds$targetCohortDefinitionId
  )

heading <- 'Indications'
ciIds <- indicationCis$cohortDefinitionId

result <- knitr::knit_child(text = knitr::knit_expand("cohort_incidence_template.qmd", quiet = TRUE, envir = environment()), quiet = TRUE, envir = environment())
cat(result, sep = '\n')
}

# comparators
if(!is.null(comparators)){
comparatorCis <- cohorts %>%
  filter(
    .data$subsetParent %in% !!comparators$cohortDefinitionId
  ) %>%
 filter(
    .data$subsetParent %in% !!ciTargetIds$targetCohortDefinitionId
  )

heading <- 'Comparators'
ciIds <- comparatorCis$cohortDefinitionId

result <- knitr::knit_child(text = knitr::knit_expand("cohort_incidence_template.qmd", quiet = TRUE, envir = environment()), quiet = TRUE, envir = environment())
cat(result, sep = '\n')
}





```

:::

```{r}
#| echo: false
#| results: 'asis'

cat('\n# Summary\n')

cat(paste0('\nA retrospective epidemiological analysis was conducted to assess the relationship between ', targetName, ' and ', outcomeName,'.'))

if(includeCohortMethod | includeSccs){
  cat(paste0(
    "In addition to descriptive analyses, ",sum(c(includeCohortMethod ,includeSccs))," causal analytic designs were employed:", 
    ifelse(includeCohortMethod , "an active comparator new user cohort design", ""), 
    ifelse(sum(c(includeCohortMethod ,includeSccs)) == 2, ' and ', ''),
    ifelse(includeSccs, "a self-controlled case series design", ""),             
    ifelse(includeCohortMethod ,paste0("In the active comparator new user cohort design, new users of ", targetName, " were compared to new users of ", comparatorText,"."), ""),
    ifelse(includeSccs, paste0(" In the self-controlled case series design, the incidence of ", outcomeName, " was compared between periods of ",targetName ," use and periods of non-use."), "")
  ))
}
cat("\n\n")

if(includePrediction){
  cat(paste0(
    "A patient-level prediction model was developed to see whether we can identify a sub population of ",targetName," that is high-risk of developing ", outcomeName,".  The model learns uses the retrospectively collected data to find associations between predictors and the development of ", outcomeName," during an at risk period of time following exposure to ",targetName,"." 
  ))
}
cat("\n\n")

```

```{r}
#| label: ci_extract
#| results: 'asis'
#| include: false

  # ===================================
  # Get CI
  # ===================================
  cohortIncidence <- OhdsiReportGenerator::getIncidenceRates(
    connectionHandler = connectionHandler, 
    schema = params$schema, 
    targetIds = unique(c(targetIdsOfInterest, comparatorIdsOfInterest, indicationIdsOfInterest)), 
    outcomeIds = outcomeIdsOfInterest
  ) %>%
  dplyr::filter(
    .data$ageGroupName == 'Any',
    .data$genderName == 'Any',
    .data$startYear == 'Any'
  )

# add the friendly names to the results

tempDf <- cohortDefinitions[,c('friendlyName','cohortDefinitionId',"subsetText")]
colnames(tempDf) <- paste0(colnames(tempDf), 'Target')
cohortIncidence <- merge(cohortIncidence, tempDf, by.x = 'targetId', by.y = 'cohortDefinitionIdTarget')
tempDf <- cohortDefinitions[,c('friendlyName','cohortDefinitionId',"subsetText")]
colnames(tempDf) <- paste0(colnames(tempDf), 'Outcome')
cohortIncidence <- merge(cohortIncidence, tempDf, by.x = 'outcomeId', by.y = 'cohortDefinitionIdOutcome')

```

# Cohort Incidence 

## Method
Incidence proportion per 100 patients, in each database, was calculated by finding the number of patients with `r targetName` recorded in the database who have `r outcomeName` recorded during a time-at-risk relative to exposure divided by the number of patients with `r targetName` recorded in the database multiplied by 100.

Incidence rate per 100 patient years, in each database, was calculated by finding the number of patients with `r targetName` recorded in the database who have `r outcomeName` recorded during a time-at-risk relative to exposure divided by the total number of patient years exposed to `r targetName` in the database multiplied by 100.

## Results
```{r}
#| label: ci_results
#| echo: false
#| results: 'asis'

headings <- cohortIncidence %>%
  dplyr::select('outcomeId', "cleanWindow","tar") %>%
  dplyr::distinct()

if(nrow(headings) > 0){
for(ind in 1:nrow(headings)){
      
      cohortIncidenceOfInt <-  cohortIncidence %>%
        filter(.data$outcomeId == !!headings$outcomeId[ind]) %>%
        filter(.data$tar == !!headings$tar[ind]) %>%
        filter(.data$cleanWindow == !!headings$cleanWindow[ind])
      
      boxTitle <- paste0('The incidence proportion/rate for outcome ', cohortDefinitions$friendlyName[cohortDefinitions$cohortDefinitionId == headings$outcomeId[ind]], ' occuring between ', headings$tar[ind], ' after exposure when the outcome clean window is ', headings$cleanWindow[ind] )
      

      cat('\n\n::: {.callout-note collapse="true"}')
      cat(paste0("\n# ",boxTitle, '\n\n'))
      
      if(nrow(cohortIncidenceOfInt) > 0){
        cat(gt::as_raw_html(
         cohortIncidenceOfInt %>% 
  dplyr::select("friendlyNameTarget","subsetTextTarget", "databaseName","personsAtRisk", "personDays", "personOutcomes", "incidenceProportionP100p", "incidenceRateP100py") %>%
  dplyr::group_by(
    .data$friendlyNameTarget,
    .data$subsetTextTarget
  ) %>%
  gt::gt() %>%
  gt::fmt_integer(
    columns = c("personsAtRisk", "personDays", "personOutcomes"), 
    suffixing = TRUE
  ) %>%
  gt::fmt_number(
    columns = c("incidenceProportionP100p", "incidenceRateP100py"),
    decimals = 4
  ) %>%
  gt::fmt(
    columns = c("personOutcomes","incidenceProportionP100p", "incidenceRateP100py"),
    fns = function(x){ 
      x <- round(x, digits = 4)
      if(sum(x < 0) > 0){ 
        x[x < 0] <- paste('<', abs(x[x < 0]))
      }
      return(x)
    }
  ) %>%
  gt::cols_label(
    databaseName = 'Database',
    personsAtRisk = "Persons at risk", 
    personDays = "Person Days",
    personOutcomes = "Outcome count",
    incidenceProportionP100p = " Incidence proportion per 100 persons",
    incidenceRateP100py = "Incidence rate per 100 person years"
  ) %>%
  gt::tab_options(
    heading.align = "left",
    data_row.padding = gt::px(7),
    column_labels.font.size = gt::px(12),
    column_labels.background.color = "#EB1800",
    column_labels.font.weight = "bold",
    row_group.background.color = "#D3D3D3",
    row_group.font.weight = "bold"
  ) %>%
  gt::tab_style(
    style = gt::cell_text(
      color = "darkgrey",
      font = gt::google_font("Source Sans Pro"),
      transform = "uppercase"
    ),
    locations = gt::cells_column_labels(gt::everything())
  ) %>%
  gt::tab_options(quarto.disable_processing = TRUE)
        ))
        cat('\n')
      } else{
        cat('No results to display\n')
      }
      
      cat('\n\n:::\n')
}
} else{
  cat('No cohort incidence results to display\n')
}

```

---
  output: html_document
---
  
```{r, echo=FALSE, results='asis'}

getExternalVal <- function(
  connectionHandler,
  resultSchema,
  myTableAppend,
  databaseTableAppend,
  modelDesignId,
  devDatabase,
  tempEmulationSchema = NULL
){
  
    sql <- "select per.performance_id, dm.cdm_source_abbreviation as validation_database
      from 
      (select performance_id, validation_database_id from 
      @result_schema.@my_table_appendperformances 
      where model_design_id = @model_design_id 
      and development_database_id != validation_database_id) per
      inner join @result_schema.@my_table_appenddatabase_details d
      on d.database_id = per.validation_database_id
      inner join @result_schema.@database_table_appenddatabase_meta_data dm
      on d.database_meta_data_id = dm.database_id
      where d.database_id = @dev_database
      
      ;
    "
    
  res <- connectionHandler$queryDb(
    sql = sql, 
    result_schema = resultSchema,
    my_table_append = myTableAppend,
    database_table_append = databaseTableAppend,
    model_design_id = modelDesignId,
    dev_database = devDatabase
  )
  
  if(is.null(res$performanceId) || length(res$performanceId)==0){
    return(NULL)
  }
  
  # get the performances for the performanceId
  result <- list()
  length(result) <- length(res$performanceId)
  
  for(i in 1:length(result)){
  result[[i]] <- list()
  tableNames <- c(
    'attrition', 
    'prediction_distribution',
    'covariate_summary',
    'threshold_summary',
    'calibration_summary',
    'evaluation_statistics',
    'demographic_summary'
    )
  length(result[[i]]) <- length(tableNames)+1
  names(result[[i]]) <- c('database',tableNames)
  result[[i]][['database']] <- res$validationDatabase[i]
  
  for(tabName in tableNames){
  
  sql <- "select * from 
    @result_schema.@my_table_append@table_name 
    where performance_id = @performance_id 
    "
  
  result[[i]][[tabName]] <- connectionHandler$queryDb(
    sql = sql, 
    result_schema = resultSchema,
    table_name = tabName,
    my_table_append = myTableAppend,
    performance_id = res$performanceId[i]
  )
  }
  }
  
  return(result)
}

resultList <- getExternalVal(
  connectionHandler = params$connectionHandler,
  resultSchema = params$resultSchema,
  myTableAppend = params$myTableAppend,
  databaseTableAppend = params$databaseTableAppend,
  modelDesignId = params$modelDesignIds[i],
  devDatabase = devDatabase
  )
```

### External Validation

The external validation is the performance of the models when validated using a new database.

```{r results='asis', echo=FALSE}
if(!is.null(resultList)){
  for (i in 1:length(resultList)) {
    result <- resultList[[i]]
    if(!is.null(result)){
      modelDesign <- knitr::knit_child(
        "plp-results-template.Rmd", 
        quiet = TRUE, 
        envir = environment()
        )
      cat(modelDesign, sep = '\n')
      }
    }
  } else{
    cat('\n No external validation available \n')
  }

```

## Self Controlled Case Series
  
::: {style='font-size: 50%;'}
::: {.panel-tabset .nav-pills}

### Study Diagnostics
:::: {.columns}

::: {.column width="85%"}

```{r}
#| label: sccs diagostics

  sccsTableDiag <- OhdsiReportGenerator::getSccsDiagnosticsData(
    connectionHandler = connectionHandler,
    schema = resultsSchema,
    targetIds = targets$cohortDefinitionId,
    outcomeIds = outcomes$cohortDefinitionId
)
  
  # add table
  reactable::reactable(
    data = sccsTableDiag %>%
      dplyr::select(
        "databaseName",
        "targetName",
        "outcomeName",
        'indicationName',
        "analysisId",
        "summaryValue"
      )
    ,
    columns = list(
      databaseName = reactable::colDef(
        name = 'Database'
      ),
      summaryValue = reactable::colDef(
        name = "Overall Diagnostic",
        style = function(value) {
          color <- 'orange'
          if(is.na(value)){
            color <- 'black'
          }else if(value == 'Pass'){
            color <- '#AFE1AF'
          }else if(value == 'Fail'){
            color <- '#E97451'
          }
          list(background = color)
        }
      )
    ), 
    defaultPageSize = defaultPageSize,
    highlight = TRUE, 
    striped = TRUE, 
    searchable = TRUE,
    filterable = TRUE
  )
```

:::
  
::: {.column width="15%"}

> Fail: Study failed ASSURE diagnostics

> Pass: Study passed without warnings

> Number: Passed wth Number of warnings

:::
  
::::
  
```{r}
#| label: sccs estimation

  sccsTable <- OhdsiReportGenerator::getSccsEstimation(
    connectionHandler = connectionHandler,
    schema = resultsSchema,
    targetIds = targets$cohortDefinitionId,
    outcomeIds = outcomes$cohortDefinitionId
  )
  
  sccsMeta <- OhdsiReportGenerator::getSccsMetaEstimation(
    connectionHandler = connectionHandler,
    schema = resultsSchema,
    targetIds = targets$cohortDefinitionId,
    outcomeIds = outcomes$cohortDefinitionId
  )
```

### Table Results
```{r}
#| label: sccs estimation table

  reactable::reactable(
    data = sccsTable %>% 
      dplyr::bind_rows(sccsMeta) %>% 
      dplyr::select(
        'databaseName',
        'targetName',
        'outcomeName',
        'indicationName',
        'description',
        'calibratedRr',
        'calibratedCi95Lb',
        'calibratedCi95Ub',
        'calibratedP',
        'calibratedLogRr',
        'calibratedSeLogRr'
      ),
    columns = list(
      description = reactable::colDef(
        minWidth = 300
      ),
      calibratedRr = reactable::colDef(
        format = reactable::colFormat(digits = 2)
      ),
      calibratedCi95Lb = reactable::colDef(
        format = reactable::colFormat(digits = 2)
      ),
      calibratedCi95Ub = reactable::colDef(
        format = reactable::colFormat(digits = 2)
      ),
      calibratedP = reactable::colDef(
        format = reactable::colFormat(digits = 2)
      ),
      calibratedLogRr = reactable::colDef(
        format = reactable::colFormat(digits = 2)
      ),
      calibratedSeLogRr = reactable::colDef(
        format = reactable::colFormat(digits = 2)
      )
    ),
    defaultPageSize = defaultPageSize,
    highlight = TRUE, 
    striped = TRUE,
    searchable = TRUE,
    filterable = TRUE
  )
```

### Plot Results
::: {.panel-tabset}
```{r}
#| label: sccs estimation plot
#| echo: false
#| results: 'asis'

  analyses <- unique(sccsTable$description)
  analysisIds <- unique(sccsTable$analysisId)
  
  for (i in seq_along(analyses)) {
    analysis <- analyses[[i]]
    analysisId <- analysisIds[[i]]
    cat(paste0('\n\n#### ', analysis ,'\n\n'))
    
   # for each outcome?
for(j in 1:nrow(outcomes)){

mdata <- sccsTable %>% 
    dplyr::filter(.data$outcomeId == outcomes$cohortDefinitionId[j])
esData <- sccsMeta %>% 
    dplyr::filter(.data$outcomeId == outcomes$cohortDefinitionId[j])
if(nrow(esData) == 0){
esData <- NULL
}
if(nrow(mdata) >0 ){
    cat(paste0('\n',outcomes$cohortFriendlyName[j], ' plot: \n\n')) 
    p <- OhdsiReportGenerator::plotSccsEstimates(
      sccsData = mdata,
      sccsMeta = esData,
      targetName = targets$cohortFriendlyName[targets$cohortDefinitionId == params$targetId], # can this handle multiple targets?
      selectedAnalysisId = analysisId
    )
    print(p)
}
}
}

```
:::
  
:::
:::

### Risk Factors
```{r}
#| echo: false
#| results: 'asis'

# Should this only be for the target?
  

# TODO restrict to T and O cohorts of interest
targetIds <- cohortDefinitions %>%
  dplyr::filter(.data$cohortDefinitionId %in% targetIdsOfInterest) %>%
  dplyr::arrange(.data$order, .data$cohortDefinitionId) %>%
  dplyr::select("cohortDefinitionId") 

charIdsOfInt <- unique(data.frame(
  targetCohortId = rep(targetIds$cohortDefinitionId, length(outcomeIdsOfInterest)),
  outcomeCohortId = unlist(lapply(1:length(outcomeIdsOfInterest), function(oi) rep(outcomeIdsOfInterest[oi], length(targetIds$cohortDefinitionId))))
))

# order by id 
charIdsOfInt <- merge(
  x = charIdsOfInt, 
  y = cohortDefinitions[c('cohortDefinitionId', 'order')], 
  by.x = 'targetCohortId', 
  by.y = 'cohortDefinitionId'
) %>%
  dplyr::arrange(dplyr::desc(-1*.data$order), .data$targetCohortId)

if( nrow(charIdsOfInt) >0 ){
  
  for(cind in 1:nrow(charIdsOfInt)){
    # need to run this per t and o
    riskFactorsBinary <- OhdsiReportGenerator::getBinaryRiskFactors(
      connectionHandler = connectionHandler, 
      schema = params$schema, 
      targetId = charIdsOfInt$targetCohortId[cind], #12401, 
      outcomeId = charIdsOfInt$outcomeCohortId[cind], #19970,
      analysisIds = c(1,4,5,210,410)
    )
    
    notNull <- !is.null(riskFactorsBinary)
    hasRows <- FALSE
    if(notNull){
      hasRows <- nrow(riskFactorsBinary) > 0 
    }
    if(notNull & hasRows ){
    riskFactorsBinary <- OhdsiReportGenerator::formatBinaryCovariateName(riskFactorsBinary)
    riskFactorsBinary <- OhdsiReportGenerator::addTarColumn(riskFactorsBinary)
    
    csets <- unique(riskFactorsBinary[, c('tar', 'minPriorObservation','outcomeWashoutDays')])
    
    for(cind2 in 1:nrow(csets)){
      minPriorObservation <- csets$minPriorObservation[cind2]
      outcomeWashoutDays <- csets$outcomeWashoutDays[cind2]
      tar <- csets$tar[cind2]
      
      boxTitle <- paste0(
          'Risk Factors for ', 
          cohortDefinitions$friendlyName[cohortDefinitions$cohortDefinitionId == charIdsOfInt$outcomeCohortId[cind]],
          ' occuring for the first time in ',
          outcomeWashoutDays,
          ' days during ',
          tar,
          ' for patients exposed to ',
          cohortDefinitions$friendlyName[cohortDefinitions$cohortDefinitionId == charIdsOfInt$targetCohortId[cind]], ' ' , cohortDefinitions$subsetText[cohortDefinitions$cohortDefinitionId == charIdsOfInt$targetCohortId[cind]], 
          ' with >= ',
          minPriorObservation,
          ' days prior observation'
        )
      
      # print the binary risk factors in a collapseable box
      cat('\n\n::: {.callout-note collapse="true"}')
      cat(paste0("\n# ",boxTitle, '\n\n'))
      
      temp <- riskFactorsBinary %>%
          dplyr::arrange(-1*.data$absSmd) %>%
          dplyr::filter(
            .data$minPriorObservation == !!minPriorObservation,
            .data$outcomeWashoutDays ==  !!outcomeWashoutDays,
            .data$tar == !!tar,
            .data$absSmd > 0.1,
            .data$caseCount >= 0 # make this an input?
          ) %>%
          head(
            n = 200 # select top 200
          ) %>%
         dplyr::select(
          "covariateName",
          "databaseName",
          "caseCount", 
          "caseAverage",
          "nonCaseCount", 
          "nonCaseAverage",
          "smd", 
          "absSmd"
          )

         print(htmltools::tagList(
    temp %>%
      dplyr::group_by(.data$covariateName) %>%
      gt::gt() %>%
      gt::fmt_percent(
        columns = c("caseAverage","nonCaseAverage"), 
        decimals = 2
      ) %>%
      gt::fmt_number(
        columns = c("smd", "absSmd"), 
        decimals = 2
      ) %>%
      gt::fmt_integer(
        columns = c("caseCount", "nonCaseCount")
      ) %>%
      gt::cols_label(
        covariateName = "Covariate"
      ) %>%
      gt::tab_options(
        heading.align = "left",
        data_row.padding = gt::px(7),
        column_labels.font.size = gt::px(12),
        column_labels.background.color = "#EB1800",
        column_labels.font.weight = "bold",
        row_group.background.color = "#D3D3D3",
        row_group.font.weight = "bold"
      ) %>%
      gt::tab_style(
        style = gt::cell_text(
          color = "darkgrey",
          font = gt::google_font("Source Sans Pro"),
          transform = "uppercase"
        ),
        locations = gt::cells_column_labels(gt::everything())
      ) 
  ))
  
      cat("\n\n:::\n\n")
      
    }}}} else{
      cat('\n\n::: {.callout-note collapse="true"}')
      cat(paste0("\n# No risk factors \n\n"))
      cat("\n\n No risk factors for the target of interest")
      cat("\n\n:::\n\n")
    }

```

# Prediction {.break-slides}
  
## Models
  
::: {style='font-size: 50%;'}

:::: {.panel-tabset .nav-pills}

### Model Summary

:::: {.columns}

::: {.column width="80%"}

```{r}
#| label: prediction_model_summary
#| echo: false
#| results: 'asis'

# panel for outcomes
cat('\n::: {.panel-tabset .nav-pills}')
for(outcomeId in outcomeIds){
  
  outcomeName <- outcomeNames[which(outcomeIds == outcomeId)]
  cat(paste0('\n\n#### ', outcomeName, '\n'))
  
  pTable <- OhdsiReportGenerator::getPredictionPerformances(
    connectionHandler = connectionHandler,
    schema = resultsSchema
  )
  
 OhdsiReportGenerator::printReactable(
        data = pTable %>% 
dplyr::filter(
.data$developmentTargetId == !!targetId &
.data$developmentOutcomeId == !!outcomeId
) %>%
          dplyr::select(
            'modelDesignId',
            #'modelType',
            'developmentTargetName',
            'developmentOutcomeName',
            'developmentDatabase',
            'validationDatabase',
            'validationTimeAtRisk',
            'populationSize',
            'outcomeCount',
            'auroc',
            'eStatistic',
            'brierScore'
          ),
        columns = list(
          auroc = reactable::colDef(
            name = 'AUROC',
            format = reactable::colFormat(digits = 2)
          ),
          eStatistic = reactable::colDef(
            name = 'e-stat',
            format = reactable::colFormat(digits = 2)
          ),
          brierScore = reactable::colDef(
            name = 'brier',
            format = reactable::colFormat(digits = 2)
          ),
          populationSize = reactable::colDef(
            name = 'N'
          )
        ),
        defaultPageSize = defaultPageSize,
        highlight = TRUE, 
        striped = TRUE
      )
  
}
cat('\n:::') # outcome tabs

```

:::
  
::: {.column width="20%"}

> AUROC: A measure of model discrimination. 1 is best.  Probability a random person with the outcome has a higher risk than an random person without the outcome.

> e-stat: A measure of calibration.  Smaller is better. How accurate the predicted risks are.

> brier: Overall error. Smaller is better.

:::
  
::::
  
  
### Top Predictors
  
```{r}
#| label: prediction_top_pred
#| echo: false
#| results: 'asis'

# panel for outcomes
cat('\n\n:::: {.panel-tabset .nav-pills}')
for(outcomeId in outcomeIds){
  
  outcomeName <- outcomeNames[which(outcomeIds == outcomeId)]
  cat(paste0('\n\n#### ', outcomeName, '\n\n'))
  
  topPreds <- OhdsiReportGenerator::getPredictionTopPredictors(
    connectionHandler = connectionHandler,
    schema = resultsSchema,
    targetIds = targetId,
    outcomeIds = outcomeId, # ToDo loop outcomes
    numberPredictors = 50
  )
  
  if(nrow(topPreds) >0 ){
    databases <- unique(topPreds$databaseName)
    
    cat('\n\n::: {.panel-tabset .nav-pills}')
    for(database in databases){
      
      cat(paste0('\n\n##### ', database))
      cat('\n\n')
      
      OhdsiReportGenerator::printReactable(
            data = topPreds %>% 
              dplyr::filter(.data$databaseName == !!database) %>%
              dplyr::mutate(
                tar = paste0(
                  '( ',.data$tarStartAnchor, ' + ', .data$tarStartDay, ' ) - ( ', 
                  .data$tarEndAnchor, ' + ', .data$tarEndDay, ' )'
                )
              ) %>%
              dplyr::select(
                'tar',
                'covariateName',
                'covariateValue',
                'withNoOutcomeCovariateMean',
                'withOutcomeCovariateMean',
                'standardizedMeanDiff'
              ),
            columns = list(
              standardizedMeanDiff = reactable::colDef(
                name = 'SMD',
                format = reactable::colFormat(digits = 2)
              ),
              withOutcomeCovariateMean = reactable::colDef(
                name = 'With Outcome',
                format = reactable::colFormat(digits = 2)
              ),
              withNoOutcomeCovariateMean = reactable::colDef(
                name = 'No Outcome',
                format = reactable::colFormat(digits = 2)
              ),
              covariateValue = reactable::colDef(
                name = 'Value',
                format = reactable::colFormat(digits = 2)
              ),
              
              
              covariateName = reactable::colDef(
                name = 'Predictor'
              ),
              tar = reactable::colDef(
                name = 'Time-at-risk'
              )
            ),
            defaultPageSize = defaultPageSize,
            highlight = TRUE, 
            striped = TRUE
          )
    }
    cat('\n\n:::')
  }
}
cat('\n\n::::')
```

::::
  
:::

# Definitions

```{r}
#| label: exposure
#| echo: false
#| results: 'asis'


cat('\n\n## Exposure Identification\n')

cat(paste0('Patients in the ', targetName,' cohort were identified using the following logic: \n\n'))

cat('\n::: {.callout-important}\n')
cat(paste0('\n### ', targetName, ' definition\n'))
cat(
  tryCatch({
    CirceR::cohortPrintFriendly(cohortDefinitions$json[cohortDefinitions$cohortDefinitionId == params$targetId])
  }, error = function(e){
   print(e); return('Error rendering friendly cohort...')
  })
)
cat('\n:::\n')

if(!is.null(indicationIds)){
cat("\n\n")
cat(paste0("The analysis was implemented for ", length(indicationIds)," indication/s (",
           paste0(unique(cohortDefinitions$friendlyName[cohortDefinitions$subsetParent %in% indicationIds]), collapse = '/')
           ,")."))

}

```   

```{r}
#| label: outcome
#| echo: false
#| results: 'asis'

outcomeNames <- params$cohortNames[params$cohortIds %in% params$outcomeIds]

for(ind in 1:length(outcomeNames)){
  cat(paste0("\n\n## Outcome: ",outcomeNames[ind]," \n"))
  
  cat(paste0('Patients in the ', outcomeNames[ind],' cohort were identified using the following logic: \n\n'))
  
  cat('\n::: {.callout-important}\n')
  cat(paste0('\n### ', outcomeNames[ind], ' definition\n'))
  
  cat(
  tryCatch({
    CirceR::cohortPrintFriendly(cohortDefinitions$json[cohortDefinitions$cohortDefinitionId == params$outcomeIds[ind]])
  }, error = function(e){
   print(e); return('Error rendering friendly cohort...')
  })
)
  
  cat('\n:::\n')
  
}

```   

```{r}
#| label: indications
#| echo: false
#| results: 'asis'

if(!is.null(indicationIds)){
for(i in 1:length(indicationIds)){
  if(!indicationIds[i] == ''){
    indication <- cohortDefinitions$friendlyName[cohortDefinitions$cohortDefinitionId == indicationIds[i]]
    
    cat(paste0("\n\n## Indication: ",indication," \n"))
    
    cat(paste0('Patients in the ', indication,' cohort were identified using the following logic: \n\n'))
    
    cat('\n::: {.callout-important}\n')
    cat(paste0('\n### ', indication, ' definition\n'))
    
      cat(
  tryCatch({
    CirceR::cohortPrintFriendly(cohortDefinitions$json[cohortDefinitions$cohortDefinitionId == indicationIds[i]])
  }, error = function(e){
   print(e); return('Error rendering friendly cohort...')
  })
)
   
    cat('\n:::\n')
    
  }
}
}


```   

```{r}
#| label: comparators_def
#| echo: false
#| results: 'asis'

if(length(comparatorIds) > 0 ){
  for(i in 1:length(comparatorIds)){
    if(!is.na(comparatorIds[i])){  
      name <- cohortDefinitions$friendlyName[cohortDefinitions$cohortDefinitionId == comparatorIds[i]]
      
      cat(paste0("\n\n## Comparator: ",name," \n"))
      
      cat(paste0('Patients in the ', name,' cohort were identified using the following logic: \n\n'))
      
      cat('\n::: {.callout-important}\n')
      cat(paste0('\n### ', name, ' definition\n'))
      
 cat(
  tryCatch({
    CirceR::cohortPrintFriendly(cohortDefinitions$json[cohortDefinitions$cohortDefinitionId == comparatorIds[i]])
  }, error = function(e){
   print(e); return('Error rendering friendly cohort...')
  })
)
            
      cat('\n:::\n')
    }
  }
}


```       

---
title: "PatientLevelPrediction Protocol "
date: "`r Sys.Date() `"
params:
  include_diagnostic: TRUE
  include_results: TRUE
  plpCitation: "Reps JM, Schuemie MJ, Suchard MA, Ryan PB, Rijnbeek P (2018). 'Design and implementation of a standardized framework to generate and evaluate patient level prediction models using observational healthcare data'. Journal of the American Medical Informatics Association, 25(8), 969-975."
  tripodCitation: "Collins, G., et al. (2017). 'Transparent reporting of a multivariable prediction model for individual prognosis or diagnosis (TRIPOD): the TRIPOD statement.' Journal of British Surgery 102.3 (2015): 148-158."
  progressCitation: "Steyerberg EW, Moons KG, van der Windt DA, Hayden JA, Perel P, Schroter S, Riley RD, Hemingway H, Altman DG; PROGRESS Group. Prognosis Research Strategy (PROGRESS) 3: prognostic model research. PLoS Med. 2013;10(2):e1001381."
  rCitation: "R Core Team (2013). R: A language and environment for statistical computing. R Foundation for Statistical Computing, Vienna, Austria. URL http://www.R-project.org/."
  connectionHandler: ""
  resultSchema: "plp_results"
  myTableAppend: ""
  databaseTableAppend: ""
  cohortTableAppend: ""
  modelDesignIds: 1
format: 
  html:
    toc: true
    toc-location: left
    toc_depth: 5
    toc_float: true
---

# Project Details
  
Study Title: PatientLevelPrediction Report
Prepared on: ``r Sys.Date()``

# Abbreviations

```{r abbreviations, echo=FALSE}

abb <- data.frame(rbind(
  c("AUROC", "Area Under the Receiver Operating Characteristic Curve"),
  c("CDM","Common Data Model"),
  c("O","Outcome Cohort"),
  c("OHDSI","Observational Health Data Sciences & Informatics"),
  c("OMOP","Observational Medical Outcomes Partnership"),
  c("T", "Target Cohort"),
  c("TAR", "Time at Risk")
))
names(abb) <- c("Abbreviation","Phrase")
abb <- abb[order(abb$Abbreviation),]

knitr::kable(x = abb, caption = 'List of Abbreviations')

```


# Framework

In this study the model development follows the framework presented in ``r  params$plpCitation ``

# Evaluation Metrics and Plots

```{r metrics, child = "plp-metrics.Rmd"}
```

```{r output, child = "plp-output.Rmd"}
```

```{r plots, child = "plp-plots.Rmd"}
```


```{r design diag results, results='asis', echo=FALSE}
    for (i in 1:length(params$modelDesignIds)) {
      modelDesignId <- params$modelDesignIds[i]
      modelDesign <- knitr::knit_child("model-design.Rmd", quiet = TRUE, envir = environment())
      cat(modelDesign, sep = '\n')
      
      if(params$include_diagnostic){
      diagnostic <- knitr::knit_child("plp-diagnostic.Rmd", quiet = TRUE, envir = environment())
      cat(diagnostic , sep = '\n')
      }
      
      if(params$include_results){
      results <- knitr::knit_child("plp-results.Rmd", quiet = TRUE, envir = environment())
      cat(results , sep = '\n')
      }
    }

```


# Tools

This study will be designed using OHDSI tools and run with R

``r params$rCitation ``

# Causal Inference

## Methods
### Active Comparator New User Design
In the active comparator new user design, large-scale propensity scores were used to match new users of `r targetName` to new users of the comparator in a 1:n ratio (max n=100). Propensity score models included all variables available in the observational medical history, including all previously observed conditions, diagnoses, medications dispensed, measurements, and procedures recorded in the 30 and 365 days preceding the index date. Comorbidity indices calculated using all pre-index medical history and demographic variables (age, sex, race/ethnicity, index year, index month) were also included. Poisson models were used to estimate incidence rate ratios comparing the incidence of `r outcomeName` between new users of `r targetName` and new users of the comparator.

```{r, echo = FALSE, results = 'asis'}

nNegativeControls <- 100

if(includeCohortMethod){
  
  if(is.null(indicationIds)){
    cmIndicationIds <- ""
  } else{
    cmIndicationIds <- indicationIds
  }
  
for(i in 1:length(cmIndicationIds)){
  
  indication <- 'No indication'
  if(!cmIndicationIds[i] == ''){
    indication <- cohortDefinitions$friendlyName[cohortDefinitions$cohortDefinitionId == cmIndicationIds[i]]
  }
  
  exposures <- cohortDefinitions$friendlyName[cohortDefinitions$cohortDefinitionId %in% comparatorIdsOfInterest]
  
  cat(paste0("\n\n##### ", indication))
  
  cat(paste0("\n", length(exposures)+1, " exposure cohorts were created for the active comparator new user cohort design: new users of ", targetName, " (target) and new users of ", paste0(exposures, collapse ='/')," (comparators)."))
  
  cat(paste0("The index date was defined as the first observed use of either the, ", targetName, ", or ",paste0(exposures, collapse ='/'),  ". "))
  
  if(indication != 'No indication'){
    cat(paste0("Patients were only included if they had a history of ", indication, ".\n")) 
  }
  
  if(!is.null(params$excludeText)){ # replace this using cohort subset logic
    cat(
      paste0(
        "Patients were excluded if ", 
        params$excludeText, 
        "."
      )
    )
  }
  
  
  cat(paste0(
    "\nPatients were followed from the index date until the earliest of discontinuation of the index ", targetName, "  or switch to or addition of the comparator agent, end of observation in the database, or the occurrence of ",  outcomeName, " for an on-treatment analysis. Patients were considered to have discontinued their index exposure at the first occurrence of a gap of > ",  params$drugGap, " days between the end of one dispensing’s supply and the beginning of the next dispensing. We restricted the cohorts to patients with >= 365 days of prior database observation."
  ))
  
  cat(paste0(
    "\n\nFor the self-controlled case series design and incidence rate the exposure cohort was all exposures of ",  targetName,". Patients were only included if they had a history of ", indication, ". "
  )
  )
  
  
}
}

```


### Self-Controlled Case Series Design
In the self-controlled case series design, Poisson regression was used to compare the incidence of `r outcomeName` between the day of `r targetName` exposure and periods of non-use. Spline terms for calendar time, season, and age were included in the model to adjust for time-varying confounding.

### Diagnostics
In both designs, negative control outcomes were used to quantify and correct for systematic bias via empirical calibration (Schuemie 2016; 2018). `r nNegativeControls` conditions with no association to `r targetName`, as identified in the literature, drug product labels, or AE reports, were selected as negative controls. Objective diagnostics related to statistical power and systematic error observed in negative controls were employed in both designs, while additional diagnostics related to covariate balance and temporal trends in incidence were employed in the active comparator new user and self-controlled case series analyses, respectively. These diagnostics were used to determine whether a given analysis could be unblinded and included in meta-analyses and are described in the table below.

All analyses were conducted separately within each database and Bayesian random-effects meta-analysis was used to summarize across databases.


| Diagnostic | Description | Relevant Analytic Design | 
  |-------|--------|-------|
  | Equipoise |  	We compute the percent of the population with a preference score between 0.3 and 0.7, called equipoise. The preference score is a linear transformation of the propensity score. The diagnostic failure threshold is an equipoise value of less than 20%. |	Comparative Cohort |
  | Covariate Balance |	We compute the standardized difference of mean (SDM) for every covariate that was used to balance any two exposure groups. The diagnostic failure threshold is any covariate having a SDM greater than or equal to 0.1. |	Comparative Cohort |
  | Systematic Error |	We fit a systematic error distribution using negative control estimates and summarize this as the Expected Absolute Systematic Error (EASE). An EASE of 0 means all variations in the negative control estimates can be explained by random error (as expressed for example in the CI). The diagnostic failure threshold for EASE is a value of greater than or equal to 0.25. |	Comparative Cohort; SCCS |
  | Study Power |	We compute the minimally detectable rate ratio which represents a transformation of study power noting the smallest observed effect estimate that we could expect to generate a significant finding (at an alpha level of 0.05), given the observed number of people and outcomes within the relevant study populations.  The diagnostic failure threshold for MDRR is a value of greater than or equal to 10. |	Comparative Cohort; SCCS |
  | Time Trend | The rate of an outcome may change as a function of calendar time. This may occur in instances because of data capture lag (incurred but not documented is expected to occur close to the date of data extraction), leading to (within subject) confounding. Although our design adjusts for this using a spline function over calendar time, we test if this adjustment was sufficient. For each month, we compute the rate of the outcome after adjustment using the spline, and test whether the rate after adjustment differs from the mean monthly rate before adjustment. We use a family-wise alpha of 0.05 and apply a Bonferroni correction for the number of months. The diagnostic failure threshold is the detection of at least one month having a rate statistically different from the mean. |	SCCS |
  | Reverse Causality | We test if the risk of the outcome is already increased just prior to the outcome. We compute p for (rate 30 days before exposure) > (rate 30 days after exposure). The diagnostic failure threshold for reverse causality test is a P < 0.05. |	SCCS |
  : Table 1:	Diagnostics For Each Analysis Design
  
  
## Results  
### Active Comparator New User Design Diagnostics
```{r}
#| label: cohort_method_results
#| echo: false
#| results: 'asis'

if(includeCohortMethod){

tempDf <- cohortDefinitions[, c("friendlyName", 'cohortDefinitionId', 'subsetText')]
colnames(tempDf) <- paste0(colnames(tempDf), 'Target')
cohortMethodDiagnostics <- merge(cohortMethodDiagnostics, 
                                 tempDf, by.x = 'targetId', by.y = 'cohortDefinitionIdTarget')
tempDf <- cohortDefinitions[, c("friendlyName", 'cohortDefinitionId')]
colnames(tempDf) <- paste0(colnames(tempDf), 'Outcome')
cohortMethodDiagnostics <- merge(cohortMethodDiagnostics, 
                                 tempDf, by.x = 'outcomeId', by.y = 'cohortDefinitionIdOutcome')
tempDf <- cohortDefinitions[, c("friendlyName", 'cohortDefinitionId')]
colnames(tempDf) <- paste0(colnames(tempDf), 'Comp')
cohortMethodDiagnostics <- merge(cohortMethodDiagnostics, 
                                 tempDf, by.x = 'comparatorId', by.y = 'cohortDefinitionIdComp')

descriptions <- unique(cohortMethodDiagnostics$description)

for(description in descriptions){
  
  cat(paste0('\n\n#### ',description,'\n\n'))
  
  cohortMethodDiagnosticsTemp <- cohortMethodDiagnostics %>% 
    dplyr::filter(.data$description == !!description)
  cmIds <- unique(cohortMethodDiagnosticsTemp[,c('targetId', 'outcomeId')])
  
  for(cmInd in 1:nrow(cmIds)){
   cat(gt::as_raw_html(cohortMethodDiagnostics %>% 
                           dplyr::filter(.data$description == !!description) %>%
                           dplyr::filter(.data$targetId == !!cmIds$targetId[cmInd]) %>%
                           dplyr::filter(.data$outcomeId == !!cmIds$outcomeId[cmInd]) %>%
                           dplyr::select("databaseName", "subsetTextTarget", #"description",
                                         "friendlyNameComp", "indicationName",
                                         #"friendlyNameTarget","friendlyNameOutcome",
                                         "balanceDiagnostic", "sharedBalanceDiagnostic","equipoiseDiagnostic",
                                         "mdrrDiagnostic", "attritionDiagnostic", "easeDiagnostic",
                                         'unblindForEvidenceSynthesis',
                                         'unblind')%>%
                           dplyr::group_by(
                             .data$friendlyNameComp,
                             .data$subsetTextTarget,
                             .data$indicationName
                           ) %>%
                           gt::gt() %>%
                           gt::cols_label(
                             databaseName = 'Database',
                             #description = "Analysis Description", 
                             friendlyNameComp = "Comparator",
                             indicationName = "Indication",
                             balanceDiagnostic = "balance",
                             sharedBalanceDiagnostic = "shared balance",
                             equipoiseDiagnostic = "equipoise",
                             mdrrDiagnostic = "mdrr", 
                             attritionDiagnostic = "attrition", 
                             easeDiagnostic = "ease",
                             unblindForEvidenceSynthesis = "unblind For ES"
                           ) %>%
                           gt::tab_options(
                             heading.align = "left",
                             data_row.padding = gt::px(7),
                             column_labels.font.size = gt::px(12),
                             column_labels.background.color = "#EB1800",
                             column_labels.font.weight = "bold",
                             row_group.background.color = "#D3D3D3",
                             row_group.font.weight = "bold"
                           ) %>%
                           gt::tab_style(
                             style = gt::cell_text(
                               color = "darkgrey",
                               font = gt::google_font("Source Sans Pro"),
                               transform = "uppercase"
                             ),
                             locations = gt::cells_column_labels(gt::everything())
                           ) %>%
                           gt::tab_header(
                             title = paste0('Target: ', cohortDefinitions$friendlyName[cohortDefinitions$cohortDefinitionId == cmIds$targetId[cmInd]]), 
                             subtitle = paste0('Outcome: ', cohortDefinitions$friendlyName[cohortDefinitions$cohortDefinitionId == cmIds$outcomeId[cmInd]])
                           )
    ))
  }
}

}

```

### Active Comparator New User Design Estimates
```{r}
#| label: cohort_method_diagnostics
#| echo: false
#| results: 'asis'

if(includeCohortMethod){
  
  # add Evidence synth if the results exist
  if(!is.null(cohortMethodES)){
    cohortMethodResults <- dplyr::bind_rows(cmResults, cohortMethodES)
  } 

tempDf <- cohortDefinitions[, c("friendlyName", 'cohortDefinitionId', 'subsetText')]
colnames(tempDf) <- paste0(colnames(tempDf), 'Target')
cohortMethodResults <- merge(cohortMethodResults, 
                             tempDf, by.x = 'targetId', by.y = 'cohortDefinitionIdTarget')
tempDf <- cohortDefinitions[, c("friendlyName", 'cohortDefinitionId')]
colnames(tempDf) <- paste0(colnames(tempDf), 'Outcome')
cohortMethodResults <- merge(cohortMethodResults, 
                             tempDf, by.x = 'outcomeId', by.y = 'cohortDefinitionIdOutcome')
tempDf <- cohortDefinitions[, c("friendlyName", 'cohortDefinitionId')]
colnames(tempDf) <- paste0(colnames(tempDf), 'Comp')
cohortMethodResults <- merge(cohortMethodResults, 
                             tempDf, by.x = 'comparatorId', by.y = 'cohortDefinitionIdComp')


descriptions <- unique(cohortMethodResults$description)

for(description in descriptions){
  
  cat(paste0('\n\n#### ',description,'\n\n'))
  
  cohortMethodResultsTemp <- cohortMethodResults %>% 
    dplyr::filter(.data$description == !!description)
  cmIds <- unique(cohortMethodResultsTemp[,c('targetId', 'outcomeId')])
  
  for(cmInd in 1:nrow(cmIds)){
    cat(gt::as_raw_html(cohortMethodResults %>% 
                           dplyr::filter(.data$description == !!description) %>%
                           dplyr::filter(.data$targetId == !!cmIds$targetId[cmInd]) %>%
                           dplyr::filter(.data$outcomeId == !!cmIds$outcomeId[cmInd]) %>% 
                           dplyr::select("databaseName","subsetTextTarget",
                                         "friendlyNameComp", "indicationName", 
                                         "targetSubjects", "comparatorSubjects",
                                         "targetOutcomes", "comparatorOutcomes",
                                         "calibratedRr", "calibratedCi95Lb",
                                         "calibratedP","calibratedOneSidedP")%>%
                           dplyr::group_by(
                             .data$friendlyNameComp,
                             .data$subsetTextTarget,
                             .data$indicationName
                           ) %>%
                           gt::gt() %>%
                           gt::cols_label(
                             databaseName = 'Database',
                             #description = "Analysis Description", 
                             friendlyNameComp = "Comparator",
                             indicationName = "Indication",
                             targetSubjects = "Target N",
                             comparatorSubjects = "Comp N",
                             calibratedRr  = "Cal. RR", 
                             calibratedCi95Lb = "Cal. Ci95%Lb",
                             calibratedP = "Cal. P",
                             calibratedOneSidedP = "Cal. OneSidedP"
                           ) %>%
                           gt::fmt_integer(
                             columns = c("targetSubjects", "comparatorSubjects")
                           ) %>%
                           gt::fmt_number(
                             columns = c( "calibratedRr", "calibratedCi95Lb",
                                          "calibratedP","calibratedOneSidedP"), 
                             decimals = 4
                           ) %>%
                           gt::fmt(
                             columns = c("targetOutcomes", "comparatorOutcomes"),
                             fns = function(x){ 
                               if(sum(x < 0) > 0){ 
                                 x[x < 0] <- paste('<', abs(x[x < 0]))
                               }
                               return(x)
                             }) %>%
                           gt::tab_options(
                             heading.align = "left",
                             data_row.padding = gt::px(7),
                             column_labels.font.size = gt::px(12),
                             column_labels.background.color = "#EB1800",
                             column_labels.font.weight = "bold",
                             row_group.background.color = "#D3D3D3",
                             row_group.font.weight = "bold"
                           ) %>%
                           gt::tab_style(
                             style = gt::cell_text(
                               color = "darkgrey",
                               font = gt::google_font("Source Sans Pro"),
                               transform = "uppercase"
                             ),
                             locations = gt::cells_column_labels(gt::everything())
                           ) %>%
                           gt::tab_header(
                             title = paste0('Target: ', cohortDefinitions$friendlyName[cohortDefinitions$cohortDefinitionId == cmIds$targetId[cmInd]]), 
                             subtitle = paste0('Outcome: ', cohortDefinitions$friendlyName[cohortDefinitions$cohortDefinitionId == cmIds$outcomeId[cmInd]])
                           )
    ))
  }
}

}
```


### Active Comparator New User Design Plots
```{r}
#| label: cohort_method_plots
#| echo: false
#| results: 'asis'
#| fig-width: 13
#| out-width: "100%"
#| fig-align: "center"

if(includeCohortMethod){
  
  analysisIds <- unique(cmResults$analysisId)
  
  for(analysisId in analysisIds){
  
    plots <- OhdsiReportGenerator::plotCmEstimates(
              cmData = cmResults, 
              cmDiagnostics = cohortMethodDiagnostics, 
              cmMeta = cohortMethodES, 
              includeCounts = TRUE, 
              selectedAnalysisId = analysisId
              )
    
    for(i in 1:length(plots)){
      cat('\n\n::: {.callout-note collapse="true"}\n')
      cat(paste0('## ', names(plots)[i], '\n\n'))
      print(plots[[i]])
      cat('\n:::\n')
    }
    
  }

  
}
```


### Self-Controlled Case Series Design Diagnostics
```{r}
#| label: sccs_diagnostics
#| echo: false
#| results: 'asis'

if(includeSccs){

tempDf <- cohortDefinitions[, c("friendlyName", 'cohortDefinitionId', 'subsetText')]
colnames(tempDf) <- paste0(colnames(tempDf), 'Target')
sccsDiagnostics <- merge(sccsDiagnostics, 
                         tempDf, by.x = 'targetId', by.y = 'cohortDefinitionIdTarget')
tempDf <- cohortDefinitions[, c("friendlyName", 'cohortDefinitionId')]
colnames(tempDf) <- paste0(colnames(tempDf), 'Outcome')
sccsDiagnostics <- merge(sccsDiagnostics, 
                         tempDf, by.x = 'outcomeId', by.y = 'cohortDefinitionIdOutcome')

# join by "indicationId" to add friendlyNameIndication
tempDf <- cohortDefinitions[, c("friendlyName", 'cohortDefinitionId')]
colnames(tempDf) <- paste0(colnames(tempDf), 'Indication')
sccsDiagnostics <- merge(sccsDiagnostics, 
                     tempDf, by.x = 'indicationId', by.y = 'cohortDefinitionIdIndication', all.x = TRUE)


sccsIds <- unique(sccsDiagnostics[,c('targetId', 'outcomeId')])

for(sccsInd in 1:nrow(sccsIds)){
  cat(gt::as_raw_html(sccsDiagnostics %>% 
                         dplyr::filter(.data$targetId == !!sccsIds$targetId[sccsInd]) %>%
                         dplyr::filter(.data$outcomeId == !!sccsIds$outcomeId[sccsInd]) %>%
                         dplyr::select("databaseName", "subsetTextTarget", "description","friendlyNameIndication",
                                       #"friendlyNameTarget","friendlyNameOutcome",
                                       "mdrrDiagnostic", "easeDiagnostic","timeTrendDiagnostic",
                                       "preExposureDiagnostic", 
                                       "unblindForEvidenceSynthesis",
                                       'unblind'
                         )%>%
                         dplyr::group_by(
                           .data$description,
                           .data$subsetTextTarget,
                           .data$friendlyNameIndication
                         ) %>%
                         gt::gt() %>%
                         gt::cols_label(
                           databaseName = 'Database',
                           description = "Analysis Description",
                           mdrrDiagnostic = "mdrr", 
                           easeDiagnostic = "ease",
                           timeTrendDiagnostic = "timeTrend",
                           preExposureDiagnostic = "preExposure",
                           unblindForEvidenceSynthesis = "unblind for ES"
                         ) %>%
                         gt::tab_options(
                           heading.align = "left",
                           data_row.padding = gt::px(7),
                           column_labels.font.size = gt::px(12),
                           column_labels.background.color = "#EB1800",
                           column_labels.font.weight = "bold",
                           row_group.background.color = "#D3D3D3",
                           row_group.font.weight = "bold"
                         ) %>%
                         gt::tab_style(
                           style = gt::cell_text(
                             color = "darkgrey",
                             font = gt::google_font("Source Sans Pro"),
                             transform = "uppercase"
                           ),
                           locations = gt::cells_column_labels(gt::everything())
                         ) %>%
                         gt::tab_header(
                           title = paste0('Target: ', cohortDefinitions$friendlyName[cohortDefinitions$cohortDefinitionId == sccsIds$targetId[sccsInd]]), 
                           subtitle = paste0('Outcome: ', cohortDefinitions$friendlyName[cohortDefinitions$cohortDefinitionId == sccsIds$outcomeId[sccsInd]])
                         )
  ))
}

}

```

### Self-Controlled Case Series Design Estimates
```{r}
#| label: sccs_results
#| echo: false
#| results: 'asis'


if(includeSccs){
  # add evidence synth results exist
  if(!is.null(sccsES)){
    sccsCombined <- dplyr::bind_rows(sccsResults, sccsES)
  } else{
    sccsCombined <- sccsResults
  }

tempDf <- cohortDefinitions[, c("friendlyName", 'cohortDefinitionId', 'subsetText')]
colnames(tempDf) <- paste0(colnames(tempDf), 'Target')
sccsCombined <- merge(sccsCombined, 
                     tempDf, by.x = 'targetId', by.y = 'cohortDefinitionIdTarget')
tempDf <- cohortDefinitions[, c("friendlyName", 'cohortDefinitionId')]
colnames(tempDf) <- paste0(colnames(tempDf), 'Outcome')
sccsCombined <- merge(sccsCombined, 
                     tempDf, by.x = 'outcomeId', by.y = 'cohortDefinitionIdOutcome')

# join by "indicationId" to add friendlyNameIndication
tempDf <- cohortDefinitions[, c("friendlyName", 'cohortDefinitionId')]
colnames(tempDf) <- paste0(colnames(tempDf), 'Indication')
sccsCombined <- merge(sccsCombined, 
                     tempDf, by.x = 'indicationId', by.y = 'cohortDefinitionIdIndication', all.x = TRUE)


sccsIds <- unique(sccsCombined[,c('targetId', 'outcomeId')])

for(sccsInd in 1:nrow(sccsIds)){
  cat(gt::as_raw_html(sccsCombined %>% 
                         dplyr::filter(.data$targetId == !!sccsIds$targetId[sccsInd]) %>%
                         dplyr::filter(.data$outcomeId == !!sccsIds$outcomeId[sccsInd]) %>%
                         dplyr::select("databaseName","subsetTextTarget", "description",
                                       'friendlyNameIndication',
                                       #"friendlyNameOutcome","friendlyNameTarget", 
                                       "outcomeSubjects", "covariateSubjects",
                                       "outcomeEvents", "covariateOutcomes",
                                       "calibratedRr", "calibratedCi95Lb", 
                                       "calibratedP","calibratedOneSidedP") %>%
                         dplyr::group_by(
                           .data$description,
                           .data$subsetTextTarget,
                           .data$friendlyNameIndication
                         ) %>%
                         gt::gt() %>%
                         gt::cols_label(
                           databaseName = 'Database',
                           description = "Analysis Description",
                           covariateSubjects = "Covariate N",
                           outcomeSubjects = "Outcome N",
                           calibratedRr  = "Cal. RR", 
                           calibratedCi95Lb = "Cal. Ci95%Lb",
                           calibratedP = "Cal. P",
                           calibratedOneSidedP = "Cal. OneSidedP"
                         ) %>%
                         gt::fmt_integer(
                           columns = c("outcomeSubjects")
                         ) %>%
                         gt::fmt_number(
                           columns = c( "calibratedRr", "calibratedCi95Lb", "calibratedP","calibratedOneSidedP"), 
                           decimals = 4
                         ) %>%
                         gt::fmt(
                           columns = c("covariateSubjects","outcomeEvents", "covariateOutcomes"),
                           fns = function(x){ 
                             if(sum(x < 0) > 0){ 
                               x[x < 0] <- paste('<', abs(x[x < 0]))
                             }
                             return(x)
                           }) %>%
                         gt::tab_options(
                           heading.align = "left",
                           data_row.padding = gt::px(7),
                           column_labels.font.size = gt::px(12),
                           column_labels.background.color = "#EB1800",
                           column_labels.font.weight = "bold",
                           row_group.background.color = "#D3D3D3",
                           row_group.font.weight = "bold"
                         ) %>%
                         gt::tab_style(
                           style = gt::cell_text(
                             color = "darkgrey",
                             font = gt::google_font("Source Sans Pro"),
                             transform = "uppercase"
                           ),
                           locations = gt::cells_column_labels(gt::everything())
                         ) %>%
                         gt::tab_header(
                           title = paste0('Target: ', cohortDefinitions$friendlyName[cohortDefinitions$cohortDefinitionId == sccsIds$targetId[sccsInd]]), 
                           subtitle = paste0('Outcome: ', cohortDefinitions$friendlyName[cohortDefinitions$cohortDefinitionId == sccsIds$outcomeId[sccsInd]])
                         )
  ))
}

}

```


### Self-Controlled Case Series Design Plots
```{r}
#| label: sccs_plots
#| echo: false
#| results: 'asis'
#| fig-width: 13
#| out-width: "100%"
#| fig-align: "center"

if(includeSccs){
  
  analysisIds <- unique(sccsResults$analysisId)
  
  for(analysisId in analysisIds){
  
    plots <- OhdsiReportGenerator::plotSccsEstimates(
              sccsData = sccsResults, 
              sccsDiagnostics = sccsDiagnostics, 
              sccsMeta = sccsES, 
              includeCounts = TRUE, 
              selectedAnalysisId = analysisId
              )
    
    for(i in 1:length(plots)){
      cat('\n\n::: {.callout-note collapse="true"}\n')
      cat(paste0('## ', names(plots)[i], '\n\n'))
      print(plots[[i]])
      cat('\n:::\n')
    }
    
  }

  
}
```

```{r}
#| echo: false
#| results: 'asis'
#| fig-width: 12

modelPerf <- OhdsiReportGenerator::getPredictionPerformances(
  connectionHandler = connectionHandler, 
  schema = schema, 
  plpTablePrefix = params$plpTablePrefix, 
  cgTablePrefix = params$cgTablePrefix, 
  databaseTable = 'database_meta_data', 
  databaseTablePrefix = params$databaseTablePrefix, 
  modelDesignId = modelDesignOfIntCohorts$modelDesignId, 
  developmentDatabaseId = NULL
    )

cat(paste0('\n\n### ', TA))
cat('\n\n')

if(nrow(modelPerf) > 0){
modelPerf  %>%
  ggplot2::ggplot( ggplot2::aes(
    x = developmentTargetName, 
    y = developmentOutcomeName, 
    fill = auroc
    ) 
    ) +
      ggplot2::geom_tile() +
  ggplot2::facet_grid(. ~ .data$developmentDatabase) +
      #ggplot2::scale_fill_gradient(low="white", high="#006400") +
  ggplot2::scale_fill_gradientn(
    colors = c("black", "#494848", "#636363","#909090",
                "#D4D4D4", "white", "white" 
                 ),
    values = c(1, 0.9, 0.8, 0.7, 0.6, 0.5, 0), 
    limits = c(0, 1)
    ) +
  ggplot2::theme(axis.title.x=ggplot2::element_blank(),
        axis.text.x=ggplot2::element_blank(),
        axis.ticks.x=ggplot2::element_blank(),
        axis.title.y=ggplot2::element_blank(),
        axis.text.y=ggplot2::element_blank(),
        axis.ticks.y=ggplot2::element_blank(),
    strip.text.y.right = ggplot2::element_text(angle = 0)
        )
}
cat('\n\n')
  
```

---
output: html_document
---
  
### Participants

The target population ``r target$name`` is defined as:

```{r results='asis', echo=FALSE}

targetDetails <- OhdsiReportGenerator::getCohortDefinitions(
  connectionHandler = connectionHandler, 
  schema = schema, 
  targetIds = c(modelDesign$developmentTargetId, floor(modelDesign$developmentTargetId/1000))
  )

targetJsonMain <- targetDetails$json[targetDetails$cohortDefinitionId == targetDetails$subsetParent]
targetJsonSubset <- targetDetails$subsetDefinitionJson[targetDetails$cohortDefinitionId == modelDesign$developmentTargetId]


markdown <- tryCatch(
  {CirceR::cohortPrintFriendly(targetJson)}, 
  error = function(e) return('')
)

markdown <- gsub('###', '####',markdown)
cat(markdown)

# TODO add summary text of subset 

```

Additional inclusion criteria:
```{r, echo=FALSE, results='asis'}

if(populationSettings$firstExposureOnly){
  cat('\n - Patients are only in the cohort once.  The earliest index date is used per patient. \n')
}

if(restrictPlpDataSettings$washoutPeriod > populationSettings$washoutPeriod){
  washout <- restrictPlpDataSettings$washoutPeriod
} else{
  washout <- populationSettings$washoutPeriod
}

if(!is.null(target$expression)){
if(washout > ParallelLogger::convertJsonToSettings(targetJsonMain)$PrimaryCriteria$ObservationWindow[1]){
  cat('\n - Patients with less than ',
      washout,' days observation prior to index',
      ' are excluded \n')
}
}

if(populationSettings$removeSubjectsWithPriorOutcome){
  cat('\n - Patients with the outcome within ',
      populationSettings$priorOutcomeLookback,' days prior to index',
      ' are excluded \n')
}

if(populationSettings$requireTimeAtRisk){
  cat('\n - Patients who are lost to follow up within ',
      populationSettings$minTimeAtRisk,' days post index',
      ' are excluded \n')
  if(populationSettings$includeAllOutcomes){
  cat('\n - Except patients with the outcome who are lost to follow up within',
      populationSettings$minTimeAtRisk,' days post index',
  ' (they are still included) \n')
  }
}

if(restrictPlpDataSettings$studyStartDate != ""){
  cat('\n - Restrict to patients with an index date on or after ',restrictPlpDataSettings$studyStartDate,' \n')
}

if(restrictPlpDataSettings$studyEndDate != ""){
  cat('\n - Restrict to patients with an index date on or before ',restrictPlpDataSettings$studyEndDate,' \n')
}

  

```

**Sampling**

```{r, echo=FALSE, results='asis'}

if(!is.null(restrictPlpDataSettings$sampleSize)){
  cat('\n Randomly sample ',restrictPlpDataSettings$sampleSize,' patients from the target population \n')
}

if(!is.null(attr(sampleSettings[[1]],"fun"))){
  if(attr(sampleSettings[[1]],"fun") == 'sameData'){
    cat('\n No over/under-sampling of the patients based on class was done \n')
  }
  
  if(attr(sampleSettings[[1]],"fun") != 'sameData'){
    cat('\n An over/under-sampling of the patients based on class was done using ',
        attr(sampleSettings[[1]],"fun"),' with the  settings: \n')

    sampleDat <- data.frame(
      names = names(sampleSettings[[1]]),
      values = unlist(lapply(sampleSettings[[1]], function(x) paste(x, sep='-', collapse = '-')))
    )

    print(knitr::kable(x = sampleDat, caption = paste('Sampling settings')))
  }
}



```

## Cohort Method
  
::: {style='font-size: 50%;'}

```{r}
#| label: cohort method 
#| echo: false
#| results: 'asis'

# for each outcome have a tab

cmSubsetIds <- c()

# panel for outcomes
cat('\n\n::: {.panel-tabset .nav-pills}')
for(outcomeId in outcomes$cohortDefinitionId){
  
  outcomeName <- outcomes$cohortFriendlyName[which(outcomes$cohortDefinitionId == outcomeId)]
  cat(paste0('\n\n### ', outcomeName))
  
  # panel for diag/table/plot
  cat('\n::::: {.panel-tabset .nav-pills}')  
  cat('\n\n#### Study Diagnostics')
  cat("\n\n:::: {.columns}")
  cat('\n\n::: {.column width="85%"} \n\n')
  
  cmTableDiag <- OhdsiReportGenerator::getCmDiagnosticsData(
    connectionHandler = connectionHandler,
    schema = resultsSchema,
    targetIds = targets$cohortDefinitionId,
    outcomeIds = outcomeId
  )
  
  if(nrow(cmTableDiag) > 0){

subsets <- cohorts %>%
filter(cohortDefinitionId %in% cmTableDiag$targetId) %>%
select("subsetDefinitionId")
cmSubsetIds <- unique(c(cmSubsetIds,subsets$subsetDefinitionId))

    
    # add table
    OhdsiReportGenerator::printReactable(
          data = cmTableDiag %>%
            dplyr::select(
              "databaseName",
              "targetName",
              "analysisId",
              "comparatorName",
              "summaryValue"
            )
          ,
          columns = list(
            databaseName = reactable::colDef(
              name = 'Database'
            ),
           targetName = reactable::colDef(
              filterable = TRUE
            ),
            comparatorName = reactable::colDef(
              filterable = TRUE
            ),
            summaryValue = reactable::colDef(
              name = "Overall Diagnostic",
              style = function(value) {
                color <- 'orange'
                if(is.na(value)){
                  color <- 'black'
                }else if(value == 'Pass'){
                  color <- '#AFE1AF'
                }else if(value == 'Fail'){
                  color <- '#E97451'
                }
                list(background = color)
              }
            )
          ), 
          defaultPageSize = defaultPageSize,
          highlight = TRUE, 
          striped = TRUE
        )
  }
  
  cat('\n\n:::')
  cat('\n::: {.column width="15%"}')
  cat('\n\n> Fail: Study failed ASSURE diagnostics')
  cat('\n\n> Pass: Study passed without warnings')
  cat('\n\n> Number: Passed wth Number of warnings')
  # end column
  cat('\n:::')
  # end columns
  cat('\n::::') 
  
  cat('\n\n#### Table Results\n\n')
  
  cmTable <- OhdsiReportGenerator::getCMEstimation(
    connectionHandler = connectionHandler,
    schema = resultsSchema,
    targetIds = targets$cohortDefinitionId,
    outcomeIds = outcomeId
  )
  
  cmMeta <- OhdsiReportGenerator::getCmMetaEstimation(
    connectionHandler = connectionHandler,
    schema = resultsSchema,
    targetIds = targets$cohortDefinitionId,
    outcomeIds = outcomeId
  )
  
  OhdsiReportGenerator::printReactable(
        data = cmTable %>% 
      dplyr::bind_rows(cmMeta) %>% 
          dplyr::select(
            'databaseName',
            'targetName',
            'comparatorName',
            'description',
            'calibratedRr',
            'calibratedCi95Lb',
            'calibratedCi95Ub',
            'calibratedOneSidedP',
            'calibratedP',
            'calibratedLogRr',
            'calibratedSeLogRr'
          ),
        columns = list(
        targetName = reactable::colDef(
            filterable = TRUE
          ),
          comparatorName = reactable::colDef(
            filterable = TRUE
          ),
          calibratedRr = reactable::colDef(
            format = reactable::colFormat(digits = 2)
          ),
          calibratedCi95Lb = reactable::colDef(
            format = reactable::colFormat(digits = 2)
          ),
          calibratedCi95Ub = reactable::colDef(
            format = reactable::colFormat(digits = 2)
          ),
          calibratedOneSidedP = reactable::colDef(
            format = reactable::colFormat(digits = 2)
          ),
          calibratedP = reactable::colDef(
            format = reactable::colFormat(digits = 2)
          ),
          calibratedLogRr = reactable::colDef(
            format = reactable::colFormat(digits = 2)
          ),
          calibratedSeLogRr = reactable::colDef(
            format = reactable::colFormat(digits = 2)
          )
        ),
        defaultPageSize = defaultPageSize,
        highlight = TRUE, 
        striped = TRUE
      )
  
  if(nrow(cmTable)>0){
    cat('\n\n#### Plot Results')
    # add Justin's nice code for cmTable
    # loop over tar
    analyses <- unique(cmTable$description)
    analysisIds <- unique(cmTable$analysisId)
    cmRes <- unique(cmTable[,c('description','analysisId', 'comparatorId')])
    
    # panel for analysis tab
    cat('\n\n::: {.panel-tabset .nav-pills}\n')
    
    for (i in seq_along(analyses)) {
      analysis <- analyses[[i]]
      analysisId <- analysisIds[[i]]
      
      cat(paste0('\n\n##### ', analysis ,'\n\n'))
      
      comps <- cmRes$comparatorId[cmRes$analysisId == analysisId]

      for(comparatorId in comps){
        cmdata <- cmTable %>% 
                dplyr::filter(
                  .data$comparatorId == !!comparatorId
                )

for(cmTargetId in unique(cmdata$targetId)){
  if(nrow(cmdata) > 0){
     cmdata <- cmdata %>% 
                dplyr::filter(
                  .data$targetId == !!cmTargetId
                )
  }
  
# adding this to handle no meta data for T and C
cmMetaT <- cmMeta %>% dplyr::filter(
                  .data$comparatorId == !!comparatorId &
                  .data$targetId == !!cmTargetId)
if(nrow(cmMetaT) == 0){
cmMetaT <- NULL
}

        
        if(nrow(cmdata) > 0 ){
          p <- tryCatch({
            OhdsiReportGenerator::plotCmEstimates(
              cmData = cmdata,
              cmMeta = cmMetaT,
              targetName = targets$cohortFriendlyName[targets$cohortDefinitionId == params$targetId],
              comparatorName = comparators$cohortFriendlyName[comparators$cohortDefinitionId == cohorts$subsetParent[cohorts$cohortDefinitionId == comparatorId]], # loop this
              selectedAnalysisId = analysisId
            )},
            error = function(e){return(as.character(e))}
          )
          
          if(!is.null(p)){
cat(paste0('\n\nFull Target Name: ', cohorts$cohortFriendlyName[cohorts$cohortDefinitionId == cmTargetId]), '\n')
cat(paste0('\nFull Comparator Name: ', cohorts$cohortFriendlyName[cohorts$cohortDefinitionId == comparatorId]), '\n\n')
            print(p)
          }
        } 
} # target
      }
    }
    # end panel for analysis tab
    cat('\n\n:::')
  }
  
  # end panel for diag/table/plot
  cat('\n\n::::')
  
} # end outcome 
# end panel for outcomes
cat('\n\n:::::')

```

:::

## Database

::: {style="font-size: 50%;"}
```{r}
#| label: database_sub
#| echo: false
#| results: 'asis'

# add table with the dbs
databases <- connectionHandler$queryDb("SELECT * FROM @schema.database_meta_data;", schema = resultsSchema)

reactable::reactable( 
  data = databases %>% 
    select("cdmSourceAbbreviation", "sourceDescription", "sourceReleaseDate",
         "sourceDocumentationReference",
         "cdmEtlReference", "maxObsPeriodEndDate"),  
  columns = list(
cdmSourceAbbreviation = reactable::colDef(
name = 'Database'
  ),
sourceDescription = reactable::colDef(
name = 'Description',
minWidth = 400,
  )),
  searchable = TRUE, 
  filterable = TRUE, 
  highlight = TRUE, 
  striped = TRUE
)

```
:::

## Signal Evaluation Overview

```{r}
#| label: overview_sub
#| echo: false
#| results: 'asis'

cat('\n\n:::: {.columns}')
cat('\n\n::: {.column width="50%"}')
cat(paste0('\n\n> Target: ', targetName,' @sec-target'))

if(!is.null(comparators)){
cat(paste0('\n\n> Comparators: ',paste0(comparators$cohortFriendlyName, ' @sec-',OhdsiReportGenerator::removeSpaces(comparators$cohortFriendlyName), collapse =', ')))
}

if(!is.null(indications)){
cat(paste0('\n\n> Indication/s: ', paste0(indications$cohortFriendlyName,' @sec-',OhdsiReportGenerator::removeSpaces(indications$cohortFriendlyName), collapse =', ')))
}

cat(paste0('\n\n> Outcome/s: ', paste0(outcomeNames, ' @sec-',OhdsiReportGenerator::removeSpaces(outcomeNames), collapse =', ')))
cat('\n\n:::')

cat('\n\n::: {.column width="10%"}')
cat('\n\n<!-- empty column to create gap -->')
cat('\n\n:::')

cat('\n\n::: {.column width="40%"}')
cat('\n\n::: {.border}')
cat('\n\n::: {style="font-size: 50%;"}')

cat('\n\n| Study Period: ')
cat(paste0('\n| ', studyPeriod))

cat('\n\n| Restrictions:') 
cat(paste0('\n| ', restrictions))

cat('\n\n| Analytic Methods:') 
if(params$includeCharacterization | params$includeCI){
cat('\n|   * Characterization')
}
if(params$includeCM){
cat('\n|   * Propensity-matched comparative cohort on treatment')
}
if(params$includeSCCS){
cat('\n|   * Self-controlled case series (SCCS)')
}
if(params$includePLP){
cat('\n|   * Patient-level prediction')
}

# TODO - update this based on databases in results
cat('\n\n| Databases: ')
cat(paste0('\n|   *  ', ifelse(is.null(dbs[['us claims']]),0,dbs[['us claims']]), ' US Claims'))
cat(paste0('\n|   *  ', ifelse(is.null(dbs[['us ehr']]),0,dbs[['us ehr']]), '  US EHR'))
cat(paste0('\n|   *  ', ifelse(is.null(dbs[['non-us claims']]),0,dbs[['non-us claims']]), '  Non-US Claims'))

cat('\n\n:::')
cat('\n\n:::')
cat('\n\n:::')

cat('\n\n::::')

```

# Characterization {.break-slides}

## Cohort Demographics

::: {style='font-size: 50%;'}

```{r}
#| label: cohort demographics
#| echo: false
#| results: 'asis'
  # add text for outcome tabs


# figure out which targets are in char
targetCharDf <- OhdsiReportGenerator::getCharacterizationTargets(
  connectionHandler = connectionHandler, 
  schema = resultsSchema, 
  cTablePrefix = 'c_', 
  useRf = TRUE, 
  useTte = FALSE, 
  useDcrc = FALSE
  )

charTargetIds <- unique(targetCharDf$cohortDefinitionId)

targetsC <- targets %>% filter(
.data$cohortDefinitionId %in% !!charTargetIds
)

# add targets loop
#cat('\n::: {.panel-tabset .nav-pills}')
for(targetInd in 1:length(targetsC$cohortDefinitionId)){
cat(paste0('\n\n### ',targetsC$cohortFriendlyName[targetInd], ' \n\n'))

  cat('\n\n:::: {.panel-tabset .nav-pills}')
  for(outcomeInd in 1:length(outcomes$cohortDefinitionId)){
    outcomeId <- outcomes$cohortDefinitionId[outcomeInd]
    outcomeName <- outcomes$cohortFriendlyName[outcomeInd]
    
    ageData <- OhdsiReportGenerator::getCharacterizationDemographics(
      connectionHandler = connectionHandler,
      schema = resultsSchema,
      targetId = targetsC$cohortDefinitionId[targetInd],
      outcomeId = outcomeId,
      type = 'age'
    )
   sexData <- OhdsiReportGenerator::getCharacterizationDemographics(
      connectionHandler = connectionHandler,
      schema = resultsSchema,
      targetId = targetsC$cohortDefinitionId[targetInd],
      outcomeId = outcomeId,
      type = 'sex'
    )
    
    if(nrow(sexData) >0 | nrow(ageData) > 0){
      # add outcome tab
      cat(paste0('\n\n### ',outcomeName, ' \n\n'))
      
      # loop over tar settings as tabs
      tars <- OhdsiReportGenerator:::getTars(
        data = sexData,  
        tarColumnNames = c(
          'startAnchor', 
          'riskWindowStart',
          'endAnchor',
          'riskWindowEnd'
        )
      )
      
      cat('\n::: {.panel-tabset .nav-pills}')
      for(tar in tars){
        cat(paste0('\n#### TAR: ',paste0('(',tar$startAnchor, " + ", tar$riskWindowStart, ') - (', tar$endAnchor, " + ", tar$riskWindowEnd, ')'),' \n'))
        
sexPlot <- NULL
        if(nrow(sexData)>0){
          sexData$caseAverage <- as.double(sexData$caseAverage)
          sexData$nonCaseAverage <- as.double(sexData$nonCaseAverage)

  
        sexPlot <- OhdsiReportGenerator::plotSexDistributions(
          sexData = sexData,
          riskWindowStart = tar$riskWindowStart,
          riskWindowEnd = tar$riskWindowEnd, 
          startAnchor = tar$startAnchor, 
          endAnchor = tar$endAnchor
        )
        }
agePlot <- NULL
if(nrow(ageData)>0){
  
  ageData$caseAverage <- as.double(ageData$caseAverage)
  ageData$nonCaseAverage <- as.double(ageData$nonCaseAverage)
  
  agePlot <- OhdsiReportGenerator::plotAgeDistributions(
          ageData = ageData,
          riskWindowStart = tar$riskWindowStart,
          riskWindowEnd = tar$riskWindowEnd, 
          startAnchor = tar$startAnchor, 
          endAnchor = tar$endAnchor
        )
}
        
        print(
          ggpubr::annotate_figure(
            p = ggpubr::ggarrange(
              sexPlot, 
              agePlot, 
              heights = c(1, 2),
              ncol = 1, 
              nrow = 2,
              labels = c("A", "B") 
            ), 
            fig.lab = "Demographics for patients with outcome during time-at-risk vs patients without outcome during time-at-risk when first exposed.", 
            fig.lab.pos = "bottom.left"
          )
        )
      }
      cat('\n\n:::') # tars
    }
  }
  cat('\n\n::::\n') # outcomes
}
#cat('\n\n:::') # targets

```

:::
  
## Cohort Characterization
  
::: {style='font-size: 50%;'}

```{r}
#| label: cohort characterization
#| echo: false
#| results: 'asis'

for(targetInd in 1:length(targetsC$cohortDefinitionId)){
cat(paste0('\n\n### ',targetsC$cohortFriendlyName[targetInd], ' \n\n'))

  # add text for outcome tabs
  cat('\n:::: {.panel-tabset .nav-pills}')
  for(outcomeInd in 1:length(outcomes$cohortDefinitionId)){
    outcomeId <- outcomes$cohortDefinitionId[outcomeInd]
    outcomeName <- outcomes$cohortFriendlyName[outcomeInd]
    
    chars <- OhdsiReportGenerator::getBinaryRiskFactors(
      connectionHandler = connectionHandler,
      schema = resultsSchema,
      targetId = targetsC$cohortDefinitionId[targetInd],
      outcomeId = outcomeId,
      analysisIds = c(201,401,210,410)
    )
    
    if(nrow(chars) > 0 ){
      
      # add outcome tab
      cat(paste0('\n\n### ',outcomeName, ' \n\n'))
      
      tars <- OhdsiReportGenerator:::getTars(
        data = chars,  
        tarColumnNames = c(
          'startAnchor', 
          'riskWindowStart',
          'endAnchor',
          'riskWindowEnd'
        )
      )
      
      cat('\n::: {.panel-tabset .nav-pills}')
      for(tar in tars){
        
        charsData <- chars %>% 
         dplyr::filter(
          .data$startAnchor == !!tar$startAnchor &
          .data$endAnchor == !!tar$endAnchor &
          .data$riskWindowStart == !!tar$riskWindowStart &
          .data$riskWindowEnd == !!tar$riskWindowEnd
         )
        
        
        if(nrow(charsData) > 0 ){
          # only include tab if there are results
          cat(paste0('\n\n#### TAR: ',paste0('(',tar$startAnchor, " + ", tar$riskWindowStart, ') - (', tar$endAnchor, " + ", tar$riskWindowEnd, ')'),' \n\n')) 
          
          #print(
            #htmltools::tagList(
              #reactable::reactable(
          OhdsiReportGenerator::printReactable(
                data = charsData, 
                columns = list(
                  covariateName = reactable::colDef(
                    filterable = TRUE, 
                    minWidth = 300 
                  ),
                 caseAverage = reactable::colDef(
                    aggregate = "mean",
                    name = 'With Outcome',
                    format = reactable::colFormat(digits = 2)
                  ),
                  nonCaseAverage = reactable::colDef(
                    aggregate = "mean",
                    name = 'No Outcome',
                    format = reactable::colFormat(digits = 2)
                  ),
                  smd = reactable::colDef(
                    aggregate = "mean",
                    name = 'SMD',
                    format = reactable::colFormat(digits = 2)
                  ),
                  absSmd = reactable::colDef(
                    aggregate = "mean",
                    name = 'SMD',
                    format = reactable::colFormat(digits = 2)
                  )
                ),
                groupBy = c('covariateName'), 
                defaultPageSize = defaultPageSize,
                highlight = TRUE, 
                striped = TRUE
              )
            #)
          #)
        }
        
      }
      cat('\n:::')
    }
  }
  cat('\n::::')

} # end target loop

```

:::

---
  output: html_document
---
  
```{r, echo=FALSE, results='asis'}

getInternalVal <- function(
  connectionHandler,
  resultSchema,
  myTableAppend,
  modelDesignId,
  databaseTableAppend,
  devDatabase,
  tempEmulationSchema = NULL
){
  
    sql <- "select per.performance_id, dm.cdm_source_abbreviation as validation_database
      from 
      (select performance_id, validation_database_id from 
      @result_schema.@my_table_appendperformances 
      where model_design_id = @model_design_id 
      and development_database_id = validation_database_id) per
      inner join @result_schema.@my_table_appenddatabase_details d
      on d.database_id = per.validation_database_id
      inner join @result_schema.@database_table_appenddatabase_meta_data dm
      on d.database_meta_data_id = dm.database_id
      where d.database_id = @dev_database
      ;
    "
  res <- connectionHandler$queryDb(
        sql = sql, 
    result_schema = resultSchema,
    my_table_append = myTableAppend,
    database_table_append = databaseTableAppend,
    model_design_id = modelDesignId,
    dev_database = devDatabase
  )
  
  if(is.null(res$performanceId)){
    return(NULL)
  }
  
  # get the performances for the performanceId
  result <- list()
  tableNames <- c(
    'attrition', 
    'prediction_distribution',
    'covariate_summary',
    'threshold_summary',
    'calibration_summary',
    'evaluation_statistics',
    'demographic_summary'
    )
  length(result) <- length(tableNames)+1
  names(result) <- c('database',tableNames)
  result[['database']] <- res$validationDatabase
  
  for(tabName in tableNames){
  
  sql <- "select * from 
    @result_schema.@my_table_append@table_name 
    where performance_id = @performance_id 
    "
  
  result[[tabName]] <- connectionHandler$queryDb(
        sql = sql, 
    result_schema = resultSchema,
    table_name = tabName,
    my_table_append = myTableAppend,
    performance_id = res$performanceId
  )
  }
  
  return(result)
}

result <- getInternalVal(
  connectionHandler = params$connectionHandler,
  resultSchema = params$resultSchema,
  myTableAppend = params$myTableAppend,
  databaseTableAppend = params$databaseTableAppend,
  modelDesignId = params$modelDesignIds[i],
  devDatabase = devDatabase
  )
```

### Internal Validation

The internal validation is the performance of the models when validated using held-out data from the database used to develop the model.

```{r, child = if(!is.null(result)) "plp-results-template.Rmd"}
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Indexes.R
\name{createCharacterizationIndexes}
\alias{createCharacterizationIndexes}
\title{A function to add indexes to the Characterization results}
\usage{
createCharacterizationIndexes(connectionHandler, schema, cTablePrefix = "c_")
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{cTablePrefix}{The prefix used for the characterization results tables}
}
\description{
A function to add indexes to the Characterization results
}
\details{
Specify the connectionHandler, the schema and the prefixes
}
\seealso{
Other Indexes: 
\code{\link{createCohortIndexes}()},
\code{\link{createIncidenceIndexes}()},
\code{\link{createSccsIndexes}()}
}
\concept{Indexes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CharacterzationQueries.R
\name{getCharacterizationTargets}
\alias{getCharacterizationTargets}
\title{A function to extarct the targets found in characterization}
\usage{
getCharacterizationTargets(
  connectionHandler,
  schema,
  cTablePrefix = "c_",
  cgTablePrefix = "cg_",
  printTimes = FALSE,
  useTte = TRUE,
  useDcrc = TRUE,
  useRf = TRUE,
  useTb = TRUE,
  useCs = TRUE
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{cTablePrefix}{The prefix used for the characterization results tables}

\item{cgTablePrefix}{The prefix used for the cohort generator results tables}

\item{printTimes}{Print the time it takes to run each query}

\item{useTte}{whether to determine what cohorts are used in time to event}

\item{useDcrc}{whether to determine what cohorts are used in dechal-rechal}

\item{useRf}{whether to determine what cohorts are used in risk factor}

\item{useTb}{whether to determine what cohorts are used in target baseline}

\item{useCs}{whether to determine what cohorts are used in case-series}
}
\value{
A data.frame with the characterization target cohort ids, names and which characterization analyses the cohorts are used in.
}
\description{
A function to extarct the targets found in characterization
}
\details{
Specify the connectionHandler, the schema and the prefixes
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

cohorts <- getCharacterizationTargets(
  connectionHandler = connectionHandler, 
  schema = 'main'
)

}
\seealso{
Other Characterization: 
\code{\link{getBinaryCaseSeries}()},
\code{\link{getBinaryRiskFactors}()},
\code{\link{getBinaryTargetBaseline}()},
\code{\link{getCaseCounts}()},
\code{\link{getCaseTargetCounts}()},
\code{\link{getCharacterizationCohortBinary}()},
\code{\link{getCharacterizationCohortContinuous}()},
\code{\link{getCharacterizationDemographics}()},
\code{\link{getCharacterizationOutcomes}()},
\code{\link{getContinuousCaseSeries}()},
\code{\link{getContinuousRiskFactors}()},
\code{\link{getDechallengeRechallenge}()},
\code{\link{getDechallengeRechallengeFails}()},
\code{\link{getIncidenceOutcomes}()},
\code{\link{getIncidenceRates}()},
\code{\link{getIncidenceTargets}()},
\code{\link{getTargetBinaryFeatures}()},
\code{\link{getTargetContinuousFeatures}()},
\code{\link{getTimeToEvent}()},
\code{\link{plotAgeDistributions}()},
\code{\link{plotSexDistributions}()},
\code{\link{viewIncidenceRate}()}
}
\concept{Characterization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SummaryReports.R
\name{generateSummaryPredictionReport}
\alias{generateSummaryPredictionReport}
\title{generateSummaryPredictionReport}
\usage{
generateSummaryPredictionReport(
  connectionHandler,
  schema,
  targetIds = NULL,
  outcomeIds = NULL,
  plpTablePrefix = "plp_",
  databaseTablePrefix = "",
  cgTablePrefix = "cg_",
  outputFolder,
  outputFileName = "plp-summary.html",
  intermediatesDir = file.path(tempdir(), "plp-prot"),
  overwrite = FALSE
)
}
\arguments{
\item{connectionHandler}{The connection handler to the results database}

\item{schema}{The result database schema}

\item{targetIds}{The target cohort IDs of interest}

\item{outcomeIds}{The outcome cohort IDs of interest}

\item{plpTablePrefix}{The prediction table prefix}

\item{databaseTablePrefix}{The database table name e.g., database_meta_data}

\item{cgTablePrefix}{The cohort generator table prefix}

\item{outputFolder}{The folder name where file will be save to}

\item{outputFileName}{The file name of the saved report}

\item{intermediatesDir}{The work directory for rmarkdown}

\item{overwrite}{whether to overwrite any existing file at the outputFolder/outputFileName}
}
\value{
A html file is created with the summary report
}
\description{
Generates a summary report for a given targets and outcomes
}
\details{
Specify the connection handler to the result database, the schema name
and the cohortId of interest to generate a html report summarizing the
performance of prediction models in the database.
}
\seealso{
Other Reporting: 
\code{\link{createPredictionReport}()},
\code{\link{generateFullReport}()}
}
\concept{Reporting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CharacterzationQueries.R
\name{getContinuousCaseSeries}
\alias{getContinuousCaseSeries}
\title{A function to extract case series continuous feature characterization results}
\usage{
getContinuousCaseSeries(
  connectionHandler,
  schema,
  cTablePrefix = "c_",
  cgTablePrefix = "cg_",
  databaseTable = "database_meta_data",
  targetId = NULL,
  outcomeId = NULL,
  databaseIds = NULL,
  riskWindowStart = NULL,
  riskWindowEnd = NULL,
  startAnchor = NULL,
  endAnchor = NULL
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{cTablePrefix}{The prefix used for the characterization results tables}

\item{cgTablePrefix}{The prefix used for the cohort generator results tables}

\item{databaseTable}{The name of the table with the database details (default 'database_meta_data')}

\item{targetId}{An integer corresponding to the target cohort ID}

\item{outcomeId}{Am integer corresponding to the outcome cohort ID}

\item{databaseIds}{(optional) One or more unique identifiers for the databases}

\item{riskWindowStart}{(optional) A riskWindowStart to restrict to}

\item{riskWindowEnd}{(optional) A riskWindowEnd to restrict to}

\item{startAnchor}{(optional) A startAnchor to restrict to}

\item{endAnchor}{(optional) An endAnchor to restrict to}
}
\value{
A data.frame with the characterization case series results
}
\description{
A function to extract case series continuous feature characterization results
}
\details{
Specify the connectionHandler, the schema and the target/outcome cohort IDs
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

cs <- getContinuousCaseSeries(
  connectionHandler = connectionHandler, 
  schema = 'main',
  targetId = 1, 
  outcomeId = 3
)

}
\seealso{
Other Characterization: 
\code{\link{getBinaryCaseSeries}()},
\code{\link{getBinaryRiskFactors}()},
\code{\link{getBinaryTargetBaseline}()},
\code{\link{getCaseCounts}()},
\code{\link{getCaseTargetCounts}()},
\code{\link{getCharacterizationCohortBinary}()},
\code{\link{getCharacterizationCohortContinuous}()},
\code{\link{getCharacterizationDemographics}()},
\code{\link{getCharacterizationOutcomes}()},
\code{\link{getCharacterizationTargets}()},
\code{\link{getContinuousRiskFactors}()},
\code{\link{getDechallengeRechallenge}()},
\code{\link{getDechallengeRechallengeFails}()},
\code{\link{getIncidenceOutcomes}()},
\code{\link{getIncidenceRates}()},
\code{\link{getIncidenceTargets}()},
\code{\link{getTargetBinaryFeatures}()},
\code{\link{getTargetContinuousFeatures}()},
\code{\link{getTimeToEvent}()},
\code{\link{plotAgeDistributions}()},
\code{\link{plotSexDistributions}()},
\code{\link{viewIncidenceRate}()}
}
\concept{Characterization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Helpers.R
\name{addTarColumn}
\alias{addTarColumn}
\title{addTarColumn}
\usage{
addTarColumn(data)
}
\arguments{
\item{data}{The data.frame with the individual TAR columns that you want to 
combine into one column}
}
\value{
The data data.frame object with the tar column added if seperate TAR columns are found
}
\description{
Finds the four TAR columns and creates a new column called tar that pastes the columns
into a nice string
}
\details{
Create a friendly single tar column
}
\examples{
addTarColumn(data.frame(
tarStartWith = 'cohort start',
tarStartOffset = 1,
tarEndWith = 'cohort start',
tarEndOffset = 0
))

}
\seealso{
Other helper: 
\code{\link{formatBinaryCovariateName}()},
\code{\link{getExampleConnectionDetails}()},
\code{\link{getOutcomeTable}()},
\code{\link{getTargetTable}()},
\code{\link{kableDark}()},
\code{\link{removeSpaces}()}
}
\concept{helper}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Helpers.R
\name{kableDark}
\alias{kableDark}
\title{output a nicely formatted html table}
\usage{
kableDark(data, caption = NULL, position = NULL)
}
\arguments{
\item{data}{A data.frame containing data of interest to show via a table}

\item{caption}{A caption for the table}

\item{position}{The position for the table if used within a quarto document.
This is the "real" or say floating position for the latex table environment. 
The kable only puts tables in a table environment when a caption is provided. 
That is also the reason why your tables will be floating around if you specify 
captions for your table. Possible choices are h (here), t (top, default), 
b (bottom) and p (on a dedicated page).}
}
\value{
An object of class `knitr_kable` that will show the data via a nice html table
}
\description{
This returns a html table with the input data
}
\details{
Input the data that you want to be shown via a dark html table
}
\examples{
kableDark(
data = data.frame(a=1,b=4), 
caption = 'A made up table to demonstrate this function',
position = 'h'
)

}
\seealso{
Other helper: 
\code{\link{addTarColumn}()},
\code{\link{formatBinaryCovariateName}()},
\code{\link{getExampleConnectionDetails}()},
\code{\link{getOutcomeTable}()},
\code{\link{getTargetTable}()},
\code{\link{removeSpaces}()}
}
\concept{helper}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Generate.R
\name{generateFullReport}
\alias{generateFullReport}
\title{generateFullReport}
\usage{
generateFullReport(
  server,
  username,
  password,
  dbms,
  resultsSchema = NULL,
  targetId = 1,
  outcomeIds = 3,
  comparatorIds = 2,
  indicationIds = NULL,
  restrictTargetToIndications = FALSE,
  cohortNames = c("target name", "outcome name", "comp name"),
  cohortIds = c(1, 3, 2),
  includeCI = TRUE,
  includeCharacterization = TRUE,
  includeCohortMethod = TRUE,
  includeSccs = TRUE,
  includePrediction = TRUE,
  webAPI = NULL,
  authMethod = NULL,
  webApiUsername = NULL,
  webApiPassword = NULL,
  outputLocation,
  outputName = paste0("full_report_", gsub(":", "_", gsub(" ", "_",
    as.character(date()))), ".html"),
  intermediateDir = tempdir(),
  pathToDriver = Sys.getenv("DATABASECONNECTOR_JAR_FOLDER")
)
}
\arguments{
\item{server}{The server containing the result database}

\item{username}{The username for an account that can access the result database}

\item{password}{The password for an account that can access the result database}

\item{dbms}{The dbms used to access the result database}

\item{resultsSchema}{The result database schema}

\item{targetId}{The cohort definition id for the target cohort}

\item{outcomeIds}{The cohort definition id for the outcome}

\item{comparatorIds}{(optional) The cohort definition id for any comparator cohorts.  If NULL the report will find and include all possible comparators in the results if includeCohortMethod is TRUE.}

\item{indicationIds}{The cohort definition id for any indication cohorts to show in characterization. If no indication use NULL.}

\item{restrictTargetToIndications}{If you only want the results for targets that are nested by the indicationIds set this to TRUE otherwise results for all children of the targetId will be generated.}

\item{cohortNames}{Friendly names for any cohort used in the study}

\item{cohortIds}{The corresponding Ids for the cohortNames}

\item{includeCI}{Whether to include the cohort incidence slides}

\item{includeCharacterization}{Whether to include the characterization slides}

\item{includeCohortMethod}{Whether to include the cohort method slides}

\item{includeSccs}{Whether to include the self controlled case series slides}

\item{includePrediction}{Whether to include the patient level prediction slides}

\item{webAPI}{The ATLAS web API to use for the characterization index breakdown (set to NULL to not include)}

\item{authMethod}{The authorization method for the webAPI}

\item{webApiUsername}{The username for the webAPI authorization}

\item{webApiPassword}{The password for the webAPI authorization}

\item{outputLocation}{The file location and name to save the protocol}

\item{outputName}{The name of the html protocol that is created}

\item{intermediateDir}{The work directory for quarto}

\item{pathToDriver}{Path to a folder containing the JDBC driver JAR files.}
}
\value{
An html document containing the full results for the target, comparators, indications and outcomes specified.
}
\description{
Generates a full report from a Strategus analysis
}
\details{
Specify the connection details to the result database and the schema name
to generate the full report.
}
\seealso{
Other Reporting: 
\code{\link{createPredictionReport}()},
\code{\link{generateSummaryPredictionReport}()}
}
\concept{Reporting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CharacterzationQueries.R
\name{getCaseCounts}
\alias{getCaseCounts}
\title{Extract the outcome cohort counts result}
\usage{
getCaseCounts(
  connectionHandler,
  schema,
  cTablePrefix = "c_",
  cgTablePrefix = "cg_",
  databaseTable = "database_meta_data",
  targetIds = NULL,
  outcomeIds = NULL,
  databaseIds = NULL,
  riskWindowStart = NULL,
  riskWindowEnd = NULL,
  startAnchor = NULL,
  endAnchor = NULL
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{cTablePrefix}{The prefix used for the characterization results tables}

\item{cgTablePrefix}{The prefix used for the cohort generator results tables}

\item{databaseTable}{The name of the table with the database details (default 'database_meta_data')}

\item{targetIds}{A vector of integers corresponding to the target cohort IDs}

\item{outcomeIds}{A vector of integers corresponding to the outcome cohort IDs}

\item{databaseIds}{(optional) A vector of database IDs to restrict to}

\item{riskWindowStart}{(optional) A vector of time-at-risk risk window starts to restrict to}

\item{riskWindowEnd}{(optional) A vector of time-at-risk risk window ends to restrict to}

\item{startAnchor}{(optional) A vector of time-at-risk start anchors to restrict to}

\item{endAnchor}{(optional) A vector of time-at-risk end anchors to restrict to}
}
\value{
Returns a data.frame with the columns:
\itemize{
 \item{databaseName the name of the database}
 \item{databaseId the unique identifier of the database}
 \item{targetName the target cohort name}
 \item{targetId the target cohort unique identifier}
 \item{outcomeName the outcome name}
 \item{outcomeId the outcome unique identifier}
 \item{rowCount the number of entries in the cohort}
 \item{personCount the number of people in the cohort}
 \item{minPriorObservation the minimum required observation days prior to index for an entry}
 \item{outcomeWashoutDays patients with the outcome occurring within this number of days prior to index are excluded (NA means no exclusion)}
\item{riskWindowStart the number of days ofset the start anchor that is the start of the time-at-risk}
\item{startAnchor the start anchor is either the target cohort start or cohort end date}
\item{riskWindowEnd the number of days ofset the end anchor that is the end of the time-at-risk}
\item{endAnchor the end anchor is either the target cohort start or cohort end date}
}
}
\description{
This function extracts outcome cohort counts across databases in the results for specified target and outcome cohorts.
}
\details{
Specify the connectionHandler, the schema and the target/outcome cohort IDs
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

cc <- getCaseCounts(
connectionHandler = connectionHandler, 
schema = 'main'
)

}
\seealso{
Other Characterization: 
\code{\link{getBinaryCaseSeries}()},
\code{\link{getBinaryRiskFactors}()},
\code{\link{getBinaryTargetBaseline}()},
\code{\link{getCaseTargetCounts}()},
\code{\link{getCharacterizationCohortBinary}()},
\code{\link{getCharacterizationCohortContinuous}()},
\code{\link{getCharacterizationDemographics}()},
\code{\link{getCharacterizationOutcomes}()},
\code{\link{getCharacterizationTargets}()},
\code{\link{getContinuousCaseSeries}()},
\code{\link{getContinuousRiskFactors}()},
\code{\link{getDechallengeRechallenge}()},
\code{\link{getDechallengeRechallengeFails}()},
\code{\link{getIncidenceOutcomes}()},
\code{\link{getIncidenceRates}()},
\code{\link{getIncidenceTargets}()},
\code{\link{getTargetBinaryFeatures}()},
\code{\link{getTargetContinuousFeatures}()},
\code{\link{getTimeToEvent}()},
\code{\link{plotAgeDistributions}()},
\code{\link{plotSexDistributions}()},
\code{\link{viewIncidenceRate}()}
}
\concept{Characterization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Helpers.R
\name{getExampleConnectionDetails}
\alias{getExampleConnectionDetails}
\title{create a connection detail for an example OHDSI results database}
\usage{
getExampleConnectionDetails(exdir = tempdir())
}
\arguments{
\item{exdir}{a directory to unzip the example result data into.  Default is tempdir().}
}
\value{
An object of class `ConnectionDetails` with the details to connect to the example OHDSI result database
}
\description{
This returns an object of class `ConnectionDetails` that lets you connect via `DatabaseConnector::connect()` to the example result database.
}
\details{
Finds the location of the example result database in the package and calls `DatabaseConnector::createConnectionDetails` to create a `ConnectionDetails` object for connecting to the database.
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

}
\seealso{
Other helper: 
\code{\link{addTarColumn}()},
\code{\link{formatBinaryCovariateName}()},
\code{\link{getOutcomeTable}()},
\code{\link{getTargetTable}()},
\code{\link{kableDark}()},
\code{\link{removeSpaces}()}
}
\concept{helper}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PredictionQueries.R
\name{getPredictionAggregateTopPredictors}
\alias{getPredictionAggregateTopPredictors}
\title{Extract the top N predictors across a set of models}
\usage{
getPredictionAggregateTopPredictors(
  connectionHandler,
  schema,
  plpTablePrefix = "plp_",
  cgTablePrefix = "cg_",
  databaseTable = "database_meta_data",
  modelDesignIds = NULL
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{plpTablePrefix}{The prefix used for the patient level prediction results tables}

\item{cgTablePrefix}{The prefix used for the cohort generator results tables}

\item{databaseTable}{The name of the table with the database details (default 'database_meta_data')}

\item{modelDesignIds}{One or more model design IDs to restrict to}
}
\value{
Returns a data.frame with the columns: 
\itemize{
 \item{databaseName the name of the database the model was developed on}
 \item{tarStartDay the time-at-risk start day}
 \item{tarStartAnchor whether the time-at-risk start is relative to cohort start or end}
 \item{tarEndDay the time-at-risk end day}
 \item{tarEndAnchor whether the time-at-risk end is relative to cohort start or end}
 \item{covariateId the FeatureExtraction covariate identifier}
 \item{covariateName the name of the covariate}
 \item{conceptId the covariates corresponding concept or 0}
 \item{sumCovariateValue the total absolute feature importance or coefficient value}
 \item{numberOfTimesPredictive number of models that contained the covariate}
}
}
\description{
This function extracts the top N predictors across models by 
finding the sum of the absolute coefficient value across models.
}
\details{
Specify the connectionHandler, the resultDatabaseSettings and (optionally) any modelDesignIds to restrict to
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

topPreds <- getPredictionAggregateTopPredictors(
  connectionHandler = connectionHandler, 
  schema = 'main',
  modelDesignIds = c(1,2,5)
)

}
\seealso{
Other Prediction: 
\code{\link{getFullPredictionPerformances}()},
\code{\link{getPredictionCohorts}()},
\code{\link{getPredictionCovariates}()},
\code{\link{getPredictionDiagnosticTable}()},
\code{\link{getPredictionDiagnostics}()},
\code{\link{getPredictionHyperParamSearch}()},
\code{\link{getPredictionIntercept}()},
\code{\link{getPredictionLift}()},
\code{\link{getPredictionModelDesigns}()},
\code{\link{getPredictionOutcomes}()},
\code{\link{getPredictionPerformanceTable}()},
\code{\link{getPredictionPerformances}()},
\code{\link{getPredictionTargets}()},
\code{\link{getPredictionTopPredictors}()}
}
\concept{Prediction}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CharacterizationPlots.R
\name{viewIncidenceRate}
\alias{viewIncidenceRate}
\title{View the Incidence Rates}
\usage{
viewIncidenceRate(
  incidenceData,
  ageData = NULL,
  genderData = NULL,
  stratification = "overall",
  maxAgeSampleSize = 5000
)
}
\arguments{
\item{incidenceData}{The data extracted using 'getIncidenceRates'}

\item{ageData}{The data extracted using 'getBinaryTargetBaseline' with analysisIds = 3}

\item{genderData}{The data extracted using 'getBinaryTargetBaseline' with analysisIds = 1}

\item{stratification}{Pick either overall/age/sex/year to specify whether to view the overall rates or stratified by age/sex/year}

\item{maxAgeSampleSize}{When creating the age distributions this is the max age vector length to create}
}
\value{
Returns a gt table that displays the incidence rates
}
\description{
Creates a table with the incidence rates and optionally demographics
}
\details{
Input the incidence rate data (and optionally demographic data)
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)
schema <- 'main'

incidenceData <- getIncidenceRates(
  connectionHandler = connectionHandler , 
  schema = schema
  )
  
  # incidence data does not have rate values to imputing them
  incidenceData$incidenceRateP100py <- 1 +
    sample(c(-1,1),replace = TRUE)*runif(nrow(incidenceData))
  incidenceData$incidenceProportionP100p <- 0.5 +
    sample(c(-1,1),replace = TRUE)*runif(nrow(incidenceData))
  
 ageData <- getBinaryTargetBaseline(
  connectionHandler = connectionHandler, 
  schema = schema,  
  analysisIds = 3
 )
 
 genderData <- getBinaryTargetBaseline(
  connectionHandler = connectionHandler, 
  schema = schema,  
  analysisIds = 1
 )

viewIncidenceRate(
  incidenceData = incidenceData,
  ageData = ageData,
  genderData = genderData
  )
  

}
\seealso{
Other Characterization: 
\code{\link{getBinaryCaseSeries}()},
\code{\link{getBinaryRiskFactors}()},
\code{\link{getBinaryTargetBaseline}()},
\code{\link{getCaseCounts}()},
\code{\link{getCaseTargetCounts}()},
\code{\link{getCharacterizationCohortBinary}()},
\code{\link{getCharacterizationCohortContinuous}()},
\code{\link{getCharacterizationDemographics}()},
\code{\link{getCharacterizationOutcomes}()},
\code{\link{getCharacterizationTargets}()},
\code{\link{getContinuousCaseSeries}()},
\code{\link{getContinuousRiskFactors}()},
\code{\link{getDechallengeRechallenge}()},
\code{\link{getDechallengeRechallengeFails}()},
\code{\link{getIncidenceOutcomes}()},
\code{\link{getIncidenceRates}()},
\code{\link{getIncidenceTargets}()},
\code{\link{getTargetBinaryFeatures}()},
\code{\link{getTargetContinuousFeatures}()},
\code{\link{getTimeToEvent}()},
\code{\link{plotAgeDistributions}()},
\code{\link{plotSexDistributions}()}
}
\concept{Characterization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CharacterizationPlots.R
\name{plotAgeDistributions}
\alias{plotAgeDistributions}
\title{Plots the age distributions using the binary age groups}
\usage{
plotAgeDistributions(
  ageData,
  riskWindowStart = "1",
  riskWindowEnd = "365",
  startAnchor = "cohort start",
  endAnchor = "cohort start"
)
}
\arguments{
\item{ageData}{The age data extracted using 'getCharacterizationDemographics(type = 'age')'}

\item{riskWindowStart}{The time at risk window start}

\item{riskWindowEnd}{The time at risk window end}

\item{startAnchor}{The anchor for the time at risk start}

\item{endAnchor}{The anchor for the time at risk end}
}
\value{
Returns a ggplot with the distributions
}
\description{
Creates bar charts for the target and case age groups.
}
\details{
Input the data returned from 'getCharacterizationDemographics(type = 'age')' and the time-at-risk
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

ageData <- getCharacterizationDemographics(
connectionHandler = connectionHandler, 
schema = 'main',
targetId = 1, 
outcomeId = 3, 
type = 'age'
)

plotAgeDistributions(ageData = ageData)

}
\seealso{
Other Characterization: 
\code{\link{getBinaryCaseSeries}()},
\code{\link{getBinaryRiskFactors}()},
\code{\link{getBinaryTargetBaseline}()},
\code{\link{getCaseCounts}()},
\code{\link{getCaseTargetCounts}()},
\code{\link{getCharacterizationCohortBinary}()},
\code{\link{getCharacterizationCohortContinuous}()},
\code{\link{getCharacterizationDemographics}()},
\code{\link{getCharacterizationOutcomes}()},
\code{\link{getCharacterizationTargets}()},
\code{\link{getContinuousCaseSeries}()},
\code{\link{getContinuousRiskFactors}()},
\code{\link{getDechallengeRechallenge}()},
\code{\link{getDechallengeRechallengeFails}()},
\code{\link{getIncidenceOutcomes}()},
\code{\link{getIncidenceRates}()},
\code{\link{getIncidenceTargets}()},
\code{\link{getTargetBinaryFeatures}()},
\code{\link{getTargetContinuousFeatures}()},
\code{\link{getTimeToEvent}()},
\code{\link{plotSexDistributions}()},
\code{\link{viewIncidenceRate}()}
}
\concept{Characterization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EstimationPlots.R
\name{plotSccsEstimates}
\alias{plotSccsEstimates}
\title{Plots the self controlled case series results for one analysis}
\usage{
plotSccsEstimates(
  sccsData,
  sccsDiagnostics = NULL,
  sccsMeta = NULL,
  includeCounts = TRUE,
  selectedAnalysisId
)
}
\arguments{
\item{sccsData}{The self controlled case series data}

\item{sccsDiagnostics}{(optional) The self controlled case series diagnostic data}

\item{sccsMeta}{(optional) The self controlled case series evidence synthesis data}

\item{includeCounts}{Whether to include count on the plot}

\item{selectedAnalysisId}{The analysis ID of interest to plot}
}
\value{
Returns a ggplot with the estimates
}
\description{
Creates nice self controlled case series plots
}
\details{
Input the self controlled case series data
}
\examples{

conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

sccsEst <- getSccsEstimation(
  connectionHandler = connectionHandler, 
  schema = 'main',
  targetIds = 1,
  outcomeIds = 3
)
plotSccsEstimates(
  sccsData = sccsEst, 
  sccsMeta = NULL, 
  selectedAnalysisId = 1
)

}
\seealso{
Other Estimation: 
\code{\link{.getCmVersion}()},
\code{\link{.getSccsVersion}()},
\code{\link{getCMEstimation}()},
\code{\link{getCmDiagnosticsData}()},
\code{\link{getCmMetaEstimation}()},
\code{\link{getCmNegativeControlEstimates}()},
\code{\link{getCmOutcomes}()},
\code{\link{getCmPropensityModel}()},
\code{\link{getCmTable}()},
\code{\link{getCmTargets}()},
\code{\link{getSccsDiagnosticsData}()},
\code{\link{getSccsEstimation}()},
\code{\link{getSccsMetaEstimation}()},
\code{\link{getSccsModel}()},
\code{\link{getSccsNegativeControlEstimates}()},
\code{\link{getSccsOutcomes}()},
\code{\link{getSccsTable}()},
\code{\link{getSccsTargets}()},
\code{\link{getSccsTimeToEvent}()},
\code{\link{plotCmEstimates}()}
}
\concept{Estimation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EstimationQueries.R
\name{getCmPropensityModel}
\alias{getCmPropensityModel}
\title{Extract the cohort method model}
\usage{
getCmPropensityModel(
  connectionHandler,
  schema,
  cmTablePrefix = "cm_",
  targetId = NULL,
  comparatorId = NULL,
  indicationId = NULL,
  analysisId = NULL,
  databaseId = NULL
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{cmTablePrefix}{The prefix used for the cohort method results tables}

\item{targetId}{An integer corresponding to the target cohort ID}

\item{comparatorId}{the comparator ID of interest}

\item{indicationId}{The indications that the target & comparator was nested to}

\item{analysisId}{the analysis ID to restrict to}

\item{databaseId}{the database ID to restrict to}
}
\value{
Returns a data.frame with the cohort method model
}
\description{
This function extracts the cohort method model.
}
\details{
Specify the connectionHandler, the schema and optionally the target/comparator/analysis/database IDs
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

cmModel <- getCmPropensityModel(
  connectionHandler = connectionHandler, 
  schema = 'main'
)

}
\seealso{
Other Estimation: 
\code{\link{.getCmVersion}()},
\code{\link{.getSccsVersion}()},
\code{\link{getCMEstimation}()},
\code{\link{getCmDiagnosticsData}()},
\code{\link{getCmMetaEstimation}()},
\code{\link{getCmNegativeControlEstimates}()},
\code{\link{getCmOutcomes}()},
\code{\link{getCmTable}()},
\code{\link{getCmTargets}()},
\code{\link{getSccsDiagnosticsData}()},
\code{\link{getSccsEstimation}()},
\code{\link{getSccsMetaEstimation}()},
\code{\link{getSccsModel}()},
\code{\link{getSccsNegativeControlEstimates}()},
\code{\link{getSccsOutcomes}()},
\code{\link{getSccsTable}()},
\code{\link{getSccsTargets}()},
\code{\link{getSccsTimeToEvent}()},
\code{\link{plotCmEstimates}()},
\code{\link{plotSccsEstimates}()}
}
\concept{Estimation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CharacterzationQueries.R
\name{getTargetBinaryFeatures}
\alias{getTargetBinaryFeatures}
\title{Extract aggregate statistics of binary feature analysis IDs of interest for targets (ignoring excluding people with prior outcome)}
\usage{
getTargetBinaryFeatures(
  connectionHandler,
  schema,
  cTablePrefix = "c_",
  cgTablePrefix = "cg_",
  databaseTable = "database_meta_data",
  targetId = NULL,
  databaseIds = NULL,
  analysisIds = NULL,
  conceptIds = NULL
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{cTablePrefix}{The prefix used for the characterization results tables}

\item{cgTablePrefix}{The prefix used for the cohort generator results tables}

\item{databaseTable}{The name of the table with the database details (default 'database_meta_data')}

\item{targetId}{An integer corresponding to the target cohort ID}

\item{databaseIds}{(optional) A vector of database ids to restrict to}

\item{analysisIds}{(optional) The feature extraction analysis ID of interest (e.g., 201 is condition)}

\item{conceptIds}{(optional) The feature extraction concept ID of interest to restrict to}
}
\value{
Returns a data.frame with the columns:
\itemize{
 \item{databaseName the name of the database}
 \item{databaseId the unique identifier of the database}
 \item{targetName the target cohort name}
 \item{targetId the target cohort unique identifier}
 \item{minPriorObservation the minimum required observation days prior to index for an entry}
 \item{covariateId the id of the feature}
 \item{covariateName the name of the feature}
 \item{sumValue the number of target patients who have the feature value of 1 (target patients are restricted to first occurrence and require min prior obervation days)}
 \item{averageAvalue the fraction of target patients who have the feature value of 1 (target patients are restricted to first occurrence and require min prior obervation days)}
}
}
\description{
This function extracts the feature extraction results for targets corresponding to specified target
 but does not exclude any patients with the outcome during the outcome washout (so it agnostic to the outcome of interest).
}
\details{
Specify the connectionHandler, the schema and the target cohort IDs
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

tbf <- getTargetBinaryFeatures (
connectionHandler = connectionHandler, 
schema = 'main',
targetId = 1
)

}
\seealso{
Other Characterization: 
\code{\link{getBinaryCaseSeries}()},
\code{\link{getBinaryRiskFactors}()},
\code{\link{getBinaryTargetBaseline}()},
\code{\link{getCaseCounts}()},
\code{\link{getCaseTargetCounts}()},
\code{\link{getCharacterizationCohortBinary}()},
\code{\link{getCharacterizationCohortContinuous}()},
\code{\link{getCharacterizationDemographics}()},
\code{\link{getCharacterizationOutcomes}()},
\code{\link{getCharacterizationTargets}()},
\code{\link{getContinuousCaseSeries}()},
\code{\link{getContinuousRiskFactors}()},
\code{\link{getDechallengeRechallenge}()},
\code{\link{getDechallengeRechallengeFails}()},
\code{\link{getIncidenceOutcomes}()},
\code{\link{getIncidenceRates}()},
\code{\link{getIncidenceTargets}()},
\code{\link{getTargetContinuousFeatures}()},
\code{\link{getTimeToEvent}()},
\code{\link{plotAgeDistributions}()},
\code{\link{plotSexDistributions}()},
\code{\link{viewIncidenceRate}()}
}
\concept{Characterization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PredictionQueries.R
\name{getPredictionDiagnostics}
\alias{getPredictionDiagnostics}
\title{Extract the model design diagnostics for a specific development database}
\usage{
getPredictionDiagnostics(
  connectionHandler,
  schema,
  plpTablePrefix = "plp_",
  cgTablePrefix = "cg_",
  databaseTable = "database_meta_data",
  modelDesignIds = NULL,
  threshold1_2 = 0.9
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{plpTablePrefix}{The prefix used for the patient level prediction results tables}

\item{cgTablePrefix}{The prefix used for the cohort generator results tables}

\item{databaseTable}{The name of the table with the database details (default 'database_meta_data')}

\item{modelDesignIds}{The identifier for a model design  to restrict results to}

\item{threshold1_2}{A threshold for probast 1.2}
}
\value{
Returns a data.frame with the columns:
\itemize{   
 \item{modelDesignId the unique identifier for the model design}
 \item{diagnosticId the unique identifier for diagnostic result}
 \item{developmentDatabaseName the name for the database used to develop the model}
 \item{developmentTargetName the name for the development target population}
 \item{developmentOutcomeName the name for the development outcome}
 \item{probast1_1 Were appropriate data sources used, e.g., cohort, RCT, or nested case-control study data?}
 \item{probast1_2 Were all inclusions and exclusions of paticipants appropriate?}
 \item{probast2_1 Were predictors defined and assessed in a similar way for all participants?}
 \item{probast2_2 Were predictors assessments made without knowledge of outcome data?}
 \item{probast2_3 All all predictors available at the time the model is intended to be used?}
 \item{probast3_4 Was the outcome defined and determined in a similar way for all participants?}
 \item{probast3_6 Was the time interval between predictor assessment and outcome determination appropriate?}
 \item{probast4_1 Were there a reasonable number of participants with the outcome?}
 }
}
\description{
This function extracts the PROBAST diagnostics
}
\details{
Specify the connectionHandler, the resultDatabaseSettings and (optionally) a modelDesignId and threshold1_2 a threshold value to use for the PROBAST 1.2
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

diag <- getPredictionDiagnostics(
  connectionHandler = connectionHandler, 
  schema = 'main'
)

}
\seealso{
Other Prediction: 
\code{\link{getFullPredictionPerformances}()},
\code{\link{getPredictionAggregateTopPredictors}()},
\code{\link{getPredictionCohorts}()},
\code{\link{getPredictionCovariates}()},
\code{\link{getPredictionDiagnosticTable}()},
\code{\link{getPredictionHyperParamSearch}()},
\code{\link{getPredictionIntercept}()},
\code{\link{getPredictionLift}()},
\code{\link{getPredictionModelDesigns}()},
\code{\link{getPredictionOutcomes}()},
\code{\link{getPredictionPerformanceTable}()},
\code{\link{getPredictionPerformances}()},
\code{\link{getPredictionTargets}()},
\code{\link{getPredictionTopPredictors}()}
}
\concept{Prediction}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CharacterzationQueries.R
\name{getIncidenceRates}
\alias{getIncidenceRates}
\title{Extract the cohort incidence result}
\usage{
getIncidenceRates(
  connectionHandler,
  schema,
  ciTablePrefix = "ci_",
  cgTablePrefix = "cg_",
  databaseTable = "database_meta_data",
  targetIds = NULL,
  outcomeIds = NULL
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{ciTablePrefix}{The prefix used for the cohort incidence results tables}

\item{cgTablePrefix}{The prefix used for the cohort generator results tables}

\item{databaseTable}{The name of the table with the database details (default 'database_meta_data')}

\item{targetIds}{A vector of integers corresponding to the target cohort IDs}

\item{outcomeIds}{A vector of integers corresponding to the outcome cohort IDs}
}
\value{
Returns a data.frame with the columns:
\itemize{
 \item{databaseName the name of the database}
 \item{databaseId the unique id of the database}
 \item{targetName the target cohort name}
 \item{targetId the target cohort unique identifier}
 \item{outcomeName the outcome name}
 \item{outcomeId the outcome unique identifier}
 \item(tar the friendly time-at-risk string)
 \item{cleanWindow clean windown around outcome}
 \item{subgroupName name for the result subgroup}
 \item{ageGroupName name for the result age group}
 \item{genderName name for the result gender group}
 \item{startYear name for the result start year}
 \item{tarStartWith time at risk start reference}
 \item{tarStartOffset time at risk start offset from reference}
 \item{tarEndWith time at risk end reference}
 \item{tarEndOffset time at risk end offset from reference}
 \item{personsAtRiskPe persons at risk per event}
 \item{personsAtRisk persons at risk}
 \item{personDaysPe person days per event}
 \item{personDays person days}
 \item{personOutcomesPe person outcome per event}
 \item{personOutcomes persons outcome}
 \item{outcomesPe number of outcome per event}
 \item{outcomes number of outcome}
 \item{incidenceProportionP100p incidence proportion per 100 persons}
 \item{incidenceRateP100py incidence rate per 100 person years}
 }
}
\description{
This function extracts all incidence rates across databases in the results for specified target and outcome cohorts.
}
\details{
Specify the connectionHandler, the schema and the target/outcome cohort IDs
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

ir <- getIncidenceRates(
connectionHandler = connectionHandler, 
schema = 'main'
)

}
\seealso{
Other Characterization: 
\code{\link{getBinaryCaseSeries}()},
\code{\link{getBinaryRiskFactors}()},
\code{\link{getBinaryTargetBaseline}()},
\code{\link{getCaseCounts}()},
\code{\link{getCaseTargetCounts}()},
\code{\link{getCharacterizationCohortBinary}()},
\code{\link{getCharacterizationCohortContinuous}()},
\code{\link{getCharacterizationDemographics}()},
\code{\link{getCharacterizationOutcomes}()},
\code{\link{getCharacterizationTargets}()},
\code{\link{getContinuousCaseSeries}()},
\code{\link{getContinuousRiskFactors}()},
\code{\link{getDechallengeRechallenge}()},
\code{\link{getDechallengeRechallengeFails}()},
\code{\link{getIncidenceOutcomes}()},
\code{\link{getIncidenceTargets}()},
\code{\link{getTargetBinaryFeatures}()},
\code{\link{getTargetContinuousFeatures}()},
\code{\link{getTimeToEvent}()},
\code{\link{plotAgeDistributions}()},
\code{\link{plotSexDistributions}()},
\code{\link{viewIncidenceRate}()}
}
\concept{Characterization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreatmentPatternsQueries.R
\name{getTreatmentPathways}
\alias{getTreatmentPathways}
\title{Extracts treatment pathways}
\usage{
getTreatmentPathways(
  connectionHandler,
  schema,
  tpTablePrefix = "tp_",
  databaseTable = "database_meta_data",
  age = "all",
  sex = "all",
  indexYear = "all",
  analysisIds = NULL,
  databaseIds = NULL,
  databaseNames = NULL,
  targetIds = NULL
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{tpTablePrefix}{The prefix used for the cohort generator results tables}

\item{databaseTable}{The name of the table with the database details (default 'database_meta_data')}

\item{age}{(optional) a string representing an age bucket to restrict (e.g., "0-17", "18-34", "65+")}

\item{sex}{(optional) A string "male" or "female" to restrict}

\item{indexYear}{(optional) A string with a four-digit year to restrict}

\item{analysisIds}{(optional) A vector of analysis ids to restrict to}

\item{databaseIds}{(optional) A vector of database ids to restrict to}

\item{databaseNames}{(optional) A vector of database Names to restrict to}

\item{targetIds}{(optional) A vector of target cohort ids to restrict to}
}
\value{
Returns a data.frame with the columns:
\itemize{
 \item{databaseName the name of the database}
 \item{databaseId the unique identifier of the database}
 \item{analysisId the unique identifier of a treament patterns run}
 \item{targetCohortName the target cohort name}
 \item{targetCohortId the target cohort unique identifier}
 \item{pathway a string representing the progression of events for a target. Use '-' to separate sequential steps and '+' for combination of events at that step}
 \item{freq the count of pathway occurance}
 \item{age the stratifyed pathways for age}
 \item{indexYear the stratifyed pathways for index year}
 \item{sex the stratifyed pathways for sex}
 }
}
\description{
This function extracts results pathways for specified analysis ids and target cohorts
}
\details{
Specify the connectionHandler and the schema
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

tp <- getTreatmentPathways(
  connectionHandler = connectionHandler,
  schema = "main"
)
}
\seealso{
Other TreatmentPatterns: 
\code{\link{getAnalysisCohorts}()},
\code{\link{getEventDuration}()}
}
\concept{TreatmentPatterns}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PredictionQueries.R
\name{getPredictionLift}
\alias{getPredictionLift}
\title{Extract model lift at given model sensitivity}
\usage{
getPredictionLift(
  connectionHandler,
  schema,
  plpTablePrefix = "plp_",
  modelDesignIds = NULL,
  performanceIds = NULL,
  sensitivity = 0.1
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{plpTablePrefix}{The prefix used for the patient level prediction results tables}

\item{modelDesignIds}{(optional) restrict to the input modelDesignIds}

\item{performanceIds}{(optional) restrict to the input performanceIds}

\item{sensitivity}{(default 0.1) the lift at the threshold with the sensitivity closest to this value is return}
}
\value{
Returns a data.frame with the columns: modelDesignId, performanceId, evaluation, sensitivity, outcomeCount, positivePredictiveValue, outcomeRate and lift.
}
\description{
This function extracts the model lift (PPV/outcomeRate)
}
\details{
Specify the connectionHandler, the resultDatabaseSettings and (optionally) modelDesignIds or performanceIds to filter to
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

liftsAt0p15 <- getPredictionLift(
  connectionHandler = connectionHandler, 
  schema = 'main', 
  sensitivity = 0.15
)

}
\seealso{
Other Prediction: 
\code{\link{getFullPredictionPerformances}()},
\code{\link{getPredictionAggregateTopPredictors}()},
\code{\link{getPredictionCohorts}()},
\code{\link{getPredictionCovariates}()},
\code{\link{getPredictionDiagnosticTable}()},
\code{\link{getPredictionDiagnostics}()},
\code{\link{getPredictionHyperParamSearch}()},
\code{\link{getPredictionIntercept}()},
\code{\link{getPredictionModelDesigns}()},
\code{\link{getPredictionOutcomes}()},
\code{\link{getPredictionPerformanceTable}()},
\code{\link{getPredictionPerformances}()},
\code{\link{getPredictionTargets}()},
\code{\link{getPredictionTopPredictors}()}
}
\concept{Prediction}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Helpers.R
\name{formatBinaryCovariateName}
\alias{formatBinaryCovariateName}
\title{formatBinaryCovariateName}
\usage{
formatBinaryCovariateName(data)
}
\arguments{
\item{data}{The data.frame with the covariateName column}
}
\value{
The data data.frame object with the ovariateName column changed to be more friendly
}
\description{
Removes the long part of the covariate name to make it friendly
}
\details{
Makes the covariateName more friendly and shorter
}
\examples{
formatBinaryCovariateName(data.frame(
covariateName = c("fdfgfgf: dgdgff","made up test")
))

}
\seealso{
Other helper: 
\code{\link{addTarColumn}()},
\code{\link{getExampleConnectionDetails}()},
\code{\link{getOutcomeTable}()},
\code{\link{getTargetTable}()},
\code{\link{kableDark}()},
\code{\link{removeSpaces}()}
}
\concept{helper}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PredictionQueries.R
\name{getPredictionTopPredictors}
\alias{getPredictionTopPredictors}
\title{Extract the top N predictors per model}
\usage{
getPredictionTopPredictors(
  connectionHandler,
  schema,
  plpTablePrefix = "plp_",
  cgTablePrefix = "cg_",
  databaseTable = "database_meta_data",
  targetIds = NULL,
  outcomeIds = NULL,
  numberPredictors = 100
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{plpTablePrefix}{The prefix used for the patient level prediction results tables}

\item{cgTablePrefix}{The prefix used for the cohort generator results tables}

\item{databaseTable}{The database table name}

\item{targetIds}{A vector of integers corresponding to the target cohort IDs}

\item{outcomeIds}{A vector of integers corresponding to the outcome cohort IDs}

\item{numberPredictors}{the number of predictors per model to return}
}
\value{
Returns a data.frame with the columns: 
\itemize{
 \item{databaseName the name of the database the model was developed on}
 \item{tarStartDay the time-at-risk start day}
 \item{tarStartAnchor whether the time-at-risk start is relative to cohort start or end}
 \item{tarEndDay the time-at-risk end day}
 \item{tarEndAnchor whether the time-at-risk end is relative to cohort start or end}
 \item{performanceId a unique identifier for the performance}
 \item{covariateId the FeatureExtraction covariate identifier}
 \item{covariateName the name of the covariate}
 \item{conceptId the covariates corresponding concept or 0}
 \item{covariateValue the feature importance or coefficient value}
 \item{covariateCount how many people had the covariate}
 \item{covariateMean the fraction of the target population with the covariate}
 \item{covariateStDev the standard deviation}
 \item{withNoOutcomeCovariateCount the number of the target population without the outcome with the covariate}
 \item{withNoOutcomeCovariateMean the fraction of the target population without the outcome with the covariate}
 \item{withNoOutcomeCovariateStDev the covariate standard deviation of the target population without the outcome}
 \item{withOutcomeCovariateCount the number of the target population with the outcome with the covariate}
 \item{withOutcomeCovariateMean the fraction of the target population with the outcome with the covariate}
 \item{withOutcomeCovariateStDev the covariate standard deviation of the target population with the outcome}
 \item{standardizedMeanDiff the standardized mean difference comparing the target population with outcome and without the outcome}
 \item{rn the row number showing the covariate rank}
}
}
\description{
This function extracts the top N predictors per model from the prediction results tables
}
\details{
Specify the connectionHandler, the resultDatabaseSettings and (optionally) any targetIds or outcomeIds to restrict models to
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

topPreds <- getPredictionTopPredictors(
  connectionHandler = connectionHandler, 
  schema = 'main',
  targetIds = 1,
  outcomeIds = 3
)

}
\seealso{
Other Prediction: 
\code{\link{getFullPredictionPerformances}()},
\code{\link{getPredictionAggregateTopPredictors}()},
\code{\link{getPredictionCohorts}()},
\code{\link{getPredictionCovariates}()},
\code{\link{getPredictionDiagnosticTable}()},
\code{\link{getPredictionDiagnostics}()},
\code{\link{getPredictionHyperParamSearch}()},
\code{\link{getPredictionIntercept}()},
\code{\link{getPredictionLift}()},
\code{\link{getPredictionModelDesigns}()},
\code{\link{getPredictionOutcomes}()},
\code{\link{getPredictionPerformanceTable}()},
\code{\link{getPredictionPerformances}()},
\code{\link{getPredictionTargets}()}
}
\concept{Prediction}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CohortQueries.R
\name{getCohortSubsetDefinitions}
\alias{getCohortSubsetDefinitions}
\title{Extract the cohort subset definition details}
\usage{
getCohortSubsetDefinitions(
  connectionHandler,
  schema,
  cgTablePrefix = "cg_",
  subsetIds = NULL
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{cgTablePrefix}{The prefix used for the cohort generator results tables}

\item{subsetIds}{A vector of subset cohort ids or NULL}
}
\value{
Returns a data.frame with the cohort subset details
}
\description{
This function extracts all cohort subset definitions for the subsets of interest.
}
\details{
Specify the connectionHandler, the schema and the subset IDs
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

subsetDef <- getCohortSubsetDefinitions(
  connectionHandler = connectionHandler, 
  schema = 'main'
)

}
\seealso{
Other Cohorts: 
\code{\link{getCohortAttrition}()},
\code{\link{getCohortCounts}()},
\code{\link{getCohortDefinitions}()},
\code{\link{getCohortInclusionRules}()},
\code{\link{getCohortInclusionStats}()},
\code{\link{getCohortInclusionSummary}()},
\code{\link{getCohortMeta}()},
\code{\link{getCohortSubsetAttrition}()},
\code{\link{getSubsetText}()},
\code{\link{processCohortDefinitionsForQuarto}()},
\code{\link{processCohorts}()},
\code{\link{restrictCohortDefinitionsForQuarto}()}
}
\concept{Cohorts}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PredictionQueries.R
\name{getPredictionModelDesigns}
\alias{getPredictionModelDesigns}
\title{Extract the model designs from the prediction results}
\usage{
getPredictionModelDesigns(
  connectionHandler,
  schema,
  plpTablePrefix = "plp_",
  cgTablePrefix = "cg_",
  targetIds = NULL,
  outcomeIds = NULL,
  modelDesignIds = NULL
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{plpTablePrefix}{The prefix used for the patient level prediction results tables}

\item{cgTablePrefix}{The prefix used for the cohort generator results tables}

\item{targetIds}{A vector of integers corresponding to the target cohort IDs}

\item{outcomeIds}{A vector of integers corresponding to the outcome cohort IDs}

\item{modelDesignIds}{(Optional) A set of model design ids to restrict to}
}
\value{
Returns a data.frame with the columns:
\itemize{ 
 \item{modelDesignId a unique identifier in the database for the model design}
 \item{modelType the type of classifier or surival model}
 \item{developmentTargetId a unique identifier for the development target ID}
 \item{developmentTargetName the name of the development target cohort}
 \item{developmentTargetJson the json of the target cohort}
 \item{developmentOutcomeId a unique identifier for the development outcome ID}
 \item{developmentOutcomeName the name of the development outcome cohort}
 \item{timeAtRisk the time at risk string}
 \item{developmentOutcomeJson the json of the outcome cohort}
 \item{covariateSettingsJson the covariate settings json}
 \item{populationSettingsJson the population settings json}
 \item{tidyCovariatesSettingsJson the tidy covariate settings json}
 \item{plpDataSettingsJson the plp data extraction settings json}
 \item{featureEngineeringSettingsJson the feature engineering settings json}
 \item{splitSettingsJson the split settings json}
 \item{sampleSettingsJson the sample settings json}
 \item{modelSettingsJson the model settings json}
 }
}
\description{
This function extracts the model design settings
}
\details{
Specify the connectionHandler, the resultDatabaseSettings and (optionally) any targetIds or outcomeIds to restrict model designs to
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

modDesign <- getPredictionModelDesigns(
  connectionHandler = connectionHandler, 
  schema = 'main'
)

}
\seealso{
Other Prediction: 
\code{\link{getFullPredictionPerformances}()},
\code{\link{getPredictionAggregateTopPredictors}()},
\code{\link{getPredictionCohorts}()},
\code{\link{getPredictionCovariates}()},
\code{\link{getPredictionDiagnosticTable}()},
\code{\link{getPredictionDiagnostics}()},
\code{\link{getPredictionHyperParamSearch}()},
\code{\link{getPredictionIntercept}()},
\code{\link{getPredictionLift}()},
\code{\link{getPredictionOutcomes}()},
\code{\link{getPredictionPerformanceTable}()},
\code{\link{getPredictionPerformances}()},
\code{\link{getPredictionTargets}()},
\code{\link{getPredictionTopPredictors}()}
}
\concept{Prediction}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CharacterzationQueries.R
\name{getIncidenceTargets}
\alias{getIncidenceTargets}
\title{A function to extract the targets found in incidence}
\usage{
getIncidenceTargets(
  connectionHandler,
  schema,
  ciTablePrefix = "ci_",
  cgTablePrefix = "cg_"
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{ciTablePrefix}{The prefix used for the cohort incidence results tables}

\item{cgTablePrefix}{The prefix used for the cohort generator results tables}
}
\value{
A data.frame with the incidence target cohort ids and names
}
\description{
A function to extract the targets found in incidence
}
\details{
Specify the connectionHandler, the schema and the prefixes
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

cohorts <- getIncidenceTargets(
  connectionHandler = connectionHandler, 
  schema = 'main'
)

}
\seealso{
Other Characterization: 
\code{\link{getBinaryCaseSeries}()},
\code{\link{getBinaryRiskFactors}()},
\code{\link{getBinaryTargetBaseline}()},
\code{\link{getCaseCounts}()},
\code{\link{getCaseTargetCounts}()},
\code{\link{getCharacterizationCohortBinary}()},
\code{\link{getCharacterizationCohortContinuous}()},
\code{\link{getCharacterizationDemographics}()},
\code{\link{getCharacterizationOutcomes}()},
\code{\link{getCharacterizationTargets}()},
\code{\link{getContinuousCaseSeries}()},
\code{\link{getContinuousRiskFactors}()},
\code{\link{getDechallengeRechallenge}()},
\code{\link{getDechallengeRechallengeFails}()},
\code{\link{getIncidenceOutcomes}()},
\code{\link{getIncidenceRates}()},
\code{\link{getTargetBinaryFeatures}()},
\code{\link{getTargetContinuousFeatures}()},
\code{\link{getTimeToEvent}()},
\code{\link{plotAgeDistributions}()},
\code{\link{plotSexDistributions}()},
\code{\link{viewIncidenceRate}()}
}
\concept{Characterization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CohortQueries.R
\name{processCohorts}
\alias{processCohorts}
\title{Extract the cohort parents and children cohorts (cohorts derieved from the parent cohort)}
\usage{
processCohorts(cohort)
}
\arguments{
\item{cohort}{The data.frame extracted using `getCohortDefinitions()`}
}
\value{
Returns a list containing parents: a named vector of all the parent cohorts and cohortList: a list 
the same length as the parent vector with the first element containing all the children
of the first parent cohort, the second element containing the children of the second parent, etc.
}
\description{
This function lets you split the cohort data.frame into the parents and the children per parent.
}
\details{
Finds the parent cohorts and children cohorts
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

cohortDef <- getCohortDefinitions(
  connectionHandler = connectionHandler, 
  schema = 'main'
)

parents <- processCohorts(cohortDef)

}
\seealso{
Other Cohorts: 
\code{\link{getCohortAttrition}()},
\code{\link{getCohortCounts}()},
\code{\link{getCohortDefinitions}()},
\code{\link{getCohortInclusionRules}()},
\code{\link{getCohortInclusionStats}()},
\code{\link{getCohortInclusionSummary}()},
\code{\link{getCohortMeta}()},
\code{\link{getCohortSubsetAttrition}()},
\code{\link{getCohortSubsetDefinitions}()},
\code{\link{getSubsetText}()},
\code{\link{processCohortDefinitionsForQuarto}()},
\code{\link{restrictCohortDefinitionsForQuarto}()}
}
\concept{Cohorts}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PredictionQueries.R
\name{getPredictionPerformances}
\alias{getPredictionPerformances}
\title{Extract the model performances}
\usage{
getPredictionPerformances(
  connectionHandler,
  schema,
  plpTablePrefix = "plp_",
  cgTablePrefix = "cg_",
  databaseTable = "database_meta_data",
  databaseTablePrefix = "",
  modelDesignId = NULL,
  developmentDatabaseId = NULL
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{plpTablePrefix}{The prefix used for the patient level prediction results tables}

\item{cgTablePrefix}{The prefix used for the cohort generator results tables}

\item{databaseTable}{The name of the table with the database details (default 'database_meta_data')}

\item{databaseTablePrefix}{A prefix to the database table, either '' or 'plp_'}

\item{modelDesignId}{The identifier for a model design  to restrict results to}

\item{developmentDatabaseId}{The identifier for the development database to restrict results to}
}
\value{
Returns a data.frame with the columns:
\itemize{  
 \item{performanceId the unique identifier for the performance}
 \item{modelDesignId the unique identifier for the model design}
 \item{modelType the type of classifier}
 \item{algorithmName the model algorithm name}
 \item{developmentDatabaseId the unique identifier for the database used to develop the model}
 \item{validationDatabaseId the unique identifier for the database used to validate the model}
 \item{developmentTargetId the unique cohort id for the development target population}
 \item{developmentTargetName the name for the development target population}
 \item{developmentOutcomeId the unique cohort id for the development outcome}
 \item{developmentOutcomeName the name for the development outcome}
 \item{developmentDatabase the name for the database used to develop the model}
 \item{validationDatabase the name for the database used to validate the model}
 \item{validationTargetName the name for the validation target population}
 \item{validationOutcomeName the name for the validation outcome}
 \item{timeStamp the date/time when the analysis occurred}
 \item{auroc the test/validation AUROC value for the model}
 \item{auroc95lb the test/validation lower bound of the 95 percent CI AUROC value for the model}
 \item{auroc95ub the test/validation upper bound of the 95 percent CI AUROC value for the model}
 \item{calibrationInLarge the test/validation calibration in the large value for the model}
 \item{eStatistic the test/validation calibration e-statistic value for the model}
 \item{brierScore the test/validation brier value for the model}
 \item{auprc the test/validation discrimination AUPRC value for the model}
 \item{populationSize the test/validation population size used to develop the model}
 \item{outcomeCount the test/validation outcome count used to develop the model}
 \item{evalPercent the percentage of the development data used as the test set}
 \item{outcomePercent the outcome percent in the development data}
 \item{validationTimeAtRisk time at risk for the validation}
 \item{predictionResultType development or validation}
 
}
}
\description{
This function extracts the model performances
}
\details{
Specify the connectionHandler, the resultDatabaseSettings and (optionally) a modelDesignId and/or developmentDatabaseId to restrict models to
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

perf <- getPredictionPerformances(
  connectionHandler = connectionHandler, 
  schema = 'main'
)

}
\seealso{
Other Prediction: 
\code{\link{getFullPredictionPerformances}()},
\code{\link{getPredictionAggregateTopPredictors}()},
\code{\link{getPredictionCohorts}()},
\code{\link{getPredictionCovariates}()},
\code{\link{getPredictionDiagnosticTable}()},
\code{\link{getPredictionDiagnostics}()},
\code{\link{getPredictionHyperParamSearch}()},
\code{\link{getPredictionIntercept}()},
\code{\link{getPredictionLift}()},
\code{\link{getPredictionModelDesigns}()},
\code{\link{getPredictionOutcomes}()},
\code{\link{getPredictionPerformanceTable}()},
\code{\link{getPredictionTargets}()},
\code{\link{getPredictionTopPredictors}()}
}
\concept{Prediction}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CohortQueries.R
\name{restrictCohortDefinitionsForQuarto}
\alias{restrictCohortDefinitionsForQuarto}
\title{Function to figure out the target, comparator, outcome and indication ids of interest
for the quarto report based on the user inputs}
\usage{
restrictCohortDefinitionsForQuarto(
  connectionHandler,
  schema,
  cohortDefinitions,
  targetId,
  outcomeIds,
  comparatorIds,
  restrictTargetToIndications,
  indicationIds,
  includeCohortMethod
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{cohortDefinitions}{The output of `processCohortDefinitionsForQuarto()`}

\item{targetId}{a parent target id}

\item{outcomeIds}{a vector of outcome ids}

\item{comparatorIds}{NULL or a vector of comparator parent ids}

\item{restrictTargetToIndications}{whether to restrict the target ids to cohorts nested in indicationIds}

\item{indicationIds}{The indication ids of interest for restrictTargetToIndications}

\item{includeCohortMethod}{If TRUE, when comparatorIds is NULL all comparators included in CohortMethod are included in the report}
}
\value{
A list of targetIdsOfInterest that is a vector of cohortIds that are targets of interest to include in the report,
comparatorIds a vector of cohortIds that are comparators, 
comparatorIdsOfInterest a vector of cohortIds that are comparators of interest, 
outcomeIdsOfInterest  a vector of cohortIds that are outcomes of interest and
indicationIdsOfInterest  a vector of cohortIds that are indications of interest.
}
\description{
Function to figure out the target, comparator, outcome and indication ids of interest
for the quarto report based on the user inputs
}
\details{
This function finds the targets, comparators, indications and outcomes of interest based
on the user inputs for quarto report generation.
}
\seealso{
Other Cohorts: 
\code{\link{getCohortAttrition}()},
\code{\link{getCohortCounts}()},
\code{\link{getCohortDefinitions}()},
\code{\link{getCohortInclusionRules}()},
\code{\link{getCohortInclusionStats}()},
\code{\link{getCohortInclusionSummary}()},
\code{\link{getCohortMeta}()},
\code{\link{getCohortSubsetAttrition}()},
\code{\link{getCohortSubsetDefinitions}()},
\code{\link{getSubsetText}()},
\code{\link{processCohortDefinitionsForQuarto}()},
\code{\link{processCohorts}()}
}
\concept{Cohorts}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Helpers.R
\name{removeSpaces}
\alias{removeSpaces}
\title{removeSpaces}
\usage{
removeSpaces(x)
}
\arguments{
\item{x}{A string}
}
\value{
A string without spaces
}
\description{
Removes spaces and replaces with under scroll
}
\details{
Removes spaces and replaces with under scroll
}
\examples{
removeSpaces(' made up.   string')

}
\seealso{
Other helper: 
\code{\link{addTarColumn}()},
\code{\link{formatBinaryCovariateName}()},
\code{\link{getExampleConnectionDetails}()},
\code{\link{getOutcomeTable}()},
\code{\link{getTargetTable}()},
\code{\link{kableDark}()}
}
\concept{helper}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EstimationQueries.R
\name{getCmDiagnosticsData}
\alias{getCmDiagnosticsData}
\title{Extract the cohort method diagostic results}
\usage{
getCmDiagnosticsData(
  connectionHandler,
  schema,
  cmTablePrefix = "cm_",
  cgTablePrefix = "cg_",
  databaseTable = "database_meta_data",
  targetIds = NULL,
  outcomeIds = NULL,
  comparatorIds = NULL,
  indicationIds = NULL,
  analysisIds = NULL,
  databaseIds = NULL
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{cmTablePrefix}{The prefix used for the cohort method results tables}

\item{cgTablePrefix}{The prefix used for the cohort generator results tables}

\item{databaseTable}{The name of the table with the database details (default 'database_meta_data')}

\item{targetIds}{A vector of integers corresponding to the target cohort IDs}

\item{outcomeIds}{A vector of integers corresponding to the outcome cohort IDs}

\item{comparatorIds}{A vector of integers corresponding to the comparator cohort IDs}

\item{indicationIds}{A vector of cohort ids for the indication (nesting id) to restrict to}

\item{analysisIds}{An optional vector of analysisIds to filter to}

\item{databaseIds}{An optional vector of databaseIds to filter to}
}
\value{
Returns a data.frame with the columns:
\itemize{
 \item{databaseName the name of the database}
 \item{databaseId the unqiue identifier of the database}
 \item{analysisId the analysis unique identifier}
 \item{description a description of the analysis}
 \item{targetName the target cohort name}
 \item{targetId the target cohort unique identifier}
 \item{comparatorName the comparator cohort name}
 \item{comparatorId the comparator cohort unique identifier}
 \item{outcomeName the outcome name}
 \item{outcomeId the outcome cohort unique identifier}
 \item{maxSdm max allowed standardized difference of means when comparing the target to the comparator after PS adjustment for the ballance diagnostic diagnostic to pass.}
 \item{sharedMaxSdm max allowed standardized difference of means when comparing the target to the comparator after PS adjustment for the ballance diagnostic diagnostic to pass.}
 \item{equipoise the bounds on the preference score to determine whether a subject is in equipoise.}
 \item{mdrr the maximum passable minimum detectable relative risk (mdrr) value.  If the mdrr is greater than this the diagnostics will fail.}
 \item{attritionFraction (depreciated) the minmum attrition before the diagnostics fails.}
 \item{ease The expected absolute systematic error (ease) measures residual bias.}
 \item{balanceDiagnostic whether the balance diagnostic passed or failed.}
 \item{sharedBalanceDiagnostic whether the shared balance diagnostic passed or failed.}
 \item{equipoiseDiagnostic whether the equipose diagnostic passed or failed.}
 \item{mdrrDiagnostic whether the mdrr (power) diagnostic passed or failed.}
 \item{attritionDiagnostic (depreciated) whether the attrition diagnostic passed or failed.}
 \item{easeDiagnostic whether the ease diagnostic passed or failed.}
 \item{unblindForEvidenceSynthesis whether the results can be unblinded for the meta analysis.}
 \item{unblind whether the results can be unblinded.}
 \item{summaryValue summary of diagnostics results. FAIL, PASS or number of warnings.}
 }
}
\description{
This function extracts the cohort method diagnostics that examine whether the analyses were sufficiently powered
and checks for different types of bias.
}
\details{
Specify the connectionHandler, the schema and the target/comparator/outcome cohort IDs
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

cmDiag <- getCmDiagnosticsData(
  connectionHandler = connectionHandler, 
  schema = 'main',
  targetIds = 1,
  outcomeIds = 3
)

}
\seealso{
Other Estimation: 
\code{\link{.getCmVersion}()},
\code{\link{.getSccsVersion}()},
\code{\link{getCMEstimation}()},
\code{\link{getCmMetaEstimation}()},
\code{\link{getCmNegativeControlEstimates}()},
\code{\link{getCmOutcomes}()},
\code{\link{getCmPropensityModel}()},
\code{\link{getCmTable}()},
\code{\link{getCmTargets}()},
\code{\link{getSccsDiagnosticsData}()},
\code{\link{getSccsEstimation}()},
\code{\link{getSccsMetaEstimation}()},
\code{\link{getSccsModel}()},
\code{\link{getSccsNegativeControlEstimates}()},
\code{\link{getSccsOutcomes}()},
\code{\link{getSccsTable}()},
\code{\link{getSccsTargets}()},
\code{\link{getSccsTimeToEvent}()},
\code{\link{plotCmEstimates}()},
\code{\link{plotSccsEstimates}()}
}
\concept{Estimation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EstimationQueries.R
\name{getCmMetaEstimation}
\alias{getCmMetaEstimation}
\title{Extract the cohort method meta analysis results}
\usage{
getCmMetaEstimation(
  connectionHandler,
  schema,
  cmTablePrefix = "cm_",
  cgTablePrefix = "cg_",
  esTablePrefix = "es_",
  targetIds = NULL,
  outcomeIds = NULL,
  indicationIds = NULL,
  comparatorIds = NULL,
  includeOneSidedP = TRUE
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{cmTablePrefix}{The prefix used for the cohort method results tables}

\item{cgTablePrefix}{The prefix used for the cohort generator results tables}

\item{esTablePrefix}{The prefix used for the evidence synthesis results tables}

\item{targetIds}{A vector of integers corresponding to the target cohort IDs}

\item{outcomeIds}{A vector of integers corresponding to the outcome cohort IDs}

\item{indicationIds}{A vector of cohort ids for the indication (nesting id) to restrict to}

\item{comparatorIds}{A vector of integers corresponding to the comparator cohort IDs}

\item{includeOneSidedP}{This lets you extract from older results that do not have the one sided p by setting this to FALSE}
}
\value{
Returns a data.frame with the columns:
\itemize{
 \item{databaseName the name of the database}
 \item{analysisId the analysis unique identifier}
 \item{description a description of the analysis}
 \item{targetName the target cohort name}
 \item{targetId the target cohort unique identifier}
 \item{comparatorName the comparator cohort name}
 \item{comparatorId the comparator cohort unique identifier}
 \item{outcomeName the outcome name}
 \item{outcomeId the outcome cohort unique identifier}
 \item{calibratedRr the calibrated relative risk}
 \item{calibratedRrCi95Lb the calibrated relative risk 95 percent confidence interval lower bound}
 \item{calibratedRrCi95Ub the calibrated relative risk 95 percent confidence interval upper bound}
 \item{calibratedP the two sided calibrated p value}
 \item{calibratedOneSidedP the one sided calibrated p value}
 \item{calibratedLogRr the calibrated relative risk logged}
 \item{calibratedSeLogRr the standard error of the calibrated relative risk logged}
 
 \item{targetSubjects the number of people in the target cohort across included database}
 \item{comparatorSubjects the number of people in the comparator cohort across included database}
 \item{targetDays the total number of days at risk across the target cohort people across included database}
 \item{comparatorDays the total number of days at risk across the comparator cohort people across included database}
 \item{targetOutcomes the total number of outcomes occuring during the time at risk for the target cohort people across included database}
 \item{comparatorOutcomes the total number of outcomes occuring during the time at risk for the comparator cohort people across included database}
 
 \item{unblind whether the results can be unblinded.}
 \item{nDatabases the number of databases included}
 }
}
\description{
This function extracts any meta analysis estimation results for cohort method.
}
\details{
Specify the connectionHandler, the schema and the target/comparator/outcome cohort IDs
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

cmMeta <- getCmMetaEstimation(
  connectionHandler = connectionHandler, 
  schema = 'main',
  targetIds = 1,
  outcomeIds = 3
)

}
\seealso{
Other Estimation: 
\code{\link{.getCmVersion}()},
\code{\link{.getSccsVersion}()},
\code{\link{getCMEstimation}()},
\code{\link{getCmDiagnosticsData}()},
\code{\link{getCmNegativeControlEstimates}()},
\code{\link{getCmOutcomes}()},
\code{\link{getCmPropensityModel}()},
\code{\link{getCmTable}()},
\code{\link{getCmTargets}()},
\code{\link{getSccsDiagnosticsData}()},
\code{\link{getSccsEstimation}()},
\code{\link{getSccsMetaEstimation}()},
\code{\link{getSccsModel}()},
\code{\link{getSccsNegativeControlEstimates}()},
\code{\link{getSccsOutcomes}()},
\code{\link{getSccsTable}()},
\code{\link{getSccsTargets}()},
\code{\link{getSccsTimeToEvent}()},
\code{\link{plotCmEstimates}()},
\code{\link{plotSccsEstimates}()}
}
\concept{Estimation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CharacterzationQueries.R
\name{getTargetContinuousFeatures}
\alias{getTargetContinuousFeatures}
\title{Extract aggregate statistics of continuous feature analysis IDs of interest for targets}
\usage{
getTargetContinuousFeatures(
  connectionHandler,
  schema,
  cTablePrefix = "c_",
  cgTablePrefix = "cg_",
  databaseTable = "database_meta_data",
  targetIds = NULL,
  analysisIds = NULL,
  databaseIds = NULL
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{cTablePrefix}{The prefix used for the characterization results tables}

\item{cgTablePrefix}{The prefix used for the cohort generator results tables}

\item{databaseTable}{The name of the table with the database details (default 'database_meta_data')}

\item{targetIds}{A vector of integers corresponding to the target cohort IDs}

\item{analysisIds}{The feature extraction analysis ID of interest (e.g., 201 is condition)}

\item{databaseIds}{(Optional) A vector of database IDs to restrict to}
}
\value{
Returns a data.frame with the columns:
\itemize{
 \item{databaseName the name of the database}
 \item{databaseId the unique identifier of the database}
 \item{targetName the target cohort name}
 \item{targetId the target cohort unique identifier}
 \item{minPriorObservation the minimum required observation days prior to index for an entry}
 \item{covariateName the name of the feature}
 \item{covariateId the id of the feature}
 \item{countValue the number of cases who have the feature}
 \item{minValue the minimum value observed for the feature}
 \item{maxValue the maximum value observed for the feature}
 \item{averageValue the mean value observed for the feature}
 \item{standardDeviation the standard deviation of the value observed for the feature}
 \item{medianValue the median value observed for the feature}
 \item{p10Value the 10th percentile of the value observed for the feature}
 \item{p25Value the 25th percentile of the value observed for the feature}
 \item{p75Value the 75th percentile of the value observed for the feature}
 \item{p90Value the 90th percentile of the value observed for the feature}
 
}
}
\description{
This function extracts the continuous feature extraction results for targets corresponding to specified target cohorts.
}
\details{
Specify the connectionHandler, the schema and the target/outcome cohort IDs
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

tcf <- getTargetContinuousFeatures(
connectionHandler = connectionHandler, 
schema = 'main'
)

}
\seealso{
Other Characterization: 
\code{\link{getBinaryCaseSeries}()},
\code{\link{getBinaryRiskFactors}()},
\code{\link{getBinaryTargetBaseline}()},
\code{\link{getCaseCounts}()},
\code{\link{getCaseTargetCounts}()},
\code{\link{getCharacterizationCohortBinary}()},
\code{\link{getCharacterizationCohortContinuous}()},
\code{\link{getCharacterizationDemographics}()},
\code{\link{getCharacterizationOutcomes}()},
\code{\link{getCharacterizationTargets}()},
\code{\link{getContinuousCaseSeries}()},
\code{\link{getContinuousRiskFactors}()},
\code{\link{getDechallengeRechallenge}()},
\code{\link{getDechallengeRechallengeFails}()},
\code{\link{getIncidenceOutcomes}()},
\code{\link{getIncidenceRates}()},
\code{\link{getIncidenceTargets}()},
\code{\link{getTargetBinaryFeatures}()},
\code{\link{getTimeToEvent}()},
\code{\link{plotAgeDistributions}()},
\code{\link{plotSexDistributions}()},
\code{\link{viewIncidenceRate}()}
}
\concept{Characterization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PredictionQueries.R
\name{getPredictionDiagnosticTable}
\alias{getPredictionDiagnosticTable}
\title{Extract specific diagnostic table}
\usage{
getPredictionDiagnosticTable(
  connectionHandler,
  schema,
  plpTablePrefix = "plp_",
  table = "diagnostic_participants",
  diagnosticId = NULL
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{plpTablePrefix}{The prefix used for the patient level prediction results tables}

\item{table}{The table to extract}

\item{diagnosticId}{(optional) restrict to the input diagnosticId}
}
\value{
Returns a data.frame with the specified table
}
\description{
This function extracts the specified diagnostic table
}
\details{
Specify the connectionHandler, the resultDatabaseSettings, the table of interest and (optionally) a diagnosticId to filter to
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

diagPred <- getPredictionDiagnosticTable(
  connectionHandler = connectionHandler, 
  schema = 'main',
  table = 'diagnostic_predictors'
)

}
\seealso{
Other Prediction: 
\code{\link{getFullPredictionPerformances}()},
\code{\link{getPredictionAggregateTopPredictors}()},
\code{\link{getPredictionCohorts}()},
\code{\link{getPredictionCovariates}()},
\code{\link{getPredictionDiagnostics}()},
\code{\link{getPredictionHyperParamSearch}()},
\code{\link{getPredictionIntercept}()},
\code{\link{getPredictionLift}()},
\code{\link{getPredictionModelDesigns}()},
\code{\link{getPredictionOutcomes}()},
\code{\link{getPredictionPerformanceTable}()},
\code{\link{getPredictionPerformances}()},
\code{\link{getPredictionTargets}()},
\code{\link{getPredictionTopPredictors}()}
}
\concept{Prediction}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EstimationQueries.R
\name{.getSccsVersion}
\alias{.getSccsVersion}
\title{An internal function to determine the version of SCCS used 
to store results}
\usage{
.getSccsVersion(connectionHandler, schema, sccsTablePrefix = "sccs_")
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{sccsTablePrefix}{The prefix used for the cohort generator results tables}
}
\value{
A integer with the major version number of cohort method
}
\description{
An internal function to determine the version of SCCS used 
to store results
}
\details{
Specify the connectionHandler, the schema and the prefixes. This
query will attempt to identify if SelfControlledCaseSeries v6.1.4 was used by 
inspecing the migration table. When the migration_order is >= 3
then v6.1.4 of SelfControlledCaseSeries was used.
}
\seealso{
Other Estimation: 
\code{\link{.getCmVersion}()},
\code{\link{getCMEstimation}()},
\code{\link{getCmDiagnosticsData}()},
\code{\link{getCmMetaEstimation}()},
\code{\link{getCmNegativeControlEstimates}()},
\code{\link{getCmOutcomes}()},
\code{\link{getCmPropensityModel}()},
\code{\link{getCmTable}()},
\code{\link{getCmTargets}()},
\code{\link{getSccsDiagnosticsData}()},
\code{\link{getSccsEstimation}()},
\code{\link{getSccsMetaEstimation}()},
\code{\link{getSccsModel}()},
\code{\link{getSccsNegativeControlEstimates}()},
\code{\link{getSccsOutcomes}()},
\code{\link{getSccsTable}()},
\code{\link{getSccsTargets}()},
\code{\link{getSccsTimeToEvent}()},
\code{\link{plotCmEstimates}()},
\code{\link{plotSccsEstimates}()}
}
\concept{Estimation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CharacterzationQueries.R
\name{getDechallengeRechallenge}
\alias{getDechallengeRechallenge}
\title{Extract the dechallenge rechallenge results}
\usage{
getDechallengeRechallenge(
  connectionHandler,
  schema,
  cTablePrefix = "c_",
  cgTablePrefix = "cg_",
  databaseTable = "database_meta_data",
  targetIds = NULL,
  outcomeIds = NULL
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{cTablePrefix}{The prefix used for the characterization results tables}

\item{cgTablePrefix}{The prefix used for the cohort generator results tables}

\item{databaseTable}{The name of the table with the database details (default 'database_meta_data')}

\item{targetIds}{A vector of integers corresponding to the target cohort IDs}

\item{outcomeIds}{A vector of integers corresponding to the outcome cohort IDs}
}
\value{
Returns a data.frame with the columns:
\itemize{
 \item{databaseName the name of the database}
 \item{databaseId the unique identifier of the database}
 \item{targetName the target cohort name}
 \item{targetId the target cohort unique identifier}
 \item{outcomeName the outcome name}
 \item{outcomeId the outcome unique identifier}
 \item{dechallengeStopInterval An integer specifying the how much time to add to the cohort_end when determining whether the event starts during cohort and ends after}
 \item{dechallengeEvaluationWindow A period of time evaluated for outcome recurrence after discontinuation of exposure, among patients with challenge outcomes}
 \item{numExposureEras Distinct number of exposure events (i.e. drug eras) in a given target cohort}
 \item{numPersonsExposed Distinct number of people exposed in target cohort. A person must have at least 1 day exposure to be included}
 \item{numCases Distinct number of persons in outcome cohort. A person must have at least 1 day of observation time to be included}
 \item{dechallengeAttempt Distinct count of people with observable time after discontinuation of the exposure era during which the challenge outcome occurred}
 \item{dechallengeFail Among people with challenge outcomes, the distinct number of people with outcomes during dechallengeEvaluationWindow}
 \item{dechallengeSuccess Among people with challenge outcomes, the distinct number of people without outcomes during the dechallengeEvaluationWindow}
 \item{rechallengeAttempt Number of people with a new exposure era after the occurrence of an outcome during a prior exposure era}
 \item{rechallengeFail Number of people with a new exposure era during which an outcome occurred, after the occurrence of an outcome during a prior exposure era}
 \item{rechallengeSuccess Number of people with a new exposure era during which an outcome did not occur, after the occurrence of an outcome during a prior exposure era}
 \item{pctDechallengeAttempt Percent of people with observable time after discontinuation of the exposure era during which the challenge outcome occurred}
 \item{pctDechallengeFail Among people with challenge outcomes, the percent of people without outcomes during the dechallengeEvaluationWindow}
 \item{pctDechallengeSuccess Among people with challenge outcomes, the percent of people with outcomes during dechallengeEvaluationWindow}
 \item{pctRechallengeAttempt Percent of people with a new exposure era after the occurrence of an outcome during a prior exposure era}
 \item{pctRechallengeFail Percent of people with a new exposure era during which an outcome did not occur, after the occurrence of an outcome during a prior exposure era}
 \item{pctRechallengeSuccess Percent of people with a new exposure era during which an outcome occurred, after the occurrence of an outcome during a prior exposure era}
 }
}
\description{
This function extracts all dechallenge rechallenge results across databases for specified target and outcome cohorts.
}
\details{
Specify the connectionHandler, the schema and the target/outcome cohort IDs
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

dcrc <- getDechallengeRechallenge(
connectionHandler = connectionHandler, 
schema = 'main'
)

}
\seealso{
Other Characterization: 
\code{\link{getBinaryCaseSeries}()},
\code{\link{getBinaryRiskFactors}()},
\code{\link{getBinaryTargetBaseline}()},
\code{\link{getCaseCounts}()},
\code{\link{getCaseTargetCounts}()},
\code{\link{getCharacterizationCohortBinary}()},
\code{\link{getCharacterizationCohortContinuous}()},
\code{\link{getCharacterizationDemographics}()},
\code{\link{getCharacterizationOutcomes}()},
\code{\link{getCharacterizationTargets}()},
\code{\link{getContinuousCaseSeries}()},
\code{\link{getContinuousRiskFactors}()},
\code{\link{getDechallengeRechallengeFails}()},
\code{\link{getIncidenceOutcomes}()},
\code{\link{getIncidenceRates}()},
\code{\link{getIncidenceTargets}()},
\code{\link{getTargetBinaryFeatures}()},
\code{\link{getTargetContinuousFeatures}()},
\code{\link{getTimeToEvent}()},
\code{\link{plotAgeDistributions}()},
\code{\link{plotSexDistributions}()},
\code{\link{viewIncidenceRate}()}
}
\concept{Characterization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreatmentPatternsQueries.R
\name{getAnalysisCohorts}
\alias{getAnalysisCohorts}
\title{Extracts the different analyses ran for each target and event cohorts}
\usage{
getAnalysisCohorts(connectionHandler, schema, tpTablePrefix = "tp_")
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{tpTablePrefix}{The prefix used for the cohort generator results tables}
}
\value{
Returns a data.frame with the columns:
\itemize{
 \item{databaseName a concatinated string of all the database names ran for that analysis}
 \item{databaseId a concatinated string of all the database ids ran for that analysis}
 \item{analysisId the analysis ids for the treament patterns run}
 \item{targetCohortName the target cohort name}
 \item{targetCohortId the target cohort unique identifier}
 \item{eventCohortList a concatinated string of all the event cohort names ran for that target}
 \item{exitCohortList a concatinated string of all the exit cohort names ran for that target}
 }
}
\description{
This function extracts analysis ids, events cohorts, and databases per target from treatment patterns
}
\details{
Specify the connectionHandler and the schema
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

cohortAnalysis <- getAnalysisCohorts(
  connectionHandler = connectionHandler,
  schema = "main"
)
}
\seealso{
Other TreatmentPatterns: 
\code{\link{getEventDuration}()},
\code{\link{getTreatmentPathways}()}
}
\concept{TreatmentPatterns}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CohortQueries.R
\name{getCohortMeta}
\alias{getCohortMeta}
\title{Extract the cohort meta}
\usage{
getCohortMeta(
  connectionHandler,
  schema,
  cgTablePrefix = "cg_",
  databaseTable = "database_meta_data",
  cohortIds = NULL
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{cgTablePrefix}{The prefix used for the cohort generator results tables}

\item{databaseTable}{The name of the table with the database details (default 'database_meta_data')}

\item{cohortIds}{Optionally a list of cohortIds to restrict to}
}
\value{
Returns a data.frame with the cohort inclusion rules
}
\description{
This function extracts all cohort meta for the cohorts of interest.
}
\details{
Specify the connectionHandler, the schema and the cohort IDs
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

cohortMeta <- getCohortMeta(
  connectionHandler = connectionHandler, 
  schema = 'main'
)

}
\seealso{
Other Cohorts: 
\code{\link{getCohortAttrition}()},
\code{\link{getCohortCounts}()},
\code{\link{getCohortDefinitions}()},
\code{\link{getCohortInclusionRules}()},
\code{\link{getCohortInclusionStats}()},
\code{\link{getCohortInclusionSummary}()},
\code{\link{getCohortSubsetAttrition}()},
\code{\link{getCohortSubsetDefinitions}()},
\code{\link{getSubsetText}()},
\code{\link{processCohortDefinitionsForQuarto}()},
\code{\link{processCohorts}()},
\code{\link{restrictCohortDefinitionsForQuarto}()}
}
\concept{Cohorts}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TargetOutcomeInfo.R
\name{getOutcomeTable}
\alias{getOutcomeTable}
\title{Extract the outcome cohorts and where they are used in the analyses.}
\usage{
getOutcomeTable(
  connectionHandler,
  schema,
  cgTablePrefix = "cg_",
  cTablePrefix = "c_",
  ciTablePrefix = "ci_",
  cmTablePrefix = "cm_",
  sccsTablePrefix = "sccs_",
  plpTablePrefix = "plp_",
  databaseTable = "database_meta_data",
  targetId = NULL,
  getIncidenceInclusion = TRUE,
  getCharacterizationInclusion = TRUE,
  getPredictionInclusion = TRUE,
  getCohortMethodInclusion = TRUE,
  getSccsInclusion = TRUE,
  printTimes = FALSE
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{cgTablePrefix}{The prefix used for the cohort generator results tables}

\item{cTablePrefix}{The prefix used for the characterization results tables}

\item{ciTablePrefix}{The prefix used for the cohort incidence results tables}

\item{cmTablePrefix}{The prefix used for the cohort method results tables}

\item{sccsTablePrefix}{The prefix used for the cohort generator results tables}

\item{plpTablePrefix}{The prefix used for the patient level prediction results tables}

\item{databaseTable}{The name of the table with the database details (default 'database_meta_data')}

\item{targetId}{An integer corresponding to the target cohort ID}

\item{getIncidenceInclusion}{Whether to check usage of the cohort in incidence}

\item{getCharacterizationInclusion}{Whether to check usage of the cohort in characterization}

\item{getPredictionInclusion}{Whether to check usage of the cohort in prediction}

\item{getCohortMethodInclusion}{Whether to check usage of the cohort in cohort method}

\item{getSccsInclusion}{Whether to check usage of the cohort in SCCS}

\item{printTimes}{whether to print the time it takes to run each SQL query}
}
\value{
Returns a data.frame with the columns: 
\itemize{
 \item{cohortId the number id for the target cohort}
 \item{cohortName the name of the cohort}
 \item{subsetParent the number id of the parent cohort}
 \item{subsetDefinitionId the number id of the subset}
 \item{numDatabase number of databases with the cohort}
 \item{databaseString all the names of the databases with the cohort}
 \item{databaseIdString all the ids of the databases with the cohort}
 \item{databaseStringCount all the names of the databases with the cohort plus their counts}
 \item{databaseCount all the names of the databases with the cohort and their sizes}
 \item{minSubjectCount number of subjects in databases with lowest count}
 \item{maxSubjectCount number of subjects in databases with highest count}
 \item{minEntryCount number of entries in databases with lowest count}
 \item{maxEntryCount number of entries in databases with highest count}
 \item{cohortIncidence whether the cohort was used in cohort incidence}
 \item{dechalRechal whether the cohort was used in dechallenge rechallenge}
 \item{riskFactors whether the cohort was used in risk factors}
 \item{caseSeries whether the cohort was used in case series analysis}
 \item{timeToEvent whether the cohort was used in time to event}
 \item{prediction whether the cohort was used in prediction}
 \item{cohortMethod whether the cohort was used in cohort method}
 \item{selfControlledCaseSeries whether the cohort was used in self controlled case series}
}
}
\description{
This function extracts the outcome cohorts, the number of subjects/entries and where the cohort was used.
}
\details{
Specify the connectionHandler, the schema and the table prefixes
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

outcomeTable <- getOutcomeTable(
  connectionHandler = connectionHandler, 
  schema = 'main'
)

}
\seealso{
Other helper: 
\code{\link{addTarColumn}()},
\code{\link{formatBinaryCovariateName}()},
\code{\link{getExampleConnectionDetails}()},
\code{\link{getTargetTable}()},
\code{\link{kableDark}()},
\code{\link{removeSpaces}()}
}
\concept{helper}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EstimationQueries.R
\name{getSccsEstimation}
\alias{getSccsEstimation}
\title{Extract the self controlled case series (sccs) results}
\usage{
getSccsEstimation(
  connectionHandler,
  schema,
  sccsTablePrefix = "sccs_",
  cgTablePrefix = "cg_",
  databaseTable = "database_meta_data",
  targetIds = NULL,
  outcomeIds = NULL
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{sccsTablePrefix}{The prefix used for the cohort generator results tables}

\item{cgTablePrefix}{The prefix used for the cohort generator results tables}

\item{databaseTable}{The name of the table with the database details (default 'database_meta_data')}

\item{targetIds}{A vector of integers corresponding to the target cohort IDs}

\item{outcomeIds}{A vector of integers corresponding to the outcome cohort IDs}
}
\value{
Returns a data.frame with the columns:
\itemize{
 \item{databaseName the database name}
 \item{databaseId the database id}
 \item{exposuresOutcomeSetId the exposure outcome set identifier}
 \item{analysisId the analysis unique identifier}
 \item{description an analysis description}
 \item{targetName the target name}
 \item{targetId the target cohort id}
 \item{outcomeName the outcome name}
 \item{outcomeId the outcome cohort id}
 \item{indicationName the indication name}
 \item{indicatonId the indication cohort id}
 \item{covariateName whether main or secondary analysis}
 \item{covariateId the analysis id}
 \item{outcomeSubjects The number of subjects with at least one outcome.}
 \item{outcomeEvents The number of outcome events.}
 \item{outcomeObservationPeriods The number of observation periods containing at least one outcome.}
 \item{covariateSubjects The number of subjects having the covariate.}
 \item{covariateDays The total covariate time in days.}
 \item{covariateEras The number of continuous eras of the covariate.}
 \item{covariateOutcomes The number of outcomes observed during the covariate time.}
 \item{observedDays The number of days subjects were observed.}
 \item{rr the relative risk}
 \item{ci95Lb the lower bound of the 95 percent confidence interval for the relative risk}
 \item{ci95Ub the upper bound of the 95 percent confidence interval for the relative risk}
 \item{p the p-value for the relative risk}
 \item{logRr the log of the relative risk}
 \item{seLogRr the standard error or the log of the relative risk}
 \item{calibratedRr the calibrated relative risk}
 \item{calibratedCi95Lb the lower bound of the 95 percent confidence interval for the calibrated relative risk}
 \item{calibratedCi95Ub the upper bound of the 95 percent confidence interval for the calibrated relative risk}
 \item{calibratedP the calibrated p-value}
 \item{calibratedOneSidedP the calibrated one sided p-value}
 \item{calibratedLogRr the calibrated log of the relative risk}
 \item{calibratedSeLogRr the calibrated log of the relative risk standard error}
 \item{llr The log of the likelihood ratio (of the MLE vs the null hypothesis of no effect).}
 \item{mdrr The minimum detectable relative risk.}
 \item{unblind Whether the results can be unblinded}
 \item{unblindForEvidenceSynthesis whether the results can be unblinded for the meta analysis.}
 }
}
\description{
This function extracts the single database sccs estimates
}
\details{
Specify the connectionHandler, the schema and the target/outcome cohort IDs
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

sccsEst <- getSccsEstimation(
  connectionHandler = connectionHandler, 
  schema = 'main',
  targetIds = 1,
  outcomeIds = 3
)

}
\seealso{
Other Estimation: 
\code{\link{.getCmVersion}()},
\code{\link{.getSccsVersion}()},
\code{\link{getCMEstimation}()},
\code{\link{getCmDiagnosticsData}()},
\code{\link{getCmMetaEstimation}()},
\code{\link{getCmNegativeControlEstimates}()},
\code{\link{getCmOutcomes}()},
\code{\link{getCmPropensityModel}()},
\code{\link{getCmTable}()},
\code{\link{getCmTargets}()},
\code{\link{getSccsDiagnosticsData}()},
\code{\link{getSccsMetaEstimation}()},
\code{\link{getSccsModel}()},
\code{\link{getSccsNegativeControlEstimates}()},
\code{\link{getSccsOutcomes}()},
\code{\link{getSccsTable}()},
\code{\link{getSccsTargets}()},
\code{\link{getSccsTimeToEvent}()},
\code{\link{plotCmEstimates}()},
\code{\link{plotSccsEstimates}()}
}
\concept{Estimation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CohortQueries.R
\name{processCohortDefinitionsForQuarto}
\alias{processCohortDefinitionsForQuarto}
\title{Function processes the cohortDefinitions object ready for use in the main quarto report.}
\usage{
processCohortDefinitionsForQuarto(
  cohortDefinitions,
  friendlyCohortIds,
  friendlyCohortNames,
  restrictTargetToIndications,
  indicationIds
)
}
\arguments{
\item{cohortDefinitions}{The output of `getCohortDefinitions()`}

\item{friendlyCohortIds}{a vector of parent cohort ids that you want to rename}

\item{friendlyCohortNames}{a vector of new names for the friendlyCohortIds}

\item{restrictTargetToIndications}{whether to restrict the results to cohorts nested in certain indications}

\item{indicationIds}{The indication ids of interest for restrictTargetToIndications}
}
\value{
A cohortDefinitions object with extra columns: friendlyName,
}
\description{
Function processes the cohortDefinitions object ready for use in the main quarto report.
}
\details{
Function processes the cohortDefinitions object by adding friendly names for 
specified parent cohorts, determines which cohorts are nested in the specified
indication cohort ids and ...
}
\seealso{
Other Cohorts: 
\code{\link{getCohortAttrition}()},
\code{\link{getCohortCounts}()},
\code{\link{getCohortDefinitions}()},
\code{\link{getCohortInclusionRules}()},
\code{\link{getCohortInclusionStats}()},
\code{\link{getCohortInclusionSummary}()},
\code{\link{getCohortMeta}()},
\code{\link{getCohortSubsetAttrition}()},
\code{\link{getCohortSubsetDefinitions}()},
\code{\link{getSubsetText}()},
\code{\link{processCohorts}()},
\code{\link{restrictCohortDefinitionsForQuarto}()}
}
\concept{Cohorts}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EstimationQueries.R
\name{getSccsOutcomes}
\alias{getSccsOutcomes}
\title{A function to extract the outcomes found in self controlled case series}
\usage{
getSccsOutcomes(
  connectionHandler,
  schema,
  sccsTablePrefix = "sccs_",
  cgTablePrefix = "cg_",
  targetId = NULL
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{sccsTablePrefix}{The prefix used for the cohort generator results tables}

\item{cgTablePrefix}{The prefix used for the cohort generator results tables}

\item{targetId}{An integer corresponding to the target cohort ID}
}
\value{
A data.frame with the self controlled case series outcome ids and names.
}
\description{
A function to extract the outcomes found in self controlled case series
}
\details{
Specify the connectionHandler, the schema and the prefixes
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

outcomes <- getSccsOutcomes(
  connectionHandler = connectionHandler, 
  schema = 'main'
)

}
\seealso{
Other Estimation: 
\code{\link{.getCmVersion}()},
\code{\link{.getSccsVersion}()},
\code{\link{getCMEstimation}()},
\code{\link{getCmDiagnosticsData}()},
\code{\link{getCmMetaEstimation}()},
\code{\link{getCmNegativeControlEstimates}()},
\code{\link{getCmOutcomes}()},
\code{\link{getCmPropensityModel}()},
\code{\link{getCmTable}()},
\code{\link{getCmTargets}()},
\code{\link{getSccsDiagnosticsData}()},
\code{\link{getSccsEstimation}()},
\code{\link{getSccsMetaEstimation}()},
\code{\link{getSccsModel}()},
\code{\link{getSccsNegativeControlEstimates}()},
\code{\link{getSccsTable}()},
\code{\link{getSccsTargets}()},
\code{\link{getSccsTimeToEvent}()},
\code{\link{plotCmEstimates}()},
\code{\link{plotSccsEstimates}()}
}
\concept{Estimation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PredictionReport.R
\name{createPredictionReport}
\alias{createPredictionReport}
\title{createPredictionReport}
\usage{
createPredictionReport(
  connectionHandler,
  schema,
  plpTablePrefix,
  databaseTablePrefix = plpTablePrefix,
  cgTablePrefix = plpTablePrefix,
  modelDesignId,
  output,
  intermediatesDir = file.path(tempdir(), "plp-prot"),
  outputFormat = "html_document"
)
}
\arguments{
\item{connectionHandler}{The connection handler to the results database}

\item{schema}{The result database schema}

\item{plpTablePrefix}{The prediction table prefix}

\item{databaseTablePrefix}{The database table name e.g., database_meta_data}

\item{cgTablePrefix}{The cohort generator table prefix}

\item{modelDesignId}{The model design ID of interest}

\item{output}{The folder name where main.html will be save to}

\item{intermediatesDir}{The work directory for rmarkdown}

\item{outputFormat}{the type of outcome html_document or html_fragment}
}
\value{
An named R list with the elements 'standard' and 'source'
}
\description{
Generates a report for a given prediction model design
}
\details{
Specify the connection handler to the result database, the schema name
and the modelDesignId of interest to generate a html report summarizing the
performance of models developed across databases.
}
\seealso{
Other Reporting: 
\code{\link{generateFullReport}()},
\code{\link{generateSummaryPredictionReport}()}
}
\concept{Reporting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EstimationQueries.R
\name{getSccsDiagnosticsData}
\alias{getSccsDiagnosticsData}
\title{Extract the self controlled case series (sccs) diagostic results}
\usage{
getSccsDiagnosticsData(
  connectionHandler,
  schema,
  sccsTablePrefix = "sccs_",
  cgTablePrefix = "cg_",
  databaseTable = "database_meta_data",
  targetIds = NULL,
  outcomeIds = NULL
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{sccsTablePrefix}{The prefix used for the cohort generator results tables}

\item{cgTablePrefix}{The prefix used for the cohort generator results tables}

\item{databaseTable}{The name of the table with the database details (default 'database_meta_data')}

\item{targetIds}{A vector of integers corresponding to the target cohort IDs}

\item{outcomeIds}{A vector of integers corresponding to the outcome cohort IDs}
}
\value{
Returns a data.frame with the columns:
\itemize{
 \item{databaseName the database name}
 \item{databaseId the unique identifier for the database}
 \item{analysisId the analysis unique identifier}
 \item{description an analysis description}
 \item{targetName the target name}
 \item{targetId the target cohort id}
 \item{outcomeName the outcome name}
 \item{outcomeId the outcome cohort id}
 \item{indicationName the indication name}
 \item{indicatonId the indication cohort id}
 \item{covariateName whether main or secondary analysis}
 \item{mdrr the maximum passable minimum detectable relative risk (mdrr) value.  If the mdrr is greater than this the diagnostics will fail.}
 \item{ease The expected absolute systematic error (ease) measures residual bias.}
 \item{timeTrendP (Depreciated to timeStabilityP) The p for whether the mean monthly ratio between observed and expected is no greater than 1.25.}
 \item{preExposureP (Depreciated) One-sided p-value for whether the rate before expore is higher than after, against the null of no difference.}
 \item{mdrrDiagnostic whether the mdrr (power) diagnostic passed or failed.}
 \item{easeDiagnostic whether the ease diagnostic passed or failed.}
 \item{timeStabilityP (New) The p for whether the mean monthly ratio between observed and expected exceeds the specified threshold.}
 \item{eventExposureLb (New) Lower bound of the 95\% CI for the pre-expososure estimate.}
 \item{eventExposureUb (New) Upper bound of the 95\% CI for the pre-expososure estimate.}
 \item{eventObservationLb (New) Lower bound of the 95\% CI for the end of observation probe estimate.}
 \item{eventObservationUb (New) Upper bound of the 95\% CI for the end of observation probe estimate.}
 \item{rareOutcomePrevalence (New) The proportion of people in the underlying population who have the outcome at least once.}
 \item{timeTrendDiagnostic (Depreciated) Pass / warning / fail / not evaluated classification of the time trend (unstalbe months) diagnostic.}
 \item{preExposureDiagnostic (Depreciated) Pass / warning / fail / not evaluated classification of the time trend (unstalbe months) diagnostic.}
 \item{timeStabilityDiagnostic (New) Pass / fail / not evaluated classification of the time stability diagnostic.}
 \item{eventExposureDiagnostic (New) Pass / fail / not evaluated classification of the event-exposure independence diagnostic.}
 \item{eventObservationDiagnostic (New) Pass / fail / not evaluated classification of the event-observation period dependence diagnostic.}
 \item{rareOutcomeDiagnostic (New) Pass / fail / not evaluated classification of the rare outcome diagnostic.}
 \item{unblind whether the results can be unblinded.}
 \item{unblindForEvidenceSynthesis whether the results can be unblinded for the meta analysis.}
 \item{summaryValue summary of diagnostics results. FAIL, PASS or number of warnings.}
 }
}
\description{
This function extracts the sccs diagnostics that examine whether the analyses were sufficiently powered
and checks for different types of bias.
}
\details{
Specify the connectionHandler, the schema and the target/outcome cohort IDs
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

sccsDiag <- getSccsDiagnosticsData(
  connectionHandler = connectionHandler, 
  schema = 'main',
  targetIds = 1,
  outcomeIds = 3
)

}
\seealso{
Other Estimation: 
\code{\link{.getCmVersion}()},
\code{\link{.getSccsVersion}()},
\code{\link{getCMEstimation}()},
\code{\link{getCmDiagnosticsData}()},
\code{\link{getCmMetaEstimation}()},
\code{\link{getCmNegativeControlEstimates}()},
\code{\link{getCmOutcomes}()},
\code{\link{getCmPropensityModel}()},
\code{\link{getCmTable}()},
\code{\link{getCmTargets}()},
\code{\link{getSccsEstimation}()},
\code{\link{getSccsMetaEstimation}()},
\code{\link{getSccsModel}()},
\code{\link{getSccsNegativeControlEstimates}()},
\code{\link{getSccsOutcomes}()},
\code{\link{getSccsTable}()},
\code{\link{getSccsTargets}()},
\code{\link{getSccsTimeToEvent}()},
\code{\link{plotCmEstimates}()},
\code{\link{plotSccsEstimates}()}
}
\concept{Estimation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EstimationQueries.R
\name{getCMEstimation}
\alias{getCMEstimation}
\title{Extract the cohort method results}
\usage{
getCMEstimation(
  connectionHandler,
  schema,
  cmTablePrefix = "cm_",
  cgTablePrefix = "cg_",
  databaseTable = "database_meta_data",
  targetIds = NULL,
  outcomeIds = NULL,
  indicationIds = NULL,
  comparatorIds = NULL
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{cmTablePrefix}{The prefix used for the cohort method results tables}

\item{cgTablePrefix}{The prefix used for the cohort generator results tables}

\item{databaseTable}{The name of the table with the database details (default 'database_meta_data')}

\item{targetIds}{A vector of integers corresponding to the target cohort IDs}

\item{outcomeIds}{A vector of integers corresponding to the outcome cohort IDs}

\item{indicationIds}{A vector of cohort ids for the indication (nesting id) to restrict to}

\item{comparatorIds}{A vector of integers corresponding to the comparator cohort IDs}
}
\value{
Returns a data.frame with the columns:
\itemize{
 \item{databaseName the name of the database}
 \item{databaseId the unqiue identifier of the database}
 \item{analysisId the analysis design unique identifier}
 \item{description the analysis design description}
 \item{targetName the target cohort name}
 \item{targetId the target cohort unique identifier}
 \item{comparatorName the comparator cohort name}
 \item{comparatorId the comparator cohort unique identifier}
 \item{outcomeName the outcome name}
 \item{outcomeId the outcome unique identifier}
 \item{calibratedRr the calibrated relative risk}
 \item{calibratedRrCi95Lb the calibrated relative risk 95 percent confidence interval lower bound}
 \item{calibratedRrCi95Ub the calibrated relative risk 95 percent confidence interval upper bound}
 \item{calibratedP the two sided calibrated p value}
 \item{calibratedOneSidedP the one sided calibrated p value}
 \item{calibratedLogRr the calibrated relative risk logged}
 \item{calibratedSeLogRr the standard error of the calibrated relative risk logged}
 \item{targetSubjects the number of people in the target cohort}
 \item{comparatorSubjects the number of people in the comparator cohort}
 \item{targetDays the total number of days at risk across the target cohort people}
 \item{comparatorDays the total number of days at risk across the comparator cohort people}
 \item{targetOutcomes the total number of outcomes occuring during the time at risk for the target cohort people}
 \item{comparatorOutcomes the total number of outcomes occuring during the time at risk for the comparator cohort people}
 \item{Unblind Whether the results passed diagnostics and were unblinded}
 \item{unblindForEvidenceSynthesis whether the results can be unblinded for the meta analysis.}
 \item{targetEstimator ...}
 }
}
\description{
This function extracts the single database cohort method estimates for results that can be unblinded and have a calibrated RR
}
\details{
Specify the connectionHandler, the schema and the target/comparator/outcome cohort IDs
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

cmEst <- getCMEstimation(
  connectionHandler = connectionHandler, 
  schema = 'main',
  targetIds = 1,
  outcomeIds = 3
)

}
\seealso{
Other Estimation: 
\code{\link{.getCmVersion}()},
\code{\link{.getSccsVersion}()},
\code{\link{getCmDiagnosticsData}()},
\code{\link{getCmMetaEstimation}()},
\code{\link{getCmNegativeControlEstimates}()},
\code{\link{getCmOutcomes}()},
\code{\link{getCmPropensityModel}()},
\code{\link{getCmTable}()},
\code{\link{getCmTargets}()},
\code{\link{getSccsDiagnosticsData}()},
\code{\link{getSccsEstimation}()},
\code{\link{getSccsMetaEstimation}()},
\code{\link{getSccsModel}()},
\code{\link{getSccsNegativeControlEstimates}()},
\code{\link{getSccsOutcomes}()},
\code{\link{getSccsTable}()},
\code{\link{getSccsTargets}()},
\code{\link{getSccsTimeToEvent}()},
\code{\link{plotCmEstimates}()},
\code{\link{plotSccsEstimates}()}
}
\concept{Estimation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PredictionQueries.R
\name{getPredictionCohorts}
\alias{getPredictionCohorts}
\title{Extract a complete set of cohorts used in the prediction results}
\usage{
getPredictionCohorts(
  connectionHandler,
  schema,
  plpTablePrefix = "plp_",
  cgTablePrefix = "cg_"
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{plpTablePrefix}{The prefix used for the patient level prediction results tables}

\item{cgTablePrefix}{The prefix used for the cohort generator results tables}
}
\value{
Returns a data.frame with the columns: 
\itemize{
 \item{cohortId the cohort definition ID}
 \item{cohortName the name of the cohort}
 \item{type whether the cohort was used as a target or outcome cohort}
}
}
\description{
This function extracts the target and outcome cohorts used to develop any model in the results
}
\details{
Specify the connectionHandler, the resultDatabaseSettings and any targetIds or outcomeIds to restrict models to
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

predCohorts <- getPredictionCohorts(
  connectionHandler = connectionHandler, 
  schema = 'main'
)

}
\seealso{
Other Prediction: 
\code{\link{getFullPredictionPerformances}()},
\code{\link{getPredictionAggregateTopPredictors}()},
\code{\link{getPredictionCovariates}()},
\code{\link{getPredictionDiagnosticTable}()},
\code{\link{getPredictionDiagnostics}()},
\code{\link{getPredictionHyperParamSearch}()},
\code{\link{getPredictionIntercept}()},
\code{\link{getPredictionLift}()},
\code{\link{getPredictionModelDesigns}()},
\code{\link{getPredictionOutcomes}()},
\code{\link{getPredictionPerformanceTable}()},
\code{\link{getPredictionPerformances}()},
\code{\link{getPredictionTargets}()},
\code{\link{getPredictionTopPredictors}()}
}
\concept{Prediction}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Database.R
\name{getDatabaseDetails}
\alias{getDatabaseDetails}
\title{Extract the database used in the analyses}
\usage{
getDatabaseDetails(
  connectionHandler,
  schema,
  databaseTable = "database_meta_data"
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{databaseTable}{The name of the table with the database details (default 'database_meta_data')}
}
\value{
Returns a data.frame with the columns:
\itemize{
 \item{databaseFullName the full name of the database}
 \item{databaseName the friendly name of the database}
 \item{cdmHolder the license holder of the database}
 \item{sourceDescription a description of the database}
 \item{sourceDocumentationReference a link to the database information document}
 \item{cdmEtlReference a link to the ETL document}
 \item{sourceReleaseDatethe release date for the source database}
 \item(cdmReleaseDate the release date for the database mapped to the OMOP CDM)
 \item{cdmVersion the OMOP CDM version of the database}
 \item{cdmVersionConceptId the CDM version concept ID}
 \item{vocabularyVersion the database's vocabulary version}
 \item{databaseId a unique identifier for the database}
 \item{maxObsPeriodEndDate the last observational period end date in the database}
 }
}
\description{
This function extracts the databases and their information.
}
\details{
Specify the connectionHandler, the schema and the database table name
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

ir <- getIncidenceRates(
connectionHandler = connectionHandler, 
schema = 'main'
)

}
\concept{Database}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreatmentPatternsQueries.R
\name{getEventDuration}
\alias{getEventDuration}
\title{Extracts summary of event duration}
\usage{
getEventDuration(
  connectionHandler,
  schema,
  analysisIds,
  tpTablePrefix = "tp_",
  databaseTable = "database_meta_data",
  databaseIds = NULL,
  databaseNames = NULL,
  targetIds = NULL
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{analysisIds}{A vector of analysis ids to restrict to}

\item{tpTablePrefix}{The prefix used for the cohort generator results tables}

\item{databaseTable}{The name of the table with the database details (default 'database_meta_data')}

\item{databaseIds}{(optional) A vector of database ids to restrict to}

\item{databaseNames}{(optional) A vector of database Names to restrict to}

\item{targetIds}{(optional) A vector of target cohort ids to restrict to}
}
\value{
Returns a data.frame with the columns:
\itemize{
 \item{databaseName the name of the database}
 \item{databaseId the unique identifier of the database}
 \item{analysisId the unique identifier of a treament patterns run}
 \item{targetCohortId the target cohort unique identifier}
 \item{targetCohortName the target cohort name}
 \item{eventName a string representing an events for a target. Uses '+' for combination of event cohorts}
 \item{rank the step number of event occurance}
 \item{eventCount the count of event occurance at rank}
 \item{durationAverage the average duration of event}
 \item{durationMax the maximum duration of event}
 \item{durationMin the minimum duration of event}
 \item{durationMedian the median duration of event}
 \item{p25Value the 25th percentile for duration of event}
 \item{p75Value the 75th percentile for duration of event}
 \item{standardDeviation the standard deviation for duration of event}
 }
}
\description{
This function extracts results summary stats of event duration for specified analysis ids and target cohorts
}
\details{
Specify the connectionHandler, the schema, and the analysisIds
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

ed <- getEventDuration(
  connectionHandler = connectionHandler,
  schema = "main",
  analysisIds = c(1)
)
}
\seealso{
Other TreatmentPatterns: 
\code{\link{getAnalysisCohorts}()},
\code{\link{getTreatmentPathways}()}
}
\concept{TreatmentPatterns}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PredictionQueries.R
\name{getPredictionIntercept}
\alias{getPredictionIntercept}
\title{Extract model interception (for logistic regression)}
\usage{
getPredictionIntercept(
  connectionHandler,
  schema,
  plpTablePrefix = "plp_",
  modelDesignId = NULL,
  databaseId = NULL
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{plpTablePrefix}{The prefix used for the patient level prediction results tables}

\item{modelDesignId}{The identifier for a model design  to restrict to}

\item{databaseId}{The identifier for the development database to restrict  to}
}
\value{
Returns a single value corresponding to the model intercept or NULL if not a logistic regression model
}
\description{
This function extracts the interception value
}
\details{
Specify the connectionHandler, the resultDatabaseSettings, the modelDesignId and the databaseId
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

intercepts <- getPredictionIntercept(
  connectionHandler = connectionHandler, 
  schema = 'main'
)

}
\seealso{
Other Prediction: 
\code{\link{getFullPredictionPerformances}()},
\code{\link{getPredictionAggregateTopPredictors}()},
\code{\link{getPredictionCohorts}()},
\code{\link{getPredictionCovariates}()},
\code{\link{getPredictionDiagnosticTable}()},
\code{\link{getPredictionDiagnostics}()},
\code{\link{getPredictionHyperParamSearch}()},
\code{\link{getPredictionLift}()},
\code{\link{getPredictionModelDesigns}()},
\code{\link{getPredictionOutcomes}()},
\code{\link{getPredictionPerformanceTable}()},
\code{\link{getPredictionPerformances}()},
\code{\link{getPredictionTargets}()},
\code{\link{getPredictionTopPredictors}()}
}
\concept{Prediction}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OhdsiReportGenerator.R
\docType{package}
\name{OhdsiReportGenerator}
\alias{OhdsiReportGenerator-package}
\alias{OhdsiReportGenerator}
\title{OhdsiReportGenerator}
\description{
A package for extracting analyses results and creating reports.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://ohdsi.github.io/OhdsiReportGenerator/}
  \item \url{https://github.com/OHDSI/OhdsiReportGenerator}
  \item Report bugs at \url{https://github.com/OHDSI/OhdsiReportGenerator/issues}
}

}
\author{
\strong{Maintainer}: Jenna Reps \email{jreps@its.jnj.com}

Authors:
\itemize{
  \item Anthony Sena \email{sena@ohdsi.org}
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EstimationPlots.R
\name{plotCmEstimates}
\alias{plotCmEstimates}
\title{Plots the cohort method results for one analysis}
\usage{
plotCmEstimates(
  cmData,
  cmDiagnostics = NULL,
  cmMeta = NULL,
  cohortNames = NULL,
  includeCounts = TRUE,
  selectedAnalysisId = NULL
)
}
\arguments{
\item{cmData}{The cohort method data}

\item{cmDiagnostics}{(optional) The cohort method diagnostic data}

\item{cmMeta}{(optional) The cohort method evidence synthesis data}

\item{cohortNames}{A data.frame with columns cohortId and cohortName}

\item{includeCounts}{Whether to include the target/comp size and event counts}

\item{selectedAnalysisId}{The analysis ID of interest to plot}
}
\value{
Returns a ggplot with the estimates
}
\description{
Creates nice cohort method plots
}
\details{
Input the cohort method data
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

cmEst <- getCMEstimation(
  connectionHandler = connectionHandler, 
  schema = 'main',
  targetIds = 1002,
  outcomeIds = 3
)
plotCmEstimates(
  cmData = cmEst, 
  cmMeta = NULL, 
  selectedAnalysisId = 1
)

}
\seealso{
Other Estimation: 
\code{\link{.getCmVersion}()},
\code{\link{.getSccsVersion}()},
\code{\link{getCMEstimation}()},
\code{\link{getCmDiagnosticsData}()},
\code{\link{getCmMetaEstimation}()},
\code{\link{getCmNegativeControlEstimates}()},
\code{\link{getCmOutcomes}()},
\code{\link{getCmPropensityModel}()},
\code{\link{getCmTable}()},
\code{\link{getCmTargets}()},
\code{\link{getSccsDiagnosticsData}()},
\code{\link{getSccsEstimation}()},
\code{\link{getSccsMetaEstimation}()},
\code{\link{getSccsModel}()},
\code{\link{getSccsNegativeControlEstimates}()},
\code{\link{getSccsOutcomes}()},
\code{\link{getSccsTable}()},
\code{\link{getSccsTargets}()},
\code{\link{getSccsTimeToEvent}()},
\code{\link{plotSccsEstimates}()}
}
\concept{Estimation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CohortQueries.R
\name{getSubsetText}
\alias{getSubsetText}
\title{Function that converts a subsetDefinitionJson into text description}
\usage{
getSubsetText(subsetDefinitionJson, cohortDefinitions)
}
\arguments{
\item{subsetDefinitionJson}{The subset logic json}

\item{cohortDefinitions}{A data.frame with the columns cohortDefinitionId, cohortName and optionally friendlyName that will
be used to know the friendly cohort name for any subsetting that nests in other cohorts}
}
\value{
A text string describing the subsetting
}
\description{
Function that converts a subsetDefinitionJson into text description
}
\details{
The function takes a subsetDefinitionJson and converts it into friendly text describing the 
subset logic
}
\seealso{
Other Cohorts: 
\code{\link{getCohortAttrition}()},
\code{\link{getCohortCounts}()},
\code{\link{getCohortDefinitions}()},
\code{\link{getCohortInclusionRules}()},
\code{\link{getCohortInclusionStats}()},
\code{\link{getCohortInclusionSummary}()},
\code{\link{getCohortMeta}()},
\code{\link{getCohortSubsetAttrition}()},
\code{\link{getCohortSubsetDefinitions}()},
\code{\link{processCohortDefinitionsForQuarto}()},
\code{\link{processCohorts}()},
\code{\link{restrictCohortDefinitionsForQuarto}()}
}
\concept{Cohorts}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CohortQueries.R
\name{getCohortSubsetAttrition}
\alias{getCohortSubsetAttrition}
\title{Get cohort subset attrition}
\usage{
getCohortSubsetAttrition(
  connectionHandler,
  schema,
  cgTablePrefix = "cg_",
  databaseTable = "database_meta_data",
  cohortIds = NULL
)
}
\arguments{
\item{connectionHandler}{A connection handler object.}

\item{schema}{The database schema name.}

\item{cgTablePrefix}{Prefix for cohort generator tables. Default is 'cg_'.}

\item{databaseTable}{Name of the database metadata table. Default is 'database_meta_data'.}

\item{cohortIds}{Optional vector of cohort IDs to filter.}
}
\value{
A data.frame with attrition details for each cohort subset.
}
\description{
Retrieves attrition information for specified cohort subsets from the database.
}
\seealso{
Other Cohorts: 
\code{\link{getCohortAttrition}()},
\code{\link{getCohortCounts}()},
\code{\link{getCohortDefinitions}()},
\code{\link{getCohortInclusionRules}()},
\code{\link{getCohortInclusionStats}()},
\code{\link{getCohortInclusionSummary}()},
\code{\link{getCohortMeta}()},
\code{\link{getCohortSubsetDefinitions}()},
\code{\link{getSubsetText}()},
\code{\link{processCohortDefinitionsForQuarto}()},
\code{\link{processCohorts}()},
\code{\link{restrictCohortDefinitionsForQuarto}()}
}
\concept{Cohorts}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PredictionQueries.R
\name{getFullPredictionPerformances}
\alias{getFullPredictionPerformances}
\title{Extract the model performances per evaluation}
\usage{
getFullPredictionPerformances(
  connectionHandler,
  schema,
  plpTablePrefix = "plp_",
  cgTablePrefix = "cg_",
  databaseTable = "database_meta_data",
  databaseTablePrefix = "",
  modelDesignId = NULL,
  developmentDatabaseId = NULL
)
}
\arguments{
\item{connectionHandler}{A connection handler that connects to the database and extracts sql queries.  Create a connection handler via `ResultModelManager::ConnectionHandler$new()`.}

\item{schema}{The result database schema (e.g., 'main' for sqlite)}

\item{plpTablePrefix}{The prefix used for the patient level prediction results tables}

\item{cgTablePrefix}{The prefix used for the cohort generator results tables}

\item{databaseTable}{The name of the table with the database details (default 'database_meta_data')}

\item{databaseTablePrefix}{A prefix to the database table, either '' or 'plp_'}

\item{modelDesignId}{The identifier for a model design  to restrict results to}

\item{developmentDatabaseId}{The identifier for the development database to restrict results to}
}
\value{
Returns a data.frame with the columns:
\itemize{  
 \item{timeStamp the date/time when the analysis occurred}
 \item{performanceId the unique identifier for the performance}
 \item{modelDesignId the unique identifier for the model design}
 \item{modelType the type of classifier}
 \item{algorithmName the model algorithm name}
 \item{covariateName a summary name for the candidate covariates}
 \item{developmentDatabaseId the unique identifier for the database used to develop the model}
 \item{validationDatabaseId the unique identifier for the database used to validate the model}
 \item{developmentTargetId the unique cohort id for the development target population}
 \item{developmentTargetName the name for the development target population}
 \item{validationTargetId the id for the validation target population}
 \item{validationTargetName the name for the validation target population if different from development}
 \item{developmentOutcomeId the unique cohort id for the development outcome}
 \item{developmentOutcomeName the name for the development outcome}
 \item{validationOutcomeId the id for the validation outcome}
 \item{validationOutcomeName the name for the validation outcome if different from development}
 \item{developmentDatabase the name for the database used to develop the model}
 \item{validationDatabase the name for the database used to validate the model if different from development}
 \item{validationTarId the validation time at risk id}
 \item{validationTimeAtRisk the time at risk used when evaluating the model if different from development}
 \item{developmentTarId the development time at risk id}
 \item{developmentTimeAtRisk the time at risk used when developing the model}
 \item{evaluation The type of evaluation: Test/Train/CV/Validation}
 \item{populationSize the test/validation population size used to develop the model}
 \item{outcomeCount the test/validation outcome count used to develop the model}
 \item{AUROC the AUROC value for the model}
 \item{95 lower AUROC: the lower bound of the 95 percent CI AUROC value for the model}
 \item{95 upper AUROC: the upper bound of the 95 percent CI AUROC value for the model}
 \item{AUPRC the discrimination AUPRC value for the model}
 \item{brier score: the brier value for the model}
 \item{brier score scaled: the scaled brier value for the model}
 \item{Average Precision: the average precision value for the model}
 \item{Eavg the calibration average error e-statistic value for the model}
 \item{E90 the calibration 90 percent upper bound e-statistic value for the model}
 \item{Emax the calibration max error e-statistic value for the model}
 \item{calibrationInLarge mean prediction: the  calibration in the large mean predicted risk value for the model}
 \item{calibrationInLarge observed risk: the calibration in the large mean observed risk value for the model}
 \item{calibrationInLarge intercept: the calibration in the large value intercept for the model}
 \item{weak calibration intercept: the weak calibration intercept for the model}
 \item{weak calibration gradient: the weak calibration gradient for the model}
 \item{Hosmer Lemeshow calibration intercept: the Hosmer Lemeshow calibration intercept for the model}
 \item{Hosmer Lemeshow calibration gradient: the Hosmer Lemeshow calibration gradient for the model}
 \item{... Additional metrics that are added to PLP}
}
}
\description{
This function extracts the model performances per evaluation
}
\details{
Specify the connectionHandler, the resultDatabaseSettings and (optionally) a modelDesignId and/or developmentDatabaseId to restrict models to
}
\examples{
conDet <- getExampleConnectionDetails()

connectionHandler <- ResultModelManager::ConnectionHandler$new(conDet)

perf <- getFullPredictionPerformances(
  connectionHandler = connectionHandler, 
  schema = 'main'
)

}
\seealso{
Other Prediction: 
\code{\link{getPredictionAggregateTopPredictors}()},
\code{\link{getPredictionCohorts}()},
\code{\link{getPredictionCovariates}()},
\code{\link{getPredictionDiagnosticTable}()},
\code{\link{getPredictionDiagnostics}()},
\code{\link{getPredictionHyperParamSearch}()},
\code{\link{getPredictionIntercept}()},
\code{\link{getPredictionLift}()},
\code{\link{getPredictionModelDesigns}()},
\code{\link{getPredictionOutcomes}()},
\code{\link{getPredictionPerformanceTable}()},
\code{\link{getPredictionPerformances}()},
\code{\link{getPredictionTargets}()},
\code{\link{getPredictionTopPredictors}()}
}
\concept{Prediction}
